//******************************************************************************
// Header file for color palettes                                    CSI_color.h
//
//  for Large display project
//              
//==============================================================================
#ifndef CSI_COLOR_H_
#define CSI_COLOR_H_

#include <stdint.h>

// RGB structure
typedef struct
{
  unsigned char red;
  unsigned char green;
  unsigned char blue;
} RGB_color;

// RGB triplets
extern RGB_color cg_font;
extern RGB_color cg_background;
extern RGB_color highlight;
extern RGB_color border;

extern unsigned char cur_foreground;  
extern unsigned char cur_background;

extern RGB_color current_palette[16];

extern uint8_t day_bright_val;
extern uint8_t night_bright_val;

/* not being used

// test and see if the 24bit packed struct takes less space than
// 3 separate unsigned char
// use if saving space is needed
typedef struct
{
  unsigned int red   :8;
  unsigned int green :8;
  unsigned int blue  :8;
} Packed_RGB_color;
*/

// VGA standard palette, HTML standard 4.01
extern RGB_color standard_palette[16];
// Rainbow palette
extern RGB_color rainbow_palette[16];
// Greyscale palette
extern RGB_color greyscale_palette[16];
// Default palette, more useful and less garish than VGA
extern RGB_color default_palette[16];
// Designer palette, fancy!
extern RGB_color designer_palette[16];

// color functions
int  get_pal_color( uint8_t color_num );
void set_pal_color( uint8_t color_num, int pal_num );

void display_color_block( uint16_t startX, uint16_t startY, int num,
                         uint16_t width, uint16_t height );

void save_colorset( void );
void load_colorset( void );
void restore_default_colors( void );
void erase_colorset( void );

extern void load_palette( RGB_color *palette );
extern uint8_t color_check( RGB_color *palette );
void show_palette( uint16_t startX, uint16_t startY, RGB_color *palette);
void show_pal_selector( uint16_t startX, uint16_t startY,
                       RGB_color *palette, uint8_t col_num);
void show_pal_selector_frame( int16_t startX, uint16_t startY,
                       RGB_color *palette, uint8_t pal_num);

void display_color_selector( uint16_t startX, uint16_t startY, int num); 
void show_brightness_control( uint8_t dayn );

void set_font_colors( unsigned char fore_color, unsigned char back_color );
void invert_font_colors( void );
void setup_color( unsigned int );
void RGB_copy( RGB_color *dest, RGB_color *source );

#endif /* CSI_COLOR_H_ */

