//******************************************************************************
//
// Mar, 20, 2018		Design Documents                     error_log.c
//
// log for hardware, program, and user errors
// eventlog for debugging events 
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Constants
//
// History
//  03/20/2018 copied from CS7000 project 
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  error_log                     stores an error number into the error log
//  get_error_text                returns a pointer to an error text string
//  log_event                     stores an event into the event log
//  get_event                     returns a pointer to an event text string
//  log_packet                    stores the packet number into the packet log

// Status: written not used
//  error_log_flush               clears all errors from the log

// Status: written not working
//  display_error                 prints an error, uses printf

//==============================================================================
#include "CSI_ERROR_LOG.h"
#include <stdint.h>
#include <string.h>
#include <stdio.h>

// rewrite display routines for Large Display
//void display_string(uint8_t, uint8_t, char *, uint8_t);
//void clear_display( void );

// index value of position in error log
// position will resart at 0 if log is full, overwritting old errors
unsigned int current_error = 0;
unsigned int error_count = 0;
int log_size = 100;
// set up rolling log for errors
int error_record[100];

int error_log( int error_number)
{
  if (current_error >= log_size) current_error = 0;
  error_record[current_error] = error_number;
  current_error++;
  error_count++;
  return(1);
}
//
//  clear all errors in the log
//  
void error_log_flush( void )
{
  for(int i=0; i < log_size; i++) error_record[i] =0; 
}
// 
//  Error message data
//
struct error_data
  {
  int error_number;
  char *error_text; 
  }error_list[50] =
  {
  // error messages should be no longer than 20 characters  
  // Hardware errors ( 0 - 999)
  101,  "HSE not ready",
  102,  "PLL not ready",
  // Internal Peripheral Errors (1000 - 1999)
  //     |                |
  1401, "I2C busy timeout",
  1402, "I2C bus error",
  1403, "I2C arb. lost",
  1404, "I2C ack failure",
  1405, "I2C overrun err",  
  1406, "I2C PEC error",
  1407, "I2C timeout err",
  1408, "I2C SMB alert",
  1501, "Flash erase timeout",  
  1502, "Flash write timeout",  
  1503, "Flash read timeout",
  1601, "SPI under run",      
  1602, "SPI over run",
  1603, "SPI CRC error",
  1604, "SPI mode fault",
  1605, "SPI frame error",
  1606, "SPI rx dat timeout",
  1701, "Uart data overrun",
  1702, "Uart data noise",
  1703, "Uart wd timeout",
  1704, "Uart data nopacket",
  1705, "Uart tx err",
  // External Peripheral Errors (2000 - 2999)
  2100, "SSD1963 no PLL lock",
  2200, "EPROM timeout",
  2202, "EPROM page err",
  2203, "EPROM timeout err",
  2301, "FLASH address err",
  2401, "TOUCH_P timeoutRx",
  2402, "TOUCH_P timeoutTx",
  // Software Errors (3000 - 3999)
  3001, "Default case occured",
  3002, "Bad state number",
  3003, "Null pointer",
  3021, "Buffer wrap around",
  3022, "Buffer empty",
  3023, "SPI buffer err",
  3031, "Flash page err",
  3041, "Menu error",
  3051, "Unknown Packet",
  3052, "Packet info err",
  3061, "Text too long",
  3071, "Packet FIFO overrun",
  3081, "Bad button assigned",
  3100, "Bad fav number",
  3200, "Process not fin",
  3300, "Missed key",
  3400, "Checksum err",
  3500, "Stack err",
  // User Errors (4000 - 4999)
  4010, "Bad color choice", 
  };
//******************************************************************************
//
// May 5, 2016			Design Documents                   display_error
//
// Displays error text message on top line of lcd display.   
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//   
// Functions used
//  display_string
// 
// External variables used
//  error_list[] array of error numbers and text
//
// History
//  05/05/2016 written, tested
//  03/20/2018 revised for Large Display project
//
// Status
//  written, not working using printf
//
//******************************************************************************
//
//  Display error
//
void display_error( int error_num )
{
  int i = 0;
  // find error number
  while(error_list[i].error_number != error_num)i++;
  // display error message  
  printf("%s", error_list[ i ].error_text);
  //clear_display();
  //display_string(0, 2, error_list[ i ].error_text, 1);
}

// =============================================================================
//
//  get error text
//    returns a pointer to the text sting of an error message
//
char* get_error_text( int error_num )
{
  int i = 0;
  // find error number
  while(error_list[i].error_number != error_num)i++;
  // return error message  
  return error_list[ i ].error_text;

}


//******************************************************************************
//
// Sept 19, 2016		Design Documents                     log_event.c
//
// log for events that need to be tracked. Description lengths should be <20
//   This and all calls should be removed after debugging is done
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  09/19/2016 started
//  03/20/2018 added to Large Display project, no changes
//  10/11/2018 modified to count events if the same event repeats 
//
// Status
//  written
//
//******************************************************************************

typedef struct{
    char desc[20];
} event; 

event event_log[100];
unsigned int event_log_indx = 0;

char prev_event[24] = "";
int event_rep_count = 1;

void log_event( char* ev_text )
{
  char count_txt[6] = "";
  char event_txt[26] = "                        ";
  //int n;
  
  /*
  // for debugging, stop on an empty string to find which packet sent it
  if(strncmp(ev_text, "", 1) == 0)
  {
    n = 1;
  }
  */
  
  // trim long descriptions, just in case
  if(strlen(ev_text) > 19) ev_text[19] = '\0';
  
  // copy passed string to local variable
  strncpy( event_txt, ev_text, 20);
  
  if(strncmp( ev_text, prev_event, 19) == 0)
  {
    event_rep_count++;
//    n = sprintf((char*) count_txt, " %i", event_rep_count);
    sprintf((char*) count_txt, " %i", event_rep_count);
  }
  else
  {
    event_rep_count = 1; 
    event_log_indx++;
    if (event_log_indx > 99) event_log_indx=0;    
  }
  
  strncpy( prev_event, ev_text, 19 );
  // add count to local string
  strncat( event_txt, count_txt, 6 );
  
  // store event
  strncpy(event_log[event_log_indx].desc, event_txt, 20);
}
// =============================================================================
char* get_event( int index )
{
  return event_log[ index].desc;
}

// =============================================================================
//
// February 4, 2019		                                    log_packet.c
//
// log reception and transmission of packets to or from the radio
//  added for debugging of packet handling
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
// 02/04/2019
//
// Status
//  written
//
//******************************************************************************

typedef struct{
    char desc[10];
} packet_id; 

packet_id packet_log[100];
unsigned int packet_log_indx = 0;

char prev_packet[5] = "";
int packet_rep_count = 1;

void log_packet( char* id_text )
{
  char count_txt[8] = "";
  char packet_txt[5] = "     ";
//  int n;
  
  // trim long descriptions, just in case
  if(strlen(id_text) > 4) id_text[5] = '\0';
  
  // copy passed string to local variable
  strncpy( packet_txt, id_text, 5);
  
  if(strncmp( id_text, prev_packet, 5) == 0)
  {
    packet_rep_count++;
//    n = sprintf((char*) count_txt, " %i", packet_rep_count);
    sprintf((char*) count_txt, " %i", packet_rep_count);
  }
  else
  {
    packet_rep_count = 1; 
    packet_log_indx++;
    if (packet_log_indx > 99) packet_log_indx=0;    
  }
  
  strncpy( prev_packet, id_text, 5 );
  // add count to local string
  strncat( packet_txt, count_txt, 6 );
  
  // store event
  strncpy(packet_log[packet_log_indx].desc, packet_txt, 10 );
}

