//******************************************************************************
// Header file for Winbond 64 Mbit serial flash memory            CSI_ExtFlash.h
//
//  for Large display project
//              
//  part number W25Q64FV
//
#define FLASH_SIZE    0x800000  //verify !!

// mfr id for W25Q64FV
#define FLASH_MFR_ID     0xEF16
//
// Instruction codes
//          

#define WRITE_ENABLE          (uint8_t) 0x06
#define VOL_SR_WRITE_ENABLE   (uint8_t) 0x50
#define WRITE_DISABLE         (uint8_t) 0x04
#define READ_STATUS_REG1      (uint8_t) 0x05
#define READ_STATUS_REG2      (uint8_t) 0x35
#define WRITE_STATUS_REG      (uint8_t) 0x01
#define PAGE_PROGRAM          (uint8_t) 0x02
#define SECTOR_ERASE          (uint8_t) 0x20
#define BLOCK_ERASE_32K       (uint8_t) 0x52
#define BLOCK_ERASE_64K       (uint8_t) 0xD8
#define CHIP_ERASE            (uint8_t) 0x60  
#define CHIP_ERASE_ALT        (uint8_t) 0xC7  // alternate erase
#define ERASE_SUSPEND         (uint8_t) 0x75
#define ERASE_RESUME          (uint8_t) 0x7A
#define POWER_DOWN            (uint8_t) 0xB9
#define READ_DATA             (uint8_t) 0x03
#define FAST_READ             (uint8_t) 0x0B
#define RELEASE_POWER_DOWN    (uint8_t) 0xAB
#define MFR_DEVICE_ID         (uint8_t) 0x90
#define JEDEC_ID              (uint8_t) 0x9F
#define READ_UNIQUE_ID        (uint8_t) 0x4B
#define READ_SFDP_REG         (uint8_t) 0x5A
#define ERASE_SECUR_REGS      (uint8_t) 0x44
#define PROGRAM_SECUR_REGS    (uint8_t) 0x42
#define READ_SECUR_REGS       (uint8_t) 0x48
#define ENABLE_QP1            (uint8_t) 0x38
#define ENABLE_RESET          (uint8_t) 0x66
#define RESET                 (uint8_t) 0x99

// quad SPI depends on package,
/// not available on large panel display
#define QUAD_PAGE_PROGRAM     (uint8_t) 0x32
#define FAST_READ_QUAD_3ADR   (uint8_t) 0x6B
#define FAST_READ_QUAD_IO     (uint8_t) 0xEB
#define WORD_READ_QUAD_IO     (uint8_t) 0xE7
#define OCTAL_W_READ_QUAD_IO  (uint8_t) 0xE3
#define SET_BURST_W_WRAP      (uint8_t) 0x77
#define QUAR_MFR_DEVICE_ID    (uint8_t) 0x94

// function prototypes
void initialize_flash_memory(void);
void flash_command( uint8_t command );
void flash_busy_wait(void);
uint8_t flash_read_stat1(void);
uint8_t flash_read_stat2(void);

int flash_mfrid(void);
uint32_t flash_jdecid(void);
uint64_t flash_uniqueid(void);
void flash_reset(void);

void flash_read( uint32_t address, uint32_t numbytes );
void flash_write ( uint8_t *data, uint32_t address, uint8_t numbytes );
void flash_erase( uint32_t address, uint8_t size );

