//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// December 11, 2018         Design Documents                     CSI_FEATURES.h
//
//		       header for enhanced features
//                     large panel display project
//
// Copyright
//  Connect Systems Inc.
//
// =============================================================================
#ifndef FEATURES_H_
#define FEATURES_H_

#include <stdint.h>

extern int zf_num;                   // favorite number
extern uint8_t zone_fav_changed;     // changed flag
extern int cf_num;                   // favorite number
extern uint8_t channel_fav_changed;  // changed flag

// state variables, all need to be reset on menu abort with Knob push
// reset is in menus module
extern uint8_t zf_state;
extern uint8_t cf_state;
extern uint8_t vfo_state;
extern uint8_t e_chan_state;
extern uint8_t chansc_state;
extern uint8_t gsd_state;
extern uint8_t sv_state;
extern uint8_t enhscan_state;

extern uint8_t chansc_flag;    // channel screen return
extern uint8_t chansc_channel_type;
extern uint8_t current_bin;

extern unsigned long scan_step_timer;
extern unsigned long scan_rx_timer;

extern uint8_t dial_rotation;

// eeprom address for color settings
// 0x2000 - 0x2039
// eeprom address for Function button settings
// 0x2100 - 0x212D
// eeprom address for zone favorites
// 0x2200 - 0x22F8
// 0x2300 - 0x2378
#define FAV_ZONE_HI  0x22
#define FAV_ZONE_LO  0x00
// eeprom address for channel favorites
// 0x2200 - 0x22F8
// 0x2300 - 0x2378
#define FAV_CHAN_HI  0x24
#define FAV_CHAN_LO  0x00

// structure for zone and channel favorites
typedef struct
{
  char     name[17];
  uint16_t zone_num;
  uint16_t channel_num;
} fav_info;

extern fav_info favorite_zone[16];
extern fav_info favorite_channel[16];

void EnhancedMonitorMode( void );
void EnhancedChannelEntry( void );
void EnhancedScanningMode( void );
// void VfoMode( void );
void DirectZoneChannelSelect( void );
void display_fav_zones( uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_fav_channels( uint16_t pos_x, uint16_t pos_y, uint8_t font );
void read_favorites( fav_info* favorite, uint8_t Loc_HI, uint8_t Loc_LO );
void save_all_favorites( fav_info* favorite, uint8_t Loc_HI, uint8_t Loc_LO );
void save_favorite( uint8_t fav_num, fav_info* favorite,
                   uint8_t Loc_HI, uint8_t Loc_LO );
void clear_all_favorites( fav_info* favorite  );

void display_chan_menuitem( int item_num );

// channel screen, vfo, enhanced
void process_zone_favs( uint8_t nav_key );
void process_channel_favs( uint8_t nav_key );
void process_channel_favs( uint8_t nav_key );
void process_vfo_mode( uint8_t nav_key );
void process_e_chan_mode( uint8_t nav_key );
void process_channel_screen( uint8_t nav_key );
void process_spectrum_view( uint8_t nav_key );
void process_enhscan_mode( uint8_t nav_key );

void display_frequency( uint32_t freq, uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_contact_id( uint32_t contact, uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_squelch_type( uint8_t stype, uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_spectrum_graph( uint16_t pos_x, uint16_t pos_y );
void display_integer_item( int menu_item, int value );
uint8_t check_freq_inrange( uint32_t freq );

void get_spectrum_data( uint8_t nav_key );

/*
void AF_DZCS_State_00(void);
void AF_DZCS_State_01(void);
void AF_DZCS_State_02(void);
void AF_DZCS_State_03(void);
void AF_DZCS_State_04(void);
void AF_DZCS_State_05(void);
*/


#endif /* FEATURES_H_ */