//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// January 23, 2018	        Design Documents             CSI_FT857_screens.c 
// 
//       Routines for display of radio screens and screen information
//                for FT857 large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  01/23/2019  started
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  FT857_display_screen            displays the FT857 screen
//  FT857_draw_button               displays a button on the screen
//  FT857_display_volume_bar        displays the volume bar
//  FT857_erase_volume_bar          erases the volume bar
//  FT857_display_squelch_bar       displays the squelch/ RF gain bar
//  FT857_erase_squelch_bar         erases the squelch/ RF gain bar
//  FT857_clear_radio_screen        clears just the radio info screen area
//
// Status: written, not used
//  FT857_display_message           displays a message on a given line
//  FT857_clear_message             clears a given line 

//
//==============================================================================
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include "CSI_FT857_SCREENS.h"
#include "CSI_FT857D.h"
#include "CSI_SSD1963.h"
#include "CSI_COLOR.h"
#include "CSI_GRAPHICS.h"
#include "CSI_ICONS.h"

// =============================================================================
// January 23, 2019  		Design Documents            FT857_display_screen
//
//     Display a screen
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  01/23/2019 copied from 800D display_screen and revised for FT857
//
// Status
//  writing
//

int FT857_active_screen_num = 0;

// for display modes
// 0 normal
//  split??
// multiple screens may be used by a mode

extern int display_mode;
extern uint8_t screen_refresh; 
  
// screen button info table
// is_active, x , y, text, font, icon 
button_info FT857_Buttons[14] = {
// top
{ 1,   8, 10,  "Left" , 4, ICON_LEFT_ARROW  }, // left arrow
{ 1, 122, 10,  "Right", 4, ICON_RIGHT_ARROW }, // right arrow
{ 1, 236, 10,  " DSP",  3, 0                }, // DSP
{ 1, 350, 10,  "Down",  4, ICON_NAV_DOWN    }, // Down
{ 1, 464, 10,  "Up",    4, ICON_NAV_UP      }, // UP
{ 1, 578, 10,  "V/M",   3, 0                }, // VFO / Memory
{ 1, 692, 10,  "Lock",  3, 0                }, // Lock
// side
{ 1,   8, 100, " Home", 4, 0                }, // Home
{ 1,   8, 190, "  Clar",4, 0                }, // Clarifier
{ 1,   8, 280, " Func", 4, 0                }, // Function
// bottom
/*
{ 1, 122, 375, "A",     3, 0                }, // A
{ 1, 350, 375, "B",     3, 0                }, // B  PROBABLY WRONG
{ 1, 578, 375, "C",     3, 0                }, // C
{ 1, 380, 430, "Power", 4, ICON_POWER       }, // Power
*/
{ 1, 200, 365, "A",     3, 0                }, // A
{ 1, 385, 365, "B",     3, 0                }, // B PROBABLY RIGHT
{ 1, 558, 365, "C",     3, 0                }, // C
{ 1, 390, 430, "Power", 4, ICON_POWER       }, // Power
};

/*
// location on mode 0 screen 1 for standard messages
int message1_X = 30;
int message1_Y = 405;
int message2_X = 30;
int message2_Y = 435;
*/

//extern unsigned int ticks;
//int char_time = 0;

void FT857_display_screen( uint8_t screen_num, uint8_t clear )
{
  int i;
  
  switch( screen_num )
  {
  case 1:     // normal radio display screen
    {
    // screen 1 uses display mode 0
    display_mode = 0;

    // font colors 
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    // set shadow color
    RGB_copy( &highlight, &current_palette[get_pal_color( 4 )] );
    // set border color
    RGB_copy( &border, &current_palette[get_pal_color( 2 )] );
   
    if(clear)
    {
      ssd1963_cg_clear_display();
      screen_refresh = 1;
    }
    // set active after clearing
    FT857_active_screen_num = screen_num;
    // border shaded frame
    draw_shaded_frame( 0, 0, 800, 480, 5, border, highlight);
    // Radio shaded frame
    draw_shaded_frame( 120, 65, 665, 300, 5, border, highlight);
    
    //   Volume 1-32  
//    FT857_display_volume_bar( 10 );
    // Squelch
//    FT857_display_squelch_bar( 10 );
    // draw buttons
    for( i = 0; i < 13; i++)
    {
      FT857_draw_button( i, get_pal_color( 2 ), get_pal_color( 3 ));
    }
    
    clear_radio_screen_and_rewrite_lines();    
    
    /* need to set up touch points
    
    // activate touch points for this screen
    touch_point[0].is_active = 1;  // volume up
    touch_point[0].is_active = 1;  // volume down
    touch_point[0].is_active = 1;  // squelch up
    touch_point[0].is_active = 1;  // squelch down

    touch_point[0].is_active = 1;  // Left
    touch_point[0].is_active = 1;  // Right 
    touch_point[0].is_active = 1;  // DSP
    touch_point[0].is_active = 1;  // Down
    touch_point[0].is_active = 1;  // UP
    touch_point[0].is_active = 1;  // V/M
    touch_point[0].is_active = 1;  // Lock

    touch_point[0].is_active = 1;  // A
    touch_point[0].is_active = 1;  // B
    touch_point[0].is_active = 1;  // C

    touch_point[0].is_active = 1;  // Home
    touch_point[0].is_active = 1;  // Clar
    touch_point[0].is_active = 1;  // Func
    touch_point[0].is_active = 1;  // Power
    */

    // power button
    // draw_icon( ICON_POWER, FT857_Buttons[13].x_pos, FT857_Buttons[13].y_pos,
    //            cur_foreground, cur_background );
    
    // done refeshing
    screen_refresh = 0;
    
    break;
    }
  }
}

// =============================================================================
// January 23, 2019  		Design Documents               FT897_draw_button
//
//     Display a button on the screen
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 
//  01/23/2019 copied from 800D draw_button and revised for FT857
//
// Status
//  written
//

// pixel width is in process_CS800D and could move
extern uint16_t pixel_line_width( char* text, uint8_t font, uint8_t spacing);

void FT857_draw_button( uint8_t B_num, uint8_t bordr_color, uint8_t face_color )
{
  unsigned int bordr_size;
  unsigned int tlen;
  int c_width;    // offset needed to center 
  int c_height;
  
  // draw the box
  bordr_size = 4;
  // draw buttons 100 wide 50 high
  draw_shaded_frame( FT857_Buttons[B_num].x_pos, FT857_Buttons[B_num].y_pos,
                  100, 50, bordr_size,
                  current_palette[face_color], current_palette[bordr_color] );
  
  if( FT857_Buttons[B_num].icon == 0 )
  {
    // calc centering offset
    c_width = (100 - pixel_line_width( FT857_Buttons[B_num].text,
                                      FT857_Buttons[B_num].font, 2 ) ) / 2; 
    // write text in the box
    tlen = strlen(FT857_Buttons[B_num].text);
    memcpy( cg_line_buffer, FT857_Buttons[B_num].text, tlen );  
    ssd1963_cg_write_line( tlen, FT857_Buttons[B_num].font,
                               FT857_Buttons[B_num].x_pos + c_width,
                               FT857_Buttons[B_num].y_pos + bordr_size + 4, 2);  
  }
  else
  {
    // draw the icon in the box
    // calc centering offsets
    c_width = (100 - *FT857_Buttons[B_num].icon ) / 2; 
    c_height = (50 - *(FT857_Buttons[B_num].icon + 1)) / 2;     
    
    draw_icon( FT857_Buttons[B_num].icon,
               FT857_Buttons[B_num].x_pos + c_width,
               FT857_Buttons[B_num].y_pos + c_height,
               cur_foreground, cur_background);
  }
  
}
// =============================================================================
// January 23, 2019		Design Documents        FT857_display_volume_bar
//
// This program displays the volume bar. 
//
//  Uses draw_progress_bar which:
//   Locates the bar on the screen at x,y location
//   defines the width and height
//   selects the border, bar, and background colors
//
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   volume_value: This is the value of the volume bar and is passed as
//   a parameter
//
// Exit Conditions
//  The volume bar is displayed.
//
// External functions called
//  draw_progress_bar 
//
// History 
//  01/23/2019 copied and revised from CS800D screens.c
//  05/08/2019 added title above bar
//
// Status
//  written
//

// touch controls are not in use

// touch button location variables, for screen 1
// x and y are swapped, need to adjust in controller

/*
// xy adjusted here to test
uint16_t volume_up_touch_miny = 760;
uint16_t volume_up_touch_maxy = 800; 
uint16_t volume_up_touch_minx = 375; 
uint16_t volume_up_touch_maxx = 415;

uint16_t volume_dn_touch_miny = 535;
uint16_t volume_dn_touch_maxy = 575;
uint16_t volume_dn_touch_minx = 375;
uint16_t volume_dn_touch_maxx = 415;
*/
uint16_t vol_bar_xpos = 540;
uint16_t vol_bar_ypos = 440;

void FT857_display_volume_bar(uint8_t volume_value)
{
  uint16_t val;
  
  RGB_color prev_bkcolor;
  RGB_color prev_frcolor;
 
  // save current colors
  RGB_copy( &prev_frcolor, &cg_font ); 
  RGB_copy( &prev_bkcolor, &cg_background );
  
  // if volume is displayed on the screen
  // update volume display
  
  // volume level is 0 - 255
  // progress bar is 0-100%
  // Multiply by 100 which gives us 0 - 25500
  // Divide by 255 which gives us 100 max

  val = (uint16_t) ((volume_value * 100 ) / 255);
  
  if(val > 100) val = 100;  // in case of a garbage value
  // current_radio_volume = val;

  // set to screen colors  
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );    
  // title
  strncpy( (char * ) cg_line_buffer, "Volume", 6);
  ssd1963_cg_write_line( 6, 4, vol_bar_xpos, vol_bar_ypos - 24, 2);
  // restore current colors
  RGB_copy( &cg_font, &prev_frcolor ); 
  RGB_copy( &cg_background, &prev_bkcolor );
  
  draw_progress_bar( val, 0, vol_bar_xpos, vol_bar_ypos, 200, 30,
                    get_pal_color( 2 ), get_pal_color( 7 ), 1);
//  draw_icon( ICON_LEFT_ADJ, vol_bar_xpos - 21, vol_bar_ypos - 2,
//                get_pal_color( 6 ), cur_background );
//  draw_icon( ICON_RIGHT_ADJ, vol_bar_xpos + 204, vol_bar_ypos - 2,
//                get_pal_color( 6 ), cur_background ); 
}
// =============================================================================
//
//
void FT857_erase_volume_bar(void)
{
  RGB_color prev_bkcolor;
  RGB_color prev_frcolor;
 
  // save current colors
  RGB_copy( &prev_frcolor, &cg_font ); 
  RGB_copy( &prev_bkcolor, &cg_background );
  
  // set to screen colors  
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );  
  clear_rectangle(  vol_bar_xpos, vol_bar_ypos - 24, 200, 54); 

  // restore current colors
  RGB_copy( &cg_font, &prev_frcolor ); 
  RGB_copy( &cg_background, &prev_bkcolor );
  
}
// =============================================================================
// January 23, 2019		Design Documents       FT857_display_squelch_bar
//
// This program displays the squelch bar. 
//
//  Uses draw_progress_bar which:
//   Locates the bar on the screen at x,y location
//   defines the width and height
//   selects the border, bar, and background colors
//
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   squelch_value: This is the value of the squelch bar and is passed as
//   a parameter
//
// Exit Conditions
//  The squelch bar is displayed.
//
// External functions called
//  draw_progress_bar 
//
// History 
//  01/23/2019 copied and revised from CS800D screens.c volume_bar
//  05/08/2019 added title above bar
//
// Status
//  written, working
//

uint16_t sql_bar_xpos = 60;
uint16_t sql_bar_ypos = 440;

void FT857_display_squelch_bar(uint8_t squelch_value)
{
  RGB_color prev_bkcolor;
  RGB_color prev_frcolor;
 
  // save current colors
  RGB_copy( &prev_frcolor, &cg_font ); 
  RGB_copy( &prev_bkcolor, &cg_background );
  
  // squelch level is 0 - 255
  // progress bar is 0-100%
  // Multiply by 100 which gives us 0 - 25500
  // Divide by 255 which gives us 100 max

  squelch_value = (uint16_t) ((squelch_value * 100 ) / 255);

  // set to screen colors  
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );  
  
  // bar title
  if( FT857D_rfsql_mode == 2 )  //squelch 
  {  
    strncpy( (char * ) cg_line_buffer, "Squelch", 7);
    ssd1963_cg_write_line( 7, 4, sql_bar_xpos ,sql_bar_ypos - 24, 2);    
  }
  if( FT857D_rfsql_mode == 1 )  //rf gain
  {
    strncpy( (char * ) cg_line_buffer, "RF gain", 7);
    ssd1963_cg_write_line( 7, 4, sql_bar_xpos ,sql_bar_ypos - 24, 2);
  }
  // restore current colors
  RGB_copy( &cg_font, &prev_frcolor ); 
  RGB_copy( &cg_background, &prev_bkcolor );
  
  draw_progress_bar( squelch_value, 0, sql_bar_xpos, sql_bar_ypos, 200, 30,
                    get_pal_color( 2 ), get_pal_color( 7 ), 1);
//  draw_icon( ICON_LEFT_ADJ, sql_bar_xpos - 21, sql_bar_ypos - 2,
//                get_pal_color( 6 ), cur_background );
//  draw_icon( ICON_RIGHT_ADJ, sql_bar_xpos + 204, sql_bar_ypos - 2,
//                get_pal_color( 6 ), cur_background );  
}
// =============================================================================
//
//
void FT857_erase_squelch_bar(void)
{
  RGB_color prev_bkcolor;
  RGB_color prev_frcolor;
 
  // save current colors
  RGB_copy( &prev_frcolor, &cg_font ); 
  RGB_copy( &prev_bkcolor, &cg_background );
  
  // set to screen colors  
  set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );  
  clear_rectangle( sql_bar_xpos ,sql_bar_ypos - 24, 200, 54);

  // restore current colors
  RGB_copy( &cg_font, &prev_frcolor ); 
  RGB_copy( &cg_background, &prev_bkcolor );

}

// =============================================================================
//
//   display text on a line based on table locations and font
//

// screen message info table
// is_active, x , y, font
message_info FT857_Message[5] =
{
  {1, 270,  75, 7 },   // line 0
  {1, 200, 135, 7 },   // line 1
  {1, 230, 195, 5 },   // line 2
  {1, 155, 210, 7 },   // line 3
  {1, 155, 310, 7 },   // line 4
};

//  early version of message display
//  Status: not in use

void FT857_display_message( uint8_t line_num, char* text )
{
  int n;
  // prevent null pointer error 
  if( text == 0 ) return;

  n = strlen( text );
  strncpy( (char * ) cg_line_buffer, text, n );
  ssd1963_cg_write_line( n, FT857_Message[line_num].font,
                            FT857_Message[line_num].x_pos,
                            FT857_Message[line_num].y_pos, 8);
}
// =============================================================================
//
//   clear radio screen
//    clears the central portion of the FT857 screen inside the border
//    used after radio is turned off

// Status: written working

void FT857_clear_radio_screen( void )
{
  clear_rectangle( 128, 73, 648, 284 );
}
// =============================================================================
//
//  clear message
//     clears the message for the line number sent
//     clears to the edge of the radio frame except for line 3 which clears to
//     the beginning of line 2
//
// 
// Status: not in use 
void FT857_clear_message( uint8_t line_num )
{
  int m_width;
  // get font max sizes, and spacing
  ssd1963_cg_read_char_size( FT857_Message[line_num].font ); 

  // exception for line 3. It ends at the beginning of line 2
  if( line_num == 3 )
  {  
    m_width = FT857_Message[2].x_pos - FT857_Message[3].x_pos;
  }
  else m_width = 780 - FT857_Message[line_num].x_pos;


  clear_rectangle( FT857_Message[line_num].x_pos,
                   FT857_Message[line_num].y_pos,
                   m_width, cg_max_vert_size );  
}
