//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
//  March 19, 2018	    Design Documents                          CSI_MAIN.c 
// 
//             Main program for the large display project
//                           STM32F405VG
//
// Programmer
//  Jerry, Wanger, Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  03/09/2018  written for test
//  10/15/2018  first release version 1.0.0 
//  10/26/2018  second release version 1.0.1
//  11/16/2018  third release version 1.0.2 for 20.41
//  11/20/2018  release version 1.0.3 for 20.43 Black Friday
//  12/06/2018  release version 1.0.4 for 20.45
//  12/21/2018  release version 1.0.5 for 20.47
//  01/04/2018  release version 1.0.6 for 20.47
//  01/07/2019  writing version 1.0.7 for 20.48,49,52
//  01/23/2019  New version for FT857 demo
//  01/28/2019  release version 1.0.8 for 20.53
//  03/14/2019  release version 1.0.9 for 20.53
//  04/10/2019  beta version    1.1.0 for 20.57 
//  05/02/2019  beta version    1.1.1a for 20.57 
//  05/21/2019  release version 1.1.2  for 20.57,20.58 
//  07/22/2019  release version 1.1.3  for 20.58, 30.04
//  08/29/2019  release version 1.1.4  for 30.04-30.11
//  08/29/2019  release version 1.1.5  for 30.11 30.14
//  11/12/2019  release version 1.1.6  for 30.14
//
// Status
//  writing version 1.1.7
//
//============================================================================== 

#include <stdint.h>
#include "CSI_GPIO.h"
#include "CSI_initialize.h"
#include "CSI_SCREENS.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_PROGRAMMING.h"
#include "CSI_ExtFlash.h"
#include "CSI_TouchPanel.h"
#include "CSI_SWITCHES.h"
#include "CSI_Menus.h"
#include "CSI_FEATURES.h"
#include "CSI_RADIO.h"
#include "CSI_TIMERS.h"

#include "CSI_CS800_TGMonitor.h"

#ifdef radio_FT857D
#include "CSI_FT857_SCREENS.h"
#include "CSI_FT857D.h"
#endif /* FT857D */

// for sniffing
#include "CSI_UART.h"

// for debug
//#include "CSI_diagnostics.h"

uint8_t startup_mode = 0;

// This is the starting radio type
// change this to have the panel start for the chosen radio type
uint8_t current_radio_type = CS800D;
//uint8_t current_radio_type = FT857D;

void main(void)
{
  // Set up the hardware 
  
  initialize();
    
  switch(startup_mode)
  {
  case 1:  // P1 held programming start
    {
      // programming screen
      display_screen( 8 , 1);
      lcd_backlight_on();
      // wait here till reboot
      // blink red green       
      while(1)
      {
        blink_alternate();
        msec_delay(500);
      }
        
      break;
    }
  case 2:  // p2 held RCDB
    {
      // RCDB screen
      display_screen( 9 , 1);
      lcd_backlight_on();
      // solid green led
      rx_led_on();
      // wait here till startup packet is rec'd
      while( packet_buffer_output_type != 4);
      break;
    }
  case 0: // normal start
  default: break;
  }
  radio_watchdog_timer = 3000;
  // splash startup screen
  display_screen( 7 , 1);
  lcd_backlight_on();

  blink_red(10);
  blink_green(10);
  
  // need to read the startup packet during the splash screen
  if( current_radio_type == CS800D )
  {
     process_all_packet();    
  }
  msec_delay(500); 
  
  // debug force to 800D
  // current_radio_type = CS800D;  
  
  switch( current_radio_type )
  {
  case CS800D:
    {
    // main display screen
    display_screen( 1 , 1);
    msec_delay(1000);    // wait while radio ID is displayed.
    
    TGM_monitor_on = 1;   // continuous monitoring
    
    // This is the idle loop of the system. 
    while (1)
    {
      process_all_packet();
      // This section initializes the flash memory if the user needs to initialize
      // the system because of some problem. This routine is entered when the user
      // presses **123456** from the keypad
      if (factory_initialize_flag)
      {
        initialize_flash_memory();
        factory_initialize_flag = 0;
      }
      // This section activates the user progamming from the keypad. This routine is
      // entered when the user presses ##123456# from the keypad. 
      if (ProgramFlag)
      {
        parameter_program();
        ProgramFlag = 0;
      }
      // These routines process the background tasks for normal operation.
      // screen mode switch
      if( read_switch_s2() )
      {
        // standard version
        // if not on the menu screen where the knob is used for select
        if(active_screen_num != 3)
        {
          // display menu for panel controls
          if(display_mode == 0)
          {
            reset_panel_menu();  // reset menu variables before starting
            display_mode = 1;
            display_screen( 6 , 1);  // 5 for debug orig, 6 for front panel menu
          }
          else
          // regular radio screen  
          {
            // need to reset panel menu variables, so it's ready for next 
            reset_panel_menu(); 
            display_mode = 0;
            display_screen( 1 , 1);         
          }
        }
      }        
      // handle panel menu
      process_panel_menu();
    }      
    break;
    }
// ==============================    
#ifdef radio_FT857D    
  case FT857D:
  case FT857D2:
    {
      // main display screen
      FT857_display_screen( 1, 1 );
      active_screen_num = 0;      
      // idle loop
      while(1)
      {
        if(!FT857D_startup_timer ) process_FT857D_packets();
        process_blinking_cursor();        
        if( read_switch_s2() ) // right knob press
        {
          if(active_screen_num == 6 )
          {
            FT857_display_screen( 1, 1 );
            active_screen_num = 0;
          }
          else
          {
            display_mode = 1;          
            display_screen( 6 , 1);
          }
        }
        /*
        // for FT857D packet sniffing only, also change uartrx routine 
        if( read_switch_s4() )  // P2 button
        {
          display_rxdata_dump( uart1_rx_buffer );
        }
         */
        if(active_screen_num == 6 ) process_panel_menu();
      }
      break;
    }
#endif /* FT857D */
// ==============================    
    default:
      {
        // force to CS800D
        current_radio_type = CS800D;
        break;
      }
  }
}