//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// August, 6 2018  		Design Documents                     CSI_menus.h 
// 
//       Header for Menu routines for CS800D
//           for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  08/06/2018  started
//
// Status
//  writing
//
//==============================================================================
#ifndef CSI_MENUS_H_
#define CSI_MENUS_H_

#include <stdint.h>

extern uint8_t num_menu_levels;
extern int submenu_max_lines;
extern int submenu_num_items;
extern int submenu_max_lines;
extern int menu_current_level;

extern uint8_t info_box_active;
extern uint8_t menu_exit;

extern int select_box_width;

extern uint8_t msg1_buf[16];
extern uint8_t msg2_buf[16];

enum menu_action { MAIN, SUB, TEXT, INFO, ONOFF, LIST, COLR, SLID};

enum nav_action { NONE, CANCEL, UP, DOWN, SELECT };

enum menu_page { NOPAGE, VFOMODE, ENHMON, ECHAN, ENHSCAN, CHANFAV, ZONEFAV, SPECTV,
                 RADSEL, ERRLIST, EVTLIST, COLRSEL, QBOX };

// menu_table
typedef struct { 
  int  level;
  int  item_num;
  int  action_type;
  int  sub_indx;
  int pos_x;
  int pos_y;
  int font;
  char *msg1;
  char *msg2;
} Menu_item;

// info box location and size
// menupage   xloc, yloc, width, height
typedef struct { 
  int  x_loc;
  int  y_loc;
  int  width;
  int  height;
} box_spec;

extern box_spec info_box_loc[6];

void display_menu( uint8_t menu_page, uint8_t action_type,
                  char * msg_1, int n1, char * msg_2, int n2 );
void display_submenu( int index );
int get_menu_index( int current_level, int current_item);
int get_nav_key( void );
void display_text_box( uint16_t x_loc, uint16_t y_loc, uint16_t width, uint16_t height);
void question_box( uint16_t pos_x, uint16_t pos_y, uint16_t width,
               uint16_t height, char* question, uint8_t butn1, uint8_t butb2 );
void message_box( uint16_t pos_x, uint16_t pos_y,
                 uint16_t width, uint16_t height, char* message );

void process_question_box( uint8_t nav_key );
void reset_panel_menu( void ); 
void display_sub_menu( int sub_num, uint8_t select_num, uint16_t box_width, uint8_t font );

void process_color_selector( uint8_t nav_key );
void display_color_menupage( void );
void process_menu_level( uint8_t nav_key );
void process_submenu_level2( uint8_t nav_key );
void process_e_list( uint8_t nav_key );
void display_panel_menu( void );
void process_panel_menu( void );
void display_error_list( void );
void display_event_list( void );

void process_radio_select( uint8_t nav_key );
void display_radio_choices( uint16_t pos_x, uint16_t pos_y, uint8_t font );

// menu stack functions
void push_menu_stack( uint8_t page_num );
void pop_menu_stack( void );
uint8_t get_menu_stack( void );
void reset_menu_stack( void );

#endif /* CSI_MENUS_H_ */