//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
///
// Mar 5, 2018			Design Documents             CSI_PROCESS_CODES.c 
// 
//            Routines for CS800D radio programming codes
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//      03/05/2018 started
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, untested

//  process_codes                   code entry controller, reset or prgramming 
//  factory_reset_code              check for entry of factory reset code
//  programming_code                handle programming codes entered

//==============================================================================
#include <stdint.h>
#include "CSI_MC59.h"
#include "CSI_PROGRAMMING.h"
//
// ****************************************************************************
// process_codes       determines if a valid access code or initialization code
// factory_reset_code  determines if **123456** entered
// programming_code    determines if **123456** available from radio
// ****************************************************************************
// 
static uint8_t factory_reset_string[10] = {
"**123456**"
};
static uint8_t factory_programming_string[9] = {
  "##123450#"
};
//
//******************************************************************************
//
// March 5, 2018		Design Documents                   process_codes
//
// Determines if the factory reset code of the factory programming code is
// entered from the keypad. It should be noted that each key pressed has to be
// within five seconds of the previous key otherwise the program resets itself.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If key is pressed, then the following two programs are called:
//  1. factory_reset_code
//  2. programming_code 
//
// External functions called
//  factory_reset_code
//  programming_code
//
// External variables used
//  dtmf_code_flag
//  dtmf_char
//  dtmf_code_char
//  factory_reset_state
//  programming_code_state
//  dtmf_codes_timer
//
// Internal variables used   
//
// Constants
//
// History 
//
// Status
//  written
//
void process_codes(void)
{
  if (dtmf_code_flag)
  {
    dtmf_code_flag = 0;
    if (!dtmf_codes_timer)
    {
      factory_reset_state = 0;
      programming_code_state = 0;
    }
    dtmf_codes_timer = 50;
    factory_reset_code();
    programming_code();
  }
}
//
//******************************************************************************
//
// March 5, 2018		Design Documents              factory_reset_code
//
// This program compares the incoming string from the microphone against the 
// factory initialization string of **123456**
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If the sequence is found, then factory_initialize_flag is set to 1
//
// External functions called
//
// External variables used
//  dtmf_code_char
//  factory_reset_state
//  factory_initialize_flag
//
// Internal variables used   
//
// Constants
//  factory_reset_string
//
// History 
//
// Status
//  written
//
//
void factory_reset_code(void)
{
  if (factory_reset_string[factory_reset_state++] != dtmf_code_char)
  {
    if (dtmf_code_char == '*')
    {
      if (factory_reset_state == 3)
      {
        factory_reset_state = 2;
      }
      else
      {
        factory_reset_state = 1; 
      }
    }
    else
    {
      factory_reset_state = 0;
    }
  }
  if (factory_reset_state == 10)
  {
    factory_reset_state = 0;
    factory_initialize_flag = 1;
  }
}
//
//******************************************************************************
//
// March 5, 2018		Design Documents              factory_reset_code
//
// This program compares the incoming string from the microphone against the 
// programming string of ##123456#
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  If the sequence is found, then factory_initialize_flag is set to 1
//
// External functions called
//
// External variables used
//  dtmf_code_char
//  programming_code_state
//  ProgramFlag
//
// Internal variables used   
//
// Constants
//  factory_programming_string
//
// History 
//
// Status
//  written
//
void programming_code(void)
{
  if (factory_programming_string[programming_code_state++]  != dtmf_code_char)
  {
    if (dtmf_code_char == '*')
    {
      if (programming_code_state == 3)
      {
        programming_code_state = 2;
      }
      else
      {
        programming_code_state = 1; 
      }
    }
    else
    {
      programming_code_state = 0;
    }
  }
  if (programming_code_state == 9)
  {
    programming_code_state = 0;
    ProgramFlag = 1;
  }
}
