//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// April 29, 2018		Design Documents            CSI_PROCESS_CS800D.c 
// 
//                        CS800D radio packet handling
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//      04/29/2018
//      06/12/2019  changed extract_message to extract_message_to,
//                   writes packet data directly to the msg_buffer
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
// convert_LSB_bytes_to_int       covert 4 bytes stored LSB to 32 bit integer
// store_4bytes_LSB               store 32bit integer as 4 bytes LSB
// process_CS800D                 determines purpose of packet
//       -- radio command packets --
// RCDB_start                     send packet for RCDB start 
// set_zone_channel               send zone and channel number command 
// set_temp_DCH                   send temporary digital channel command
// set_temp_ACH                   send temporary analog channel command
// request_RSSI_value             send RSSI value request to radio
//       -- acknowledge handlers --
// packet_type_A                  normal packet acknowledge handler
// packet_type_B                  packet acknowledge after a command reply
// packet_type_C                  packet acknowledge after a RSSI request
// packet_type_0                  heartbeat
// packet_type_2                  firmware version
//       -- packet info output routines --
// process_all_packet             Process all packets for output
// packet_buffer_0_output         heartbeat
// packet_buffer_1_output         LED and backlight control
// packet_buffer_2_output         firmware version
// packet_buffer_3_output         volume control
// packet_buffer_4_output         two lines centered
// packet_buffer_5_output         channel info, icons, frequencies, volume, rssi
// packet_buffer_6_output         menu selection
// packet_buffer_7_output         data entry type 1
// packet_buffer_8_output         data entry type 2
// packet_buffer_9_output         data entry type 3
// packet_buffer_10_output        two lines centered
// packet_buffer_11_output        data entry type 4
// packet_buffer_12_output        data entry type 5
// packet_buffer_13_output        two lines centered, receive info
// packet_buffer_14_output        data entry for text (using mic keypad)
// packet_buffer_15_output        item selected in a list
// packet_buffer_16_output        advanced channel info (new packet 5)
// packet_buffer_17_output        DMR Rx activity
// packet_buffer_18_output        Analog Rx activity
// packet_buffer_19_output        programmed button assignemnt info
// packet_buffer_20_output        acknowledgement of Temp_DCH
// packet_buffer_21_output        acknowledgement of Temp_ACH
// packet_buffer_22_output        RSSI value, after RSSI request 
//       -- display routines ---
// extract_message_to             Get ASCII message store directly in buffer 
// set_channel_bits               Set packet channel info to screen control bits
// set_pkt1_bits                  Set packet to screen control bits
// check_channel_bits             check which channel bits are set
// check_pkt1_bits                check which packet bits are set

// packet_buffer_5_display        displays pkt 5 items
// packet_buffer_16_display       displays pkt 16 items
// packet_buffer_17_display       displays DMR Rx items
// packet_buffer_18_display       displays Analog Rx items
// packet_buffer_19_display       does nothing with button names 

// display_pkt1_volume            displays volume bar, or volume text message            
// display_pkt1_3_msg1            displays pkt 3 message 1 and icons
// display_pkt1_3_msg2            displays pkt 3 message 2 
// display_pkt1_4_msg1            displays pkt 4 message 1
// display_pkt1_4_msg2            displays pkt 4 message 2
// display_channel_alias          displays channel name
// display_channel_rx_frequency   displays channel receive frequency
// display_channel_tx_frequency   displays channel transmit frequency
// display_zone_alias             displays zone name
// display_channel_number_in_zone displays channel number in zone
// display_zone_number            displays zone number
// display_channel_number         displays channel number
// display_channel_rssi           displays RSSI icon with signal bars
// display_channel_scan           displays scan icon
// display_channel_power          displays power level
// display_channel_privacy        displays privacy icon (lock) 
// display_channel_message        displays message icon (envelope)
// display_channel_gps            displays gps icon
// display_channel_disable_tone   displays disable tone icon
// display_channel_talk_around    displays talk around icon
// display_channel_monitor        displays digital monitor icon
// display_channel_type           displays channel type, DMR or analog
// display_channel_volume         displays volume bar
// display_pkt1_6_msg1            displays pkt 6 msg 1, top level menu 
// display_pkt1_7_msg1            displays pkt 7 msg 1, submenu
// display_pkt1_7_msg2            displays pkt 7 msg 2, submenu
// display_pkt1_8_msg1            displays pkt 8 msg 1, menu action select
// display_pkt1_8_msg2            displays pkt 8 msg 2, menu action select
// display_pkt1_9_msg1            displays pkt 9 msg 1, info from menu item
// display_pkt1_9_msg2            displays pkt 9 msg 2, info from menu item
// display_pkt1_10_msg1           displays pkt 10 msg 1, startup, shutdown
// display_pkt1_10_msg2           displays pkt 10 msg 2, startup, shutdown
// display_pkt1_11_msg1           displays pkt 11 msg 1, menu data entry
// display_pkt1_11_msg2           displays pkt 11 msg 2, menu data entry
// display_pkt1_12_msg1           displays pkt 12 msg 1, menu list pop up
// display_pkt1_12_msg2           displays pkt 12 msg 2, menu list pop up
// display_pkt1_13_icon           displays DMR call type icon
// display_pkt1_13_msg1           diplays call sign or name, DMR last call info
// display_pkt1_13_msg2           diplays DMR group name
// display_pkt1_14_msg1           displays pkt 14 msg 1, in menu item
// display_pkt1_14_msg2           displays pkt 14 msg 2, pop up text box
// display_pkt1_15_msg1           displays pkt 15 msg 1,
// display_pkt1_15_msg2           displays pkt 15 msg 2, pop up text box
// pixel_line_width               calc number of pixels in a text string     
// clear_pckt_message             clear a message area
// panel_sleep                    display radio off screen, wait for activity

// Status: written, tested, not used
//
// set_normal_ack                  send a standard acknowledge reponse
// packet_type_1                   process idle command 2 (packet type A)
// display_debug_p                 display debugging messages in debug mode

// Status: written, not tested, not used
//
// store_temp_CH                   store temporary info command
// display_channel_battery         does nothing
// display_channel_mode            does nothing
// display_channel_emergency       does nothing
// display_channel_bt_state        does nothing
// display_channel_link            does nothing
// calculate_line_center           get pixel offset for the center of a string
// output_special_icons            does nothing
// message_type_icons              does nothing

//==============================================================================
#include "CSI_PROCESS_CS800D.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "CSI_GPIO.h"
#include "CSI_SSD1963.h"
#include "CSI_SCREENS.h"
#include "CSI_GRAPHICS.h"
#include "CSI_COLOR.h"
#include "CSI_SWITCHES.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_Menus.h"
#include "CSI_FEATURES.h"
#include "CSI_UART.h"
#include "CSI_RADIO.h"
#include "CSI_CS800_TGMonitor.h"

// variables

uint8_t  packet_process_buffer[256];
uint8_t  unknown_packet_error_buffer[256];
uint32_t packet_buffer_output_flag;
uint32_t packet_buffer_output_type;
uint32_t radio_watchdog_timer;

uint32_t packet_process_buffer_flag = 1;
uint8_t  packets_fifo[4][256];
uint32_t packet_process_watchdog_timer;

uint32_t RxFreq;
uint32_t TxFreq;
uint32_t cs_channel_monitor;
int dig_mon_on = 0;
int scan_on = 0;

uint32_t cs_pkt1_new;

// flags for redraw after msg changes
uint8_t pckt_msg1_dirty = 0;
uint8_t pckt_msg2_dirty = 0;
uint8_t pckt_msg3_dirty = 0;
uint8_t pckt_msg4_dirty = 0;

uint8_t msg1_dump[17];
uint8_t msg2_dump[17];

char pkt_id[10];

//  button structures
FB_KeysProgFunctionDef_t current_key_functions;  // for acitve use in the panel 
FB_KeysProgFunctionDef_t stored_key_functions;   // when read out from eeprom 
FB_KeysProgFunctionDef_t update_key_functions;   // when received from the radio

// first byte is long_press_time, 4 x 250 msec steps 1000 msec
// second byte is emergency quit time 4 x 0.5 sec steps 2 seconds
// Default used if current and stored are bad.
// Only channel and volume are set, all others unassigned.
FB_KeysProgFunctionDef_t default_key_functions = {  
 0x04, 0x04, 0, 0, 0, 0, 0, 0, 0, 0, n_PB_ChannelUp, 0, n_PB_ChannelDown, 0,
 0, 0, (DBProgramButton_e) n_PB_VolumeUp, (DBProgramButton_e) n_PB_VolumeDown  };

uint16_t FB_num_keys = 46;  // number of items in the FB structure
// uint16_t FB_num_funct = 52; // max number of functions assignable


/* 11/19/2018 data from mfr, structure of radio info packet
// packet 5 for MS20.43   

 typedef struct 
{ 
  uint8_t windowType; 
  uint8_t  rssiGrade   :3;      // None,RSSI 0 ~ RSSI 4          
  uint8_t  batteryGrade:3;      // Empty(0),1-4Grade              
  uint8_t  scan        :2;      // Disable(0),On(1),Prio1(2),Prio2(3)      

  uint8_t  powerLevel :2;       // None(0),Low(1),High(2)            
  uint8_t  privacy    :2;       // None(0),Disable(1),Enable(2)  
  uint8_t  message    :2;       // None(0),Unread(1),Full(2)         
  uint8_t  gps        :2;       // None(0),without data(1),with data(2)    

  uint8_t  disableTone:1;       // Disable(0),Enable(1)              
  uint8_t  talkaround :1;       // Disable(0),Enable(1)              
  uint8_t  monitor    :1;       // Disable(0),Enable(1)              
  uint8_t  cha_type   :2;       // Channel Type          
  uint8_t  cha_mode   :1;       // normal(0),emergency(1)                  
  uint8_t  e_disp_type:2;       // emerg_alias(0),emerg_char(1)            

  uint8_t  bt_state:2;          // State:0,not open;1,unconnect;2,connect  
  uint8_t  ble_state:1;         // State:0,not link;1,link                 
  uint8_t  volume_lev:5; 

  uint8_t  dmr_slot:1;          // slot1(0),slot2(1)                       
  uint8_t  dmr_cc:4; 
  uint8_t  bit_rsd:3; 

  uint8_t  byte_rsd[2]; 

  uint8_t  freq_alias[16]; 
  uint8_t  freq_space[8]; 
  uint32_t rx_freq;    
  uint32_t tx_freq; 
  uint8_t  zone_alias_content[32]; 
  uint8_t  channel_in_zone_num;                  
  uint8_t  zone_num;                  
  uint16_t channel_physic_num;    

}FB_HomeScreenWinData_t; 
*/

/*
//
// ****************************************************************************
// August 17, 2018		Design Documents                set_normal_ack()
//
//      Loads the acknowlegement buffer with the standard ack
//      Most packets have the same ack.
// 
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     11/02/2018 replaced by acknowledge process_A
//
// Status
//  written, working, no longer used
//

uint8_t normal_ack[9] = {0xB1, 0x00, 0x05, 0x80, 0x40, 0x00, 0x00, 0xC0, 0xB4};

void set_normal_ack( void )
{
  memcpy( (char*) ack_buffer, (char const*) normal_ack, 9);
}
*/
// utility routines may relocate...

// =============================================================================
//
//   Routine to convert 4 bytes stored LSB first to a 32bit unsigned int
//
//   for TX and RX frequencies
//
//  History 
//     10/25/2018 written, working.

uint32_t convert_LSB_bytes_to_int ( uint8_t* in_data )  
{
  uint32_t val = 0;
  val = in_data[0] +
       (in_data[1] << 8) +
       (in_data[2] << 16) +
       (in_data[3] << 24);  
  return val;
}
// =============================================================================
//
//   Routine to store 4 bytes LSB first from a 32bit unsigned int
//
//   for TX and RX frequencies
//
//  History 
//     06/19/2019 written, working.

void store_4bytes_LSB( uint8_t* dest, uint32_t data )  
{
  dest[0] =  (data & 0x000000FF);
  dest[1] =  (data & 0x0000FF00) >> 8;
  dest[2] =  (data & 0x00FF0000) >> 16;  
  dest[3] =  (data & 0xFF000000) >> 24;
}

// ****************************************************************************
// April 29, 2018		Design Documents                 process_CS800D
//
// This program processes packets received from UART1. This program is called
// by a 1 millisecond interrupt. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  uart1_rx_flag true indicates there is data sitting in the uart1_rx_buffer[] 
//
// Exit Conditions
//  if uart1_rx_flag true then the following happens:
//    1. uart1_rx_buffer[] is saved to the packet_process_buffer[]
//    2. The packet is taken apart and sent to a non interrupt program to be
//       displayed or otherwise handled.
//    3. The acknowledgement buffer is setup for other programs to handle
//
// External functions called
//  uint32_t packet_type_A(void)
//  uint32_t packet_type_0(void)
//  uint32_t packet_type_1(void)
//  uint32_t packet_type_2(void)
//
// External variables used
//  uart1_rx_flag
//  uart1_rx_buffer[256]
//  packet_process_buffer[256]
//  unknown_packet_error
//  unknown_packet_error_buffer[256]
//
// Internal variables used
//  uint32_t i
//  uint32_t ready_flag
//
// Constants
//
// History
//     09/05/2018 renamed for CS800D
//     11/02/2018 major revision to use header info in a table with the
//                acknowledge process to use also in the table.
//     11/21/2018 bugfix, packet output pointer & 0x031 should be 0x03 
//
// Status
//  written, working
//

// the packet info table:
  
  struct packet_info {
    uint8_t header[5];             // up to 5 bytes of packet header 
    int headlen;                   // number of bytes to check 
    uint32_t (*ack_proc)( void );  // acknowledge process for the packet 
  };  
  //  Header              Length  Acknowledge process 
  struct packet_info packet_table[24] = {
  { 0x05, 0, 0x01, 0,    0, 3, packet_type_0 }, 
  { 0x05, 0, 0x70, 0,    0, 3, packet_type_A },  //1  
  { 0x04, 0, 0x10, 0,    0, 3, packet_type_2 },
  { 0x49, 0, 0x40, 7,    1, 4, packet_type_A },  //3
  { 0x49, 0, 0x40, 0,    0, 4, packet_type_A },  //4
  { 0x50, 0, 0x40, 0,    0, 4, packet_type_A },  //5
  { 0x29, 0, 0x40, 1, 0x3F, 4, packet_type_A },  //6a 3F, 6b 2F both ok for A
  { 0x4B, 0, 0x40, 2, 0x13, 5, packet_type_A },  //7
  { 0x4B, 0, 0x40, 2, 0x2C, 5, packet_type_A },  //8
  { 0x4B, 0, 0x40, 2, 0x10, 5, packet_type_A },  //9
  { 0x47, 0, 0x40, 6,    0, 4, packet_type_A },  //10  
  { 0x4B, 0, 0x40, 2, 0x2F, 5, packet_type_A },  //11
  { 0x50, 0, 0x40, 3,    0, 4, packet_type_A },  //12
  { 0x4B, 0, 0x40, 8,    0, 4, packet_type_A },  //13 
  { 0xB8, 0, 0x40, 0,    0, 3, packet_type_A },  //14
  { 0x97, 0, 0x40, 0,    0, 3, packet_type_A },  //15
  { 0x4C, 0, 0x40, 0,    0, 4, packet_type_A },  //16
  { 0x38, 0, 0xA1, 0,    0, 3, packet_type_A },  //17
  { 0x14, 0, 0xA2, 0,    0, 3, packet_type_A },  //18
  { 0x32, 0, 0xA0, 0,    0, 3, packet_type_A },  //19  
  { 0x05, 0x80, 0xA4, 0, 0, 3, packet_type_B },  //20  temp_dch
  { 0x05, 0x80, 0xA5, 0, 0, 3, packet_type_B },  //21  temp_ach
  { 0x05, 0x80, 0xA9, 0, 0, 3, packet_type_C },  //22  get_rssi
  { 0x05, 0x80, 0xA6, 0, 0, 3, packet_type_B },  //23  store_temp_ch 
  };
  
void process_CS800D(void)
{
  uint32_t i;
  uint32_t ready_flag;
  uint32_t packets_fifo_count = 0;
  uint32_t packets_fifo_output_pointer = 0;
  uint32_t packets_fifo_input_pointer = 0;
  //uint32_t packets_fifo_overrun_error = 0;  // never used

  // Watchdog timer for packet_process_buffer_flag
  if (packet_process_watchdog_timer) packet_process_watchdog_timer--;
  if (!packet_process_watchdog_timer)
  {
    packet_process_buffer_flag = 1;
  }

  // If the programs are ready to accept more data,
  // and there is data in the packet fifo,
  // output the oldest packet to the packet_process_buffer.
  ready_flag = 0;
  if ((packet_process_buffer_flag == 1) && (packets_fifo_count))
  {
    for (i = 0; i < packets_fifo[packets_fifo_output_pointer][2] + 4; i++)
    {
      packet_process_buffer[i] = packets_fifo[packets_fifo_output_pointer][i];
    } 
    packets_fifo_output_pointer = (packets_fifo_output_pointer + 1) & 0x03;
    packets_fifo_count--;
    ready_flag = 1;
  }

  // If the programs are ready to accept more data,
  // and there is NO data in the packet fifo,
  // but there is a finished packet in the uart buffer,   
  // output the uart data directly to the packet_process_buffer.
  // flag that the data in the uart has been taken, and process.
  if ( (packet_process_buffer_flag == 1) && (packets_fifo_count == 0) &&
       (uart1_rx_flag == 1) && (ready_flag == 0)  )
  {
    for (i = 0; i < uart1_rx_buffer_index + 1; i++)
    {
      packet_process_buffer[i] = uart1_rx_buffer[i];
    }  
    packets_fifo_input_pointer = (packets_fifo_input_pointer + 1) & 0x03;
    uart1_rx_flag = 0;
    ready_flag = 1;
  }

  // If there is a finished packet in the UART1 buffer,
  // copy it directly to the packets_fifo.
  // flag that the data in the uart has been taken.
  if (uart1_rx_flag == 1)
  {
     if (packets_fifo_count < 4)
     {
       for (i = 0; i < uart1_rx_buffer_index + 1; i++)
       {
         packets_fifo[packets_fifo_input_pointer][i] = uart1_rx_buffer[i];
       }  
       packets_fifo_input_pointer = (packets_fifo_input_pointer + 1) & 0x03;
       packets_fifo_count++;
     }
     else
     {
       //packets_fifo_overrun_error = 1;
       error_log( 3071 );
     }
     uart1_rx_flag = 0;   
  }  
  // If not ready to process, exit immediately,
  // there is nothing to process at this time.
  if (!ready_flag) return;
  
  // Start processing the data to determine which program to process
  packet_process_watchdog_timer = 1000;
  packet_process_buffer_flag = 1;  // data is now in use
  
  // Determine and process the various packet types
  // ==============================================
  int pack_type = 0;
  // pack_type is the packet number and also the index in the table.
  // TODO: separate index from packet number to allow flexibility
  
  for(pack_type = 0; pack_type < 24; pack_type++)
  {
    // check the packet header with the table
    for( i = 0; i < packet_table[pack_type].headlen; i++)
    {
      if( packet_process_buffer[2 + i] != packet_table[pack_type].header[i] ) break;
    }
    // if the header is matched
    if(i == packet_table[pack_type].headlen)
    {
      // call the acknowledge process stored in the table
      packet_table[pack_type].ack_proc();
      // set packet type
      packet_buffer_output_type = pack_type;  
      // Set packet_process_buffer flag for mainline program to process
      packet_buffer_output_flag = 1;  

      return;
    }
  }
  // If we got here, we have an unknown packet type. Save the packet
  for (i = 0; i < packet_process_buffer[2]; i++)
  {
    unknown_packet_error_buffer[i] = packet_process_buffer[i];
  } 
  // display_packet_dump( unknown_packet_error_buffer );
  error_log( 3051 );
  packet_process_buffer_flag = 1; // allow processing to continue 
}
// =============================================================================
// November 1, 2018         Design Documents                          RCDB_start
// 
//  Startup panel response for RCDB (button F2 held at start)
//   This is na command packet sent by the front panel at startup.
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     11/01/2018 ver 1.0.2 added to handle proper startup 
//
// Status
//  written, working
//
void RCDB_start( void )
{
  // B1 00 07 00 23 16 00 01 00 3A B4
  // Setup key buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  0x07;
  key_buffer[3] =  0x00;
  key_buffer[4] =  0x23;
  key_buffer[5] =  0x16;
  key_buffer[6] =  0x00;
  key_buffer[7] =  0x01;
  key_buffer[8] =  0x00;
  key_buffer[9] =  0x3A;
  key_buffer[10] =  0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
}
// =============================================================================
// 11/12/2018  
// 
//    Send zone and channel number to radio
//    a radio command packet
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     11/12/2018
//
// Status
//  written, working
//
void set_zone_channel( uint8_t zone, uint16_t channel )
{
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  int i;
  
  // Setup key buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  0x08;  
  key_buffer[3] =  0x00;
  key_buffer[4] =  0xA3;
  key_buffer[5] =  zone;  // 1-250
  key_buffer[6] =  0x00;  // reserved
  key_buffer[7] =  channel >> 8;    // channel high byte 
  key_buffer[8] =  channel &0x00FF; // channel low byte
  
  for( i = 3; i < 9; i++) chk_sum += key_buffer[i];
  
  key_buffer[9]  = chk_sum >> 8;     // high byte of checksum
  key_buffer[10] = chk_sum & 0x00FF; // low byte  
  key_buffer[11] =  0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
  // log packet
  log_packet( "tA3" );  
}
// =============================================================================
// 01/03/2019  
// 
//    Send frequency settings to radio for a digital channel
//    a radio command packet
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     01/03/2019
//
// Status
//     written, working
//

//  starting temp channel defaults
//  adjust these from the vfo screen
uint8_t channel_type = 0;
uint8_t power_level = 0; // 2 bits 0=low, 1=Med, 2=High
uint8_t dmr_color = 1;   // 0-15
uint8_t dmr_slot  = 1;   // 0 or 1, means 1 or 2
//uint8_t rec_type = 0;    // 0 private, 1 group, 2 alll call
// using cs_call_type 

// uint32_t tx_contact_num = 0;
uint32_t tx_contact_num = 99;  // test value
// uint32_t tx_contact_num = 16777215;  // all call id didn't work

volatile uint8_t tempDCH_packet_ack = 0;

void set_temp_DCH( uint32_t tx_freq, uint32_t rx_freq )
{
  channel_type = 0; //DMR
  // cs_channel_monitor = 1;  // monitor mode on
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  int i;
  
  // Setup buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  20;  
  key_buffer[3] =  0x00;
  key_buffer[4] =  0xA4;
  // tx frequency
  store_4bytes_LSB( key_buffer + 5, tx_freq );
  // rx frequency
  store_4bytes_LSB( key_buffer + 9, rx_freq );  
  // type and power
  key_buffer[13] = ( channel_type << 7) +
                   ( power_level << 5 ) +
                   ( cs_channel_monitor << 4 );
  // color and slot
  key_buffer[14] = (dmr_color << 4) + (dmr_slot << 3) + (cs_call_type << 1);  
  key_buffer[15] = 0; // reserved
  key_buffer[16] = 0; // reserved
 
  // tx contact
  store_4bytes_LSB( key_buffer + 17, tx_contact_num );  
 
  for( i = 3; i < 21; i++) chk_sum += key_buffer[i];
  
  key_buffer[21] = chk_sum >> 8;   // high byte of checksum
  key_buffer[22] = chk_sum & 0x00FF; // low byte  
  key_buffer[23] = 0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
  log_packet( "tA4" ); 
}

// =============================================================================
// 01/08/2019  
// 
//    Send frequency settings to radio for an anlaog channel
//    a radio command packet
//
// additional info:
// rx_sqlmode use:
// (CTCSS or DCS ) and audio (2 tone)    0
// Audio (2 tone)                        1
// CTCSS or DCS                          2
// carrier                               3
//
// this command is used for setting an enhanced channel, scanning,
// and the spectrum view.
// In these uses carrier squelch is sufficient.  

//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     01/08/2019
//     01/18/2019  revised for MS20.52 (fixed errors)
//
// Status
//     written, working
//
uint8_t rx_sqlmode = 3;   // set to carrier type at start
uint8_t ctcss_decode_type = 0;
uint8_t ctcss_encode_type = 0;

volatile uint8_t tempACH_packet_ack = 0;

void set_temp_ACH( uint32_t tx_freq, uint32_t rx_freq )
{
  channel_type = 1;  // analog
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  int i;
 
  // Setup buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  20;   // 0x14
  key_buffer[3] =  0x00;
  key_buffer[4] =  0xA5;
  // tx frequency
  store_4bytes_LSB( key_buffer + 5, tx_freq );  
  // rx frequency
  store_4bytes_LSB( key_buffer + 9, rx_freq );  

  // type and power
  key_buffer[13] = ( channel_type << 7) +
                   ( power_level << 5 ) +
                   ( cs_channel_monitor << 4 );
  // rxsql and ctss type
  key_buffer[14] = (( rx_sqlmode ) << 4) + ((ctcss_decode_type ) << 2) + ctcss_encode_type;  
  key_buffer[15] = 0; // reserved
  key_buffer[16] = 0; // reserved
  
  switch( ctcss_decode_type )
  {
  case 0: // none
    {
      store_4bytes_LSB( key_buffer + 17, 0 );      
      break;
    }
  case 1: // ctcss
    {
      key_buffer[17] = ctcss_decode  & 0x00FF; // low byte
      key_buffer[18] = ctcss_decode >> 8;      // high byte
      key_buffer[19] = ctcss_encode  & 0x00FF;  
      key_buffer[20] = ctcss_encode >> 8; 
      break;
    }
  case 2: // DCS
  case 3: // DCS invert, same as DCS, the radio inverts     
    {
      key_buffer[17] = dcs_decode  & 0x00FF;   
      key_buffer[18] = dcs_decode >> 8; 
      key_buffer[19] = dcs_encode  & 0x00FF;   
      key_buffer[20] = dcs_encode >> 8; 
      break;      
    }
  default: break;
  }
  
  for( i = 3; i < 21; i++) chk_sum += key_buffer[i];
  
  key_buffer[21] = chk_sum >> 8;     // high byte of checksum
  key_buffer[22] = chk_sum & 0x00FF; // low byte  
  key_buffer[23] = 0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
  tempACH_packet_ack = 0;
  log_packet( "tA5" ); 
  
}
// =============================================================================
// 01/17/2019  
// 
//    Request RSSI from radio command
//    a radio command packet
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//     01/17/2019 for firmware MS20.51
//
// Status
//     written, working
//
volatile uint8_t rssi_packet_ack = 0;

void request_RSSI_value( void )
{
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  //int i;
  
  // Setup buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  5;  
  key_buffer[3] =  0x00;
  key_buffer[4] =  0xA9;
  // RSSI, 0 for the request command?
  key_buffer[5] =  0;
  // for( i = 3; i < 6; i++) chk_sum += key_buffer[i];
  chk_sum = 0x00A9;
  
  key_buffer[6] = chk_sum >> 8;   // high byte of checksum
  key_buffer[7] = chk_sum & 0x00FF; // low byte  
  key_buffer[8] = 0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
  // set for packet acknowledge 
  rssi_packet_ack = 0;
  log_packet( "tA9" );   
}
// =============================================================================
//
//  Store temporary channel
//
// 01/21/2019 testing

void store_temp_CH( uint32_t tx_freq, uint32_t rx_freq )
{
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  int i;
 
  // Setup buffer.
  key_buffer[0] =  0xB1;
  key_buffer[1] =  0x00;
  key_buffer[2] =  20;  
  key_buffer[3] =  0x00;
  key_buffer[4] =  0xA6;
  // tx frequency
  store_4bytes_LSB( key_buffer + 5, tx_freq );  
  // rx frequency
  store_4bytes_LSB( key_buffer + 9, rx_freq );  

  // type and power
  key_buffer[13] = ( channel_type << 7) +
                   ( power_level << 5 ) +
                   ( cs_channel_monitor << 4 );
  
  if( channel_type == 0 ) // DMR
  {
    // color and slot
    key_buffer[14] = (dmr_color << 4) + (dmr_slot << 3) + (cs_call_type << 1);  
    key_buffer[15] = 0; // reserved
    key_buffer[16] = 0; // reserved
    // tx contact
    store_4bytes_LSB( key_buffer + 17, tx_contact_num );    
  }
  else // analog
  {
    // rxsql and ctss type
    key_buffer[14] = ((rx_sqlmode ) << 4) + ((ctcss_decode_type ) << 2) + ctcss_encode_type;  
    key_buffer[15] = 0; // reserved
    key_buffer[16] = 0; // reserved
    
    switch( ctcss_decode_type )
    {
    case 0: // none
      {
        store_4bytes_LSB( key_buffer + 17, 0 );         
        break;
      }
    case 1: // ctcss
      {
        key_buffer[17] = ctcss_decode  & 0x00FF; // low byte
        key_buffer[18] = ctcss_decode >> 8;      // high byte
        key_buffer[19] = ctcss_encode  & 0x00FF;  
        key_buffer[20] = ctcss_encode >> 8; 
        break;
      }
    case 2: // DCS
      {
        key_buffer[17] = dcs_decode  & 0x00FF;   
        key_buffer[18] = dcs_decode >> 8; 
        key_buffer[19] = dcs_encode  & 0x00FF;   
        key_buffer[20] = dcs_encode >> 8; 
        break;      
      }
    case 3: // DCS invert  
      {
        key_buffer[17] = ~dcs_decode  & 0x00FF;   
        key_buffer[18] = ~dcs_decode >> 8; 
        key_buffer[19] = ~dcs_encode  & 0x00FF;   
        key_buffer[20] = ~dcs_encode >> 8;    
      }
    default: break;
    }
  }
  for( i = 3; i < 21; i++) chk_sum += key_buffer[i];
  
  key_buffer[21] = chk_sum >> 8;     // high byte of checksum
  key_buffer[22] = chk_sum & 0x00FF; // low byte  
  key_buffer[23] = 0xB4;
  // set flag to activate transmission  
  key_buffer_flag = 1;
  log_packet( "tA6" ); 
}
// =============================================================================
//
//    Multiple packet type handler
//      sends an acknowledgement packet for the packet received
//
//  using data stored in the table, all the packets that have just the packet
//   type and a check sum can be handled by this.
//
// History
//   11/02/2018 written for use with table
//
// Status: written, working  

uint32_t packet_type_A(void)
{
  uint16_t chk_sum = 0;  // must use 16 bit unsigned
  
  // Setup acknowlegement buffer.
  ack_buffer[0]  =  0xB1;
  ack_buffer[1]  =  0x00;
  ack_buffer[2]  =  0x05;
  ack_buffer[3]  =  0x80;
  ack_buffer[4]  =  packet_process_buffer[4];  // packet type
  ack_buffer[5]  =  0x00;
  chk_sum = ack_buffer[3] + ack_buffer[4] + ack_buffer[5];
  
  ack_buffer[6]  = chk_sum >> 8;   // high byte of checksum
  ack_buffer[7]  = chk_sum & 0x00FF; // low byte
  ack_buffer[8]  = 0xB4;
  ack_buffer_flag = 1;  
  
  // display_packet_dump( packet_process_buffer );  
   
  sprintf( pkt_id,"a%i", packet_buffer_output_type);
  log_packet( pkt_id );     
  
  // indicate packet has been processed.
  return 1;     
}
// =============================================================================
//
//    Acknowledge packet type handler
// 
//    This is for acknowledge packets that come after sending a command
//
// History
//   01/08/2019
//   01/29/2019  added acknowledgment flag for tempACH 
//
// Status: writing  

uint32_t packet_type_B(void)
{
//  int n;
  // indicate packets have been acknowledged
  if( packet_buffer_output_type == 21 ) tempACH_packet_ack = 1;

//  n = sprintf( pkt_id,"a%i", packet_buffer_output_type);
  sprintf( pkt_id,"a%i", packet_buffer_output_type);
  log_packet( pkt_id );    
    
  // log_event( "Ack packet" );   
  // indicate packet has been processed. 
  return 1;     
}
// =============================================================================
//
//    RSSI data packet type handler
// 
//    This is for the acknowledge packet that comes after sending a
//    request for RSSI
//
// History
//   01/17/2019
//
// Status: writing  

uint32_t packet_type_C(void)
{
  // log_event( "RSSI" );  // no log, using rssi graph and spectrum
  // if( packet_buffer_output_type == 22 ) rssi_packet_ack = 1;  
  //rssi_packet_ack = 1;
  // indicate packet has been processed. It is not an unknown packet
  
  log_packet( "aA9" );   
  
  return 1;     
}
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_0
//
// This command is used to determine if the display is still attached to the
// radio.
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display                     Display to Radio
// B1 00 05 00 01 01 00 02 B4         B1 00 05 80 01 01 00 82 B4        
// B1 00 05 00 01 02 00 03 B4         B1 00 05 80 01 02 00 83 B4     
// B1 00 05 00 01 03 00 04 B4         B1 00 05 80 01 03 00 84 B4     
// B1 00 05 00 01 04 00 05 B4         B1 00 05 80 01 04 00 85 B4     
// B1 00 05 00 01 05 00 06 B4         B1 00 05 80 01 05 00 86 B4     
// B1 00 05 00 01 06 00 07 B4         B1 00 05 80 01 06 00 87 B4     
// B1 00 05 00 01 07 00 08 B4         B1 00 05 80 01 07 00 88 B4     
// B1 00 05 00 01 08 00 09 B4         B1 00 05 80 01 08 00 89 B4      
// B1 00 05 00 01 09 00 0A B4         B1 00 05 80 01 09 00 8A B4     
// B1 00 05 00 01 0A 00 0B B4         B1 00 05 80 01 0A 00 8B B4  
//
// Notice that character 5 is incrementing from 0x01 through 0x0A. On the 
// acknowledgement of that string, a duplicate is sent back except character 3 
// is always 80. The checksum is calculated on the data.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_output_type contains the packet type
//  packet_buffer_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  ack_buffer_flag
//
// Internal variables used
//
// Constants
//
// History
//     11/02/2018 converted to be just the acknowledge process. Using header
//                info in table, packet determination is done before calling  
// Status
//  written, working
//
uint32_t packet_type_0(void)
{
  // Setup acknowlegement buffer.
  ack_buffer[0] =  0xB1;
  ack_buffer[1] =  0x00;
  ack_buffer[2] =  0x05;
  ack_buffer[3] =  0x80;
  ack_buffer[4] =  0x01;
  ack_buffer[5] =  packet_process_buffer[5];
  ack_buffer[6] =  0x00;
  ack_buffer[7] =  packet_process_buffer[7] + 0x80;
  ack_buffer[8] =  0xB4;  
  ack_buffer_flag = 1;
  
  // indicate packet has been processed.
  return 1;  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_1
//
// This is an idle command along with the information necessary to control the
// transmit led, receive led, and backlight for the LCD display.
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display              Display to Radio   
// B1 00 05 00 70 02 00 72 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Green LED
// B1 00 05 00 70 01 00 71 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Red LED
// B1 00 05 00 70 03 00 73 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Red + Green
// B1 00 05 00 70 00 00 70 B4    B1 00 05 80 70 00 00 F0 B4  Turn off both LEDs
// B1 00 05 00 70 0C 00 7C B4    B1 00 05 80 70 00 00 F0 B4  Turn on backlight
// B1 00 05 00 70 08 00 78 B4    B1 00 05 80 70 00 00 F0 B4  Turn off backlight
//
// Note that the response to this protocol is always the same.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_output_type contains the packet type
//  packet_buffer_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  ack_buffer_flag
//
// Internal variables used
//
// Constants
//
// History
//     11/02/2018 converted to be just the acknowledge process. Using header
//                info in table, packet determination is done before calling 
//
// Status
//  written. not used, handled by packet_type_A
//
uint32_t packet_type_1(void)
{
  // Setup acknowlegement buffer.
  ack_buffer[0] =  0xB1;
  ack_buffer[1] =  0x00;
  ack_buffer[2] =  0x05;
  ack_buffer[3] =  0x80;
  ack_buffer[4] =  0x70;   // only difference from normal
  ack_buffer[5] =  0x00;
  ack_buffer[6] =  0x00;
  ack_buffer[7] =  0xF0;
  ack_buffer[8] =  0xB4;  
  ack_buffer_flag = 1;

  log_packet( "a1" );    
  
  // indicate packet has been processed.
  return 1;  
}
// =============================================================================
// April 29, 2018		Design Documents                   packet_type_2
//
// This command request the firmware version number of the display
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display              Display to Radio 
// B1 00 04 00 10 00 10 B4       B1 00 19 80 10 00 32 2E 30 30 2E 30 38 00 00  
//                               00 00 00 00 00 00 00 0A 00 00 00 01 F0 B4
//
//  10/11/2018  revised data structure of response packet from Mfr. 
// typedef struct
// {
//   uint8_t  FirmwareVer[8];              //[6] - [13]
//   uint8_t  BtModuleWork_Flag;           //[14]  
//   uint8_t  FontLibVer[7];               //[15]-[21]
//   uint16_t LanguageTp;                  //[22]-[23] 
//   uint8_t  FbType;  //0x00:small lcd; 0x01: large lcd  [24]
//   uint8_t  Receive_Fb_Flag;  //0xAA                    [25]
// } FB_FirmwareInfo;
//
// Radio to Display              Display to Radio 
// B1 00 04 00 10 00 10 B4       B1 00 19 80 10 00 32 2E 30 30 2E 30 38 00 00  
//                               00 00 00 00 00 00 00 00 00 01 AA 02 91 B4

// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_output_type contains the packet type
//  packet_buffer_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_output_flag
//  packet_buffer_output_type
//
// Internal variables used
//
// Constants
//
// History
//     10/11/2018 revised ack for production units
//     11/01/2019 changed for firmware 2.00.14
//
// Status
//  written
//
uint32_t packet_type_2(void)
{
  uint32_t i;
  // set buffer to zero so only have to change non zero values
  for (i = 0; i < 28; i++) ack_buffer[i] = 0;

  /*
  // for prototype boards MS20.31
  // Setup acknowlegement buffer.
  ack_buffer[0]  =  0xB1;
  ack_buffer[2]  =  0x19;
  ack_buffer[3]  =  0x80;
  ack_buffer[4]  =  0x10;
  ack_buffer[6]  =  0x32; // 2
  ack_buffer[7]  =  0x2E; // .
  ack_buffer[8]  =  0x30; // 0
  ack_buffer[9]  =  0x30; // 0
  ack_buffer[10] =  0x2E; // .
  ack_buffer[11] =  0x30; // 0
  ack_buffer[12] =  0x38; // 8
  ack_buffer[22] =  0x0A;
  ack_buffer[26] =  0x01;
  ack_buffer[27] =  0xF0;
  ack_buffer[28] =  0xB4;
  ack_buffer_flag = 1;
  */
  
  // for first production units MS20.39,
  // works with 20.40, 20.41, 20.43
  // working up to 20.58  
  // Setup acknowlegement buffer.
  ack_buffer[0]  =  0xB1;
  ack_buffer[2]  =  0x19;
  ack_buffer[3]  =  0x80;
  ack_buffer[4]  =  0x10;
  ack_buffer[6]  =  0x32; // 2
  ack_buffer[7]  =  0x2E; // .
  ack_buffer[8]  =  0x30; // 0
  ack_buffer[9]  =  0x30; // 0
  ack_buffer[10] =  0x2E; // .
  ack_buffer[11] =  0x31; // 0    1  testing new firmware number
  ack_buffer[12] =  0x34; // 8    4
  ack_buffer[22] =  0x0A;    
  ack_buffer[24] =  0x01;    // new panel type code
  ack_buffer[26] =  0x01;    // 
  ack_buffer[27] =  0xEE;    //   was 1F1
  ack_buffer[28] =  0xB4;
  ack_buffer_flag = 1;

  // indicate packet has been processed.
  return 1;    
}
// end of acknowledges
// =============================================================================
// 
// ****************************************************************************
// April 29, 2018		Design Documents              process_all_packet
//
// This program processes packets for output. This program is called from the
// mainline because of the possibly lenghty time needed to process these packets 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  if packet_buffer_output_flag true, process packet
//  packet_buffer_output_type is the packet number to process
//
// Exit Conditions
//  packet is processed
//
// External functions called
//  void packet_buffer_0_output(void)
//  void packet_buffer_1_output(void)
//  void packet_buffer_2_output(void)
//  void packet_buffer_3_output(void)
//  void packet_buffer_4_output(void)
//  void packet_buffer_5_output(void)
//  void packet_buffer_6_output(void)
//  void packet_buffer_7_output(void)
//  void packet_buffer_8_output(void)
//  void packet_buffer_9_output(void)
//  void packet_buffer_10_output(void)
//  void packet_buffer_11_output(void)
//  void packet_buffer_12_output(void)
//  void packet_buffer_13_output(void)
//  void packet_buffer_14_output(void)
//  void packet_buffer_15_output(void)
//  void packet_buffer_16_output(void)
//  void packet_buffer_17_output(void)
//  void packet_buffer_18_output(void)
//  void packet_buffer_19_output(void)
//
// External variables used
//  packet_buffer_output_flag
//  packet_buffer_output_type
//
// Internal variables used
//  packet_type_err
//
// Constants
//
// History
//
// Status
//  written
//
/* for packet debug
uint32_t packet_list1[64][4];
uint32_t pac_index = 0;
*/
uint32_t packet_counter = 0;

void process_all_packet(void)
{
//  int n;
  
  if (!packet_buffer_output_flag) return;
  packet_buffer_output_flag = 0;

  int packet_type_err = 0;
  packet_counter++;
  radio_watchdog_timer = 3000;
  switch (packet_buffer_output_type)
  {
    case 0:
      packet_buffer_0_output(); break;  // heartbeat 
    case 1:
      packet_buffer_1_output(); break;  // LED's & backlight
    case 2:
      packet_buffer_2_output(); break;  // version number
    case 3:
      packet_buffer_3_output(); break;
    case 4:
      packet_buffer_4_output(); break;
    case 5:
      packet_buffer_5_output(); break;  // channel info
    case 6:
      packet_buffer_6_output(); break;
    case 7:
      packet_buffer_7_output(); break;
    case 8:
      packet_buffer_8_output(); break;
    case 9:
      packet_buffer_9_output(); break;
    case 10:
      packet_buffer_10_output(); break;
    case 11:
      packet_buffer_11_output(); break;
    case 12:
      packet_buffer_12_output(); break;
    case 13:
      packet_buffer_13_output(); break;
    case 14:
      packet_buffer_14_output(); break;
    case 15:
      packet_buffer_15_output(); break;
    case 16:
      packet_buffer_16_output(); break;  //enhanced channel info
    case 17:
      packet_buffer_17_output(); break;
    case 18:
      packet_buffer_18_output(); break;
    case 19:
      packet_buffer_19_output(); break;
    case 20:
      packet_buffer_20_output(); break;
    case 21: 
      packet_buffer_21_output(); break;
    case 22:
      packet_buffer_22_output(); break;
    
  default: 
      packet_type_err = 1; break;
  }
  /*
  // packet debug
  packet_list1[pac_index][0] = packet_process_buffer[2];
  packet_list1[pac_index][1] = packet_process_buffer[4];
  packet_list1[pac_index][2] = packet_process_buffer[5];
  packet_list1[pac_index++][3] = packet_buffer_output_type;
  if(pac_index > 63) pac_index = 0;
  */
  
  if( packet_type_err )
  {
    packet_process_buffer_flag = 0;
    // for debugging
    // copy packet to storage for later display
    
    log_event( "unknown packet" );   
  }
  else packet_process_buffer_flag = 1;
  
  // log all packets
//  n = sprintf( pkt_id,"r%i", packet_buffer_output_type);
  sprintf( pkt_id,"r%i", packet_buffer_output_type);
  log_packet( pkt_id );
  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_0_output
//
// This routine is used for process the idle message type 1 and there is no 
// further processing except used as a watch dog.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Watchdog timer reset
//
// External functions called
//
// External variables used
//  radio_watchdog_timer
//
// Internal variables used
//
// Constants
//
// History
//     08/01/2018 nothing was written, added packet message to screen
//                too many packets, turned off the message
//     12/03/2018 added setting of watchdog here
//     12/06/2018 added screen display of heartbeat indicator  
//
// Status
//  written, working
//
extern uint32_t radio_info_timer;

void packet_buffer_0_output(void)
{
  uint8_t hb_count = 0;
  // int i;
  // every packet resets the timer. 10msec timer decrements 
  // if no packets when timer is at zero, radio is not connected
  // after this packet, timer is > 0, radio is connected.
  radio_watchdog_timer = 3000;
  
  hb_count = packet_process_buffer[5] & 0x07;  // reduce to 8 count
  if( active_screen_num == 1)
  {
    display_heartbeat( 360, 110, hb_count );
    display_radio_connected( 3, 40, 130, 4 ); 
  }
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_1_output
//
// This routine is used for process the idle message type 2. The message is used
// to control the transmit led, receive led, and backlight.
//
// Character 5 contains the command as follows:
// Bit 0    Red LED:   1 = on, 0 = off
// Bit 1    Green LED: 1 = on, 0 = off
// Bit 2    clear top message line
// Bit 3    clear bottom message line
// Bit 4     unknown, program does a packet dump to check
// Bit 5     unknown, program does a packet dump to check
// Bit 6   Backlight Control: 1 = on, 0 = off
// Bit 7   BackLight
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  LED's turned on and off as dictated by the program
//
// External functions called
//  void tx_led_on(void)
//  void tx_led_off(void)
//  void rx_led_on(void)
//  void rx_led_off(void)
//  void lcd_backlight_on(void)
//  void lcd_backlight_off(void)  
//
// External variables used
//  packet_process_buffer[5]
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t radio_transmiting = 0;
uint8_t radio_receiving = 0;
uint8_t cs_program_mode = 0;

void packet_buffer_1_output(void)
{
  // process transmit (red) LED
  if (packet_process_buffer[5] & 0x01)
  {
    tx_led_on();
    // radio_transmiting = 1; is set by the screen draw routine
    // display "on air" if not in programming mode and not already on
    // and not scanning and PTT is pressed
    if(!cs_program_mode && !radio_transmiting && !scan_on
       && CS800_buttons[0].press_flag == 1 )
    {
      draw_TXonoff( 1 );
    }
  }
  else
  {
    // turn off if it was on, unless in timeout after PTT
    if(ptt_timer == 0 && !radio_transmiting ) 
    {
      tx_led_off();
      // clear on the main screen
      if( active_screen_num == 1 )
      {
        draw_TXonoff( 0 );  
      }
    }      
  }
  // process receive(green) LED
  if (packet_process_buffer[5] & 0x02 && cs_channel_scan == 0 )
  {
    rx_led_on();
    // checks rx_frequency to prevent flash at startup
    if ( cs_channel_rx_frequency > 0 ) draw_RXonoff( 1 ); 
    radio_receiving = 1;
    if( TGM_call_active == 0 )
    {
      TGM_new_call();
    }
  }
  else
  {
    
    if( cs_channel_rssi < 2 )
    {
      rx_led_off();
      draw_RXonoff( 0 );      
      radio_receiving = 0;
      if( TGM_call_active == 1 )
      {
        TGM_end_call();    
      }
    }
  }

  // ignore backlight control
  // log packet 
  // log_event( "packet 1" );
  /*
  // dump when unknown bits come in
  if( packet_process_buffer[5] & 48) // 0x30 bits 4,5 (16+32)
  {
    display_packet_dump( packet_process_buffer );  
  }
  */
  
  if( packet_process_buffer[5] & 0x08)
  {
    if(display_mode == 1)
    {
      clear_pckt_message( 4 );
      // redisplay channel
      display_channel_alias();
      pckt_msg4_dirty = 1;
    }
  }
  if( packet_process_buffer[5] & 0x04)
  {
    if(display_mode == 1)
    {
      clear_pckt_message( 3 );
      // redisplay zone
      display_zone_alias();
      pckt_msg3_dirty = 1;
    }
  }   
 
  if(display_mode == 1)
  {
    // packet 1 has no text, set msg 1 to the value of the bit command byte 
    sprintf((char*) msg1_dump, "%02X", packet_process_buffer[5]);
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 1  ", 10);

    display_debug_p();
    //ssd1963_cg_write_line( 10, 3, debug_p_x , debug_p_y + debug_p_line++ * debug_p_step, 4);
    // if(debug_p_line > 5) debug_p_line = 0;
    //ssd1963_cg_write_line( 10, 3, 40 , 240, 4);       
  }
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_2_output
//
// This routine is used for processing the version number request from the radio
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters provided by the program
//
// Exit Conditions
//  There is no processing done in this program for this packet type
//
// History
//
// Status
//    writing, was empty
//
void packet_buffer_2_output(void)
{
   // log packet 
  log_event( "packet 2" );
  if(display_mode == 1)
  {
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 2  ", 10);
    display_debug_p();
  }    
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_3_output
//
// This routine is used for processing the volume control level display.
//  also gets the call failed message here
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Volume control level is displayed
//
// External functions called
//  extract_message_to
//  ssd1963_cg_write_line
//  display_volume_bar
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint32_t cs_pkt1_volume               0                 7 
//  uint8_t  cs_pkt1_volume_msg[16]       1                 8
//  uint32_t cs_pkt1_volume_length       n/a
//
// Constants
//
// History
//    08/19/2019 added check for DMR call failed to turn off transmit indicator
//
// Status
//  written
//
uint32_t cs_pkt1_volume;
uint8_t  cs_pkt1_volume_msg[17];
uint32_t cs_pkt1_volume_length;

uint8_t  cs_pkt1_3_msg1[17];
uint32_t cs_pkt1_3_msg1_length;
uint8_t  cs_pkt1_3_msg2[17];
uint32_t cs_pkt1_3_msg2_length;


void packet_buffer_3_output(void)
{
  uint8_t vol;

  // Process volume. Volume is in location 7
  vol = packet_process_buffer[7];
  
  if( packet_process_buffer[6] == 1)
  {
    set_pkt1_bits(0, cs_pkt1_volume, vol);
    cs_pkt1_volume = vol;
    cs_channel_volume = vol;
  }
  else
  {
    // for other (not volume) packet 3 types
    // For first message, extract message and determine the number of characters.
    // The starting location for this format is position 8
    cs_pkt1_3_msg1_length = extract_message_to( cs_pkt1_3_msg1, 8 );    
    // indicate the parameter is active
    set_pkt1_bits(2,1,2);
    
    // For second message, extract message and determine the number of characters.
    // The starting location for this format is position 40.
    cs_pkt1_3_msg2_length = extract_message_to( cs_pkt1_3_msg2, 40 );    
    // indicate the parameter is active
    set_pkt1_bits(3,1,2);
    // check for call failed message
    if(strncmp("Failed!", (char const * ) cs_pkt1_3_msg2, 7) == 0)
    {
      radio_transmiting = 0;
      tx_led_off();
      draw_TXonoff( 0 );      
    }
 
  }
  // Output data to the display
  // Display volume or other
  if (check_pkt1_bits(0)) display_pkt1_volume();
  if( active_screen_num != 2 )
  {
    if (check_pkt1_bits(2)) display_pkt1_3_msg1();  
    if (check_pkt1_bits(3)) display_pkt1_3_msg2();  
  }  
  // packet_buffer_3_display();
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_4_output
//
// This is used for display two lines centered. The first line starts at
// position 8 and the second line starts at position 40
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  extract_message_to
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_4_msg1[16]           2                 8
//  uint32_t cs_pkt1_4_msg1_length       n/a
//  uint8_t  cs_pkt1_4_msg2[16]           3                40
//  uint32_t cs_pkt1_4_msg2_length       n/a
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_4_msg1[17];
uint32_t cs_pkt1_4_msg1_length;
uint8_t  cs_pkt1_4_msg2[17];
uint32_t cs_pkt1_4_msg2_length;

void packet_buffer_4_output(void)
{
  // For first message, extract message and determine the number of characters.
  // The starting location for this format is position 8
  cs_pkt1_4_msg1_length = extract_message_to( cs_pkt1_4_msg1, 8 );
  // indicate the parameter is active
  set_pkt1_bits(2,1,2);
  
  // For second message, extract message and determine the number of characters.
  // The starting location for this format is position 40.
  cs_pkt1_4_msg2_length = extract_message_to( cs_pkt1_4_msg2, 40 );  
  // indicate the parameter is active
  set_pkt1_bits(3,1,2);  

  // Output data to the display.
  if (check_pkt1_bits(2)) display_pkt1_4_msg1();  
  if (check_pkt1_bits(3)) display_pkt1_4_msg2();   
  //packet_buffer_4_display();
} 
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_5_output
//
// This is used for displaying the idle screen and the ICONS. This is one of the
// key programs as it shows much of the available parameters on a channel.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  A single line is displayed along with the ICONS
//
// External functions called
//  void   ssd1963_cg_clear_display(void)
//  uint32_t extract_message(uint32_t)
//  void ssd1963_cg_write_line(uint8_t,uint8_t,uint16_t, uint16_t, uint16_t) 
//  void output_icons_to_display(void)
//
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_channel_alias[16]            0            10-25
//  uint32_t cs_channel_alias_length         n/a
//  uint32_t cs_channel_rx_frequency         1            37-40       
//  uint32_t cs_channel_tx_frequency         2            41-44
//  uint8_t  cs_channel_zone_alias[16]       3            45-60
//  uint32_t cs_channel_zone_alias_length    n/a
//  uint32_t cs_channel_number_in_zone       4            77
//  uint32_t cs_channel_zone number          5            78
//  uint32_t cs_channel_number               6            79-80
//
//  uint32_t cs_channel_rssi                 7            6       0-2
//  uint32_t cs_channel_battery_level        8            6       3-5
//  uint32_t cs_channel_scan                 9            6       6-7
//
//  uint32_t cs_channel_power_level         10            7       0-1
//  uint32_t cs_channel_privacy             11            7       2-3
//  uint32_t cs_channel_message             12            7       4-5
//  uint32_t cs_channel_gps                 13            7       6-7
//                                     
//  uint32_t cs_channel_disable_tone        14            8       0
//  uint32_t cs_channel_talk_around         15            8       1
//  uint32_t cs_channel_monitor             16            8       2
//  uint32_t cs_channel_type                17            8       3-4
//  uint32_t cs_channel_mode                18            8       5
//  uint32_t cs_channel_emergency           19            8       6-7
//
//  uint32_t cs_channel_bt_state            20            9       0-1
//  uint32_t cs_channel_link                21            9       2
//  uint32_t cs_channel_volume              22            9       3-7
//
//  new for 20.41
//                                                packet  byte    bit     
//  uint32_t cs_color                       23     17      14     0-3 
//  uint32_t cs_slot                        24     17      14     4
//  uint32_t cs_call_type                   25     17      14     5-6
//  uint32_t cs_rx_id                       26     17      17-20
//  uint32_t cs_rx_group_id                 27     17      21-24
//  uint8_t  cs_rx_id_alias[16]             28     17      25-40
//  uint8_t  cs_rx_group_id_alias[16]       29     17      41-56


//  uint32_t cs_channel_active
//  uint32_t cs_channel_new
//

// revising

// new firmware has:

//  color code
//  slot
//  call type (same as channel type ?)
//  RX id
//  RX group id
//  RX id alias
//  RX group alias
//  temp channel DCH
//  temp channel ACH

// lots of items not covered by the channel bits
// frames
// buttons
// touch points
// menu itmes
// menu lists
// popup messages, icons
// radio messages msg1, msg2 
// DMR messages
// color controls

// probable additions
// bluetooth items
// GPS items, Lat. Long. time. connection/satellite... 
// VFO mode, band select
// multi radio items, split screen, repeater settings


// Internal variables used
//  uint8_t match;
//  uint32_t i;
//  uint32_t j;
//  uint32_t k;
//  uint32_t ChNumb;
//
// Constants
//
// History
//      11/20/2018  changed channel alias start from 10 to 13. for MS20.43
//      06/12/2019  changed extract_message to extract_message_to writing
//                  message data directly to the msg_buffer
//
// Status
//  written, working
//
uint8_t  cs_channel_alias[17];
uint32_t cs_channel_alias_length;
uint8_t  cs_channel_zone_alias[17];
uint32_t cs_channel_zone_alias_length; 
uint32_t cs_channel_rx_frequency;
uint32_t cs_channel_tx_frequency;
uint32_t cs_channel_number_in_zone;
uint32_t cs_channel_zone_number;
uint32_t cs_channel_number; 
uint32_t cs_channel_rssi;
uint32_t cs_channel_battery_level;
uint32_t cs_channel_scan;
uint32_t cs_channel_power_level;
uint32_t cs_channel_privacy;
uint32_t cs_channel_message;
uint32_t cs_channel_gps;
uint32_t cs_channel_disable_tone;
uint32_t cs_channel_talk_around;
uint32_t cs_channel_type;
uint32_t cs_channel_mode;
uint32_t cs_channel_emergency;
uint32_t cs_channel_bt_state;
uint32_t cs_channel_link;
uint32_t cs_channel_volume;

uint32_t cs_channel_active = 0;
uint32_t cs_channel_new = 0;

void packet_buffer_5_output(void)
{
  uint8_t match;
  uint32_t i;
  uint32_t j;
  uint32_t k;

  uint32_t ChNumb;
  
  // clear waitfor radio, prevents the connected message
  wait_for_radio_onscreen = 0;
  
  // Process channel alias.
  // The starting location for this variable is position 13
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  j = strlen( (char *) packet_process_buffer + 13);
  if (j != cs_channel_alias_length)
  {
    match = 0;
    cs_channel_alias_length = j;
  }
  else  // length matched, check string
  { 
    match = 1;
    for (k = 0; k < j; k++)
      if (packet_process_buffer[13+k] != cs_channel_alias[k]) match = 0 ;
  }
  if (!match) // channel alias did not match current packet data
  {
    extract_message_to( cs_channel_alias, 13);
    set_channel_bits(0,1,2);
  }
  else // it did match
  {
    // clear the new indicator  (bit 0) so it's not redisplayed
    cs_channel_new &= ~1; 
  }
  // Process Zone alias. The starting location for this variable is position 45
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  
  j = strlen( (char *) packet_process_buffer + 45);
  if (j != cs_channel_zone_alias_length) 
  {
    match = 0;
    cs_channel_zone_alias_length = j;
  }
  else  // length matched, check string
  {
    match = 1;
    for (k = 0; k < j; k++)
    if (packet_process_buffer[45+k] != cs_channel_zone_alias[k]) match = 0;
  }
  if (!match)  // zone alias did not match current packet data
  {
    for (k = 0; k < j;k++)
    {
      // blank bad data
      if(packet_process_buffer[45+k] == 0xFF)
      {
        cs_channel_zone_alias[k]= ' ';
        k++;
        cs_channel_zone_alias[k]= ' ';
      }
      else if(packet_process_buffer[45+k] > 126)
      {
        cs_channel_zone_alias[k]= ' ';
      } 
      else cs_channel_zone_alias[k]=packet_process_buffer[45+k];
    }
    cs_channel_zone_alias[k] = '\0';
    set_channel_bits(3, 1, 2);
  }
  else // it did match
  {
    // clear the new indicator (bit 3) so it's not redisplayed
    cs_channel_new &= ~8; 
  }  
  // Process Rx Frequency. The starting position for this variable is 37.
  // Position 37 is the LSB of a 32 bit variable.
  
  RxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 37 );

  set_channel_bits(1, cs_channel_rx_frequency, RxFreq);
  cs_channel_rx_frequency = RxFreq;
  // Process Rx Frequency. The starting position for this variable is 41.
  // Position 41 is the LSB of a 32 bit variable.
  
  TxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 41 );

  set_channel_bits(2, cs_channel_tx_frequency, TxFreq );
  cs_channel_tx_frequency = TxFreq;
  // Process channel number in zone. This variable is in location 77
  set_channel_bits(4, cs_channel_number_in_zone, packet_process_buffer[77]);
  cs_channel_number_in_zone = packet_process_buffer[77];
  // Process Zone Number. This variable is in location 78
  set_channel_bits(5, cs_channel_zone_number, packet_process_buffer[78] );
  cs_channel_zone_number = packet_process_buffer[78];  
  // Process Channel Number. Channel number is in locations 79 and 80. Position
  // 79 is LSB
  ChNumb =  (uint32_t) packet_process_buffer[79] + 
           ((uint32_t) packet_process_buffer[80]<< 8);
             
  set_channel_bits(6, cs_channel_number, ChNumb);
  cs_channel_number = ChNumb; 
  // Process RSSI. RSSI is in position 6 bits 0 - 2
  i = packet_process_buffer[6] & 0x07;
  set_channel_bits(7, cs_channel_rssi, i);    
  cs_channel_rssi = i;
  // Process Battery Level. Battery Level is in position 6 bits 3 - 5
  i = (packet_process_buffer[6] >> 3) & 0x07;
  set_channel_bits(8, cs_channel_battery_level, i);
  cs_channel_battery_level = i;
  // Process Scan. Scan is in position 6 bits 6-7
  i = (packet_process_buffer[6] >> 6) & 0x03; 
  set_channel_bits(9, cs_channel_scan, i);
  cs_channel_scan = i;
  // Process Power Level. Power Level is in Position 7 bits 0 - 1
  i = packet_process_buffer[7] & 0x03;
  set_channel_bits(10, cs_channel_power_level, i);
  cs_channel_power_level = i;
  // Process Privacy. Privacy is in Position 7 bits 2 - 3
  i = (packet_process_buffer[7] >> 2) & 0x03;
  set_channel_bits(11, cs_channel_privacy, i);
  cs_channel_privacy = i;   
  // Process Message. Message in position 7 bits 4-5
  i = (packet_process_buffer[7] >> 4) & 0x03;
  set_channel_bits(12, cs_channel_message, i);
  cs_channel_message = i;  
  // Process GPS. GPS in position 7 bits 6-7
  i = (packet_process_buffer[7] >> 6) & 0x03;
  set_channel_bits(13, cs_channel_gps, i);
  cs_channel_gps = i; 
  // Process Disable Tone. Disable Tone is Position 8 bit 0
  i = packet_process_buffer[8] & 0x01;
  set_channel_bits(14, cs_channel_disable_tone, i);
  cs_channel_disable_tone = i;    
  // Process Talk Around. Talk Around is Position 8 bit 1
  i = (packet_process_buffer[8] >> 1) & 0x01;
  set_channel_bits(15, cs_channel_talk_around, i);
  cs_channel_talk_around = i;  
  // Process Monitor. Monitor is Position 8 bit 2
  i = (packet_process_buffer[8] >> 2) & 0x01;
  set_channel_bits(16, cs_channel_monitor, i);
  cs_channel_monitor = i;
  // Process Channel Type. Channel Type is Position 8 bits 3-4
  i = (packet_process_buffer[8] >> 3) & 0x03;
  set_channel_bits(17, cs_channel_type, i);
  cs_channel_type = i; 
  // Process Channel Mode. Channel Mode is Position 8 bit 5
  i = (packet_process_buffer[8] >> 5) & 0x01;
  set_channel_bits(18, cs_channel_mode, i);
  cs_channel_mode = i;
  // Process Emergency Mode. Emergency Mode is Position 8 bits 6 -7
  i = (packet_process_buffer[8] >> 5) & 0x03;
  set_channel_bits(19, cs_channel_emergency, i);
  cs_channel_emergency = i;    
  // Process BT State. BT state is Position 9 bits 0 - 1
  i = packet_process_buffer[9] & 0x03;
  set_channel_bits(20, cs_channel_bt_state, i); 
  cs_channel_bt_state = i; 
  // Process Link State. Link State is Position 9 bit 2
  i = (packet_process_buffer[9] >> 2) & 0x01;
  set_channel_bits(21, cs_channel_link, i);
  cs_channel_link = i;   
  // Process Volume Level. Volume Level is Position 9 bits 3 - 7
  i = (packet_process_buffer[9] >> 3) & 0x1F;
  set_channel_bits(22, cs_channel_volume, i);
  cs_channel_volume = i;
  
  // handle a favorite zone update 
  if( zone_fav_changed == 1)
  {
    strncpy( (char *) favorite_zone[zf_num].name, (char *) cs_channel_zone_alias, 16);
    favorite_zone[zf_num].channel_num = 1;
    // write to eeprom
    save_favorite( zf_num, favorite_zone, FAV_ZONE_HI, FAV_ZONE_LO );
    zone_fav_changed = 0;
  }
    // handle a favorite channel update 
  if( channel_fav_changed == 1)
  {
    strncpy( (char *) favorite_channel[cf_num].name, (char *) cs_channel_alias, 16);
    // write to eeprom
    save_favorite( cf_num, favorite_channel, FAV_CHAN_HI, FAV_CHAN_LO );
    channel_fav_changed = 0;
  }
  if( get_menu_stack() == SPECTV ) tempACH_packet_ack = 1;
  
  // reset timer 
  //radio_info_timer = 10000;
  
  // debugging
  // display_packet_dump( packet_process_buffer );
  
  // output data to display, if not in diagnostic screen 
  if(active_screen_num != 2)  packet_buffer_5_display();

}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_6_output
//
// This is used for the start of the menu section of the program. The starting
// location for the message is 8. The index to what type it is at location 7.
//  The index is as follows:  0     Contacts
//                            1     Scan
//                            2     Zone
//                            3     Messages
//                            4     Call Log
//                            5     Utilities
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  A single line is displayed along with the ICONS
//
// External functions called
//  extract_message_to
//
// External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_6_msg1[16]           4                 8
//  uint32_t cs_pkt1_6_msg1_length       n/a
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_6_msg1[17];
uint32_t cs_pkt1_6_msg1_length;

uint8_t cs_menu_page = 0;
uint8_t cs_prev_menu_page = 0;
// flag for DMR or analog menu, 0 is DMR, 1 is analog
uint8_t dmr_ana = 0;   

void packet_buffer_6_output(void)
{
  // get menu page (index)
  cs_menu_page = packet_process_buffer[7];
  if(cs_menu_page == 0x70)
  {
    // it's an idle protocol 
    cs_pkt1_6_msg1_length = 0;
  }
  else
  {
    // Extract message
    cs_pkt1_6_msg1_length = extract_message_to( cs_pkt1_6_msg1, 8 );
    // indicate the parameter is active
    set_pkt1_bits(4,1,2);
  }
  // get DMR or analog mode, change menu 
  if( packet_process_buffer[6] == 0x3F )
  {
    dmr_ana = 0;
    num_menu_levels = 6;      
  }
  if( packet_process_buffer[6] == 0x2F )
  {
    dmr_ana = 1;
    num_menu_levels = 5;  
  }
  // Output data to the display.
  if (check_pkt1_bits(4)) display_pkt1_6_msg1(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_7_output
//
// This is used for displaying Data Entry Screen 1. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  extract_message_to
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_7_msg1[16]           5                 10
//  uint32_t cs_pkt1_7_msg1_length       n/a
//  uint8_t  cs_pkt1_7_msg2[16]           6                 42
//  uint32_t cs_pkt1_7_msg2_length       n/a
//
// Status
//  written, tested, working
//

uint8_t  cs_pkt1_7_msg1[17];
uint32_t cs_pkt1_7_msg1_length;
uint8_t  cs_pkt1_7_msg2[17];
uint32_t cs_pkt1_7_msg2_length;

// tab location, or message centered, left aligned ??
//
// 0x03  Contacts, msg1 menu, msg2 menu item
// 0x06  centered
// 0x02  Utilities, msg1 menu, msg2 sub menu item 

uint32_t cs_pkt_display_type;
// should get put into all packect output routines
//   cs_pkt_display_type = packet_process_buffer[5];

void packet_buffer_7_output(void)
{
  // Extract first message
  cs_pkt1_7_msg1_length = extract_message_to( cs_pkt1_7_msg1, 10 );
  cs_pkt_display_type = packet_process_buffer[5];
  // indicate the parameter is active
  set_pkt1_bits(5,1,2);

  // Extract second message
  cs_pkt1_7_msg2_length = extract_message_to( cs_pkt1_7_msg2, 42 );
   // indicate the parameter is active
  set_pkt1_bits(6,1,2);

  // Output data to the display.
  if (check_pkt1_bits(5)) display_pkt1_7_msg1();
  if (check_pkt1_bits(6)) display_pkt1_7_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_8_output
//
// This is used for displaying Data Entry Screen 2. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  extract_message_to
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_8_msg1[16]           7                 10
//  uint32_t cs_pkt1_8_msg1_length       n/a
//  uint8_t  cs_pkt1_8_msg2[16]           8                 42
//  uint32_t cs_pkt1_8_msg2_length       n/a
//
// Constants
//
// History
//
// Status
//  written
//

uint8_t  cs_pkt1_8_msg1[17];
uint32_t cs_pkt1_8_msg1_length;
uint8_t  cs_pkt1_8_msg2[17];
uint32_t cs_pkt1_8_msg2_length;

void packet_buffer_8_output(void)
{
  // Extract first message
  cs_pkt1_8_msg1_length = extract_message_to( cs_pkt1_8_msg1, 10 );
  // indicate the parameter is active
  set_pkt1_bits(7,1,2);

  // Extract second message
  cs_pkt1_8_msg2_length = extract_message_to( cs_pkt1_8_msg2, 42 );  
  // indicate the parameter is active
  set_pkt1_bits(8,1,2);

  // Output data to the display.
  if (check_pkt1_bits(7)) display_pkt1_8_msg1();
  if (check_pkt1_bits(8)) display_pkt1_8_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_9_output
//
// This is used for displaying Data Entry Screen 3. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  extract_message_to
//
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
// External variables used
//  uint8_t  cs_pkt1_9_msg1[16]           9                 10
//  uint32_t cs_pkt1_9_msg1_length       n/a
//  uint8_t  cs_pkt1_9_msg2[16]          10                 42
//  uint32_t cs_pkt1_9_msg2_length       n/a
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_9_msg1[17];
uint32_t cs_pkt1_9_msg1_length;
uint8_t  cs_pkt1_9_msg2[17];
uint32_t cs_pkt1_9_msg2_length;

void packet_buffer_9_output(void)
{
  // Extract first message
  cs_pkt1_9_msg1_length = extract_message_to( cs_pkt1_9_msg1, 10 );
  // indicate the parameter is active
  set_pkt1_bits(9,1,2);

  // Extract second message
  cs_pkt1_9_msg2_length = extract_message_to( cs_pkt1_9_msg2, 42 );
  // indicate the parameter is active
  set_pkt1_bits(10,1,2);

  // Output data to the display.
  if (check_pkt1_bits(9)) display_pkt1_9_msg1();
  if (check_pkt1_bits(10)) display_pkt1_9_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_10_output
//
// This is used for display two lines centered. The first line starts at
// position 6 and the second line starts at position 38
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  uint32_t extract_message_to(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_10_msg1[16]         11                  6
//  uint32_t cs_pkt1_10_msg1_length      n/a
//  uint8_t  cs_pkt1_10_msg2[16]         12                 38
//  uint32_t cs_pkt1_10_msg2_length      n/a
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_10_msg1[17];
uint32_t cs_pkt1_10_msg1_length;
uint8_t  cs_pkt1_10_msg2[17];
uint32_t cs_pkt1_10_msg2_length;

void packet_buffer_10_output(void)
{
  // Extract first message
  cs_pkt1_10_msg1_length = extract_message_to( cs_pkt1_10_msg1, 6 );
  // indicate the parameter is active
  set_pkt1_bits(11,1,2);

  // Extract second message
  cs_pkt1_10_msg2_length = extract_message_to( cs_pkt1_10_msg2, 38 );  
   // indicate the parameter is active
  set_pkt1_bits(12,1,2);
  
  // Output data to the display.
  if (check_pkt1_bits(11)) display_pkt1_10_msg1();
  if (check_pkt1_bits(12)) display_pkt1_10_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_11_output
//
// This is used for display two lines centered. The first line starts at
// position 10 and the second line starts at position 22
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  uint32_t extract_message_to(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_11_msg1[16]         13                 10
//  uint32_t cs_pkt1_11_msg1_length      n/a
//  uint8_t  cs_pkt1_11_msg2[16]         14                 (22) now at 42
//  uint32_t cs_pkt1_11_msg2_length      n/a
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_11_msg1[17];
uint32_t cs_pkt1_11_msg1_length;
uint8_t  cs_pkt1_11_msg2[17];
uint32_t cs_pkt1_11_msg2_length;

void packet_buffer_11_output(void)
{
  // Extract first message
  cs_pkt1_11_msg1_length = extract_message_to( cs_pkt1_11_msg1, 10 );  
  // indicate the parameter is active
  set_pkt1_bits(13,1,2);

  // Extract second message
  cs_pkt1_11_msg2_length = extract_message_to( cs_pkt1_11_msg2, 42 );  
  //  indicate the parameter is active
  set_pkt1_bits(14,1,2);

  // Output data to the display.
  if (check_pkt1_bits(13)) display_pkt1_11_msg1();
  if (check_pkt1_bits(14)) display_pkt1_11_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_12_output
//
// This is used for display two lines centered. The first line starts at
// position 15 and the second line starts at position 47
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  uint32_t extract_message_to(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_12_msg1[16]         15                 15
//  uint32_t cs_pkt1_12_msg1_length      n/a
//  uint8_t  cs_pkt1_12_msg2[16]         16                 47
//  uint32_t cs_pkt1_12_msg2_length      n/a
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_12_msg1[17];
uint32_t cs_pkt1_12_msg1_length;
uint8_t  cs_pkt1_12_msg2[64];  // messages are longer
uint32_t cs_pkt1_12_msg2_length;

void packet_buffer_12_output(void)
{
  // Extract first message
  cs_pkt1_12_msg1_length = extract_message_to( cs_pkt1_12_msg1, 15 );  
  // indicate the parameter is active
  set_pkt1_bits(15,1,2);

  // Extract second message
  cs_pkt1_12_msg2_length = extract_message_to( cs_pkt1_12_msg2, 47 );   
  // indicate the parameter is active
  set_pkt1_bits(16,1,2);

  // debug, dump packet to screen
  // display_packet_dump();
  // Output data to the display.
  if (check_pkt1_bits(15)) display_pkt1_12_msg1();
  if (check_pkt1_bits(16)) display_pkt1_12_msg2();  
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_13_output
//
// This is used for display two lines centered. The first line starts at
// position 7 and the second line starts at position 40
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  uint32_t extract_message_to(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint32_t cs_pkt1_13_icon             17                  6    0,1
//  uint8_t  cs_pkt1_13_msg1[16]         18                  7
//  uint32_t cs_pkt1_13_msg1_length      n/a
//  uint8_t  cs_pkt1_13_msg2[16]         19                 40
//  uint32_t cs_pkt1_13_msg2_length      n/a
//
// History
//
// Status
//  written
//
uint32_t cs_pkt1_13_icon;
uint8_t  cs_pkt1_13_msg1[17];
uint32_t cs_pkt1_13_msg1_length;
uint8_t  cs_pkt1_13_msg2[17];
uint32_t cs_pkt1_13_msg2_length;

void packet_buffer_13_output(void)
{
  // Process icons
  cs_pkt1_13_icon = packet_process_buffer[6] & 0x03;
  set_pkt1_bits(17,1,2);  

  // Extract first message
  cs_pkt1_13_msg1_length = extract_message_to( cs_pkt1_13_msg1, 7 );   
  // indicate the parameter is active
  set_pkt1_bits(18,1,2);

  // Extract second message
  cs_pkt1_13_msg2_length = extract_message_to( cs_pkt1_13_msg2, 40 );   
  // indicate the parameter is active
  set_pkt1_bits(19,1,2);

  // Output data to the display.
  if (check_pkt1_bits(17)) display_pkt1_13_icon();
  if (check_pkt1_bits(18)) display_pkt1_13_msg1();
  if (check_pkt1_bits(19)) display_pkt1_13_msg2();  
}
//
// ****************************************************************************
// August 17, 2018		Design Documents         packet_buffer_14_output
//
//
// Packet length is 187. message starts at 41, 3 bytes for checksum and final B4
// so maybe message length can be up to 143

uint8_t  cs_pkt1_14_msg1[64];  // max message length is not known !!! verify
uint32_t cs_pkt1_14_msg1_length;
uint8_t  cs_pkt1_14_msg2[17];
uint32_t cs_pkt1_14_msg2_length;

void packet_buffer_14_output(void)
{
  // Extract first message
  cs_pkt1_14_msg1_length = extract_message_to( cs_pkt1_14_msg1, 9 );   
  set_pkt1_bits(20,1,2);

  // Extract second message  
  cs_pkt1_14_msg2_length = extract_message_to( cs_pkt1_14_msg2, 41 );
  set_pkt1_bits(21,1,2);   

  // Output data to the display.  
  if (check_pkt1_bits(20)) display_pkt1_14_msg1();
  if (check_pkt1_bits(21)) display_pkt1_14_msg2();
}
//
// ****************************************************************************
// August 17, 2018		Design Documents         packet_buffer_15_output
//
// msg1 starts at 8
//

uint8_t  cs_pkt1_15_msg1[64];
uint32_t cs_pkt1_15_msg1_length;
uint8_t  cs_pkt1_15_msg2[17];
uint32_t cs_pkt1_15_msg2_length;

void packet_buffer_15_output(void)
{
  // Extract message  
  cs_pkt1_15_msg1_length = extract_message_to( cs_pkt1_15_msg1, 8 );
  // indicate the parameter is active
  set_pkt1_bits(22,1,2);  
  // no msg 2? 
  set_pkt1_bits(23,1,2);  
  // debug
  //display_packet_dump();
 
  set_pkt1_bits(23,1,2);   

  // Output data to the display.   
  if (check_pkt1_bits(22)) display_pkt1_15_msg1();
  if (check_pkt1_bits(23)) display_pkt1_15_msg2();  
}
//
// ****************************************************************************
// October 25, 2018		Design Documents         packet_buffer_16_output
//
//  MS20.40 version of packet 5
//

void packet_buffer_16_output(void)
{
  uint8_t match;
  uint32_t i;
  uint32_t j;
  uint32_t k;
  
  // Process channel alias. The starting location for this variable is position 42
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  
  j = strlen( (char * )packet_process_buffer +42 );
  if (j != cs_channel_alias_length)
  {
    match = 0;
    cs_channel_alias_length = j;
  }
  else
  {
    match = 1;
    for (k = 0; k < j; k++)
    if (packet_process_buffer[42+k] != cs_channel_alias[k]) match = 0;
  }
  if (!match) // channel alias did not match current packet data
  {
    extract_message_to( cs_channel_alias, 42);
    set_channel_bits(0,1,2);
  }
  else // it did match
  {
    // clear the new indicator  (bit 0) so it's not redisplayed
    cs_channel_new &= ~1; 
  }
  // Process RSSI. RSSI is in position 6 bits 0 - 2
  i = packet_process_buffer[6] & 0x07;
  set_channel_bits(7, cs_channel_rssi, i);    
  cs_channel_rssi = i;
  // Process Battery Level. Battery Level is in position 6 bits 3 - 5
  i = (packet_process_buffer[6] >> 3) & 0x07;
  set_channel_bits(8, cs_channel_battery_level, i);
  cs_channel_battery_level = i;
  // Process Scan. Scan is in position 6 bits 6-7
  i = (packet_process_buffer[6] >> 6) & 0x03; 
  set_channel_bits(9, cs_channel_scan, i);
  cs_channel_scan = i;
  // Process Power Level. Power Level is in Position 7 bits 0 - 1
  i = packet_process_buffer[7] & 0x03;
  set_channel_bits(10, cs_channel_power_level, i);
  cs_channel_power_level = i;
  // Process Privacy. Privacy is in Position 7 bits 2 - 3
  i = (packet_process_buffer[7] >> 2) & 0x03;
  set_channel_bits(11, cs_channel_privacy, i);
  cs_channel_privacy = i;   
  // Process Message. Message in position 7 bits 4-5
  i = (packet_process_buffer[7] >> 4) & 0x03;
  set_channel_bits(12, cs_channel_message, i);
  cs_channel_message = i;  
  // Process GPS. GPS in position 7 bits 6-7
  i = (packet_process_buffer[7] >> 6) & 0x03;
  set_channel_bits(13, cs_channel_gps, i);
  cs_channel_gps = i; 
  // Process Disable Tone. Disable Tone is Position 8 bit 0
  i = packet_process_buffer[8] & 0x01;
  set_channel_bits(14, cs_channel_disable_tone, i);
  cs_channel_disable_tone = i;    
  // Process Talk Around. Talk Around is Position 8 bit 1
  i = (packet_process_buffer[8] >> 1) & 0x01;
  set_channel_bits(15, cs_channel_talk_around, i);
  cs_channel_talk_around = i;  
  // Process Monitor. Monitor is Position 8 bit 2
  i = (packet_process_buffer[8] >> 2) & 0x01;
  set_channel_bits(16, cs_channel_monitor, i);
  cs_channel_monitor = i;
  // Process Channel Type. Channel Type is Position 8 bits 3-4
  i = (packet_process_buffer[8] >> 3) & 0x03;
  set_channel_bits(17, cs_channel_type, i);
  cs_channel_type = i; 
  // Process Channel Mode. Channel Mode is Position 8 bit 5
  i = (packet_process_buffer[8] >> 5) & 0x01;
  set_channel_bits(18, cs_channel_mode, i);
  cs_channel_mode = i;
  // Process Emergency Mode. Emergency Mode is Position 8 bits 6 -7
  i = (packet_process_buffer[8] >> 5) & 0x03;
  set_channel_bits(19, cs_channel_emergency, i);
  cs_channel_emergency = i;    
  // Process BT State. BT state is Position 9 bits 0 - 1
  i = packet_process_buffer[9] & 0x03;
  set_channel_bits(20, cs_channel_bt_state, i); 
  cs_channel_bt_state = i; 
  // Process Link State. Link State is Position 9 bit 2
  i = (packet_process_buffer[9] >> 2) & 0x01;
  set_channel_bits(21, cs_channel_link, i);
  cs_channel_link = i;   

  // set timer 
  //radio_info_timer = 10000;  
  
  // output data to display
  packet_buffer_16_display();

}
//
// ****************************************************************************
// October 25, 2018		Design Documents         packet_buffer_17_output
//
//  MS20.42 new RX_DMR_Activity packet
//
//  History: 01/18/2019 swapped RX and tx buffer locations, data was wrong
//

uint32_t cs_rx_id;
uint8_t  cs_rx_id_alias[17];
uint32_t cs_rx_id_alias_length = 0;

uint32_t cs_rx_group_id;
uint8_t  cs_rx_group_id_alias[17];
uint32_t cs_rx_group_id_alias_length = 0;

uint32_t cs_rx_color;      //    23     17      14     0-3 
uint32_t cs_rx_slot;       //    24     17      14     4
uint32_t cs_call_type;     //    25     17      14     5-6
uint32_t cs_rx_id;         //    26     17      17-20
uint32_t cs_rx_group_id;   //    27     17      21-24


void packet_buffer_17_output(void)
{
  uint8_t match;
  uint32_t j;
  uint32_t k;
  uint32_t temp;

  // Process contact id. The starting location for this variable is position 25
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  
  j = strlen((char *) packet_process_buffer + 25);
  if (j != cs_rx_id_alias_length)
  {
    match = 0;
    cs_rx_id_alias_length = j;
  }
  else
  {
    match = 1;
    for (k = 0; k < j; k++)
    if (packet_process_buffer[25+k] != cs_rx_id_alias[k]) match = 0;
  }
  if (!match) // contact ID did not match current packet data
  {
    extract_message_to( cs_rx_id_alias, 25);
    set_channel_bits(28,1,2); // new chan bits
  }
  else // it did match
  {
    // clear the new indicator  (bit 0) so it's not redisplayed
    cs_channel_new &= ~28;  // use correect bit !!!
  }
  // Process Group alias. The starting location for this variable is position 41
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  j = strlen( (char *) packet_process_buffer + 41);
  if (j != cs_rx_group_id_alias_length) 
  {
    match = 0;
    cs_rx_group_id_alias_length = j;
  }
  else
  {
    match = 1;
    for (k = 0; k < j; k++)
    if (packet_process_buffer[41+k] != cs_rx_group_id_alias[k]) match = 0;
  }
  if (!match)  // contact_group did not match current packet data
  {
    for (k = 0; k < j;k++)
    {
      // blank bad data
      if(packet_process_buffer[41+k] == 0xFF)
      {
        cs_rx_group_id_alias[k]= ' ';
        k++;
        cs_rx_group_id_alias[k]= ' ';
      }
      else if(packet_process_buffer[41+k] > 126)
      {
        cs_rx_group_id_alias[k]= ' ';
      } 
      else cs_rx_group_id_alias[k]= packet_process_buffer[41+k];
      
    }
    cs_rx_group_id_alias[k] = '\0';
    set_channel_bits(29, 1, 2); //  new bit
  }
  else // it did match
  {
    // clear the new indicator (bit 3) so it's not redisplayed
    cs_channel_new &= ~29;  // correct bt ?
  }
  strncpy( (char*) msg1_dump, (char const*) cs_rx_group_id_alias,
                              cs_rx_group_id_alias_length);
  
  // rx and tx swapped
  // Process Rx Frequency. Position 5 is the LSB of a 32 bit variable.  
  RxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 9 );
  set_channel_bits(1, cs_channel_rx_frequency, RxFreq);
  cs_channel_rx_frequency = RxFreq;

  // Process Tx Frequency. Position 9 is the LSB of a 32 bit variable.
  TxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 5 );
  set_channel_bits(2, cs_channel_tx_frequency, TxFreq );
  cs_channel_tx_frequency = TxFreq;
  // color  
  temp = packet_process_buffer[14] & 0x0F;
  set_channel_bits(23, cs_rx_color, temp);
  cs_rx_color = temp;
  // slot
  if(packet_process_buffer[14] & 0x10) temp = 2;
  else temp = 1;
  // this made 0 and 1 , not 1 and 2 
  // temp = 1 + (packet_process_buffer[14] & 0x10) >> 4;  
  set_channel_bits(24, cs_rx_slot, temp);
  cs_rx_slot = temp;
  // call type
  temp = (packet_process_buffer[14] & 0x60) >> 5;
  set_channel_bits(25, cs_call_type, temp);
  cs_call_type = temp;  
  // rx id
  temp = convert_LSB_bytes_to_int( packet_process_buffer + 17 );
  set_channel_bits(26, cs_rx_id, temp);
  cs_rx_id =  temp;
  // group id
  temp = convert_LSB_bytes_to_int( packet_process_buffer + 21 );
  set_channel_bits(27, cs_rx_group_id, temp);
  cs_rx_group_id =  temp;  
  // signal strength   
  temp = packet_process_buffer[15];
  set_channel_bits(7, cs_channel_rssi, temp);
  cs_channel_rssi = temp; 

  /*
  if(display_mode == 1)
  {
    msg1_dump[0] = 0;
    msg2_dump[0] = 0;
    strncpy( (char*) cg_line_buffer, "Packet 17  ", 10);  
    display_debug_p();     
  } 
  */
  
  // output data to display
  packet_buffer_17_display();

}
// ****************************************************************************
// October 26, 2018		Design Documents         packet_buffer_18_output
//
//  MS20.42 new RX_ANALOG_Activity packet
//
//  History: 01/18/2019 swapped RX and tx buffer locations, data was wrong
//
void packet_buffer_18_output(void)
{
  // Process Rx Frequency. The starting position for this variable is 9
  RxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 9 );
  
  set_channel_bits(1, cs_channel_rx_frequency, RxFreq);
  cs_channel_rx_frequency = RxFreq;
  // Process Tx Frequency. The starting position for this variable is 5
  TxFreq = convert_LSB_bytes_to_int( packet_process_buffer + 5 );
  
  set_channel_bits(2, cs_channel_tx_frequency, TxFreq );
  cs_channel_tx_frequency = TxFreq;
  
  // new, to be added...
  
  // this packet has
  //  channel type
  //  power level
  //  signal strength
  //  emergency_sys
  //  ctcss encode, cdcss decode, type 2
  //  ctcss encode, cdcss decode
  
  if( get_menu_stack() == SPECTV ) tempACH_packet_ack = 1;

  // output data to display
  packet_buffer_18_display();  
  
}
// ****************************************************************************
// October 26, 2018		Design Documents         packet_buffer_19_output
//
//  MS20.42 new programed keys function packet
//
void packet_buffer_19_output(void)
{
  // read data from packet and store in button assignment structure
  memcpy( &update_key_functions, &packet_process_buffer[5], FB_num_keys);
  
  // also store to current
  memcpy( &current_key_functions, &packet_process_buffer[5], FB_num_keys);

  // set buttons to the current data
  get_button_assign();  
  
  // check data read with what was stored
  if( FB_button_compare( &update_key_functions, &stored_key_functions ) == 0)
  {
    // write new data to eeprom
    FB_button_write();
  }
  
  // output data to display
  packet_buffer_19_display();  
  
}
// ****************************************************************************
// January 8, 2019		Design Documents         packet_buffer_20_output
//
//  MS20.47 Ack temp channel packet
//
//  nothing to do for now, maybe will get RSSI later
//  not being used yet
//
void packet_buffer_20_output(void)
{
  // nothing to do
  // set packet ack flag
  tempDCH_packet_ack = 1;  
}
// ****************************************************************************
// January 8, 2019		Design Documents         packet_buffer_21_output
//
//  MS20.47 Ack temp channel packet
//
//  nothing to do for now, maybe will get RSSI later
//  not being used yet
//
void packet_buffer_21_output(void)
{
  // set packet ack flag
  tempACH_packet_ack = 1;
  
}
// ****************************************************************************
// January 17, 2019		Design Documents         packet_buffer_22_output
//
//  MS20.51 RSSI data packet after sending RSSI request command
//
//
uint8_t RSSI_value = 0;
uint8_t RSSI_history[180];   // 3 minute storage
uint8_t RSSI_hist_index = 0;
uint8_t RSSI_min = 44;
uint8_t RSSI_max = 140;
float RSSI_graph_scale = 0.78;
uint8_t RSSI_range;

void packet_buffer_22_output(void)
{
  //cs_channel_rssi = packet_process_buffer[5];
  RSSI_value = packet_process_buffer[5];
  // adjust min/max for scaling put calc here instead of in the graph.
  // so that the scale value only gets calculated when the range changes
  // instead of every bar.
  
  // prevent possible div by 0
  RSSI_range = RSSI_max - RSSI_min;
  if( RSSI_range == 0 ) RSSI_range = 100; 
  
  if( RSSI_value < RSSI_min )
  {
    RSSI_min = RSSI_value;
    RSSI_graph_scale = 75.0  / RSSI_range;
  }
  if( RSSI_value > RSSI_max )
  {
    RSSI_max = RSSI_value;
    RSSI_graph_scale = 75.0  / RSSI_range;
  }
  RSSI_value -= RSSI_min;
  // store rssi in array
  RSSI_history[ RSSI_hist_index++ ] = RSSI_value;
  if( RSSI_hist_index > 179 ) RSSI_hist_index = 0;
  
  if( active_screen_num == 1 && rssi_graph_on ) display_RSSI_graph();
  
  rssi_packet_ack = 1;
}

// =============================================================================
// end of packet output section

// =============================================================================
//
//  Extract a string from packet buffer and place it in any buffer
//    location (dest) passed by pointer to the routine.
//    starts at the pointer, stops at the end of string null (0).
//    max length 64 because the destination might be larger than
//    the cg_buffer(32) see packet12. 
// 
uint32_t extract_message_to( uint8_t* dest, uint32_t start )
{
  uint32_t i;
  uint32_t numchars;
  
  // prevent a null pointer error
  if( dest == 0 ) 
  {
    error_log( 3003 );
    return 0;
  }
  
  numchars = 0;
  for (i = 0; i < 64; i++)
  {
    dest[numchars++] = packet_process_buffer[start + i];
    if (packet_process_buffer[start + i] == 0) break; 
  }
  return numchars;
}
// =============================================================================
//
// This program sets the cs_channel_new bit when
//  the new data just received is different than the old data
//
void set_channel_bits(uint32_t bits, uint32_t old_data, uint32_t new_data)
{
  uint32_t i;

  i = 1 << bits;
  // if it's the same data, it's been displayed. clear the "new" bit
  if ( old_data == new_data ) cs_channel_new &= ~i;
  
  if ( old_data != new_data ) 
  {
    cs_channel_new = cs_channel_new | i;
  }
}
// =============================================================================
//
// This program sets the cs_pkt1_new bits when
//  the new data just received is different than the old data
//
void set_pkt1_bits(uint32_t bits, uint32_t old_data, uint32_t new_data)
{
  uint32_t i;
  
  i = 1 << bits;
  if ( old_data != new_data )
  {
    cs_pkt1_new = cs_pkt1_new | i;
  }
}
// =============================================================================
//
// This program checks if the parameters need to be displayed at this time.
// It does this by looking at the cs_channel_new bit to determine if there is
// an update needed for this parameter
//
uint32_t check_channel_bits(uint32_t bits)
{
uint32_t i;
  i = 1 << bits;
  if (cs_channel_new & i) return 1;
  return 0;
}
//
// This program checks if the parameters needs to be displayed at this time.
// It does this by looking at the cs_pkt1_new bit to determine if there is
// an update needed for this parameter
//
uint32_t check_pkt1_bits(uint32_t bits)
{
uint32_t i;
  i = 1 << bits;
  if (cs_pkt1_new & i) return 1;
  return 0;
}
// *****************************************************************************
//    Channel_bits
// bit 
//  0   channel_alias
//  1   rx frequency
//  2   tx frequency
//  3   zone alias
//  4   channel number in zone
//  5   Zone Number
//  6   Channel number
//  7   RSSI ICON
//  8   battery level ICON
//  9   Scan ICON
//  10  power level ICON
//  11  privacy ICON
//  12  message ICON
//  13  GPS ICON
//  14  disable tone ICON
//  15  talk around ICON
//  16  monitor ICON
//  17  Channel type ICON
//  18  Channel mode ICON
//  19  emergency ICON
//  20  bt state ICON
//  21  link ICON
//  22  volume ICON

//
// This program outputs the data to the display if there is a change
//
int in_menu = 0;

void packet_buffer_5_display(void)
{
  // refresh screen if it needs
  if((active_screen_num != 1) && display_mode == 0)
  {
    // set rssi to inactive so it gets redrawn
    rssi_icon_isactive = 0;
    cs_channel_new = 0x7FFFFF;
    display_screen( 1 , 1);   
  }
  if((active_screen_num != 5) && display_mode == 1)
  {
    rssi_icon_isactive = 0;
    display_screen( 5 , 1);       
  }
  if(in_menu == 1)
  {
    if(active_screen_num == 5)
    {
      clear_pckt_message( 3 );
      clear_pckt_message( 4 );
    }
    if(active_screen_num == 3)
    {
      cs_channel_new = 0x7FFFFF;
      display_screen( 1 , 1);
    }
    display_channel_alias();
    display_zone_alias();      
    in_menu = 0;
  }

  // for debugging
  // log_event( "packet 5" );
  // log_event( "radio info" );
  
  // Output channel alias to the display. 
  if (check_channel_bits(0))
  {
    // packet 5
    // if the list is popped up, clear and redraw the screen
    if(lc_poped_up)
    {
      lc_poped_up = 0;
      display_screen( 1, 1 );       
    }    
   if(active_screen_num == 1)
   {
     // clear data from prev channel
      clear_pckt_message( 1 );
      clear_pckt_message( 2 );     
     
     // clear DMR color and slot
     clear_rectangle( 630, 210, 150, 20 );
     display_channel_alias();
     //display_zone_alias();  // done below on bit 3
     // clear previous contact data
     clear_rectangle(  30 , 375, 520, 35 );     
   }
  }
  
  // Output channel rx frequency to the display
  if (check_channel_bits(1)) display_channel_rx_frequency(); 

  // Output channel tx frequency to the display
  if (check_channel_bits(2)) display_channel_tx_frequency(); 

  // Output zone alias to the display.   
  if (check_channel_bits(3))
  {
    // only on main screen
    if( active_screen_num == 1 ) display_zone_alias();
  }
  // Output Channel Number in zone to the display.   
  if (check_channel_bits(4))
  {
    if ( chan_nums_active == 0 ) chan_nums_active = 1; 
    display_channel_number_in_zone( cs_channel_number_in_zone );
  }

  // Output Zone Number
  if (check_channel_bits(5)) display_zone_number(cs_channel_zone_number);

  // Output Channel number
  if (check_channel_bits(6))
  {
    if ( chan_nums_active == 0 ) chan_nums_active = 1;    
    display_channel_number( cs_channel_number );
    // if we got a new channel number turn monitor icon off
    draw_monitor_icon(0, 40, 15);
  }

  // Output Channel RSSI ICON
  if (check_channel_bits(7)) display_channel_rssi(); 

  //  Output Channel battery level ICON
  if (check_channel_bits(8)) display_channel_battery();

  //  Output Channel Scan ICON
  if (check_channel_bits(9)) display_channel_scan();

  //  Output Channel power level ICON
  if (check_channel_bits(10)) display_channel_power();   

  //  Output Channel privacy ICON
  if (check_channel_bits(11)) display_channel_privacy();   

  //  Output Channel message ICON
  if (check_channel_bits(12)) display_channel_message();      

  //  Output Channel GPS ICON
  if (check_channel_bits(13)) display_channel_gps();   

  //  Output Channel disable tone ICON 
  if (check_channel_bits(14)) display_channel_disable_tone();      

  //  Output Channel talk around ICON
  if (check_channel_bits(15)) display_channel_talk_around();   

  //  Output Channel monitor ICON 
  if (check_channel_bits(16)) display_channel_monitor();       
  
  //  Output Channel type ICON 
  if (check_channel_bits(17))
  {
    display_channel_type(cs_channel_type, 630, 230, 2);
    // if an analog channel number turn monitor icon off
    if( cs_channel_type == 1 ) draw_monitor_icon(0, 40, 15);
  }

  //  Output Channel mode ICON
  if (check_channel_bits(18)) display_channel_mode();   
   
  //  Output Channel emergency ICON
  if (check_channel_bits(19)) display_channel_emergency();      

  //  Output Channel bt state ICON
  if (check_channel_bits(20)) display_channel_bt_state();   

  //  Output Channel link ICON
  if (check_channel_bits(21)) display_channel_link();    
   
  //  Output Channel volume ICON
  if (check_channel_bits(22))
  {
    if((display_mode == 0) || (display_mode == 2))
    {
      display_channel_volume(); 
    }
  }
}
// *****************************************************************************
//
// This is a newer version of packet 5, 
//   similar information for more recent firmware
//
void packet_buffer_16_display(void)
{
  // refresh screen if it needs
  if((active_screen_num != 1) && display_mode == 0)
  {
    // set rssi to inactive so it gets redrawn
    rssi_icon_isactive = 0;
    cs_channel_new = 0x7FFFFF;
    display_screen( 1 , 1);   
  }
  if((active_screen_num != 5) && display_mode == 1)
  {
    rssi_icon_isactive = 0;
    display_screen( 5 , 1);       
  }
  if(in_menu == 1)
  {
    if(active_screen_num == 5)
    {
      clear_pckt_message( 3 );
      clear_pckt_message( 4 );
    }
    if(active_screen_num == 3)
    {
      cs_channel_new = 0x7FFFFF;
      display_screen( 1 , 1);
    }
    display_channel_alias();
    // display_zone_alias();      
    in_menu = 0;
  }

  // for debugging
  // log_event( "packet 16" );
  log_event( "radio info" );
  
  // Output channel alias to the display. 
  if (check_channel_bits(0))
  {
  
    // packet 16
    // if the list is popped up, clear and redraw the screen
    if(lc_poped_up)
    {
      lc_poped_up = 0;
      display_screen( 1, 1 );       
    }    
      
     display_channel_alias();
     //display_zone_alias();
  }
  // Output Channel RSSI ICON
  if (check_channel_bits(7))
  {
    display_channel_rssi(); 
    if( cs_channel_rssi == 0 )
    {
      rx_led_off();
      radio_receiving = 0;
      if( TGM_call_active == 1 )
      {
        TGM_end_call();    
      }
    }    
  }

  //  Output Channel battery level ICON
  if (check_channel_bits(8)) display_channel_battery();

  //  Output Channel Scan ICON
  if (check_channel_bits(9)) display_channel_scan();

  //  Output Channel power level ICON
  if (check_channel_bits(10)) display_channel_power();   

  //  Output Channel privacy ICON
  if (check_channel_bits(11)) display_channel_privacy();   

  //  Output Channel message ICON
  if (check_channel_bits(12)) display_channel_message();      

  //  Output Channel GPS ICON
  if (check_channel_bits(13)) display_channel_gps();   

  //  Output Channel disable tone ICON 
  if (check_channel_bits(14)) display_channel_disable_tone();      

  //  Output Channel talk around ICON
  if (check_channel_bits(15)) display_channel_talk_around();   

  //  Output Channel monitor ICON 
  if (check_channel_bits(16)) display_channel_monitor();       
  
  //  Output Channel type ICON 
  if (check_channel_bits(17)) display_channel_type(cs_channel_type, 630, 230, 2);

  //  Output Channel mode ICON
  if (check_channel_bits(18)) display_channel_mode();   
   
  //  Output Channel emergency ICON
  if (check_channel_bits(19)) display_channel_emergency();      

  //  Output Channel bt state ICON
  if (check_channel_bits(20)) display_channel_bt_state();   

  //  Output Channel link ICON
  if (check_channel_bits(21)) display_channel_link();
  
  if(display_mode == 1)
  {
    strncpy( (char*) msg1_dump, (char const*) cs_channel_alias,
                                          cs_channel_alias_length);
    strncpy( (char*) msg2_dump, "         ", 10) ;     
    strncpy( (char*) cg_line_buffer, "Packet 16  ", 10);  
    display_debug_p();   
  }
  if( get_menu_stack() == SPECTV ) tempACH_packet_ack = 1;
 
}
// *****************************************************************************
//
// this is for the DMR RX activity packet
//
void packet_buffer_17_display(void)
{
  uint8_t mpage = 0;
  if(active_screen_num == 1)
  {
    // Output channel rx frequency to the display
    if (check_channel_bits(1)) display_channel_rx_frequency(); 

    // Output channel tx frequency to the display
    if (check_channel_bits(2)) display_channel_tx_frequency();
    
    // if digital monitor mode display color and slot
    //if(dig_mon_on)
    if(1)
    {
      // display color code
      display_dmr_color( cs_rx_color, 630, 210, 4);
      
      // display slot
      display_dmr_slot( cs_rx_slot, 700, 210, 4);
      
      // display rssi
      if(rssi_graph_on == 0) draw_rssi_icon(cs_channel_rssi, 690, 15 );
    }
  }
  // if in VFO mode, or enh scanning mode update screen
  // with DMR slot color and contact ID
  if(active_screen_num == 6 )
  {
    mpage = get_menu_stack();
    if(mpage == VFOMODE || mpage == ENHSCAN )
    {
      if( chansc_channel_type == 0) // DMR mode
      {
        dmr_color = cs_rx_color;
        display_chan_menuitem( 5 );  // color
        display_integer_item( 5, cs_rx_color ); 
        
        tx_contact_num = cs_rx_group_id;
        display_chan_menuitem( 7 );  //TXID
        display_integer_item( 7, tx_contact_num );        
      }
    }
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 3 ); 
    strncpy( (char*) msg1_dump, (char*) cg_line_buffer, 10) ;
    strncpy( (char*) msg2_dump, "         ", 10) ;    
    strncpy( (char*) cg_line_buffer, "Packet 17  ", 10);  
    display_debug_p();   
  }   
  
  // log_event( "packet 17" );  
}
// *****************************************************************************
//
// this is for the Analog RX activity packet 
//
void packet_buffer_18_display(void)
{
  if(active_screen_num == 1)
  {
    // Output channel rx frequency to the display
    if (check_channel_bits(1)) display_channel_rx_frequency(); 
    // Output channel tx frequency to the display
    if (check_channel_bits(2)) display_channel_tx_frequency();
    // clear DMR call type icon
    draw_call_icon( 3 );  // call type 3 clears the icon
  }
  
  if(display_mode == 1)
  {
    clear_pckt_message( 3 ); 
    clear_pckt_message( 4 );
    ssd1963_cg_write_line(cs_channel_alias_length, 3, message2_X , message2_Y, 4);

    strncpy( (char*) msg1_dump, (char const*) cs_channel_alias,
                                          cs_channel_alias_length);
    strncpy( (char*) cg_line_buffer, "Packet 18  ", 10);  
    display_debug_p();   
  } 

  log_event( "Analog RX" );
}
// *****************************************************************************
//
// this is the key function packet 
//  button assignments of the cps are sent in this
//  
//
void packet_buffer_19_display(void)
{
  // TODO: later, update button names here
    
  if(display_mode == 1)
  {
    strncpy( (char*) msg1_dump, "         ", 10) ;    
    strncpy( (char*) cg_line_buffer, "Packet 19  ", 10);  
    display_debug_p();   
  } 

  log_event( "Button info" );
}

// =============================================================================
// PACKET 3 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output volume to the display
//

void  display_pkt1_volume(void)
{
  // for debugging
  //log_event( "packet 3" );
  log_event( "volume" );
  // Output bar indicating volume value. Position 7 of the packet_process_buffer[]
  // contains the value of the bar and can have a value of 0x01 through 0x20. A
  // value of 0x01 means there is no bar and a value of 0x20 means it is maximum.
  //
  cs_channel_volume = cs_pkt1_volume;

  if((display_mode == 0) || (display_mode == 2))
  {
    display_volume_bar(cs_pkt1_volume);
    // clear bit
  }
  if(display_mode == 1 && active_screen_num != 2)
    
  {
    // Transfer data to line buffer
    strncpy( (char*) cg_line_buffer, (char const*) "Radio Volume", 12);       
    // clear previous message
    clear_pckt_message( 3 );
    clear_pckt_message( 4 );
    ssd1963_cg_write_line(12, 3, message1_X , message1_Y, 4);
    
    uint16_t val;
    val = (uint16_t) ((cs_pkt1_volume - 1) * 100 ) / 31;
    
    draw_progress_bar( val, 0, 30, 160, 700, 50,
                      get_pal_color( 2 ), get_pal_color( 7 ), cur_background);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 3  ", 10);
    display_debug_p();
    // ssd1963_cg_write_line( 10, 3, 40 , 240, 4);       
  } 
  // clear the new bit
  cs_pkt1_new &= 0xFE;
}
// =============================================================================
// PACKET 3 ROUTINES
// =============================================================================
//
// Output message 1 from packet 3 to the display
//


void display_pkt1_3_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_3_msg1, cs_pkt1_3_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_3_msg1, cs_pkt1_3_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_3_msg1, cs_pkt1_3_msg1_length);
  // for debugging
  // log_event( "packet 3.1" );
  log_event( (char*) msg1_buf );
  
  // debug, dump packet to screen
  // display_packet_dump();
  // dig monitor icon
  if(active_screen_num == 1 || active_screen_num == 5)
  {
    // dig monitor icon
    if(strncmp("Dig Monitor Mode", (char const * ) cg_line_buffer, cs_pkt1_3_msg1_length) == 0)
    {
      if(strncmp("On", (char const * ) cs_pkt1_3_msg2, 2) == 0)
      {
        dig_mon_on = 1;        
        if( active_screen_num == 1 ) draw_monitor_icon(1, 40, 15);
        else draw_monitor_icon(1, 90, 15);
      }
      else
      {
        dig_mon_on = 0;
        if( active_screen_num == 1 ) draw_monitor_icon(0, 40, 15);
        else draw_monitor_icon(0, 90, 15);
      }
    }
    // scan icon  
    if(strncmp("Scan", (char const * ) cg_line_buffer, cs_pkt1_3_msg1_length) == 0)
    {
      if(strncmp("On", (char const * ) cs_pkt1_3_msg2, 2) == 0)
      {
        draw_scan_icon(1, 520, 15);
        scan_on = 1;
      }
      else
      {
        draw_scan_icon(0, 520, 15);
        scan_on = 0;
      }
    }
    // 
  }
  // Output the line to the display
  if(display_mode == 0)
  {
    // clear previous message
    clear_pckt_message( 1 );
    ssd1963_cg_write_line(cs_pkt1_3_msg1_length, 4, message1_X , message1_Y, 4);
    pckt_msg1_dirty = 1;    

  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );
    if(scan_on || dig_mon_on)
    {
      ssd1963_cg_write_line(cs_pkt1_3_msg1_length, 3, message1_X , message1_Y, 4);
      pckt_msg3_dirty = 1; 
      
    }
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {    
    clear_pckt_message( 3 );     
    ssd1963_cg_write_line(cs_pkt1_3_msg1_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;    
    }
    
  }
  // clear the new bits
  // cs_pkt1_new &= 0xF3;  
}
// =============================================================================
//
// Output message 2 from packet 3 to the display
//

void display_pkt1_3_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_3_msg2, cs_pkt1_3_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_3_msg2, cs_pkt1_3_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_3_msg2, cs_pkt1_3_msg2_length);  
  
  // for debugging
  // log_event( "packet 3.2" );
  // log_event( msg2_buf );
  
  // Output the line to the display
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    { 
      // clear previous message
      clear_pckt_message( 2 );

      ssd1963_cg_write_line(cs_pkt1_3_msg2_length, 4, message2_X , message2_Y, 4);
      pckt_msg2_dirty = 1;
      
    }
    if(active_screen_num == 3)
    { 
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      menu_current_level++;
      info_box_active = 1;
    }
  }
  if(display_mode == 1)
  {
    if(scan_on || dig_mon_on)
    {    
      clear_pckt_message( 4 );     
      ssd1963_cg_write_line(cs_pkt1_3_msg2_length, 3, message2_X , message2_Y, 4);
    }
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 3  ", 10);     
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      // if on main screen and mode 2
      clear_pckt_message( 4 );       
      ssd1963_cg_write_line(cs_pkt1_3_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      // if on menu screen and mode 2
      // ssd1963_cg_write_line(cs_pkt1_3_msg2_length, 3, message2_X , message2_Y, 4);
    }
  }
  // clear the new bits after second msg 
  cs_pkt1_new &= 0xF3;   
}
 
// =============================================================================
// PACKET 4 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output message 1 from packet 4 to the display
//
// what is assigned to P1 
// "selected" after being in the menu 

void display_pkt1_4_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  // for debugging
  // log_event( "packet 4.1" );
  log_event( (char*) msg1_buf );
  
  // debug, dump packet to screen
  // display_packet_dump();
  

  /*
  // on menu screens pop up a text box
  // on message 2 only

  if(active_screen_num == 3)
  {
    
    // clear
    clear_rectangle( 20, 200, 250, 100);    
    // draw frame
    draw_frame(20, 200, 250, 100, 5, 
                highlight);     
    // write data
     ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, 30 , 210, 4);
     menu_current_level++;
     // set chain fo return
     menu_chain[menu_current_level] = cur_sub_indx;      
    
  }
  */
  // Output the line to the display
  if(display_mode == 0)
  {
    // clear previous message
    clear_pckt_message( 1 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 4, message1_X , message1_Y, 4);
    pckt_msg1_dirty = 1;
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;    
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {    
    clear_pckt_message( 3 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;    
    }
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 4 to the display
//
void display_pkt1_4_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);  
  // for debugging
  // log_event( "packet 4.2" );
  //log_event( (char *) msg2_buf );
  
  // Output the line to the display
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    { 
      // clear previous message
      clear_pckt_message( 2 );     
      ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 4, message2_X , message2_Y, 4);
      pckt_msg2_dirty = 1;
    }
    if(active_screen_num == 3)
    { 
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      menu_current_level++;
      info_box_active = 1;
    }
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 3, message2_X , message2_Y, 4);
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 4  ", 10);     
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      // if on main screen and mode 2
      clear_pckt_message( 4 );       
      ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      // if on menu screen and mode 2
      // ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 3, message2_X , message2_Y, 4);
    }
  }
}
// =============================================================================
// PACKET 5 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output channel alias to the display
//
// 
void display_channel_alias(void)
{
  //int i;

  uint8_t font;
  
  // store channel alias to current buffer  
  strncpy( (char*) cg_line_buffer, (char const*) cs_channel_alias,
                                          cs_channel_alias_length);
  strncpy( (char*) msg1_dump, (char const*) cs_channel_alias, cs_channel_alias_length + 1);

  font = 2;
  // check pixel width, decrease font size if it doesn't fit
  while ( pixel_line_width( (char *) cs_channel_alias, font, 3 ) > 430 )
  {
    font++;
    if(font > 4)
    {
      font = 4;
      error_log( 3061 );
      cs_channel_alias[12] = '\0';  // trim
      break;
    }
  }
         
  // display buffer
  if((display_mode == 0) || (display_mode == 2))
  {  
    clear_rectangle( 170, 250, 410, 75 );
    ssd1963_cg_write_line(cs_channel_alias_length , font, 170, 265,3);
    
    // is blank sometimes after channel favorite screen
    if( strncmp( (char*) msg1_dump, "      ", 6) != 0)
    {
      log_event( (char*) msg1_dump ); 
    }
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 3 );
    clear_pckt_message( 4 );
    ssd1963_cg_write_line(cs_channel_alias_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, "Packet 5  ", 10);  
    display_debug_p();   
  }
  if(strncmp( (char * ) cs_channel_alias, "Temp ACH", 8) == 0 ||
    strncmp( (char * ) cs_channel_alias, "Temp DCH", 8) == 0)
  {
   // clear channel number in zone
   clear_rectangle( 40 , 255, 85, 60);
   // clear channel number
   clear_rectangle( 30 , 210, 95, 25);
   // if DMR
   if( strncmp( (char * ) cs_channel_alias, "Temp DCH", 8) == 0 ) // DMR
   {
     // display color code
     display_dmr_color( dmr_color, 630, 210, 4);
     // display slot
     display_dmr_slot( dmr_slot + 1, 700, 210, 4);   
   }
  }  
}
// *****************************************************************************
//
// Output channel rx frequency to the display
//
// History: 11/16/2018 fixed decimal point locator
//          12/14/2018 stopped display when last call is popped up

void display_channel_rx_frequency(void)
{
  uint32_t n;
  uint32_t i;

  if((display_mode == 0) || (display_mode == 2))
  {
    if(!lc_poped_up)
    {
      // erase previous freq
      clear_rectangle( 175, 210, 190, 25);
      n = sprintf((char*) cg_line_buffer, "Rx=%i", RxFreq);
      // make space for decimal point
      for (i = 20; i > 5 ;i--)                      
      {
        cg_line_buffer[i+1] = cg_line_buffer[i];
      }
      cg_line_buffer[6] = '.';  // add decimal point
      ssd1963_cg_write_line( n+1, 4, 175, 210, 5);
    }
  }
  // nothing for old screen
} 
// *****************************************************************************
//
// Output channel tx frequency to the display
//
// History: 11/16/2018 fixed decimal point locator
//          11/01/2019 moved to make room for day/ night control

void display_channel_tx_frequency(void)
{
  uint32_t n;
  uint32_t i;

  if((display_mode == 0) || (display_mode == 2))
  { 
    // erase previous freq
    clear_rectangle( 380, 210, 190, 25);
    n = sprintf((char*) cg_line_buffer, "Tx=%i", TxFreq);
    // make space for decimal point
    for (i = 20; i > 5 ;i--)                      
    {
      cg_line_buffer[i+1] = cg_line_buffer[i];
    }
    cg_line_buffer[6] = '.';  // add decimal point
    ssd1963_cg_write_line( n+1, 4, 380, 210, 5);
  }
  // nothing for old screen
}
// *****************************************************************************
//
// Output zone alias to the display. 
//  
void display_zone_alias(void)
{
  // int i;
  strncpy( (char*) cg_line_buffer, (char const*) cs_channel_zone_alias,
                                          cs_channel_zone_alias_length);
  strncpy( (char*) msg2_dump, (char const*) cs_channel_zone_alias, cs_channel_zone_alias_length);
  
  if((display_mode == 0) || (display_mode == 2))
  { 
    if(active_screen_num == 1)
    {
      // clear previous
      // clear a smaller area if transmitting, On Air is lit
      if( radio_transmiting ) clear_rectangle( 40, 130, 520, 48 );
      else clear_rectangle( 40, 130, 710, 48 );
      ssd1963_cg_write_line(cs_channel_zone_alias_length, 2, 40 , 130, 4);
      pckt_msg1_dirty = 1;
    }
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );
    ssd1963_cg_write_line(cs_channel_zone_alias_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;
  }
}
// =============================================================================
//  Display channel number in zone 1-999
//
// revision for dynamic channel numbers 1 to 999
// 08/06/2019

void display_channel_number_in_zone( uint32_t chnum )
{
  if((display_mode == 0) || (display_mode == 2))
  {
    if( chnum > 99 )
    {
      // three digits
      sprintf((char*) cg_line_buffer, "%i", chnum);
    }
    else if( chnum > 9 )
    {
      // leading space + two digits
      sprintf((char*) cg_line_buffer, " %i", chnum);
    }
    else 
    {
      // leading space + leading zero + one digit
      sprintf((char*) cg_line_buffer, " 0%i", chnum);
    }
    // clear previous 
    clear_rectangle( 30 , 255, 102, 65);
    ssd1963_cg_write_line( 3, 1, 30, 255, 5);
  }
}
// *****************************************************************************
//
// Output Zone Number
//
void display_zone_number( uint32_t zone)
{
  int n;
  if((display_mode == 0) || (display_mode == 2))
  {  
    n = sprintf((char*) cg_line_buffer, "Zone=%i", zone);
    ssd1963_cg_write_line( n, 4, 45, 110, 5);
  }
}
// *****************************************************************************
//
// Output Channel number, 0 - 4000
//
void display_channel_number( uint32_t chnum)
{
  int n;
  if((display_mode == 0) || (display_mode == 2))
  {  
    n = sprintf((char*) cg_line_buffer, "ch=%i", chnum);
    // clear previous 
    clear_rectangle( 30 , 210, 98, 25);
    ssd1963_cg_write_line( n, 4, 30, 210, 5);
  }
  
}
// *****************************************************************************
//
// Output Channel RSSI ICON
//
void display_channel_rssi(void)
{
  if((display_mode < 3) && (rssi_graph_on == 0))
      draw_rssi_icon(cs_channel_rssi, 690, 15 );
  
  if( cs_channel_rssi == 0 )
  {
    rx_led_off();
    radio_receiving = 0;
    if( TGM_call_active == 1 )
    {
      TGM_end_call();    
    }
  }
} 
// *****************************************************************************
//
//  Output Channel battery level ICON.
// 
void display_channel_battery(void)
{
//
// There are no batteries in the mobile radio so this routine does nothing.
//
}
// *****************************************************************************
//
//  Output Channel Scan ICON
// 
void display_channel_scan(void)
{
  draw_scan_icon(cs_channel_scan, 520, 15 );
}
// *****************************************************************************
//
//  Output Channel power level ICON
// 
void display_channel_power(void)
{
  if( active_screen_num == 1 || active_screen_num == 5 ) 
        display_tx_power(cs_channel_power_level, 0, 630, 295, 2 );
}  
// *****************************************************************************
//
//  Output Channel privacy ICON
// 
void display_channel_privacy(void)
{
  if( active_screen_num == 1 || active_screen_num == 5 )  
    draw_lock_icon(cs_channel_privacy, 206, 15 );
}  
// *****************************************************************************
//
//  Output Channel message ICON
// 
void display_channel_message(void)
{
 if( active_screen_num == 1 || active_screen_num == 5 )  
           draw_envelope_icon(cs_channel_message, 134, 18);
}  
// *****************************************************************************
//
//  Output Channel GPS ICON
// 
void display_channel_gps(void)
{
  if( active_screen_num == 1 || active_screen_num == 5 )
  {
    if (cs_channel_gps) draw_gps();
  }
}  
// *****************************************************************************
//
//  Output Channel disable tone ICON
// 
void display_channel_disable_tone(void)
{
  if( active_screen_num == 1 || active_screen_num == 5 )    
           draw_tone_disable_icon(cs_channel_disable_tone);
}  
// *****************************************************************************
//
//  Output Channel talk around ICON
// 
void display_channel_talk_around(void)
{
  // only on the correct screens
  if( active_screen_num == 1 || active_screen_num == 5 )
            draw_talk_around_icon(cs_channel_talk_around);
}  
// *****************************************************************************
//
//  Output Channel monitor ICON
// 

void display_channel_monitor(void)
{
  if( active_screen_num == 1) draw_monitor_icon(cs_channel_monitor, 40, 15);
  else draw_monitor_icon(cs_channel_monitor, 90, 15);
} 
// *****************************************************************************
//
//  Output Channel type ICON
// 
void display_channel_type( uint8_t chan_type, 
                          uint16_t pos_x, uint16_t pos_y, uint8_t font )
{  
  //uint8_t *message;
  //uint8_t i;

  if((display_mode == 0) || (display_mode == 2))
  {  
    if (chan_type == 0)
    {
      strncpy( (char*) cg_line_buffer, "DMR", 3);  
      // clear previous type
      clear_rectangle(pos_x , pos_y, 150, 50 );
      //ssd1963_cg_write_line( 3, 2, 630, 215, 4);
      ssd1963_cg_write_line( 3, font, pos_x, pos_y, 4);
    }
    else
    {
      strncpy( (char*) cg_line_buffer, "ANALOG", 6);  
      // clear previous type
      clear_rectangle(pos_x , pos_y, 150, 50 );
      ssd1963_cg_write_line( 6, font + 1, pos_x, pos_y + 5, 2);
      // clear DMR color code and slot
      clear_rectangle( pos_x, pos_y-20, 140, 20 );
    }
  }
} 
// *****************************************************************************
//
//  Output Channel mode ICON
// 
void display_channel_mode(void)
{
}  
// *****************************************************************************
//
//  Output Channel emergency tone ICON
// 
void display_channel_emergency(void)
{
//  uint32_t cs_channel_emergency
}    
// *****************************************************************************
//
//  Output Channel bt state ICON
//    Bluetooth?
// 
void display_channel_bt_state(void)
{
}  
// *****************************************************************************
//
//  Output Channel link ICON
// 
void display_channel_link(void)
{
}  
// *****************************************************************************
//
//  Output Channel volume ICON
// 
void display_channel_volume(void)
{
  if((display_mode == 0) || (display_mode == 2))
  {    
    if (cs_channel_volume == 0) 
    {
      display_volume_bar(0x20);
    }
    else
    {
      display_volume_bar(cs_channel_volume); 
    }
  }
}
// =============================================================================
// PACKET 6 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 6 to the display
//   a top level menu item
//  

void display_pkt1_6_msg1(void)
{
  // set mode DMR or analog
  
  
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
  //
  // Output the line to the display
  //
  // debug, dump packet to screen
  // display_packet_dump();  
  // for debugging
  //log_event( "packet 6.1" );
  log_event( (char*) msg1_dump );
  
  int nav_key = get_nav_key();
  in_menu = 1;
  
  if(display_mode == 0)
  {
    display_menu( cs_menu_page, MAIN, (char * )cs_pkt1_6_msg1,
                    cs_pkt1_6_msg1_length, "" , 0);

    if(nav_key == CANCEL) clear_menupage( 0 );    

    // restore text out buffer (display menu changed it)
    strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
    // clear previous message
    // clear_pckt_message( 1 );    
    // ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 4, message1_X , message1_Y, 4);   

  }
  if(display_mode == 1)
  {
    // ?? no packet 6 message 2?
    // so clear both
    clear_pckt_message( 3 );    
    clear_pckt_message( 4 );
    if( cs_menu_page != 0x70)
    {    
      ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 3,
                          message1_X + (cs_menu_page * 118) , message1_Y, 4);
    }
    pckt_msg3_dirty = 1;
    cs_channel_new = 0x7FFFFF;
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 6  ", 10);  
    display_debug_p();
  }
  if(display_mode == 2)   // normal display
  { 
    clear_rectangle( 10, 100, 780, 140);
    display_menu( cs_menu_page, MAIN, (char * )cs_pkt1_6_msg1,
                    cs_pkt1_6_msg1_length, "" , 0);      
    // deactivate list
    if( list_active )
    {
      // restore buttons
      draw_navbuttons( 1 );         
      list_active = 0;
      clear_menupage( 0 ); 
    }
    info_box_active = 0;


    // restore text out buffer (display menu changed it)
    //strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
    // clear previous message
    //clear_pckt_message( 1 );    
    //ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 4, message1_X , message1_Y, 4);  
  }  
}

// =============================================================================
// PACKET 7 ROUTINES
// =============================================================================
//
//  packet 7 occurs on entry to a menu submenu.
//
//  It also occurs after using the microphone keypad to
//  enter a call number, then pressing Ok
//
// *****************************************************************************
//
//
// Output message 1 from packet 7 to the display
//    a sub menu item 
//
void display_pkt1_7_msg1(void)
{
  // int nav_key;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg1, cs_pkt1_7_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_7_msg1, cs_pkt1_7_msg1_length);
  // debug, dump packet to screen
  //display_packet_dump();   
  // for debugging
  // log_event( "packet 7.1" );  
  
  // in_menu = 1;

  // Output the line to the display
  if(display_mode == 0)
  {    
    // clear previous message
    clear_pckt_message( 1 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  { 
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 3, message1_X , message1_Y, 4);
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 7 to the display
//  a sub menu item
//

uint8_t cur_menu_txt[17];

uint8_t sub_menu_ID = 0;

void display_pkt1_7_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
  strncpy( (char*) cur_menu_txt, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);  
  
  // for debugging
  //log_event( "packet 7.2" );
  log_event( (char*) cur_menu_txt );  
  
  // in_menu = 1;
  
  if(display_mode == 0 )
  {
    if( in_menu )
    {
      display_menu( cs_menu_page, SUB, (char * )cs_pkt1_7_msg1, cs_pkt1_7_msg1_length,
                 (char * )cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
      
      // restore text out buffer (display menu changed it)
      strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
    }
    // leave for debugging
    clear_pckt_message( 2 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 4, message2_X , message2_Y, 4); 
  }
  
  if(display_mode == 1)
  { 
    clear_pckt_message( 4 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 7  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    // deactivate list
    if( list_active )
    {
      list_active = 0;
      clear_menupage( 0 ); 
    }    
    // clear popped up infobox
    clear_rectangle( info_box_loc[cs_menu_page].x_loc,
                     info_box_loc[cs_menu_page].y_loc,
                     info_box_loc[cs_menu_page].width,
                     info_box_loc[cs_menu_page].height );
    info_box_active = 0;
    // draw submenu frame
    draw_shaded_frame( 10, 100, 780, 140, 5, border, highlight);
    
    draw_navbuttons( 1 );    
    clear_pckt_message( 4 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 3, message2_X , message2_Y, 4);
  }  
}
// =============================================================================
// PACKET 8 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 8 to the display
//
// menu action selection

void display_pkt1_8_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_8_msg1, cs_pkt1_8_msg1_length);
  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_8_msg1, cs_pkt1_8_msg1_length);

  // for debugging
  // log_event( "packet 8.1" );
  // display_packet_dump();

  // Output the line to the display
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 4, message1_X , message1_Y, 4);
    }
    // on menu screens pop up an action selector box
    if(active_screen_num == 3)
    {
      // display text popup
      // draw frame
      draw_frame(20, 200, 285, 100, 5, 
                  highlight);     
      // write data
       ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, 30 , 210, 4);   
    }    
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 4, message1_X , message1_Y, 4);
    }
    if(active_screen_num == 3)
    {
      clear_pckt_message( 3 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, message1_X , message1_Y, 4);  
    }        
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 8 to the display
//    menu action selection
//
void display_pkt1_8_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_8_msg2, cs_pkt1_8_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_8_msg2, cs_pkt1_8_msg2_length);

  // for debugging
  //log_event( "packet 8.2" );
    
  // Output the line to the display
  if(display_mode == 0)
  {     
    if(active_screen_num == 3)
    {
      // draw frame
      draw_frame(20, 200, 285, 100, 5, 
                  highlight);     
      // write data
       ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, 30 , 250, 4);
       menu_current_level++;
    }  
    clear_pckt_message( 2 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 4, message2_X , message2_Y, 4);
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, message2_X , message2_Y, 4);

    strncpy( (char*) cg_line_buffer, (char const*) "Packet 8  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, message2_X , message2_Y, 4);
   }
}
// =============================================================================
// PACKET 9 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 9 to the display
//
// radio information data from the menu

void display_pkt1_9_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_9_msg1, cs_pkt1_9_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_9_msg1, cs_pkt1_9_msg1_length);
 
  // for debugging
  //log_event( "packet 9.1" );   
  //
  // Output the line to the display
  
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 4, message1_X , message1_Y, 4);  
    }
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {  
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 4, message1_X , message1_Y, 4);  
    }    
    if(active_screen_num == 3)
    {
      //clear_pckt_message( 3 );   
      //ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 3, message1_X , message1_Y, 4);  
    }    
  }
  
}
// *****************************************************************************
//
// Output message 2 from packet 9 to the display
//
// radio information data from the menu
//
void display_pkt1_9_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_9_msg2, cs_pkt1_9_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_9_msg2, cs_pkt1_9_msg2_length);
    
  // for debugging
  //log_event( "packet 9.2" );
  log_event( (char*) msg2_buf );   
  //
  // Output the line to the display
  //
 
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 4, message2_X , message2_Y, 4);  
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      // menu_current_level++;     
    }
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 9  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
 
      // ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 3, message2_X , message2_Y, 4);
    }
  }
}
// =============================================================================
// PACKET 10 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 10 to the display
//
//  displays startup message and "powering down"

void display_pkt1_10_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_10_msg1, cs_pkt1_10_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_10_msg1, cs_pkt1_10_msg2_length);

  // for debugging
  //log_event( "packet 10.1" );
  log_event( "power up / down" );
  // debug, dump packet to screen
  // display_packet_dump();
    
  // Output the line to the display
  //
  if(display_mode == 0 && active_screen_num == 1)
  { 
    display_POM_message( 1, cs_pkt1_10_msg1_length);
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_10_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {
      display_POM_message( 1, cs_pkt1_10_msg1_length);      
    }
    // check to see if it occurs during the menu screen
    if(active_screen_num == 3)
    {
      clear_pckt_message( 3 );   
      ssd1963_cg_write_line(cs_pkt1_10_msg1_length, 3, message1_X , message1_Y, 4);
    }
  }
}
// *****************************************************************************
//
// Output message 2 from packet 10 to the display
//
void display_pkt1_10_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_10_msg2, cs_pkt1_10_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_10_msg2, cs_pkt1_10_msg2_length);
  
  // for debugging
  // log_event( "packet 10.2" );   
  //
  
  // Output the line to the display
  //
  if(display_mode == 0 && active_screen_num == 1)
  {  
    display_POM_message( 2, cs_pkt1_10_msg2_length);  
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_10_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 10 ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      display_POM_message( 2, cs_pkt1_10_msg2_length);  
    }
    if(active_screen_num == 3)
    {
      clear_pckt_message( 4 );   
      ssd1963_cg_write_line(cs_pkt1_10_msg2_length, 3, message2_X , message2_Y, 4);      
    }
  }
  //if msg1 is "powering" and message 2 is "down"  put panel to sleep
  
  if(strncmp( (char const*) cs_pkt1_10_msg1, "Powering", 8) == 0 &&
     strncmp( (char const*) cs_pkt1_10_msg2, "Down", 4) == 0)
  {
    panel_sleep();
  }
  // if msg1 is "Program Mode' then set flag so "on Air" does not display
  // when the red led for programming comes on  
  if(strncmp( (char const*) cs_pkt1_10_msg1, "Program Mode", 12) == 0 )
  {
    cs_program_mode = 1;
  }
  else cs_program_mode = 0;  
}
// =============================================================================
// PACKET 11 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 11 to the display
//
// Data entry or submenu selection in the menu
// comes up on manual dial in the contacts menu
// comes up on menu scan, view/edit/list 


void display_pkt1_11_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);

  // for debugging
  //log_event( "packet 11.1" );   
  log_event( "submenu" );   
  
  // display_packet_dump();

  // Output the line to the display
  //
  if(display_mode == 0)
  {  
    clear_pckt_message( 1 ); 
    ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1);   
      ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 4, message1_X , message1_Y, 4);
    }
    if(active_screen_num == 3)
    {
      //ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 3, message1_X , message1_Y, 4);
    }
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 11 to the display
//
void display_pkt1_11_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
    
  // for debugging
  log_event( "packet 11.2" ); 
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {
    clear_pckt_message( 2 );  
    ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 4, message2_X , message2_Y, 4);  
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 3, message2_X , message2_Y, 4);

    strncpy( (char*) cg_line_buffer, (char const*) "Packet 11 ", 10);
    display_debug_p();
    
    // debug
    // display_packet_dump();
    
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {    
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      draw_navbuttons( 1 ); 
      //clear_pckt_message( 4 );   
      //ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 3, message2_X , message2_Y, 4);      
    }
  }  
}
// =============================================================================
// PACKET 12 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 12 to the display
//
//  Sends menu items that can be displayed in a list.
//  displays CTC for contact menu and name of current contact in msg2
//  displays Zone for zone menu and name of zone in msg2

uint8_t list_active = 0;

void display_pkt1_12_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_12_msg1, cs_pkt1_12_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_12_msg1, cs_pkt1_12_msg1_length);
 
  // for debugging
  //log_event( "packet 12.1" );     
  log_event( "list" );   
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {
    clear_pckt_message( 1 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    // info displayed with list in msg2.  
  }
}
// *****************************************************************************
//
// Output message 2 from packet 12 to the display
//
//  Messages are list items of the active tab
//   contacts and zones, messages, 
//

void display_pkt1_12_msg2(void)
{
  // location for first tab list
  uint16_t list_xloc = 30; 
  uint16_t list_ytop = 105;  
  uint8_t updn = 2;
  int nav_key;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_12_msg2, cs_pkt1_12_msg2_length);    
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_12_msg2, cs_pkt1_12_msg2_length);
  
  // should put list handler in a separate subroutine
  nav_key = get_nav_key();  

  // set updn by switches
  if( nav_key == UP) updn = 1;
  if( nav_key == DOWN) updn = 0;
  if( nav_key == SELECT) updn = 0;
  
  if(display_mode == 0)
  {
    list_xloc = 30; 
    list_ytop = 105;  
    
    if(cs_menu_page == 0) list_xloc = 30;  // contacts
    if(cs_menu_page == 2) list_xloc = screen_tab[2].x_loc;  // zones
    if(cs_menu_page == 3)
    {
      // for lists on messages tab
      list_xloc = 30;
      menu_current_level++;
    }
    draw_list( list_xloc, list_ytop, cs_pkt1_12_msg2_length, updn ); 
    //clear_pckt_message( 2 );   
    //ssd1963_cg_write_line(cs_pkt1_12_msg2_length, 4, message2_X , message2_Y, 4);
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 12 ", 10);       
    display_debug_p();    
  }
  if(display_mode == 2)
  {
     // clear popped up infobox
    if(info_box_active)
    {
      clear_rectangle( info_box_loc[cs_menu_page].x_loc,
                       info_box_loc[cs_menu_page].y_loc,
                       info_box_loc[cs_menu_page].width,
                       info_box_loc[cs_menu_page].height );
      info_box_active = 0;
      // restore buttons
      draw_navbuttons( 1 );      
    }
    list_xloc = 30; 
    list_ytop = 115;  
    
    if(cs_menu_page == 0) list_xloc = 30;  // contacts
    if(cs_menu_page == 2) list_xloc = screen_tab[2].x_loc;  // zones
    if(cs_menu_page == 3)
    {
      // for lists on messages tab
      list_xloc = 30;
      menu_current_level++;
    }
    // clear frame at start of list
    if(!list_active)
    {
      // clear list data
      clear_scroll_list();      
      clear_rectangle( 10, 100, 780, 140 );
      list_active = 1;
    }
    draw_list( list_xloc, list_ytop, cs_pkt1_12_msg2_length, updn ); 
    draw_navbuttons( 2 ); 
  }
  
}
// =============================================================================
// PACKET 13 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output ICON from packet 13 to the display
//
//   2 bits of byte 6
// 0 private
// 1 group
// 2 all  ???
//
void display_pkt1_13_icon(void)
{
  if(active_screen_num == 1 || active_screen_num == 5)
  {
    draw_call_icon( cs_pkt1_13_msg1[6] & 0x03 );
  }
  // debug
  // if((cs_pkt1_13_msg1[6] & 0x03) > 1) display_packet_dump();
}
//
// Output message 1 from packet 13 to the display
//
//  displays DMR messages
//
// array of recent contacts
uint8_t last_call_recs = 10;
uint8_t last_call_indx = 0;
Last_call_info Last_call_table[10];
uint8_t prev_pkt13_msg1[17];
uint8_t lc_new = 0;

void display_pkt1_13_msg1(void)
{
  // uint32_t i;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length);     
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length);
  
  lc_new = 0;
  if( strncmp( (char const * ) Last_call_table[last_call_indx].text,
                (char const * ) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length) != 0)
  {
    lc_new = 1;
  }
  // for debugging
  //log_event( "packet 13.1" );
  // this packet info is empty when it's an icon update
  if(cs_pkt1_13_msg1_length > 1)
  {
    log_event( (char*) msg1_dump ); 
  }

  if(lc_new && (cs_pkt1_13_msg1_length > 1))
  {
    cur_call_active = 1;
    last_call_indx++;
    if(last_call_indx > last_call_recs-1) last_call_indx = 0;                  
    
    Last_call_table[last_call_indx].channel = cs_channel_number;
    
    memcpy( Last_call_table[last_call_indx].text,
            cs_pkt1_13_msg1, cs_pkt1_13_msg1_length ); 

    Last_call_table[last_call_indx].has_data = 1;
    
    if(display_mode == 0)
    {
      // Output the line to the display, larger size for this contact info
      // also clear msg 1 because the larger size overlaps
      if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );
      // clear this message space
      clear_rectangle(  30 , 375, 520, 35 );
      ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 3, 30 , 375, 4);
      display_lastcall_icon();
    }
    if(display_mode == 1)
    {
      if(pckt_msg3_dirty == 1) clear_pckt_message( 3 );
      ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 3, message1_X , message1_Y, 4);
      pckt_msg3_dirty = 1;
    }
    if(display_mode == 2)
    {
      if(active_screen_num == 1)
      {
        display_lastcall_icon();
        if(pckt_msg3_dirty == 1) clear_pckt_message( 1 );
        ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 4, message1_X , message1_Y, 4);
        pckt_msg3_dirty = 1;
      }
    }    
  }
}
// *****************************************************************************
//
// Output message 2 from packet 13 to the display
//
// packet 13 msg 1 is call sign or name
// packet 13 msg 2 is group name     

uint8_t prev_pkt13_msg2[17];

void display_pkt1_13_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length); 
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length);

  // for debugging
  // log_event( "packet 13.2" );   
  // Output the line to the display
  if(cs_pkt1_13_msg2_length > 0)
  {
    if(display_mode == 0)
    {
      if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );  
      ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 4, message2_X , message2_Y, 4);
      pckt_msg2_dirty = 1;
    }
    if(display_mode == 1)
    {
      // check msg 2, and use last_call result from msg 1
      if(strncmp( (char const * ) prev_pkt13_msg2,
                  (char const * ) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length) ||
                  (lc_new == 1))
      { 
        clear_pckt_message( 4 ); 
        ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 3, message2_X , message2_Y, 4);    
        strncpy( (char*) cg_line_buffer, (char const*) "Packet 13 ", 10);
        display_debug_p();
      }
    }
    if(display_mode == 2)
    {
      if(active_screen_num == 1)
      {
        // if it has changed
        if(strncmp( (char const * ) prev_pkt13_msg2,
                    (char const * ) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length) ||
                    (lc_new == 1))
        {
          clear_pckt_message( 2 ); 
          ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 4, message2_X , message2_Y, 4);    
        }
      }
    }
    // save message for next compare
    strncpy( (char*) prev_pkt13_msg2, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length);    
  }  
  
}
// *****************************************************************************
//
// Output message 1 from packet 14 to the display
//
uint8_t prev_pkt14_msg1[17];

void display_pkt1_14_msg1(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);

   if(display_mode == 0 && active_screen_num != 2)
  {
    // better to pop up a text box in this mode
    if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );      
    ssd1963_cg_write_line(cs_pkt1_14_msg1_length, 4, message1_X , message1_Y, 4);    
    pckt_msg1_dirty = 1;
  }   
  
  if(display_mode == 1)
  {
    // if not the same as the last
    if(strncmp((char const * ) prev_pkt14_msg1,
               (char const * ) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length))
    {
      clear_pckt_message( 3 ); 
    }
    ssd1963_cg_write_line(cs_pkt1_14_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    // handled by text box in msg2
  }
  // save message for next compare
  strncpy( (char*) prev_pkt14_msg1, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);   

}
// *****************************************************************************
//
// Output message 2 from packet 14 to the display
//
uint8_t prev_pkt14_msg2[17];

void display_pkt1_14_msg2(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);  
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);
  
  if(display_mode == 0 && active_screen_num != 2)
  {
    // need to pop up a text box in this mode
    if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );      
    ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 4, message2_X , message2_Y, 4);    
    pckt_msg2_dirty = 1;
  }    

  if(display_mode == 1)
  {
    // if not the same as the last
    if(strncmp((char const * ) prev_pkt14_msg2,
               (char const * ) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length))
    {
      clear_pckt_message( 4 );
    }
    ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 3, message2_X , message2_Y, 4); 
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 14 ", 10);
    display_debug_p();
     
  }
  
  if(display_mode == 2)
  {
    // text box
    display_text_box( info_box_loc[cs_menu_page].x_loc,
                      info_box_loc[cs_menu_page].y_loc,
                      info_box_loc[cs_menu_page].width,
                      info_box_loc[cs_menu_page].height );
    info_box_active = 1;     
    //clear_pckt_message( 4 ); 
    // ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 3, message2_X , message2_Y, 4);    
  }    
  // debug
  //display_packet_dump();
  // save message for next compare
  strncpy( (char*) prev_pkt14_msg2, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);     
 
}
// *****************************************************************************
//
// Output message 1 from packet 15 to the display
//
//
uint8_t prev_pkt15_msg1[17];

void display_pkt1_15_msg1(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);

  log_event( (char*) msg1_buf );

  if(display_mode == 0)
  {
    if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );      
    ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 4, message1_X , message1_Y, 4);    
    pckt_msg1_dirty = 1;
  } 
  
  if(display_mode == 1)
  {
    clear_pckt_message( 3 ); 
    ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 3, message1_X , message1_Y, 4);    
  }  
  if(display_mode == 2)
  {
    // clear_pckt_message( 3 ); 
    // ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 3, message1_X , message1_Y, 4);    
  }  
  // save message for next compare
  strncpy( (char*) prev_pkt15_msg1, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);  
}
// *****************************************************************************
//
// Output message 2 from packet 15 to the display
//
uint8_t prev_pkt15_msg2[17];

void display_pkt1_15_msg2(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length); 
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);
  
   if(display_mode == 0)
  {
    if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );      
    ssd1963_cg_write_line(cs_pkt1_15_msg2_length, 4, message2_X , message2_Y, 4);    
    pckt_msg2_dirty = 1;
  } 
  if(display_mode == 1)
  {  
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 15 ", 10);
    display_debug_p();
  }
  if(display_mode == 2)
  {
    display_text_box( info_box_loc[cs_menu_page].x_loc,
                      info_box_loc[cs_menu_page].y_loc,
                      info_box_loc[cs_menu_page].width,
                      info_box_loc[cs_menu_page].height );
    info_box_active = 1;   
    
   // clear_pckt_message( 4 ); 
    //ssd1963_cg_write_line(cs_pkt1_15_msg2_length, 3, message2_X , message2_Y, 4);    
  }    
  // debug
  // display_packet_dump();
  // save message for next compare
  strncpy( (char*) prev_pkt15_msg2, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);     
}
//
// *****************************************************************************
//
//  calculate line center 
//  given the number of bytes of text loaded in the text buffer. 
//  returns half the width of the text in the buffer in PIXELS for the font.
//  If the total number of pixels is odd, it returns the integer portion.
//  Then, twice the half width may be 1 pixel short of the full width. 
//
// History: 07/31/2018
//
// Status: written, untested !!!

uint16_t calculate_line_center(uint8_t font, uint8_t spacing, uint8_t numbytes)
{
   // /*
  uint16_t horiz_pixels = 0;
  int i;
  uint8_t cur_char = 0;
  // get font max sizes, and spacing
  ssd1963_cg_read_char_size( font );    
  
  for( i= 0; i < numbytes; i++)
  {
    cur_char = cg_line_buffer[i] - 0x20;
    ssd1963_cg_read_descriptor( font, cur_char );
    cg_space_before = (cg_max_hori_size - cg_hori_size)/2;
    cg_space_after = cg_space_before;
    // if it's an odd number, 0.5 part of integer was truncated, need one more bit
    if((cg_max_hori_size - cg_hori_size) & 1) cg_space_after++;
    cg_space_after++;
    // special case for a space
    if( cur_char == ' ' - 0x20) 
    {
      uint16_t spacewidth;
      // recalc hori sizes 
      if(spacing == 0)    // for centered
      {  
        spacewidth = cg_max_hori_size;
      }
      else  // for propoptional 
      {  
        spacewidth = 5 * spacing;
      }
      horiz_pixels += spacewidth + 1;
    }
    else  // not a space
    {
      // with special case handled do all  
      if(spacing == 0)    // for centered
      {  
        horiz_pixels += cg_max_hori_size;
      }
      else // proportional
      {
        horiz_pixels += cg_hori_size + spacing;
      }
    }
  }
  horiz_pixels >>= 1;  // divide by 2
  return horiz_pixels;
  // */
}
// =============================================================================
//
//  Pixel line width
//    gives the number of pixels in a string, for a font with spacing
//    string must be null terminated
//
//
//  History: 11/17/2018
//           12/12/2018  removed 16 character cutoff
//
// Status: written, working

uint16_t pixel_line_width( char* text, uint8_t font, uint8_t spacing)
{
  uint16_t horiz_pixels = 0;
  int i = 0;
  char cur_char;
  
  // get font max sizes, and spacing
  ssd1963_cg_read_char_size( font );    
  
  cur_char = text[i];

  while( cur_char != 0)   
  {
    // adjust to table index
    cur_char -= 0x20; 
    ssd1963_cg_read_descriptor( font, cur_char );
    cg_space_before = (cg_max_hori_size - cg_hori_size)/2;
    cg_space_after = cg_space_before;
    // if it's an odd number, 0.5 part of integer was truncated, need one more bit
    if((cg_max_hori_size - cg_hori_size) & 1) cg_space_after++;
    cg_space_after++;
    // special case for a space
    if( cur_char == ' ' - 0x20) 
    {
      uint16_t spacewidth;
      // recalc hori sizes 
      if(spacing == 0)    // for centered
      {  
        spacewidth = cg_max_hori_size;
      }
      else  // for propoptional 
      {  
        spacewidth = 5 * spacing;
      }
      horiz_pixels += spacewidth + 1;
    }
    else  // not a space
    {
      // with special case handled do all  
      if(spacing == 0)    // for centered
      {  
        horiz_pixels += cg_max_hori_size;
      }
      else // proportional
      {
        horiz_pixels += cg_hori_size + spacing;
      }
    }
    // next char
    i++;
    // if( i > 15 ) break;
    cur_char = text[i];    
  }
  return horiz_pixels;
}


void  output_special_icons(void)
{
}
void message_type_icons(uint32_t message_type)
{
}
// =============================================================================
//
//  clear the message area
//   for message 1, and message 2 
//   location 1 and 2 for screen 1,
//   location 3 and 4 for screen 5 
//

void clear_pckt_message( uint8_t location )
{
  if(location == 1)
  {
    clear_rectangle(  message1_X , message1_Y, 260, 25 );
    pckt_msg1_dirty = 0;
  }
  if(location == 2)
  {
    clear_rectangle(  message2_X , message2_Y, 260, 25 );
    pckt_msg2_dirty = 0;
  }
  if(location == 3)
  {
    clear_rectangle(  message1_X , message1_Y, 740, 50 );
    pckt_msg3_dirty = 0;
  }  
  if(location == 4)
  {
    clear_rectangle(  message2_X , message2_Y, 740, 50);
    pckt_msg4_dirty = 0;
  }  
}
//
// =============================================================================
//
//  Packet info display for debug on screen 5
//

void display_debug_p( void )
{
  // location for packet debug messages
  uint16_t debug_p_x = 40;
  uint16_t debug_p_y = 240;
  uint16_t debug_p_step = 25;
  uint16_t debug_p_line = 0;

  if(1)   // if debug
  {
    // clear space
    clear_rectangle( 10, debug_p_y + debug_p_line * debug_p_step, 710, 25);
    
    ssd1963_cg_write_line( 10, 4, debug_p_x, debug_p_y + debug_p_line * debug_p_step, 2);
    
    strncpy( (char*) cg_line_buffer, (char const*) msg1_dump, 16);
    ssd1963_cg_write_line( 16, 4, 200 , debug_p_y + debug_p_line * debug_p_step, 2);
    
    strncpy( (char*) cg_line_buffer, (char const*) msg2_dump, 16);
    ssd1963_cg_write_line( 16, 4, 420 , debug_p_y + debug_p_line * debug_p_step, 2);    

    // clear marker
    clear_rectangle( 0, 240, 30, 240);

    if(uart1_good_data) ssd1963_cg_output_character(' ', 4, 160, debug_p_y + debug_p_line * debug_p_step, 0);
    else 
    {
      ssd1963_cg_output_character('x', 4, 160, debug_p_y + debug_p_line * debug_p_step, 0);
      //display_packet_dump();
    }
    
    // place new marker
    ssd1963_cg_output_character('>', 3, 10, debug_p_y + debug_p_line++ * debug_p_step - 7, 2);
   
    if(debug_p_line > 7) debug_p_line = 0;
  }
}
// =============================================================================
//
//  Panel shut down, sleep
//
//  Waits for 20 seconds after powering down the radio. Then turns off the
//  screen backlight.
//  Waits for any new packet to wake up
//
// History: 12/03/2018 written, tested
//          01/29/2019 added turn on display cycle to show radio off message  
//
// Status: working with ver 1.04 MS20.43


extern unsigned long volatile seconds;

void panel_sleep( void )
{
  int till_sleep;
  unsigned long count_sec;
  int panel_on = 1;
  
  count_sec = seconds;
  till_sleep = 5;  
  
  while( till_sleep > 0 )
  {
    // delay
    if(count_sec != seconds)
    {
      till_sleep--;
      count_sec = seconds;
    }  
  }
  till_sleep = 20;
  
  ssd1963_cg_clear_display();
  strncpy( (char*) cg_line_buffer, "Radio is off", 12); 
  // set active screen to 0 to prevent diplaying "waiting for radio..."
  active_screen_num = 0;
  ssd1963_cg_write_line( 12, 2, 250 , 150, 4);
  
  // wait here while no packets rec'd
  while( packet_buffer_output_flag == 0)
  {
    // if the panel is on
    if(panel_on)
    {
      // timeout 
      if(count_sec != seconds)
      {
        till_sleep--;
        count_sec = seconds;
      }
      
      if( till_sleep == 0)
      {
        // turn off backlight;
        lcd_backlight_off();
        panel_on = 0;
        // turn off other stuff ?
        till_sleep = 10;
      }
    }
    else // panel is off
    {
      // timeout 
      if(count_sec != seconds)
      {
        till_sleep--;
        count_sec = seconds;
      }
      if( till_sleep == 0)
      {
        // turn on backlight;
        lcd_backlight_on();
        till_sleep = 1;
        panel_on = 1;        
      }      
    }
  }
  // the while breaks when a packet comes in
  // re init screen in case it's gone
  ssd1963_cg_initialize();
  // restart with main screen  
  display_screen( 1, 1);
  lcd_backlight_on();
  panel_on = 1;    
  
}


