//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June, 27 2018             Design Documents                   process_CS800D.h
//
//			header for process packets
//                    UART communication with CS800D
//                      for Large Panel Project
//
// Programmer
//  Jerry Wanger, Dean Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  06/27/2018 DSG, moved header information here from other headers
//  09/05/2018 DSG, renamed for CS800D
//
// Status
//  written
//
//==============================================================================
#ifndef PROCESS_CS800D_H_
#define PROCESS_CS800D_H_

#include <stdint.h>
#include "CSI_SCREENS.h"

extern uint32_t radio_watchdog_timer;
extern uint8_t radio_transmiting;
extern uint8_t radio_receiving;

extern uint8_t  cs_channel_alias[17];
extern uint32_t cs_channel_alias_length;
extern uint8_t  cs_channel_zone_alias[17];
extern uint32_t cs_channel_zone_alias_length; 
extern uint32_t cs_channel_rx_frequency;
extern uint32_t cs_channel_tx_frequency;
extern uint32_t cs_channel_number_in_zone;
extern uint32_t cs_channel_zone_number;
extern uint32_t cs_channel_power_level;
extern uint32_t cs_channel_type;
extern uint32_t cs_channel_volume;
extern uint32_t cs_channel_new;
extern uint32_t cs_channel_monitor;
extern uint32_t cs_channel_rssi;
extern uint32_t cs_channel_scan;

extern uint8_t  cs_rx_group_id_alias[17];

// Process Packets
extern uint8_t  packet_process_buffer[];
extern uint8_t  unknown_packet_error_buffer[256];
extern uint32_t packet_buffer_output_type;
extern uint32_t radio_watchdog_timer;

extern uint32_t packet_counter;

extern uint32_t RxFreq;
extern uint32_t TxFreq;

// DMR data
extern uint32_t cs_rx_color;   
extern uint32_t cs_rx_slot;    
extern uint32_t cs_call_type;  
extern uint32_t cs_rx_id;      
extern uint32_t cs_rx_group_id;

// DMR data for temp channel
extern uint8_t channel_type;
extern uint8_t power_level;
extern uint8_t dmr_color;
extern uint8_t dmr_slot;
extern uint8_t rec_type;
extern uint32_t ctcss_encode;
extern uint32_t ctcss_decode;
extern uint32_t dcs_encode;
extern uint32_t dcs_decode;
extern uint32_t tx_contact_num;

// Analog data for temp channel
extern uint8_t ctcss_decode_type;
extern uint8_t ctcss_encode_type;
extern uint8_t rx_sqlmode;

// for menu
extern uint8_t dmr_ana; 
extern uint8_t cs_menu_page;
extern uint16_t debug_p_line;

extern uint8_t list_active;
extern uint8_t info_box_active;

// for lastcall popup
extern uint8_t last_call_recs;
extern uint8_t last_call_indx;
extern Last_call_info Last_call_table[10];

// for RSSI graph 
extern uint8_t RSSI_history[180];  
extern uint8_t RSSI_hist_index;
extern uint8_t RSSI_value;
extern uint8_t RSSI_min;
extern uint8_t RSSI_max;
extern float RSSI_graph_scale;
extern volatile uint8_t rssi_packet_ack;
extern volatile uint8_t tempACH_packet_ack;

// Function prototypes

void process_CS800D(void);
void RCDB_start( void );
void set_zone_channel( uint8_t zone, uint16_t channel );
void set_temp_DCH( uint32_t tx_freq, uint32_t rx_freq );
void set_temp_ACH( uint32_t tx_freq, uint32_t rx_freq );
void store_temp_CH( uint32_t tx_freq, uint32_t rx_freq );
void request_RSSI_value( void );
void panel_sleep( void );

uint32_t extract_message_to( uint8_t* dest, uint32_t start );

uint32_t check_pkt1_bits(uint32_t bits);
uint32_t check_channel_bits(uint32_t bits);

uint32_t packet_type_A(void);
uint32_t packet_type_B(void);
uint32_t packet_type_C(void);

uint32_t packet_type_0(void);
uint32_t packet_type_1(void);
uint32_t packet_type_2(void);

void process_all_packet(void);
void packet_buffer_0_output(void);
void packet_buffer_1_output(void);
void packet_buffer_2_output(void);
void packet_buffer_3_output(void);
void packet_buffer_4_output(void);
void packet_buffer_5_output(void);
void packet_buffer_6_output(void);
void packet_buffer_7_output(void);
void packet_buffer_8_output(void);
void packet_buffer_9_output(void);
void packet_buffer_10_output(void);
void packet_buffer_11_output(void);
void packet_buffer_12_output(void);
void packet_buffer_13_output(void);
void packet_buffer_14_output(void);
void packet_buffer_15_output(void);
void packet_buffer_16_output(void);
void packet_buffer_17_output(void);
void packet_buffer_18_output(void);
void packet_buffer_19_output(void);
void packet_buffer_20_output(void);
void packet_buffer_21_output(void);
void packet_buffer_22_output(void);  // 20, 21 not used yet

void packet_buffer_5_display(void);
void packet_buffer_16_display(void);
void packet_buffer_17_display(void);
void packet_buffer_18_display(void);
void packet_buffer_19_display(void);

uint16_t calculate_line_center(uint8_t font, uint8_t spacing, uint8_t numbytes);
uint16_t pixel_line_width( char* text, uint8_t font, uint8_t spacing);
void  output_special_icons(void);

// packet data display functions
void  display_pkt1_volume(void);
void display_volume_bar(uint8_t);

void clear_pckt_message( uint8_t messnum );

void display_channel_alias(void);
void display_channel_rx_frequency(void);
void display_channel_tx_frequency(void);
void display_zone_alias(void); 
void display_channel_number_in_zone(uint32_t);
void display_zone_number(uint32_t);
void display_channel_number(uint32_t);
void display_channel_rssi(void);
void display_channel_battery(void);
void display_channel_scan(void);
void display_channel_power(void);
void display_channel_privacy(void);
void display_channel_message(void); 
void display_channel_gps(void); 
void display_channel_disable_tone(void);
void display_channel_talk_around(void);
void display_channel_monitor(void);

void display_channel_type( uint8_t chan_type, 
                          uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_channel_mode(void);
void display_channel_emergency(void);
void display_channel_bt_state(void);
void display_channel_link(void);
void display_channel_volume(void);

void display_pkt1_3_msg1(void);
void display_pkt1_3_msg2(void);

void display_pkt1_4_msg1(void);
void display_pkt1_4_msg2(void);

void display_pkt1_6_msg1(void);

void display_pkt1_7_msg1(void);
void display_pkt1_7_msg2(void);

void display_pkt1_8_msg1(void);
void display_pkt1_8_msg2(void);

void display_pkt1_9_msg1(void);
void display_pkt1_9_msg2(void);

void display_pkt1_10_msg1(void);
void display_pkt1_10_msg2(void);

void display_pkt1_11_msg1(void);
void display_pkt1_11_msg2(void);

void display_pkt1_12_msg1(void);
void display_pkt1_12_msg2(void);

void display_pkt1_13_icon(void);
void display_pkt1_13_msg1(void);
void display_pkt1_13_msg2(void);

void display_pkt1_14_msg1(void);
void display_pkt1_14_msg2(void);

void display_pkt1_15_msg1(void);
void display_pkt1_15_msg2(void);

void set_channel_bits(uint32_t, uint32_t, uint32_t);
void set_pkt1_bits(uint32_t, uint32_t, uint32_t);
void set_pkt2_bits(uint32_t, uint32_t, uint32_t);
void display_debug_p( void );

// =============================================================================
// function key assignment from CPS
//
// this code is directly from the Mfr.

typedef struct
{
  uint8_t long_press_time;
  uint8_t emergency_quit_time;
  uint8_t p1_short;
  uint8_t p1_long;
  uint8_t p2_short;
  uint8_t p2_long;
  uint8_t p3_short;
  uint8_t p3_long;
  uint8_t p4_short;
  uint8_t p4_long;
  uint8_t p5_short;
  uint8_t p5_long;
  uint8_t p6_short;
  uint8_t p6_long;
  uint8_t p7_short;
  uint8_t p7_long;
  uint8_t p8_forward;
  uint8_t p8_reverse;
  uint8_t k0_short;
  uint8_t k0_long;
  uint8_t k1_short;
  uint8_t k1_long;
  uint8_t k2_short;
  uint8_t k2_long;
  uint8_t k3_short;
  uint8_t k3_long;
  uint8_t k4_short;
  uint8_t k4_long;
  uint8_t k5_short;
  uint8_t k5_long;
  uint8_t k6_short;
  uint8_t k6_long;
  uint8_t k7_short;
  uint8_t k7_long;
  uint8_t k8_short;
  uint8_t k8_long;
  uint8_t k9_short;
  uint8_t k9_long;
  uint8_t ka_short;
  uint8_t ka_long;
  uint8_t kb_short;
  uint8_t kb_long;
  uint8_t kc_short;
  uint8_t kc_long;
  uint8_t kd_short;
  uint8_t kd_long;
}FB_KeysProgFunctionDef_t;

// Program Button define
typedef enum
{
  n_PB_None,                 //0
  n_PB_EmergencyOn,
  n_PB_EmergencyOff,
  n_PB_PowerSwitch,
  n_PB_RepeatTalkaround,
  n_PB_ScanSwitch,
  n_PB_LoneWorkerSwitch,
  n_PB_VoxSwitch,
  n_PB_MandownSwitch,
  n_PB_PrivacySwitch,
  n_PB_AllToneSwitch,       //10
  n_PB_Monitor,
  n_PB_KeypadLock,
  n_PB_Squelch,
  n_PB_ManualDialForPrivate,
  n_PB_OneTouchAccess1,
  n_PB_OneTouchAccess2,
  n_PB_OneTouchAccess3,
  n_PB_OneTouchAccess4,
  n_PB_OneTouchAccess5,
  n_PB_OneTouchAccess6,     //20
  n_PB_VolumeUp,            // corrected mfr spelling from Volumn
  n_PB_VolumeDown,
  n_PB_ZoneUp,
  n_PB_ZoneDown,
  n_PB_CovertMode,
  n_PB_PublicAddress,
  n_PB_Squelch_Level,
  n_PB_Squelch_Switch,
  n_PB_Zone_Toggle,
  n_PB_Batt_Indication,    //30
  n_PB_Nuisance_Del,
  n_PB_ChannelUp,
  n_PB_ChannelDown,
  n_PB_Monitor_Momentary,
  n_PB_Emergency_Alarm_Ack,
  n_PB_Home_Zone,
  n_PB_Advanced_Features,
  n_PB_Record_Normal,
  n_PB_Record_Protect,
  n_PB_Record_Play,        //40
  n_PB_Record_Delete,
  n_PB_Channel_Knob,  //Forbid Channel Change
  n_PB_Group_CA,      //Group Call Alert
  n_PB_BT_Audio_Switch,
  n_PB_Pre_Alarm_Broadcast,
  n_PB_D_Monitor_Mode,
  n_PB_D_Msg_Auto_Lock, 
  n_PB_Reset_Acc_Port1,
  n_PB_Reset_Acc_Port2, 
  n_PB_Reset_Acc_Port3,    //50
  n_PB_Ext_PA_Switch, 
}DBProgramButton_e;

/* One Touch Acess */
typedef enum
{
  e_OTA_1,
  e_OTA_2,
  e_OTA_3,
  e_OTA_4,
  e_OTA_5,
  e_OTA_6
}DB_OTA_e;

enum
{
  e_OTA_MODE_NONE,
  e_OTA_MODE_DIGITAL,
  e_OTA_MODE_ANALOG,
};

enum
{
  e_OTA_CALLTYPE_GROUP,
  e_OTA_CALLTYPE_PRIVATE,
  e_OTA_CALLTYPE_CALLALERT,
  e_OTA_CALLTYPE_TEXTMSG,
  e_OTA_CALLTYPE_DTMF1,
  e_OTA_CALLTYPE_DTMF2,
  e_OTA_CALLTYPE_DTMF3,
  e_OTA_CALLTYPE_DTMF4
};
typedef struct
{
  struct{
    uint8_t     mode:2;
    uint8_t     function_type:2; /* 0:call; 1:menu */
    uint8_t     call_type:4;
  }Byte0;

  uint8_t     text_msg_index;//Quick msg index
  uint8_t     byte_rsvd[2];
  uint32_t    contact_index;//Contact list index
}DB_OTAMember_t;

// ============================================================================= 
//  button structures
extern FB_KeysProgFunctionDef_t default_key_functions;  
extern FB_KeysProgFunctionDef_t current_key_functions;  
extern FB_KeysProgFunctionDef_t stored_key_functions;   
extern FB_KeysProgFunctionDef_t update_key_functions;   

extern uint16_t FB_num_keys;   // number of items in the function button structure
extern uint16_t FB_num_funct;  // max number of functions assignable


#endif /* PROCESS_CS800D_H_ */
