//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// September 5, 2018          Design Documents                   process_FT857.c
//
//			header for process packets
//                    UART2 communication with FT857
//                      for Large Panel Project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  09/05/2018 copied from process_packets (CS800D) for revision to FT987
//
// Status
//  writing
//
//==============================================================================
#include "CSI_PROCESS_FT857.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "CSI_RAM.h"
#include "CSI_FUNCTIONS.h"
#include "CSI_SCREENS.h"
#include "CSI_GRAPHICS.h"
#include "CSI_COLOR.h"
#include "CSI_SWITCHES.h"
#include "CSI_ERROR_LOG.h"
#include "CSI_PANELINFO.h"
#include "CSI_Menus.h"
#include "uart.h"

// variables

// uint32_t buffer_length;

extern uint32_t packet_process_buffer_flag;
extern uint32_t packets_fifo_count;
extern uint8_t  packets_fifo[4][256];
extern uint32_t packets_fifo_output_pointer;
extern uint32_t packets_fifo_input_pointer;
extern uint32_t packets_fifo_overrun_error;
extern uint32_t packet_process_watchdog_timer;

uint32_t RxFreq2;
uint32_t TxFreq2;

//uint32_t cs_pkt1_active;
//uint32_t cs_pkt2_active;

//uint32_t cs_pkt1_new;
//uint32_t cs_pkt2_new;
/*
// location for first tab list
uint16_t list_xloc = 30; 
uint16_t list_ytop = 105;

// location for packet debug messages (screen 5)
uint16_t debug_p_x = 40;
uint16_t debug_p_y = 240;
uint16_t debug_p_step = 25;
uint16_t debug_p_line = 0;

uint8_t msg1_dump[16];
uint8_t msg2_dump[16];
*/
char packet_test[6] = {0xA5, 0x04, 0x40, 0x00, 0x00, 0x40 };
char radio_start[6] = { "radio2"};

//
// ****************************************************************************
// September 5, 2018		Design Documents                set_normal_ack()
//
//
//      Loads the acknowlegement buffer with the standard ack
// 
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  09/05/2018 copied from process_packets (CS800D) for revision to FT987
//
// Status: writing, Needs to be analyzed, may not be a standard packet
//

uint8_t normal_ack2[6] = {0xA5, 0x04, 0x40, 0x00, 0x00, 0x00 };

void set_normal_ack2( void )
{
  memcpy( (char*) ack_buffer, (char const*) normal_ack2, 4);
}

// !!! ALL THESE ARE FOR CS800D may be changed or removed
// ****************************************************************************
// process_FT857           determines purpose of packet
// packet_type_0           Process idle command 1
// packet_type_1           Process idle command 2
// packet_type_2           Process firmware version
// packet_type_3           Process volume control
// packet_type_4           Process two lines centered
// packet_type_5           Process one line left justified with icons
// packet_type_6           Process menu selection
// packet_type_7           Process data entry type 1
// packet_type_8           Process data entry type 2
// packet_type_9           Process data entry type 3
// packet_type_10          Process two lines centered
// packet_type_11          Process data entry type 4
// packet_type_12          Process data entry type 5
// packet_type_13          Process the Receive screen

// packet_type_14          Process data entry for text (using mic keypad)
// packet_type_15          Item selected in a list

// process_all_packet      Process all packets for output
// packet_buffer_0_output  process packet type 0
// packet_buffer_1_output  process packet type 1
// packet_buffer_2_output  process packet type 2
// packet_buffer_3_output  process packet type 3
// packet_buffer_4_output  process packet type 4
// packet_buffer_5_output  process packet type 5
// packet_buffer_6_output  process packet type 6
// packet_buffer_7_output  process packet type 7
// packet_buffer_8_output  process packet type 8
// packet_buffer_9_output  process packet type 9
// packet_buffer_10_output process packet type 10
// packet_buffer_11_output Process packet type 11
// packet_buffer_12_output Process packet type 12
// packet_buffer_13_output Process packet type 13

// packet_buffer_14_output Process packet type 14
// packet_buffer_15_output Process packet type 15

// extract_message         Get ASCII message embedded in protocol
// setup_keypad_switches   setup keypad or front panel switches buffer
// setup_color             Setup background and foreground color
// output_icons_to_display Output the ICONS to the display
// packet_buffer_5_display Display packet 5
// 
// ****************************************************************************
// September 5, 2018		Design Documents                   process_FT87
//
// This program processes packets received from UART2.
// This program is called by a 1 millisecond interrupt. 
//
// Programmer
//  Jerry Wanger, Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  uart2_rx_flag true indicates there is data sitting in the uart2_rx_buffer[] 
//
// Exit Conditions
//  if uart2_rx_flag true then the following happens:
//    1. uart2_rx_buffer[] is saved to the packet_process_buffer[]
//    2. The packet is taken apart and sent to a non interrupt program to be
//       displayed or otherwise handled.
//    3. The acknowledgement buffer is setup for other programs to handle
//
// External functions called
//  void packet_type_0(void)
//  void packet_type_1(void)
//  void packet_type_2(void)
//  void packet_type_3(void)
//  void packet_type_4(void)
//  void packet_type_5(void)
//  void packet_type_6(void)
//  void packet_type_7(void)
//  void packet_type_8(void)
//  void packet_type_9(void)
//  void packet_type_10(void)
//
// External variables used
//  uart2_rx_flag
//  uart2_rx_buffer[256]
//  packet_process_buffer[256]
//  unknown_packet_error
//  unknown_packet_error_buffer[256]
//
// Internal variables used
//  uint32_t i
//
// Constants
//
// History
//  09/05/2018 copied from process_packets (CS800D) for revision to FT857
//
// Status
//  writing
//
void process_FT857(void)
{
  uint32_t i;
  uint32_t temp_flag;
  static uint32_t uart2_rx_flag = 0;  

  // Watchdog timer for packet_process_buffer_flag
  if (packet_process_watchdog_timer) packet_process_watchdog_timer--;
  if (!packet_process_watchdog_timer)
  {
    packet_process_buffer_flag = 1;
  }
  //
  // If the packet_process_buffer_flag = 1, implying the programs are ready
  // to accept more data, and the numbers of currently active packets in the 
  // process_packets_fifo[4][]is not zero, output the oldest packet to the 
  // packet_process_buffer[i].
  //
  temp_flag = 0;
  if ((packet_process_buffer_flag == 1) && (packets_fifo_count))
  {
    for (i = 0; i < packets_fifo[packets_fifo_output_pointer][2] + 4; i++)
    {
      packet_process_buffer[i] = packets_fifo[packets_fifo_output_pointer][i];
    } 
    packets_fifo_output_pointer = (packets_fifo_output_pointer + 1) & 0x03;
    packets_fifo_count--;
    temp_flag = 1;
  }
  //
  // If (uart2_rx_flag = 1) && (temp_flag = 0) && (packets_fifo_count = 0) &&
  // (packet_process_buffer_flag = 1), output the uart data directly to the
  // packet_process_buffer[i] and process.
  //
  if ((uart2_rx_flag == 1) && (temp_flag == 0) && (packets_fifo_count == 0) &&
  (packet_process_buffer_flag == 1))
  {
    for (i = 0; i < uart2_rx_buffer_index + 1; i++)
    {
      packet_process_buffer[i] = uart2_rx_buffer[i];
    }  
    packets_fifo_input_pointer = (packets_fifo_input_pointer + 1) & 0x03l;
    uart2_rx_flag = 0;
    goto process_packets_1;    
  }
  //
  // If we go to this point, then if there is any data available from the uart2
  // it has to go directly in the packets_fifo[][]. If packet_process_buffer_flag 
  // = 1, then we continue processing.
  //
  if (uart2_rx_flag == 1)
  {
     if (packets_fifo_count < 4)
     {
       for (i = 0; i < uart2_rx_buffer_index + 1; i++)
       {
         packets_fifo[packets_fifo_input_pointer][i] = uart2_rx_buffer[i];
       }  
       packets_fifo_input_pointer = (packets_fifo_input_pointer + 1) & 0x03l;
       packets_fifo_count++;
     }
     else
     {
       packets_fifo_overrun_error = 1; 
     }
     uart2_rx_flag = 0;   
  }  

  // If we got to this point and the temp_flag = 0, exit immediately as there
  // is nothing to process at this time.
  if (!temp_flag) return;
  
  // Start processing the data to determine which program to process
  process_packets_1:
  packet_process_watchdog_timer = 1000;
  packet_process_buffer_flag = 1;  // data is now in use

  // Determine and process the various packet types
  if (packet_type_0())  return;
  /*
  if (packet_type_1())  return;
  if (packet_type_2())  return;
  if (packet_type_3())  return;
  if (packet_type_4())  return;
  if (packet_type_5())  return; 
  if (packet_type_6())  return;
  if (packet_type_7())  return;
  if (packet_type_8())  return;
  if (packet_type_9())  return;
  if (packet_type_10()) return;
  if (packet_type_11()) return;
  if (packet_type_12()) return;
  if (packet_type_13()) return;
  if (packet_type_14()) return;
  if (packet_type_15()) return;
  */

  // If we got here, we have an unknown packet type. Save the packets
  for (i = 0; i < uart2_rx_buffer_index + 1; i++)
  {
    unknown_packet_error_buffer[i] = uart2_rx_buffer[i];
  } 
  // display_packet_dump();
  unknown_packet_error = 1;
  packet_process_buffer_flag = 1; // allow to continue processing
}

/*
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_0
//
// This command is used to determine if the display is still attached to the
// radio.
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display                     Display to Radio
// B1 00 05 00 01 01 00 02 B4         B1 00 05 80 01 01 00 82 B4        
// B1 00 05 00 01 02 00 03 B4         B1 00 05 80 01 02 00 83 B4     
// B1 00 05 00 01 03 00 04 B4         B1 00 05 80 01 03 00 84 B4     
// B1 00 05 00 01 04 00 05 B4         B1 00 05 80 01 04 00 85 B4     
// B1 00 05 00 01 05 00 06 B4         B1 00 05 80 01 05 00 86 B4     
// B1 00 05 00 01 06 00 07 B4         B1 00 05 80 01 06 00 87 B4     
// B1 00 05 00 01 07 00 08 B4         B1 00 05 80 01 07 00 88 B4     
// B1 00 05 00 01 08 00 09 B4         B1 00 05 80 01 08 00 89 B4      
// B1 00 05 00 01 09 00 0A B4         B1 00 05 80 01 09 00 8A B4     
// B1 00 05 00 01 0A 00 0B B4         B1 00 05 80 01 0A 00 8B B4  
//
// Notice that character 5 is incrementing from 0x01 through 0x0A. On the 
// acknowledgement of that string, a duplicate is sent back except character 3 
// is always 80. The checksum is calculated on the data.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_0(void)
{
  // Determine if packet type 0.
  if (packet_process_buffer[2] != 0x05) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x01) return 0;

  // Setup acknowlegement buffer.
  ack_buffer[0] =  0xB1;
  ack_buffer[1] =  0x00;
  ack_buffer[2] =  0x05;
  ack_buffer[3] =  0x80;
  ack_buffer[4] =  0x01;
  ack_buffer[5] =  packet_process_buffer[5];
  ack_buffer[6] =  0x00;
  ack_buffer[7] =  packet_process_buffer[7] + 0x80;
  ack_buffer[8] =  0xB4;  
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 0;

  // indicate packet has been processed.
  return 1;  
}

//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_1
//
// This is an idle command along with the information necessary to control the
// transmit led, receive led, and backlight for the LCD display.
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display              Display to Radio   
// B1 00 05 00 70 02 00 72 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Green LED
// B1 00 05 00 70 01 00 71 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Red LED
// B1 00 05 00 70 03 00 73 B4    B1 00 05 80 70 00 00 F0 B4  Turn on Red + Green
// B1 00 05 00 70 00 00 70 B4    B1 00 05 80 70 00 00 F0 B4  Turn off both LEDs
// B1 00 05 00 70 0C 00 7C B4    B1 00 05 80 70 00 00 F0 B4  Turn on backlight
// B1 00 05 00 70 08 00 78 B4    B1 00 05 80 70 00 00 F0 B4  Turn off backlight
//
// Note that the response to this protocol is always the same.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_1(void)
{
  // Determine if packet type 1.
  if (packet_process_buffer[2] != 0x05) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x70) return 0;

  // Setup acknowlegement buffer.
  ack_buffer[0] =  0xB1;
  ack_buffer[1] =  0x00;
  ack_buffer[2] =  0x05;
  ack_buffer[3] =  0x80;
  ack_buffer[4] =  0x70;   // different from normal
  ack_buffer[5] =  0x00;
  ack_buffer[6] =  0x00;
  ack_buffer[7] =  0xF0;
  ack_buffer[8] =  0xB4;  
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 1;

  // indicate packet has been processed.
  return 1;  
}
// April 29, 2018		Design Documents                   packet_type_2
//
// This command request the firmware version number of the display
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display              Display to Radio 
// B1 00 04 00 10 00 10 B4       B1 00 19 80 10 00 32 2E 30 30 2E 30 38 00 00  
//                               00 00 00 00 00 00 00 0A 00 00 00 01 F0 B4
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_2(void)
{
  uint32_t i;
  // Determine if packet type 2.
  if (packet_process_buffer[2] != 0x04) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x10) return 0;

  // set buffer to zero so only have to change non zero values
  for (i = 0; i < 28; i++) ack_buffer[i] = 0;

  // Setup acknowlegement buffer.
  ack_buffer[0]  =  0xB1;
  ack_buffer[2]  =  0x19;
  ack_buffer[3]  =  0x80;
  ack_buffer[4]  =  0x10;
  ack_buffer[6]  =  0x32; // 2
  ack_buffer[7]  =  0x2E; // .
  ack_buffer[8]  =  0x30; // 0
  ack_buffer[9]  =  0x30; // 0
  ack_buffer[10] =  0x2E; // .
  ack_buffer[11] =  0x30; // 0
  ack_buffer[12] =  0x38; // 8
  ack_buffer[22] =  0x0A;
  ack_buffer[26] =  0x01;
  ack_buffer[27] =  0xF0;
  ack_buffer[28] =  0xB4;
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 2;

  // indicate packet has been processed.
  return 1;    
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_3
//
// This command is used display the volume control
//
// This program checks if its the proper packet type and if it is it does:
//   1. Sets up the ack buffer to be sent back to the radio
//   2. Sets up so a non interrupt program can process the packet
//
// Radio to Display
// B1 00 49 00 40 07 01 XX 52 61 64 69 6F 20 56 6F 6C 75 6D 65
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 4E 6F 20 54 65 78 74 21 00 00 00 00 00 00 00 00 00 00 00 00
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 07 73 B4
//
// XX is the number of bars and is between 0x01 and 0x20. The checksum will
// change based on the value of XX.
//
// Display to Radio
// B1 00 05 80 40 00 00 C0 B4
//
// The message on the display (currently Radio Volume) starts at position 8 and
// ends when the next character is 0x00;
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_3(void)
{
  // Determine if packet type 3.
  if (packet_process_buffer[2] != 0x49) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x07) return 0;
  if (packet_process_buffer[6] != 0x01) return 0;
  
  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 3;
  // indicate packet has been processed.
  return 1;      
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_4
//
// Display message with both lines centered
//
// Radio to Display
// B1 00 49 00 40 07 00 01 44 69 67 20 4D 6F 6E 69 74 6F 72 20
// 4D 6F 64 65 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 4F 66 66 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 07 24 B4
//
// Display to Radio
// B1 00 05 80 40 00 00 C0 B4
//
// Probably should investigate what packet_process_buffer[5] is used for. So 
// far seen it as 6 and 7
//
// Line 1 starts at position 8 and line 2 starts at position 40. The line ends
// when the next character is 0x00;
//
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_4(void)
{
  // Determine if packet type 4.
  if (packet_process_buffer[2] != 0x49) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[6] != 0x00) return 0; 
  
  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 4;
  // indicate packet has been processed.
  return 1;      
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_5
//
// Display message left justified, single line with ICONS
//
// Radio to Display  |char 6
// B1 00 50 00 40   00 00 06 0A 00   55 20 43 68 61   6E 6E 65 6C 20  
// 31 00 00 00 00   00 00 00 00 00   00 00 00 00 00   00 00 82 60 A7
// 02 82 60 A7 02   5A 6F 6E 65 20   32 00 00 00 00   00 00 00 00 00
// 00 00 00 00 00   00 00 00 00 00   00 00 00 00 00   00 00 01 02 04
// 00 08 DA B4
//
// Display to Radio
// B1 00 05 80 40 00 00 C0 B4
//
// Character 6: 
// bits 0  2:       Value 0: RSSI no bars
//                   Value 1: RSSI 1 bar
//                   Value 2: RSSI 2 bars
//                   Value 3: RSSI 3 bars
//                   Value 4: RSSI 4 bars
//                   Value 5: RSSI 5 bars
// Bit 6             1 = Scan Icon, 0 = no Scan Icon
//
// Character 7: 
// Bits 0 -1:        Value 0 Power Level L
//                   Value 1 Power Level M
//                   Value 2 Power Level H
// Bit 2:            1 = Open Lock Icon, 0 = no Icon
// Bit 3:            1 = Closed Lock Icon, 0 = no Icon
// Bit 4:            1 = unread message Icon, 0 = no Icon
// Bit 5:            1 = Inbox Full Icon, 0 = no Icon
//
// Character 8:
// Bit 0:            1 = Tone Disable Icon, 0 = no Icon
// Bit 1:            1 = Talk Around Icon, 0 = no Icon
// Bit 2:            1 = Monitor Icon, 0 = no Icon
//
// Character 42: Start of message and ends when next character is 0x00
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_5(void)
{

  // Determine if packet type 5.
  if (packet_process_buffer[2] != 0x50) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x00) return 0;

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;
  
  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 5;
  
  // display_packet_dump();
  
  // indicate packet has been processed.
  return 1;     
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_6
//
// This is the start of the menu section of the program. Pressing the button 
// advances to the next ICON the choices from left to right is:
//   Contacts
//   Scan
//   Zone
//   Messages
//   Call Log
//   Utilities
//
// Radio to Display
// B1 00 29 00 40 01 3F 00 43 6F 6E 74 61 63 74 73 00 00 00 00
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 00 00 03 BF B4
//
// Note that packet_process_buffer[7] contains the ICON type which has a value
// from 00 to 05. packet_process_buffer[8] is the start of the ASCII message
// corresponding to the ICON choices shown above.
//
// Display to Radio
// ######################################################################################Need to check with analyzer
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_6(void)
{
  // Determine if packet type 6.
  if (packet_process_buffer[2] != 0x29) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x01) return 0;
  if (packet_process_buffer[6] != 0x3F) return 0;

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 6;
  
  // indicate packet has been processed.
  return 1;     
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_7
//
// Data Entry Screen 1
//
// Radio to Display
// B1 00 4B 00 40 02 13 00 00 00 52 61 64 69 6F 20 49 6E 66 6F
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   00 00 00 00   
// 00 00 52 61 64 69 6F 20 41 6C 69 61 73 00 00 00   00 00 00 00   
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   07 E9 B4
//
// Display to Radio
// ####################################################################################### Need to check with analyzer
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_7(void)
{
  // Determine if packet type 7.
  if (packet_process_buffer[2] != 0x4B) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x02) return 0;
  if (packet_process_buffer[6] != 0x13) return 0;

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 7;

  // indicate packet has been processed.
  return 1;       
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_8
//
// Data Entry Screen 2
//
// Radio to Display
// B1 00 4B 00   40 02 2C 00   00 00 42 61  63 6B 6C 69    67 68 74 00
// 00 00 00 00   00 00 00 00   00 00 00 00   00 00 00 00   00 00 00 00    
// 00 00 54  75  72 6E 20 4F   66 66 00 00   00 00 00 00   00 00 00 00   
// 00 00 00 00   00 00 00 00   00 00 00 00   00 00 00 00   06 DB B4
//
// Display to Radio
// ####################################################################################### Need to check with analyzer
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_8(void)
{
  // Determine if packet type 8.
  if (packet_process_buffer[2] != 0x4B) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x02) return 0;
  if (packet_process_buffer[6] != 0x2C) return 0;

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 8;

  // indicate packet has been processed.
  return 1;     
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                   packet_type_9
//
// Data Entry Screen 3
//
// Radio to Display
// B1 00 4B 00 40 02 10 00 00 00 53 69 67 6E 61 6C 20 54 79 70
// 65 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   
// 00 00 32 2D 54 4F 4E 45 00 00 00 00 00 00 00 00 00 00 00 00   
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 06 07 B4
//
// Display to Radio
// ####################################################################################### Need to check with analyzer
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_9(void)
{
  // Determine if packet type 9.
  if (packet_process_buffer[2] != 0x4B) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x02) return 0;
  if (packet_process_buffer[6] != 0x10) return 0;

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 9;

  // indicate packet has been processed.
  return 1;       
}
//
// ****************************************************************************
// April 29, 2018		Design Documents                  packet_type_10
//
// Display message with both lines centered
//
// Radio to Display
// B1 00 47 00 40 07 00 01 44 69 67 20 4D 6F 6E 69 74 6F 72 20
// 4D 6F 64 65 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 4F 66 66 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// 00 00 00 00 00 00 00 00 00 00 00 00 00 00 07 24 B4
//
// Display to Radio
// B1 00 05 80 40 00 00 C0 B4
//
// Probably should investigate what packet_process_buffer[5] is used for. So 
// far seen it as 6 and 7
//
// Line 1 starts at position 6 and line 2 starts at position 40. The line ends
// when the next character is 0x00;
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   packet_process_buffer[256] contains the complete packet to be processed
//
// Exit Conditions
//  ack_buffer[32] contains the data to be sent back to the radio
//  packet_buffer_type_output_type contains the packet type
//  packet_buffer_type_output_flag is set to a 1 to indicate packet to process
//
// External functions called
//
// External variables used
//  packet_process_buffer[256]
//  ack_buffer[32]
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t packet_type_10(void)
{
  // Determine if packet type 10.
  if (packet_process_buffer[2] != 0x47) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x06) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 10;
  
  // indicate packet has been processed.
  return 1;       
}

// *****************************************************************************
// April 29, 2018		Design Documents                  packet_type_11
//
uint32_t packet_type_11(void)
{
  // Determine if packet type 11.
  if (packet_process_buffer[2] != 0x4B) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x02) return 0; 
  if (packet_process_buffer[6] != 0x2F) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;
  
  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 11;

  // indicate packet has been processed.
  return 1;       
}
//
// *****************************************************************************
// April 29, 2018		Design Documents                  packet_type_12
//
uint32_t packet_type_12(void)
{
  // Determine if packet type 12.
  if (packet_process_buffer[2] != 0x50) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x03) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 12;

  // indicate packet has been processed.  
  return 1;       
}
//
// *****************************************************************************
// May 3, 2018	        	Design Documents                  packet_type_13
//
uint32_t packet_type_13(void)
{
  //4B  00  40  08
  // Determine if packet type 13.
  if (packet_process_buffer[2] != 0x4B) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  if (packet_process_buffer[5] != 0x08) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 13;

  // indicate packet has been processed.
  return 1; 
}
//
// *****************************************************************************
// Aug 17, 2018	        	Design Documents                  packet_type_14
//
//   type 14 is for data entry from the microphnoe keypad
//   the packet is sent on a uniformly timed repeating basis while 
//   keypad entry is being accepted. It updates the display with
//   the message as it is being typed.

uint32_t packet_type_14(void)
{
  //B8  00  40  04
  // Determine if packet type 14.
  if (packet_process_buffer[2] != 0xB8) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  // if (packet_process_buffer[5] != 0x04) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 14;

  // indicate packet has been processed.  
  return 1; 
}
//
// *****************************************************************************
// Aug 17, 2018	        	Design Documents                  packet_type_15
//
//   type 15 occurs after selecting an item in a list type menu. Type 12
//   

uint32_t packet_type_15(void)
{
  //97  00  40  04
  // Determine if packet type 15.
  if (packet_process_buffer[2] != 0x97) return 0;
  if (packet_process_buffer[3] != 0x00) return 0;
  if (packet_process_buffer[4] != 0x40) return 0;
  // if (packet_process_buffer[5] != 0x09) return 0; 

  // Setup acknowlegement buffer.
  set_normal_ack();
  ack_buffer_flag = 1;

  // Setup packet_process_buffer to be processed by mainline program
  packet_buffer_type_output_flag = 1;
  packet_buffer_type_output_type = 15;
  
  // indicate packet has been processed.
  return 1;
}
// =============================================================================
// 
// ****************************************************************************
// April 29, 2018		Design Documents              process_all_packet
//
// This program processes packets for output. This program is called from the
// mainline because of the possibly lenghty time needed to process these packets 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  if packet_buffer_type_output_flag true, process packet
//  packet_buffer_type_output_type is the packet number to process
//
// Exit Conditions
//  packet is processed
//
// External functions called
//  void packet_buffer_0_output(void)
//  void packet_buffer_1_output(void)
//  void packet_buffer_2_output(void)
//  void packet_buffer_3_output(void)
//  void packet_buffer_4_output(void)
//  void packet_buffer_5_output(void)
//  void packet_buffer_6_output(void)
//  void packet_buffer_7_output(void)
//  void packet_buffer_8_output(void)
//  void packet_buffer_9_output(void)
//  void packet_buffer_10_output(void)
//  void packet_buffer_11_output(void)
//  void packet_buffer_12_output(void)
//  void packet_buffer_13_output(void)
//  void packet_buffer_14_output(void)
//
// External variables used
//  packet_buffer_type_output_flag
//  packet_buffer_type_output_type
//
// Internal variables used
//  packet_type_err
//
// Constants
//
// History
//
// Status
//  written
//
// for packet debug
//uint32_t packet_list1[64][4];
//uint32_t pac_index = 0;



void process_all_packet(void)
{
  if (!packet_buffer_type_output_flag) return;
  packet_buffer_type_output_flag = 0;

  int packet_type_err = 0;

  switch (packet_buffer_type_output_type)
  {
    case 0:
      packet_buffer_0_output(); break;
    case 1:
      packet_buffer_1_output(); break;
    case 2:
      packet_buffer_2_output(); break;
    case 3:
      packet_buffer_3_output(); break;
    case 4:
      packet_buffer_4_output(); break;
    case 5:
      packet_buffer_5_output(); break;
    case 6:
      packet_buffer_6_output(); break;
    case 7:
      packet_buffer_7_output(); break;
    case 8:
      packet_buffer_8_output(); break;
    case 9:
      packet_buffer_9_output(); break;
    case 10:
      packet_buffer_10_output(); break;
    case 11:
      packet_buffer_11_output(); break;
    case 12:
      packet_buffer_12_output(); break;
    case 13:
      packet_buffer_13_output(); break;
    case 14:
      packet_buffer_14_output(); break;
    case 15:
      packet_buffer_15_output(); break;
      
  default: 
      packet_type_err = 1; break;
  }

  // packet debug
  //packet_list1[pac_index][0] = packet_process_buffer[2];
  //packet_list1[pac_index][1] = packet_process_buffer[4];
  //packet_list1[pac_index][2] = packet_process_buffer[5];
  //packet_list1[pac_index++][3] = packet_buffer_type_output_type;
  //if(pac_index > 63) pac_index = 0;
  
  if( packet_type_err )
  {
    packet_process_buffer_flag = 0;
    // for debugging
    log_event( "unknown packet" );   
  }
  else packet_process_buffer_flag = 1;
}

//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_0_output
//
// This routine is used for process the idle message type 1 and there is no 
// further processing except used as a watch dog.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Watchdog timer reset
//
// External functions called
//
// External variables used
//  radio_watchdog_timer
//
// Internal variables used
//
// Constants
//
// History
//     08/01/2018 nothing was written, added packet message to screen
//                too many packets, turned off the message
//
// Status
//  writing, was empty
//
void packet_buffer_0_output(void)
{
  // int i;
  // this occurs very frequently, update watchdog timer here
  // log packet 
  // log_event( "packet 0" );
  // if(display_mode == 1)
  //{  
    //char *message = "Packet 0  ";
    //for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
    //ssd1963_cg_write_line( 10, 3, 40 , 240, 4);       
  //}
 

}

//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_1_output
//
// This routine is used for process the idle message type 2. The message is used
// to control the transmit led, receive led, and backlight.
//
// Character 5 contains the command as follows:
// Bit 0    Red LED:   1 = on, 0 = off
// Bit 1    Green LED: 1 = on, 0 = off
// Bit 2    clear top message line
// Bit 3    clear bottom message line
// Bit 4
// Bit 5
// Bit 6   Backlight Control: 1 = on, 0 = off
// Bit 7   BackLight
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  LED's turned on and off as dictated by the program
//
// External functions called
//  void tx_led_on(void)
//  void tx_led_off(void)
//  void rx_led_on(void)
//  void rx_led_off(void)
//  void lcd_backlight_on(void)
//  void lcd_backlight_off(void)  
//
// External variables used
//  packet_process_buffer[5]
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//  written
//
void packet_buffer_1_output(void)
{
  // process transmit (red) LED
  if (packet_process_buffer[5] & 0x01)
  {
    tx_led_on();
  }
  else
  {
    tx_led_off();
  }
  // process receive(green) LED
  if (packet_process_buffer[5] & 0x02)
  {
    rx_led_on();
  }
  else
  {
    rx_led_off();
  } 

  // process LCD backlight
  if (packet_process_buffer[5] & 0x08)
  {
    if (packet_process_buffer[5] & 0x04)
    {
      lcd_backlight_on();
    }
    else
    {
    // lcd_backlight_off(); 
    }
  }
  // log packet 
  // log_event( "packet 1" );
  
  // dump when unknown bits come in
  if( packet_process_buffer[5] & 48) // bits 4,5 (16+32)
  {
    display_packet_dump();  
  }
  
  if( packet_process_buffer[5] & 0x08)
  {
    if(display_mode == 1)
    {
      // test for use as screen update
      // clear_pckt_message( 3 );
      clear_pckt_message( 4 );
      // redisplay zone and talkgroup
      // display_zone_alias();
      display_channel_alias();
      pckt_msg4_dirty = 1;
    }
  }
  if( packet_process_buffer[5] & 0x04)
  {
    if(display_mode == 1)
    {
      // test for use as screen update
      clear_pckt_message( 3 );
      // clear_pckt_message( 4 );
      // redisplay zone and talkgroup
      display_zone_alias();
      pckt_msg3_dirty = 1;
      //display_channel_alias();
    }
  }   
 
    if(display_mode == 1)
    {
      // packet 1 has no text, set msg 1 to the value of the bit command byte 
      //msg1_dump[0] = '\0';
      sprintf((char*) msg1_dump, "%02X", packet_process_buffer[5]);
      msg2_dump[0] = '\0';
      strncpy( (char*) cg_line_buffer, (char const*) "Packet 1  ", 10);

      display_debug_p();
      //ssd1963_cg_write_line( 10, 3, debug_p_x , debug_p_y + debug_p_line++ * debug_p_step, 4);
      // if(debug_p_line > 5) debug_p_line = 0;
      //ssd1963_cg_write_line( 10, 3, 40 , 240, 4);       
    }
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_2_output
//
// This routine is used for processing the version number request from the radio
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters provided by the program
//
// Exit Conditions
//  There is no processing done in this program for this packet type
//
// External functions called
//
// External variables used
//
// Internal variables used
//
// Constants
//
// History
//
// Status
//    writing, was empty
//
void packet_buffer_2_output(void)
{
   // log packet 
  log_event( "packet 2" );
  if(display_mode == 1)
  {
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 2  ", 10);
    display_debug_p();
  }    
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_3_output
//
// This routine is used for processing the volume control level display. 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Volume control level is displayed
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//  void ssd1963_cg_write_line(uint8_t,uint8_t,uint16_t, uint16_t, uint16_t)
//  void display_volume_bar(packet_process_buffer[uint32_t)
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint32_t cs_pkt1_volume               0                 7 
//  uint8_t  cs_pkt1_volume_msg[16]       1                 8
//  uint32_t cs_pkt1_volume_length       n/a
//
// Internal variables used
//  uint32_t j
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t cs_pkt1_volume;
uint8_t  cs_pkt1_volume_msg[16];
uint32_t cs_pkt1_volume_length;

void packet_buffer_3_output(void)
{
  uint32_t i;
  uint32_t j;
  uint32_t k; 

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Process volume. Volume is in location 7
  i = packet_process_buffer[7];
  set_pkt1_bits(0, cs_pkt1_volume, i);
  cs_pkt1_volume = i;
  //
  // Process volume message. The starting location for this variable is position 8
  // and can be up to 16 characters long. First we determine if this is a new 
  // message. If new, then we set the cs_pk1_new and cs_pkt1_active bits. If not 
  // new leave bits alone. This message might not be used in the application 
  // because currently it is always fixed.
  //
  i = 0;
  j = extract_message(8);
  if (j != cs_pkt1_volume_length)
  {
    i = 1;
    cs_pkt1_volume_length = j;
  }
  
  if (!i)
  {
    for (k = 0; k < j; k++)
    if (packet_process_buffer[8+k] != cs_pkt1_volume_msg[k]) i = 1;
  }
  
  if (i)
  {
    for (k = 0; k < j; k++) cs_pkt1_volume_msg[k] = packet_process_buffer[8+k];
    set_pkt1_bits(1,1,2);
  }

  // Output data to the display.
  packet_buffer_3_display();
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_4_output
//
// This is used for display two lines centered. The first line starts at
// position 8 and the second line starts at position 40
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_4_msg1[16]           2                 8
//  uint32_t cs_pkt1_4_msg1_length       n/a
//  uint8_t  cs_pkt1_4_msg2[16]           3                40
//  uint32_t cs_pkt1_4_msg2_length       n/a
//  
// Internal variables used
//  uint32_t j
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_4_msg1[16];
uint32_t cs_pkt1_4_msg1_length;
uint8_t  cs_pkt1_4_msg2[16];
uint32_t cs_pkt1_4_msg2_length;

void packet_buffer_4_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // For first message, extract message and determine the number of characters.
  // The starting location for this format is position 8
  j = extract_message(8);
  cs_pkt1_4_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_4_msg1[k] = packet_process_buffer[8+k];
  set_pkt1_bits(2,1,2);
  
  // For second message, extract message and determine the number of characters.
  // The starting location for this format is position 6.
  j = extract_message(40);
  cs_pkt1_4_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_4_msg2[k] = packet_process_buffer[40+k];
  set_pkt1_bits(3,1,2);  

  // Output data to the display.
  packet_buffer_4_display();
} 
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_5_output
//
// This is used for displaying the idle screen and the ICONS. This is one of the
// key programs as it shows much of the available parameters on a channel.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  A single line is displayed along with the ICONS
//
// External functions called
//  void lcd_backlight_on(void)
//  void   ssd1963_cg_clear_display(void)
//  uint32_t extract_message(uint32_t)
//  void ssd1963_cg_write_line(uint8_t,uint8_t,uint16_t, uint16_t, uint16_t) 
//  void output_icons_to_display(void)
//
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_channel_alias[16]            0            10-25
//  uint32_t cs_channel_alias_length         n/a
//  uint32_t cs_channel_rx_frequency         1            37-40       
//  uint32_t cs_channel_tx_frequency         2            41-44
//  uint8_t  cs_channel_zone_alias[16]       3            45-60
//  uint32_t cs_channel_zone_alias_length    n/a
//  uint32_t cs_channel_number_in_zone       4            77
//  uint32_t cs_channel_zone number          5            78
//  uint32_t cs_channel_number               6            79-80
//
//  uint32_t cs_channel_rssi                 7            6       0-2
//  uint32_t cs_channel_battery_level        8            6       3-5
//  uint32_t cs_channel_scan                 9            6       6-7
//
//  uint32_t cs_channel_power_level         10            7       0-1
//  uint32_t cs_channel_privacy             11            7       2-3
//  uint32_t cs_channel_message             12            7       4-5
//  uint32_t cs_channel_gps                 13            7       6-7
//                                     
//  uint32_t cs_channel_disable_tone        14            8       0
//  uint32_t cs_channel_talk_around         15            8       1
//  uint32_t cs_channel_monitor             16            8       2
//  uint32_t cs_channel_type                17            8       3-4
//  uint32_t cs_channel_mode                18            8       5
//  uint32_t cs_channel_emergency           19            8       6-7
//
//  uint32_t cs_channel_bt_state            20            9       0-1
//  uint32_t cs_channel_link                21            9       2
//  uint32_t cs_channel_volume              22            9       3-7
//
//  uint32_t cs_channel_active
//  uint32_t cs_channel_new
//
// Internal variables used
//  uint32_t j
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_channel_alias[16];
uint32_t cs_channel_alias_length;
uint8_t  cs_channel_zone_alias[16];
uint32_t cs_channel_zone_alias_length; 
uint32_t cs_channel_rx_frequency;
uint32_t cs_channel_tx_frequency;
uint32_t cs_channel_number_in_zone;
uint32_t cs_channel_zone_number;
uint32_t cs_channel_number; 
uint32_t cs_channel_rssi;
uint32_t cs_channel_battery_level;
uint32_t cs_channel_scan;
uint32_t cs_channel_power_level;
uint32_t cs_channel_privacy;
uint32_t cs_channel_message;
uint32_t cs_channel_gps;
uint32_t cs_channel_disable_tone;
uint32_t cs_channel_talk_around;
uint32_t cs_channel_monitor;
uint32_t cs_channel_type;
uint32_t cs_channel_mode;
uint32_t cs_channel_emergency;
uint32_t cs_channel_bt_state;
uint32_t cs_channel_link;
uint32_t cs_channel_volume;

uint32_t cs_channel_active = 0;
uint32_t cs_channel_new = 0;

void packet_buffer_5_output(void)
{
  uint32_t i;
  uint32_t j;
  uint32_t k;

  uint32_t ChNumb;
  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();
  // Process channel alias. The starting location for this variable is position 10
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  
  i = 0;
  j = extract_message(10);
  if (j != cs_channel_alias_length)
  {
    i = 1;
    cs_channel_alias_length = j;
  }
  
  if (!i)
  {
    for (k = 0; k < j; k++)
    if (packet_process_buffer[10+k] != cs_channel_alias[k]) i = 1;
  }
  if (i) // channel alias did not match current packet data
  {
    for (k = 0; k < j; k++) cs_channel_alias[k] = packet_process_buffer[10+k];
    set_channel_bits(0,1,2);
  }
  else // it did match
  {
    // clear the new indicator  (bit 0) so it's not redisplayed
    cs_channel_new &= ~1; 
  }
  // Process Zone alias. The starting location for this variable is position 45
  // First we determine if this is a new message. If new, then set the
  // cs_channel_new and cs_channel_active bits. If not new leave the bits alone.
  i = 0;
  j = extract_message(45);
  if (j != cs_channel_zone_alias_length) 
  {
    i = 1;
    cs_channel_zone_alias_length = j;
  }
  if (!i)
  {
    for (k = 0; k < j; k++)
    if (packet_process_buffer[45+k] != cs_channel_zone_alias[k]) i = 1;
  }
  if (i)  // zone alias did not match current packet data
  {
    for (k = 0; k < j;k++)
    {
      // blank bad data
      if(packet_process_buffer[45+k] == 0xFF)
      {
        cs_channel_zone_alias[k]= ' ';
        k++;
        cs_channel_zone_alias[k]= ' ';
      }
      else if(packet_process_buffer[45+k] > 126)
      {
        cs_channel_zone_alias[k]= ' ';
      } 
      else cs_channel_zone_alias[k]=packet_process_buffer[45+k];
      
    }
    set_channel_bits(3, 1, 2);
  }
  else // it did match
  {
    // clear the new indicator (bit 3) so it's not redisplayed
    cs_channel_new &= ~8; 
  }  
  // Process Rx Frequency. The starting position for this variable is 37.
  // Position 37 is the LSB of a 32 bit variable.
  RxFreq2 = (packet_process_buffer[37])      +
           (packet_process_buffer[38]<< 8)  +
           (packet_process_buffer[39]<< 16) +
           (packet_process_buffer[40]<< 24);
  
  set_channel_bits(1, cs_channel_rx_frequency, RxFreq2);
  cs_channel_rx_frequency = RxFreq2;
  // Process Rx Frequency. The starting position for this variable is 41.
  // Position 41 is the LSB of a 32 bit variable.
  TxFreq2 = (packet_process_buffer[41])      +
           (packet_process_buffer[42]<< 8)  +
           (packet_process_buffer[43]<< 16) +
           (packet_process_buffer[44]<< 24);
  
  set_channel_bits(2, cs_channel_tx_frequency, TxFreq2 );
  cs_channel_tx_frequency = TxFreq2;
  // Process channel number in zone. This variable is in location 77
  set_channel_bits(4, cs_channel_number_in_zone, packet_process_buffer[77]);
  cs_channel_number_in_zone = packet_process_buffer[77];
  // Process Zone Number. This variable is in location 78
  set_channel_bits(5, cs_channel_zone_number, packet_process_buffer[78] );
  cs_channel_zone_number = packet_process_buffer[78];  
  // Process Channel Number. Channel number is in locations 79 and 80. Position
  // 79 is LSB
  ChNumb =  (uint32_t) packet_process_buffer[79] + 
           ((uint32_t) packet_process_buffer[80]<< 8);
             
  set_channel_bits(6, cs_channel_number, ChNumb);
  cs_channel_number = ChNumb; 
  // Process RSSI. RSSI is in position 6 bits 0 - 2
  i = packet_process_buffer[6] & 0x07;
  set_channel_bits(7, cs_channel_rssi, i);    
  cs_channel_rssi = i;
  // Process Battery Level. Battery Level is in position 6 bits 3 - 5
  i = (packet_process_buffer[6] >> 3) & 0x07;
  set_channel_bits(8, cs_channel_battery_level, i);
  cs_channel_battery_level = i;
  // Process Scan. Scan is in position 6 bits 6-7
  i = (packet_process_buffer[6] >> 6) & 0x03; 
  set_channel_bits(9, cs_channel_scan, i);
  cs_channel_scan = i;
  // Process Power Level. Power Level is in Position 7 bits 0 - 1
  i = packet_process_buffer[7] & 0x03;
  set_channel_bits(10, cs_channel_power_level, i);
  cs_channel_power_level = i;
  // Process Privacy. Privacy is in Position 7 bits 2 - 3
  i = (packet_process_buffer[7] >> 2) & 0x03;
  set_channel_bits(11, cs_channel_privacy, i);
  cs_channel_privacy = i;   
  // Process Message. Message in position 7 bits 4-5
  i = (packet_process_buffer[7] >> 4) & 0x03;
  set_channel_bits(12, cs_channel_message, i);
  cs_channel_message = i;  
  // Process GPS. GPS in position 7 bits 6-7
  i = (packet_process_buffer[7] >> 6) & 0x03;
  set_channel_bits(13, cs_channel_gps, i);
  cs_channel_gps = i; 
  // Process Disable Tone. Disable Tone is Position 8 bit 0
  i = packet_process_buffer[8] & 0x01;
  set_channel_bits(14, cs_channel_disable_tone, i);
  cs_channel_disable_tone = i;    
  // Process Talk Around. Talk Around is Position 8 bit 1
  i = (packet_process_buffer[8] >> 1) & 0x01;
  set_channel_bits(15, cs_channel_talk_around, i);
  cs_channel_talk_around = i;  
  // Process Monitor. Monitor is Position 8 bit 2
  i = (packet_process_buffer[8] >> 2) & 0x01;
  set_channel_bits(16, cs_channel_monitor, i);
  cs_channel_monitor = i;
  // Process Channel Type. Channel Type is Position 8 bits 3-4
  i = (packet_process_buffer[8] >> 3) & 0x03;
  set_channel_bits(17, cs_channel_type, i);
  cs_channel_type = i; 
  // Process Channel Mode. Channel Mode is Position 8 bit 5
  i = (packet_process_buffer[8] >> 5) & 0x01;
  set_channel_bits(18, cs_channel_mode, i);
  cs_channel_mode = i;
  // Process Emergency Mode. Emergency Mode is Position 8 bits 6 -7
  i = (packet_process_buffer[8] >> 5) & 0x03;
  set_channel_bits(19, cs_channel_emergency, i);
  cs_channel_emergency = i;    
  // Process BT State. BT state is Position 9 bits 0 - 1
  i = packet_process_buffer[9] & 0x03;
  set_channel_bits(20, cs_channel_bt_state, i); 
  cs_channel_bt_state = i; 
  // Process Link State. Link State is Position 9 bit 2
  i = (packet_process_buffer[9] >> 2) & 0x01;
  set_channel_bits(21, cs_channel_link, i);
  cs_channel_link = i;   
  // Process Volume Level. Volume Level is Position 9 bits 3 - 7
  i = (packet_process_buffer[9] >> 3) & 0x1F;
  set_channel_bits(22, cs_channel_volume, i);
  cs_channel_volume = i;


  //if(display_mode == 1)
  //{
   // msg1_dump[0] = 0;
   // msg2_dump[0] = 0;
   // strncpy( (char*) cg_line_buffer, "Packet 5  ", 10);  
   // display_debug_p();     
  //} 

  
  // output data to display
  packet_buffer_5_display();

}

//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_6_output
//
// This is used for the start of the menu section of the program. The starting
// location for the message is 8. The index to what type it is at location 7.
//  The index is as follows:  0     Contacts
//                            1     Scan
//                            2     Zone
//                            3     Messages
//                            4     Call Log
//                            5     Utilities
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  A single line is displayed along with the ICONS
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//
// External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_6_msg1[16]           4                 8
//  uint32_t cs_pkt1_6_msg1_length       n/a
//
// Internal variables used
//  uint32_t j
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_6_msg1[16];
uint32_t cs_pkt1_6_msg1_length;

uint8_t cs_menu_page = 0;
uint8_t cs_prev_menu_page = 0;

void packet_buffer_6_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();//
  
  // get menu page (index)
  cs_menu_page = packet_process_buffer[7];
  if(cs_menu_page == 0x70)
  {
    // it's an idle protocol 
    cs_pkt1_6_msg1_length = 0;
  }
  else
  {
    // Extract message
    j = extract_message(8);
    cs_pkt1_6_msg1_length = j;

    // Move data to a fixed variable and indicate the parameter is active
    for (k = 0; k < j; k++) cs_pkt1_6_msg1[k] = packet_process_buffer[8+k];
    set_pkt1_bits(4,1,2);
  }
  
  // Output data to the display.
  packet_buffer_6_display();
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_7_output
//
// This is used for displaying Data Entry Screen 1. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_7_msg1[16]           5                 10
//  uint32_t cs_pkt1_7_msg1_length       n/a
//  uint8_t  cs_pkt1_7_msg2[16]           6                 42
//  uint32_t cs_pkt1_7_msg2_length       n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//

uint8_t  cs_pkt1_7_msg1[16];
uint32_t cs_pkt1_7_msg1_length;
uint8_t  cs_pkt1_7_msg2[16];
uint32_t cs_pkt1_7_msg2_length;

// tab location, or message centered, left aligned ??
//
// 0x03  Contacts, msg1 menu, msg2 menu item
// 0x06  centered
// 0x02  Utilities, msg1 menu, msg2 sub menu item 

uint32_t cs_pkt_display_type;
// should get put into all packect output routines
//   cs_pkt_display_type = packet_process_buffer[5];

void packet_buffer_7_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Exact first message
  j = extract_message(10);
  cs_pkt1_7_msg1_length = j;
  
  cs_pkt_display_type = packet_process_buffer[5];
  
  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_7_msg1[k] = packet_process_buffer[10+k];
  set_pkt1_bits(5,1,2);

  // Exact second message
  j = extract_message(42);
  cs_pkt1_7_msg2_length = j;
  
  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_7_msg2[k] = packet_process_buffer[42+k];
  set_pkt1_bits(6,1,2);

  // Output data to the display.
  packet_buffer_7_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_8_output
//
// This is used for displaying Data Entry Screen 2. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
//  uint8_t  cs_pkt1_8_msg1[16]           7                 10
//  uint32_t cs_pkt1_8_msg1_length       n/a
//  uint8_t  cs_pkt1_8_msg2[16]           8                 42
//  uint32_t cs_pkt1_8_msg2_length       n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//

uint8_t  cs_pkt1_8_msg1[16];
uint32_t cs_pkt1_8_msg1_length;
uint8_t  cs_pkt1_8_msg2[16];
uint32_t cs_pkt1_8_msg2_length;

void packet_buffer_8_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Exact first message
  j = extract_message(10);
  cs_pkt1_8_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_8_msg1[k] = packet_process_buffer[10+k];
  set_pkt1_bits(7,1,2);

  // Exact second message
  j = extract_message(42);
  cs_pkt1_8_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_8_msg2[k] = packet_process_buffer[42+k];
  set_pkt1_bits(8,1,2);

  // Output data to the display.
  packet_buffer_8_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents          packet_buffer_9_output
//
// This is used for displaying Data Entry Screen 3. The first line is left
// justified and the second line is centered. Has a slight variation from
// other similar formats
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  Displays two line of data using Data Entry Screen 1 style
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t)
//
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet   
// External variables used
//  uint8_t  cs_pkt1_9_msg1[16]           9                 10
//  uint32_t cs_pkt1_9_msg1_length       n/a
//  uint8_t  cs_pkt1_9_msg2[16]          10                 42
//  uint32_t cs_pkt1_9_msg2_length       n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_9_msg1[16];
uint32_t cs_pkt1_9_msg1_length;
uint8_t  cs_pkt1_9_msg2[16];
uint32_t cs_pkt1_9_msg2_length;

void packet_buffer_9_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Extract first message
  j = extract_message(10);
  cs_pkt1_9_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_9_msg1[k] = packet_process_buffer[10+k];
  set_pkt1_bits(9,1,2);

  // Extract second message
  j = extract_message(42);
  cs_pkt1_9_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_9_msg2[k] = packet_process_buffer[42+k];
  set_pkt1_bits(10,1,2);

  // Output data to the display.
  packet_buffer_9_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_10_output
//
// This is used for display two lines centered. The first line starts at
// position 6 and the second line starts at position 38
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_10_msg1[16]         11                  6
//  uint32_t cs_pkt1_10_msg1_length      n/a
//  uint8_t  cs_pkt1_10_msg2[16]         12                 38
//  uint32_t cs_pkt1_10_msg2_length      n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_10_msg1[16];
uint32_t cs_pkt1_10_msg1_length;
uint8_t  cs_pkt1_10_msg2[16];
uint32_t cs_pkt1_10_msg2_length;

void packet_buffer_10_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Exact first message
  //j = extract_message(6);
  j = extract_message(6);
  cs_pkt1_10_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  // changed 10+k to 6+k
  for (k = 0; k < j; k++) cs_pkt1_10_msg1[k] = packet_process_buffer[6+k];
  set_pkt1_bits(11,1,2);

  // Exact second message
  j = extract_message(38);
  cs_pkt1_10_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  // changed 42+k to 38+k
  for (k = 0; k < j; k++) cs_pkt1_10_msg2[k] = packet_process_buffer[38+k];
  set_pkt1_bits(12,1,2);

  // Output data to the display.
  packet_buffer_10_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_11_output
//
// This is used for display two lines centered. The first line starts at
// position 10 and the second line starts at position 22
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_11_msg1[16]         13                 10
//  uint32_t cs_pkt1_11_msg1_length      n/a
//  uint8_t  cs_pkt1_11_msg2[16]         14                 (22) now at 42
//  uint32_t cs_pkt1_11_msg2_length      n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_11_msg1[16];
uint32_t cs_pkt1_11_msg1_length;
uint8_t  cs_pkt1_11_msg2[16];
uint32_t cs_pkt1_11_msg2_length;

void packet_buffer_11_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Exact first message
  j = extract_message(10);
  cs_pkt1_11_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_11_msg1[k] = packet_process_buffer[10+k];
  set_pkt1_bits(13,1,2);

  // Exact second message
  // j = extract_message(22);  // changed to 42
  j = extract_message(42);
  cs_pkt1_11_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_11_msg2[k] = packet_process_buffer[42+k];
  set_pkt1_bits(14,1,2);

  // Output data to the display.
  packet_buffer_11_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_12_output
//
// This is used for display two lines centered. The first line starts at
// position 15 and the second line starts at position 47
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint8_t  cs_pkt1_12_msg1[16]         15                 15
//  uint32_t cs_pkt1_12_msg1_length      n/a
//  uint8_t  cs_pkt1_12_msg2[16]         16                 47
//  uint32_t cs_pkt1_12_msg2_length      n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//
uint8_t  cs_pkt1_12_msg1[16];
uint32_t cs_pkt1_12_msg1_length;
uint8_t  cs_pkt1_12_msg2[64];  // messages are longer
uint32_t cs_pkt1_12_msg2_length;

void packet_buffer_12_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Exact first message
  j = extract_message(15);
  cs_pkt1_12_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_12_msg1[k] = packet_process_buffer[15+k];
  set_pkt1_bits(15,1,2);

  // Exact second message
  j = extract_message(47);
  cs_pkt1_12_msg2_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_12_msg2[k] = packet_process_buffer[47+k];
  set_pkt1_bits(16,1,2);

  // debug, dump packet to screen
  // display_packet_dump();

  // Output data to the display.  
  packet_buffer_12_display(); 
}
//
// ****************************************************************************
// April 29, 2018		Design Documents         packet_buffer_13_output
//
// This is used for display two lines centered. The first line starts at
// position 7 and the second line starts at position 40
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//   All parameters provided by the program
//
// Exit Conditions
//  The two lines are displayed and centered
//
// External functions called
//  void lcd_backlight_on(void)
//  uint32_t extract_message(uint32_t
//
// External variables used
//  External variables used          bit_position        byte in  bit
//                            channel_active+channel_new  packet  
//  uint32_t cs_pkt1_13_icon             17                  6    0,1
//  uint8_t  cs_pkt1_13_msg1[16]         18                  7
//  uint32_t cs_pkt1_13_msg1_length      n/a
//  uint8_t  cs_pkt1_13_msg2[16]         19                 40
//  uint32_t cs_pkt1_13_msg2_length      n/a
//
// Internal variables used
//  uint32_t j
//  uint32_t k
//
// Constants
//
// History
//
// Status
//  written
//
uint32_t cs_pkt1_13_icon;
uint8_t  cs_pkt1_13_msg1[16];
uint32_t cs_pkt1_13_msg1_length;
uint8_t  cs_pkt1_13_msg2[16];
uint32_t cs_pkt1_13_msg2_length;

void packet_buffer_13_output(void)
{
  uint32_t j;
  uint32_t k;

  // Turn on the backlight of the LCD display in case it was off
  lcd_backlight_on();

  // Process icons
  cs_pkt1_13_icon = packet_process_buffer[6] & 0x03;
  set_pkt1_bits(17,1,2);  

  // Exact first message
  j = extract_message(7);
  cs_pkt1_13_msg1_length = j;

  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_13_msg1[k] = packet_process_buffer[7+k];
  set_pkt1_bits(18,1,2);

  // Exact second message
  j = extract_message(40);
  cs_pkt1_13_msg2_length = j;
  
  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_13_msg2[k] = packet_process_buffer[40+k];
  set_pkt1_bits(19,1,2);

  // Output data to the display.
  packet_buffer_13_display(); 
}
//
// ****************************************************************************
// August 17, 2018		Design Documents         packet_buffer_14_output
//
//
// Packet length is 187. message starts at 41, 3 bytes for checksum and final B4
// so maybe message length can be up to 143

uint8_t  cs_pkt1_14_msg1[64];  // max message length is not known !!! verify
uint32_t cs_pkt1_14_msg1_length;
uint8_t  cs_pkt1_14_msg2[16];
uint32_t cs_pkt1_14_msg2_length;

void packet_buffer_14_output(void)
{
  int j;
  int k;
  // get number of characters in the message
  // cs_pkt1_14_msg1_length = packet_process_buffer[6];
  // store message in buffer
  // Exact first message
  j = extract_message(9);
  cs_pkt1_14_msg1_length = j;
  
  for (k = 0; k < j; k++) cs_pkt1_14_msg1[k] = packet_process_buffer[9+k];
  set_pkt1_bits(20,1,2);
  
  j = extract_message(41);
  cs_pkt1_14_msg2_length = j;
  
  for (k = 0; k < j; k++) cs_pkt1_14_msg2[k] = packet_process_buffer[41+k];
  // debug
  // display_packet_dump();

  set_pkt1_bits(21,1,2);   
  
  packet_buffer_14_display(); 
}
//
// ****************************************************************************
// August 17, 2018		Design Documents         packet_buffer_15_output
//
// msg1 starts at 8
//

uint8_t  cs_pkt1_15_msg1[64];
uint32_t cs_pkt1_15_msg1_length;
uint8_t  cs_pkt1_15_msg2[16];
uint32_t cs_pkt1_15_msg2_length;

void packet_buffer_15_output(void)
{
  int j;
  int k;
  
  j = extract_message(8);
  cs_pkt1_15_msg1_length = j;
  // Move data to a fixed variable and indicate the parameter is active
  for (k = 0; k < j; k++) cs_pkt1_15_msg1[k] = packet_process_buffer[8 + k];
  set_pkt1_bits(22,1,2);  

  // debug
  //display_packet_dump();
 
  // no msg 2?
  set_pkt1_bits(23,1,2);   
  
  packet_buffer_15_display(); 
}
//
// *****************************************************************************
//
//
//  was set for 16 bytes max. Some messagesa are longer, raised to 64
// 
//
uint32_t extract_message(uint32_t starting_position)
{
  uint32_t i;
  uint32_t j;
  
  j = 0;
  for (i = starting_position; i < starting_position + 64; i++)
  {
    if (packet_process_buffer[i] == 0) break;    
    cg_line_buffer[j++] = packet_process_buffer[i];
  }
  return j;
}
// =============================================================================
//
// This program sets the CS_channel_active and cs_channel_new bits under the
// following circumstances:
// a. The new data just received is different than the old data
// b. The cs_channel_active bit is zero indicating this paramter is not on the
//    screen at all at this time.
//
void set_channel_bits(uint32_t bits, uint32_t old_data, uint32_t new_data)
{
uint32_t i;
  i = 1 << bits;
  
  // if it's the same data, it's been displayed. clear the "new" bit
  if ( old_data == new_data ) cs_channel_new &= ~i;
  
  if ((old_data != new_data) || (!(cs_channel_active & i)))
  {
    cs_channel_active = cs_channel_active |  i;
    cs_channel_new = cs_channel_new | i;
  }
}
//
// This program sets the CS_pkt1_active and cs_pkt1_new bits under the
// following circumstances:
// a. The new data just received is different than the old data
// b. The cs_pkt1_active bit is zero indicating this paramter is not on the
//    screen at all at this time.
//
void set_pkt1_bits(uint32_t bits, uint32_t old_data, uint32_t new_data)
{
uint32_t i;
  i = 1 << bits;
  if ((old_data != new_data) || (!(cs_pkt1_active & i)))
  {
    cs_pkt1_active = cs_pkt1_active |  i;
    cs_pkt1_new = cs_pkt1_new | i;
  }
}
//
// This program sets the CS_pkt2_active and cs_pkt2_new bits under the
// following circumstances:
// a. The new data just received is different than the old data
// b. The cs_pkt2_active bit is zero indicating this paramter is not on the
//    screen at all at this time.
//

// ??? This is never called

void set_pkt2_bits(uint32_t bits, uint32_t old_data, uint32_t new_data)
{
uint32_t i;
  i = 1 << bits;
  if ((old_data != new_data) || (!(cs_pkt2_active & i)))
  {
    cs_pkt2_active = cs_pkt2_active |  i;
    cs_pkt2_new = cs_pkt2_new | i;
  }
}
//
// This program checks if the parameters needs to be displayed at this time.
// It does this by looking at the cs_channel_new bit to determine if there is
// an update needed for this parameter
//
uint32_t check_channel_bits(uint32_t bits)
{
uint32_t i;
  i = 1 << bits;
  if (cs_channel_new & i) return 1;
  return 0;
}
//
// This program checks if the parameters needs to be displayed at this time.
// It does this by looking at the cs_pkt1_new bit to determine if there is
// an update needed for this parameter
//
uint32_t check_pkt1_bits(uint32_t bits)
{
uint32_t i;
  i = 1 << bits;
  if (cs_pkt1_new & i) return 1;
  return 0;
}
//
// This program checks if the parameters needs to be displayed at this time.
// It does this by looking at the cs_pkt2_new bit to determine if there is
// an update needed for this parameter
//

//  ??? this is never called
uint32_t check_pkt2_bits(uint32_t bits)
{
uint32_t i;
  i = 1 << bits;
  if (cs_pkt2_new & i) return 1;
  return 0;
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void  packet_buffer_3_display(void)
{
  if (check_pkt1_bits(0)) display_pkt1_volume();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void  packet_buffer_4_display(void)
{
  if (check_pkt1_bits(2)) display_pkt1_4_msg1();  
  if (check_pkt1_bits(3)) display_pkt1_4_msg2();  
}
// *****************************************************************************
//    Channel_bits
// bit 
//  0   channel_alias
//  1   rx frequency
//  2   tx frequency
//  3   zone alias
//  4   channel number in zone
//  5   Zone Number
//  6   Channel number
//  7   RSSI ICON
//  8   battery level ICON
//  9   Scan ICON
//  10  power level ICON
//  11  privacy ICON
//  12  message ICON
//  13  GPS ICON
//  14  disable tone ICON
//  15  talk around ICON
//  16  monitor ICON
//  17  Channel type ICON
//  18  Channel mode ICON
//  19  emergency ICON
//  20  bt state ICON
//  21  link ICON
//  22  volume ICON

//
// This program outputs the data to the display if there is a change
//
int in_menu = 0;

void packet_buffer_5_display(void)
{
  // refresh screen if it needs
  if((active_screen_num != 1) && display_mode == 0)
  {
    // set rssi to inactive so it gets redrawn
    rssi_icon_isactive = 0;
    cs_channel_new = 0x7FFFFF;
    display_screen( 1 , 1);   
  }
  if((active_screen_num != 5) && display_mode == 1)
  {
    rssi_icon_isactive = 0;
    display_screen( 5 , 1);       
  }
  if(in_menu == 1)
  {
    if(active_screen_num == 5)
    {
      clear_pckt_message( 3 );
      clear_pckt_message( 4 );
    }
    if(active_screen_num == 3)
    {
      cs_channel_new = 0x7FFFFF;
      display_screen( 1 , 1);
    }
    display_channel_alias();
    display_zone_alias();      
    in_menu = 0;
  }

  // for debugging
  // log_event( "packet 5" );

  // Output channel alias to the display. 
  if (check_channel_bits(0))
  {
  
  // packet 5
  // if the list is popped up, clear and redraw the screen
  if(lc_poped_up)
  {
    lc_poped_up = 0;
    display_screen( 1, 1 );       
  }    
    
   display_channel_alias();
   display_zone_alias();
  }
  // Output channel rx frequency to the display
  if (check_channel_bits(1)) display_channel_rx_frequency(); 

  // Output channel tx frequency to the display
  if (check_channel_bits(2)) display_channel_tx_frequency(); 

  // Output zone alias to the display.   
  if (check_channel_bits(3)) display_zone_alias();

  // Output Channel Number in zone to the display.   
  if (check_channel_bits(4)) display_channel_number_in_zone();

  // Output Zone Number
  if (check_channel_bits(5)) display_zone_number();

  // Output Channel number
  if (check_channel_bits(6)) display_channel_number();   

  // Output Channel RSSI ICON
  if (check_channel_bits(7)) display_channel_rssi(); 

  //  Output Channel battery level ICON
  if (check_channel_bits(8)) display_channel_battery();

  //  Output Channel Scan ICON
  if (check_channel_bits(9)) display_channel_scan();

  //  Output Channel power level ICON
  if (check_channel_bits(10)) display_channel_power();   

  //  Output Channel privacy ICON
  if (check_channel_bits(11)) display_channel_privacy();   

  //  Output Channel message ICON
  if (check_channel_bits(12)) display_channel_message();      

  //  Output Channel GPS ICON
  if (check_channel_bits(13)) display_channel_gps();   

  //  Output Channel disable tone ICON 
  if (check_channel_bits(14)) display_channel_disable_tone();      

  //  Output Channel talk around ICON
  if (check_channel_bits(15)) display_channel_talk_around();   

  //  Output Channel monitor ICON 
  if (check_channel_bits(16)) display_channel_monitor();       
  
  //  Output Channel type ICON 
  if (check_channel_bits(17)) display_channel_type();      

  //  Output Channel mode ICON
  if (check_channel_bits(18)) display_channel_mode();   
   
  //  Output Channel emergency ICON
  if (check_channel_bits(19)) display_channel_emergency();      

  //  Output Channel bt state ICON
  if (check_channel_bits(20)) display_channel_bt_state();   

  //  Output Channel link ICON
  if (check_channel_bits(21)) display_channel_link();    
   
  //  Output Channel volume ICON
  if (check_channel_bits(22))
  {
    if((display_mode == 0) || (display_mode == 2))
    {
      display_channel_volume(); 
    }
  }
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void  packet_buffer_6_display(void)
{
  if (check_pkt1_bits(4)) display_pkt1_6_msg1();   
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void  packet_buffer_7_display(void)
{
  if (check_pkt1_bits(5)) display_pkt1_7_msg1();
  if (check_pkt1_bits(6)) display_pkt1_7_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void  packet_buffer_8_display(void)
{
  if (check_pkt1_bits(7)) display_pkt1_8_msg1();
  if (check_pkt1_bits(8)) display_pkt1_8_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_9_display(void)
{
  if (check_pkt1_bits(9)) display_pkt1_9_msg1();
  if (check_pkt1_bits(10)) display_pkt1_9_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_10_display(void)
{
  if (check_pkt1_bits(11)) display_pkt1_10_msg1();
  if (check_pkt1_bits(12)) display_pkt1_10_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_11_display(void)
{
  if (check_pkt1_bits(13)) display_pkt1_11_msg1();
  if (check_pkt1_bits(14)) display_pkt1_11_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_12_display(void)
{
  if (check_pkt1_bits(15)) display_pkt1_12_msg1();
  if (check_pkt1_bits(16)) display_pkt1_12_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_13_display(void)
{
  if (check_pkt1_bits(17)) display_pkt1_13_icon();
  if (check_pkt1_bits(18)) display_pkt1_13_msg1();
  if (check_pkt1_bits(19)) display_pkt1_13_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_14_display(void)
{
  if (check_pkt1_bits(20)) display_pkt1_14_msg1();
  if (check_pkt1_bits(21)) display_pkt1_14_msg2();
}
// *****************************************************************************
//
// This program outputs the data to the display if there is a change
//
void packet_buffer_15_display(void)
{
  if (check_pkt1_bits(22)) display_pkt1_15_msg1();
  if (check_pkt1_bits(23)) display_pkt1_15_msg2();
}
// =============================================================================
// PACKET 3 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output volume to the display
//
void  display_pkt1_volume(void)
{
  // for debugging
  log_event( "packet 3" );
  // Output bar indicating volume value. Position 7 of the packet_process_buffer[]
  // contains the value of the bar and can have a value of 0x01 through 0x20. A
  // value of 0x01 means there is no bar and a value of 0x20 means it is maximum.
  //
  if((display_mode == 0) || (display_mode == 2))
  {
    display_volume_bar(cs_pkt1_volume);
  }
  if(display_mode == 1)
    
  {
    // Transfer data to line buffer
    strncpy( (char*) cg_line_buffer, (char const*) "Radio Volume", 12);       
    // clear previous message
    clear_pckt_message( 3 );
    clear_pckt_message( 4 );
    ssd1963_cg_write_line(12, 3, message1_X , message1_Y, 4);
    
    uint16_t val;
    val = (uint16_t) ((cs_pkt1_volume - 1) * 100 ) / 31;
    
    draw_progress_bar( val, 0, 30, 160, 700, 50, 5, 7, cur_background);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 3  ", 10);
    display_debug_p();
    // ssd1963_cg_write_line( 10, 3, 40 , 240, 4);       
  }  
}
// =============================================================================
// PACKET 4 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output message 1 from packet 4 to the display
//
// what is assigned to P1 
// "selected" after being in the menu 

void display_pkt1_4_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_4_msg1, cs_pkt1_4_msg1_length);
  msg1_dump[cs_pkt1_4_msg1_length] = '\0';  
  msg1_buf[cs_pkt1_4_msg1_length] = '\0'; 
  // for debugging
  log_event( "packet 4.1" );
  
  // debug, dump packet to screen
  // display_packet_dump();
  // dig monitor icon
  if(active_screen_num == 1 || active_screen_num == 5)
  {
    if(strncmp("Dig Monitor Mode", (char const * ) cg_line_buffer, cs_pkt1_4_msg1_length) == 0)
    {
      if(strncmp("On", (char const * ) cs_pkt1_4_msg2, 2) == 0)
      {
        draw_monitor_icon(1);
      }
      else
      {
        draw_monitor_icon(0);        
      }
    }
    // scan icon  
    if(strncmp("Scan", (char const * ) cg_line_buffer, cs_pkt1_4_msg1_length) == 0)
    {
      if(strncmp("On", (char const * ) cs_pkt1_4_msg2, 2) == 0)
      {
        draw_scan_icon(1);
      }
      else
      {
        draw_scan_icon(0);        
      }
    }
  }

  // on menu screens pop up a text box
  // on message 2 only

  if(active_screen_num == 3)
  {
    
    // clear
    clear_rectangle( 20, 200, 250, 100);    
    // draw frame
    draw_frame(20, 200, 250, 100, 5, 
                highlight_red, highlight_green, highlight_blue);     
    // write data
     ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, 30 , 210, 4);
     menu_current_level++;
     // set chain fo return
     menu_chain[menu_current_level] = cur_sub_indx;      
    
  }

  // Output the line to the display
  if(display_mode == 0)
  {
    // clear previous message
    clear_pckt_message( 1 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 4, message1_X , message1_Y, 4);
    pckt_msg1_dirty = 1;
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;    
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {    
    clear_pckt_message( 3 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg1_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;    
    }
    
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 4 to the display
//
void display_pkt1_4_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_4_msg2, cs_pkt1_4_msg2_length);  
  msg2_dump[cs_pkt1_4_msg2_length] = '\0'; 
  msg2_buf[cs_pkt1_4_msg2_length] = '\0';
  
  // for debugging
  log_event( "packet 4.2" );
 
  // Output the line to the display
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    { 
      // clear previous message
      clear_pckt_message( 2 );     
      ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 4, message2_X , message2_Y, 4);
      pckt_msg2_dirty = 1;
    }
    if(active_screen_num == 3)
    { 
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      menu_current_level++;
      info_box_active = 1;
    }
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );     
    ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 3, message2_X , message2_Y, 4);
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 4  ", 10);     
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      // if on main screen and mode 2
      clear_pckt_message( 4 );       
      ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screens, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      // if on menu screen and mode 2
      // ssd1963_cg_write_line(cs_pkt1_4_msg2_length, 3, message2_X , message2_Y, 4);
    }
  }
}
// =============================================================================
// PACKET 5 ROUTINES
// =============================================================================
// *****************************************************************************
//
// Output channel alias to the display
//
void display_channel_alias(void)
{
  //int i;

  strncpy( (char*) cg_line_buffer, (char const*) cs_channel_alias,
                                          cs_channel_alias_length);
  strncpy( (char*) msg1_dump, (char const*) cs_channel_alias, cs_channel_alias_length);
  msg1_dump[cs_channel_alias_length] = '\0';  
  
  // store channel alias to current
  if((display_mode == 0) || (display_mode == 2))
  {  
    clear_rectangle( 170, 250, 410, 75 );
    ssd1963_cg_write_line(cs_channel_alias_length , 2, 170, 265,3);
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 4 );
    ssd1963_cg_write_line(cs_channel_alias_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, "Packet 5  ", 10);  
    display_debug_p();   
  }  
}
// *****************************************************************************
//
// Output channel rx frequency to the display
//
void display_channel_rx_frequency(void)
{
  uint32_t i;
  uint32_t j;

  if((display_mode == 0) || (display_mode == 2))
  {  
    i = sprintf((char*) cg_line_buffer, "Rx=%i", RxFreq2);
    for (j = 20; j > 6 ;j--)                      // add decimal point
    {
      cg_line_buffer[j+1] = cg_line_buffer[j];
    }
    cg_line_buffer[6] = '.';
    ssd1963_cg_write_line( i+1, 4, 180, 210, 5);
  }
  // nothing for old screen
} 
// *****************************************************************************
//
// Output channel tx frequency to the display
//
void display_channel_tx_frequency(void)
{
  uint32_t i;
  uint32_t j;

  if((display_mode == 0) || (display_mode == 2))
  {  
    i = sprintf((char*) cg_line_buffer, "Tx=%i", TxFreq2);
    for (j = 20; j > 6 ;j--)                      // add decimal point
    {
      cg_line_buffer[j+1] = cg_line_buffer[j];
    }
    cg_line_buffer[6] = 0x2e;
    ssd1963_cg_write_line( i+1, 4, 390, 210, 5);
  }
  // nothing for old screen
}
// *****************************************************************************
//
// Output zone alias to the display. 
//  
void display_zone_alias(void)
{
  // int i;
  strncpy( (char*) cg_line_buffer, (char const*) cs_channel_zone_alias,
                                          cs_channel_zone_alias_length);
  strncpy( (char*) msg2_dump, (char const*) cs_channel_zone_alias, cs_channel_zone_alias_length);
  msg2_dump[cs_channel_zone_alias_length] = '\0';
  
  if((display_mode == 0) || (display_mode == 2))
  {  
    // clear previous 
    clear_rectangle( 40 , 130, 710, 48);
    ssd1963_cg_write_line(cs_channel_zone_alias_length, 2, 40 , 130, 4);
    pckt_msg1_dirty = 1;
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );
    ssd1963_cg_write_line(cs_channel_zone_alias_length, 3, message1_X , message1_Y, 4);
    pckt_msg3_dirty = 1;
  }
}
// *****************************************************************************
//
// Output Channel Number in zone to the display. 0 - 16
//  
void display_channel_number_in_zone(void)
{
  uint32_t i;
  if((display_mode == 0) || (display_mode == 2))
  {  
    i = cs_channel_number_in_zone;
    if (i > 9)
    {
      cg_line_buffer[0] = 0x31;
      i = i - 10;
    }
    else
    {
          cg_line_buffer[0] = 0x30;
    }
    cg_line_buffer[1] = 0x30 + i;
    // clear previous 
    clear_rectangle( 40 , 255, 85, 55);
    ssd1963_cg_write_line( 2, 1, 40, 255, 5);
  }
}
// *****************************************************************************
//
// Output Zone Number
//
void display_zone_number(void)
{
  if((display_mode == 0) || (display_mode == 2))
  {  
    int i;
    i = sprintf((char*) cg_line_buffer, "Zone=%i", cs_channel_zone_number);
    ssd1963_cg_write_line( i, 4, 45, 110, 5);
  }
}
// *****************************************************************************
//
// Output Channel number, 0 - 4000
//
void display_channel_number(void)
{
  int i;
  if((display_mode == 0) || (display_mode == 2))
  {  
    i = sprintf((char*) cg_line_buffer, "ch=%i", cs_channel_number);
    ssd1963_cg_write_line( i, 4, 30, 210, 5);
  }
  
}
// *****************************************************************************
//
// Output Channel RSSI ICON
//
void display_channel_rssi(void)
{
  draw_rssi_icon(cs_channel_rssi);
} 
// *****************************************************************************
//
//  Output Channel battery level ICON.
// 
void display_channel_battery(void)
{
//
// There are no batteries in a mobile radio so this routine does nothing.
//
}
// *****************************************************************************
//
//  Output Channel Scan ICON
// 
void display_channel_scan(void)
{
  draw_scan_icon(cs_channel_scan);
}
// *****************************************************************************
//
//  Output Channel power level ICON
// 
void display_channel_power(void)
{
  display_tx_power(cs_channel_power_level, 0, 0);
}  
// *****************************************************************************
//
//  Output Channel privacy ICON
// 
void display_channel_privacy(void)
{
  draw_lock_icon(cs_channel_privacy);
}  
// *****************************************************************************
//
//  Output Channel message ICON
// 
void display_channel_message(void)
{
 draw_envelope_icon(cs_channel_message);
}  
// *****************************************************************************
//
//  Output Channel GPS ICON
// 
void display_channel_gps(void)
{
  if (cs_channel_gps)
  {
    draw_gps();
  }
}  
// *****************************************************************************
//
//  Output Channel disable tone ICON
// 
void display_channel_disable_tone(void)
{
  draw_tone_disable_icon(cs_channel_disable_tone);
}  
// *****************************************************************************
//
//  Output Channel talk around ICON
// 
void display_channel_talk_around(void)
{
  update_talk_around_icon(cs_channel_talk_around);
}  
// *****************************************************************************
//
//  Output Channel monitor ICON
// 

void display_channel_monitor(void)
{
  draw_monitor_icon(cs_channel_monitor);
} 
// *****************************************************************************
//
//  Output Channel type ICON
// 
void display_channel_type(void)
{  
  //uint8_t *message;
  //uint8_t i;

  if((display_mode == 0) || (display_mode == 2))
  {  
    if (cs_channel_type == 0)
    {
      strncpy( (char*) cg_line_buffer, "DMR", 3);  
      // clear previous type
      draw_rectangle( 630 , 215, 150, 50,
                   cg_background_red, cg_background_green, cg_background_blue); 
      ssd1963_cg_write_line( 3, 2, 630, 215, 4);
    }
    else
    {
      strncpy( (char*) cg_line_buffer, "ANALOG", 6);  
      // clear previous type
      draw_rectangle( 630 , 215, 150, 50,
                   cg_background_red, cg_background_green, cg_background_blue);     
      ssd1963_cg_write_line( 6, 3, 630, 220, 2);
    }
  }
} 
// *****************************************************************************
//
//  Output Channel mode ICON
// 
void display_channel_mode(void)
{
}  
// *****************************************************************************
//
//  Output Channel emergency tone ICON
// 
void display_channel_emergency(void)
{
//  uint32_t cs_channel_emergency
}    
// *****************************************************************************
//
//  Output Channel bt state ICON
// 
void display_channel_bt_state(void)
{
}  
// *****************************************************************************
//
//  Output Channel link ICON
// 
void display_channel_link(void)
{
}  
// *****************************************************************************
//
//  Output Channel volume ICON
// 
void display_channel_volume(void)
{
  if((display_mode == 0) || (display_mode == 2))
  {    
    if (cs_channel_volume == 0) 
    {
      display_volume_bar(0x20);
    }
    else
    {
      display_volume_bar(cs_channel_volume); 
    }
  }
}
// =============================================================================
// PACKET 6 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 6 to the display
//
//
void display_pkt1_6_msg1(void)
{
  // Transfer data to line buffer

  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
  msg1_dump[cs_pkt1_6_msg1_length] = '\0';  
  //
  // Output the line to the display
  //
  // debug, dump packet to screen
  // display_packet_dump();  
  // for debugging
  log_event( "packet 6.1" );
  
  int nav_key = get_nav_key();
  in_menu = 1;
  
  if(display_mode == 0)
  {
    display_menu( cs_menu_page, MAIN, (char * )cs_pkt1_6_msg1,
                    cs_pkt1_6_msg1_length, "" , 0);

    if(nav_key == CANCEL) clear_menupage( 0 );    

    // restore text out buffer (display menu changed it)
    strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
    // clear previous message
    // clear_pckt_message( 1 );    
    // ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 4, message1_X , message1_Y, 4);   

  }
  if(display_mode == 1)
  {
    // ?? no packet 6 message 2?
    // so clear both
    clear_pckt_message( 3 );    
    clear_pckt_message( 4 );
    if( cs_menu_page != 0x70)
    {    
      ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 3,
                          message1_X + (cs_menu_page * 118) , message1_Y, 4);
    }
    pckt_msg3_dirty = 1;
    cs_channel_new = 0x7FFFFF;
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 6  ", 10);  
    display_debug_p();
  }
  if(display_mode == 2)
  { 
    clear_rectangle( 10, 100, 780, 140);
    display_menu( cs_menu_page, MAIN, (char * )cs_pkt1_6_msg1,
                    cs_pkt1_6_msg1_length, "" , 0);      
    // deactivate list
    if( list_active )
    {
      list_active = 0;
      clear_menupage( 0 ); 
    }
    info_box_active = 0;

    // restore text out buffer (display menu changed it)
    //strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_6_msg1, cs_pkt1_6_msg1_length);
    // clear previous message
    //clear_pckt_message( 1 );    
    //ssd1963_cg_write_line(cs_pkt1_6_msg1_length, 4, message1_X , message1_Y, 4);  
  }  
}

// =============================================================================
// PACKET 7 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 7 to the display
//
void display_pkt1_7_msg1(void)
{
  // int nav_key;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg1, cs_pkt1_7_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_7_msg1, cs_pkt1_7_msg1_length);
  msg1_dump[cs_pkt1_7_msg1_length] = '\0';     
  // debug, dump packet to screen
  //display_packet_dump();   
  // for debugging
  log_event( "packet 7.1" );  

  in_menu = 1;

  // Output the line to the display
  if(display_mode == 0)
  {    
    // clear previous message
    clear_pckt_message( 1 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  { 
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg1_length, 3, message1_X , message1_Y, 4);
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 7 to the display
//
//

uint8_t cur_menu_txt[16];

uint8_t sub_menu_ID = 0;

void display_pkt1_7_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
  strncpy( (char*) cur_menu_txt, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);  
  msg2_dump[cs_pkt1_7_msg2_length] = '\0'; 
  
  // for debugging
  log_event( "packet 7.2" );
  
  in_menu = 1;
  
  if(display_mode == 0)
  {
    display_menu( cs_menu_page, SUB, (char * )cs_pkt1_7_msg1, cs_pkt1_7_msg1_length,
               (char * )cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);
    
    // restore text out buffer (display menu changed it)
    strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_7_msg2, cs_pkt1_7_msg2_length);    
    // leave for debugging
    clear_pckt_message( 2 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 4, message2_X , message2_Y, 4);     
  }  
  
  if(display_mode == 1)
  { 
    clear_pckt_message( 4 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 7  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    // deactivate list
    if( list_active )
    {
      list_active = 0;
      clear_menupage( 0 ); 
    }    
    // clear popped up infobox
    clear_rectangle( info_box_loc[cs_menu_page].x_loc,
                     info_box_loc[cs_menu_page].y_loc,
                     info_box_loc[cs_menu_page].width,
                     info_box_loc[cs_menu_page].height );
    info_box_active = 0;
    // draw submenu frame
    draw_shaded_frame( 10, 100, 780, 140, 5, border_red, border_green, border_blue,
                      highlight_red, highlight_green, highlight_blue);
    
    draw_navbuttons( 1 );    
    clear_pckt_message( 4 );    
    ssd1963_cg_write_line(cs_pkt1_7_msg2_length, 3, message2_X , message2_Y, 4);
  }  
}
// =============================================================================
// PACKET 8 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 8 to the display
//
// menu action selection

void display_pkt1_8_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_8_msg1, cs_pkt1_8_msg1_length);
  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_8_msg1, cs_pkt1_8_msg1_length);
  msg1_dump[cs_pkt1_8_msg1_length] = '\0'; 
  // for debugging
  log_event( "packet 8.1" );
  // display_packet_dump();

  // Output the line to the display
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 4, message1_X , message1_Y, 4);
    }
    // on menu screens pop up an action selector box
    if(active_screen_num == 3)
    {
      // display text popup
      // draw frame
      draw_frame(20, 200, 285, 100, 5, 
                  highlight_red, highlight_green, highlight_blue);     
      // write data
       ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, 30 , 210, 4);   
    }    
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );    
    ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 4, message1_X , message1_Y, 4);
    }
    if(active_screen_num == 3)
    {
      clear_pckt_message( 3 );    
      ssd1963_cg_write_line(cs_pkt1_8_msg1_length, 3, message1_X , message1_Y, 4);  
    }        
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 8 to the display
//
void display_pkt1_8_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_8_msg2, cs_pkt1_8_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_8_msg2, cs_pkt1_8_msg2_length);
  msg2_dump[cs_pkt1_8_msg2_length] = '\0';  
  // for debugging
  log_event( "packet 8.2" );
   
  // Output the line to the display
  if(display_mode == 0)
  {     
    if(active_screen_num == 3)
    {
      // draw frame
      draw_frame(20, 200, 285, 100, 5, 
                  highlight_red, highlight_green, highlight_blue);     
      // write data
       ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, 30 , 250, 4);
       menu_current_level++;
    }  
    clear_pckt_message( 2 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 4, message2_X , message2_Y, 4);
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, message2_X , message2_Y, 4);

    strncpy( (char*) cg_line_buffer, (char const*) "Packet 8  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_8_msg2_length, 3, message2_X , message2_Y, 4);
   }
}
// =============================================================================
// PACKET 9 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 9 to the display
//
// radio information data from the menu

void display_pkt1_9_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_9_msg1, cs_pkt1_9_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_9_msg1, cs_pkt1_9_msg1_length);
  msg1_buf[cs_pkt1_9_msg1_length] = '\0';   
  // for debugging
  log_event( "packet 9.1" );   
  //
  // Output the line to the display
  
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 4, message1_X , message1_Y, 4);  
    }
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {  
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 4, message1_X , message1_Y, 4);  
    }    
    if(active_screen_num == 3)
    {
      //clear_pckt_message( 3 );   
      //ssd1963_cg_write_line(cs_pkt1_9_msg1_length, 3, message1_X , message1_Y, 4);  
    }    
  }
  
}
// *****************************************************************************
//
// Output message 2 from packet 9 to the display
//
void display_pkt1_9_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_9_msg2, cs_pkt1_9_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_9_msg2, cs_pkt1_9_msg2_length);
  msg2_buf[cs_pkt1_9_msg2_length] = '\0';    
  // for debugging
  log_event( "packet 9.2" );
  
  //
  // Output the line to the display
  //
 
  if(display_mode == 0)
  {
    if(active_screen_num == 1)
    {
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 4, message2_X , message2_Y, 4);  
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      // menu_current_level++;     
    }
  }
  if(display_mode == 1)
  { 
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 9  ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
 
      // ssd1963_cg_write_line(cs_pkt1_9_msg2_length, 3, message2_X , message2_Y, 4);
    }
  }
}
// =============================================================================
// PACKET 10 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 10 to the display
//
//  displays startup message and "powering down"

void display_pkt1_10_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_10_msg1, cs_pkt1_10_msg1_length);    
  // for debugging
  log_event( "packet 10.1" );
    // debug, dump packet to screen
  // display_packet_dump();
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {    
    display_POM_message( 1, cs_pkt1_10_msg1_length);
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_10_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {
      display_POM_message( 1, cs_pkt1_10_msg1_length);      
    }
    // check to see if it occurs during the menu screen
    if(active_screen_num == 3)
    {
      clear_pckt_message( 3 );   
      ssd1963_cg_write_line(cs_pkt1_10_msg1_length, 3, message1_X , message1_Y, 4);
    }
  }
}
// *****************************************************************************
//
// Output message 2 from packet 10 to the display
//
void display_pkt1_10_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_10_msg2, cs_pkt1_10_msg2_length);    
  // for debugging
  log_event( "packet 10.2" );   
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {  
    display_POM_message( 2, cs_pkt1_10_msg2_length);  
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_10_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 10 ", 10);
    display_debug_p();    
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      display_POM_message( 2, cs_pkt1_10_msg2_length);  
    }
    if(active_screen_num == 3)
    {
      clear_pckt_message( 4 );   
      ssd1963_cg_write_line(cs_pkt1_10_msg2_length, 3, message2_X , message2_Y, 4);      
    }
  }  
}
// =============================================================================
// PACKET 11 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 11 to the display
//
// Data entry or submenu selection in the menu
// comes up on manual dial in the contacts menu
// comes up on menu scan, viee/edit/list 


void display_pkt1_11_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_11_msg1, cs_pkt1_11_msg1_length);
  msg1_buf[cs_pkt1_11_msg1_length] = '\0';
  msg1_dump[cs_pkt1_11_msg1_length] = '\0';  
  // for debugging
  log_event( "packet 11.1" );   
  
  // display_packet_dump();

  // Output the line to the display
  //
  if(display_mode == 0)
  {  
    clear_pckt_message( 1 ); 
    ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  {  
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  { 
    if(active_screen_num == 1)
    {
      clear_pckt_message( 1);   
      ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 4, message1_X , message1_Y, 4);
    }
    if(active_screen_num == 3)
    {
      //ssd1963_cg_write_line(cs_pkt1_11_msg1_length, 3, message1_X , message1_Y, 4);
    }
  }  
}
// *****************************************************************************
//
// Output message 2 from packet 11 to the display
//
void display_pkt1_11_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_11_msg2, cs_pkt1_11_msg2_length);
  msg2_buf[cs_pkt1_11_msg2_length] = '\0'; 
  msg2_dump[cs_pkt1_11_msg2_length] = '\0';    
  // for debugging
  log_event( "packet 11.2" ); 
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {
    clear_pckt_message( 2 );  
    ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 4, message2_X , message2_Y, 4);  
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 3, message2_X , message2_Y, 4);

    strncpy( (char*) cg_line_buffer, (char const*) "Packet 11 ", 10);
    display_debug_p();
    
    // debug
    // display_packet_dump();
    
  }
  if(display_mode == 2)
  {
    if(active_screen_num == 1)
    {    
      clear_pckt_message( 2 );   
      ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 4, message2_X , message2_Y, 4);
    }
    if(active_screen_num == 3)
    {
      // on menu screen, pop up an info box
      display_text_box( info_box_loc[cs_menu_page].x_loc,
                        info_box_loc[cs_menu_page].y_loc,
                        info_box_loc[cs_menu_page].width,
                        info_box_loc[cs_menu_page].height );
      info_box_active = 1;
      draw_navbuttons( 1 ); 
      //clear_pckt_message( 4 );   
      //ssd1963_cg_write_line(cs_pkt1_11_msg2_length, 3, message2_X , message2_Y, 4);      
    }
  }  
}
// =============================================================================
// PACKET 12 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output message 1 from packet 12 to the display
//
//  Sends menu items that can be displayed in a list.
//  displays CTC for contact menu and name of current contact in msg2
//  displays Zone for zone menu and name of zone in msg2

uint8_t list_active = 0;

void display_pkt1_12_msg1(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_12_msg1, cs_pkt1_12_msg1_length);
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_12_msg1, cs_pkt1_12_msg1_length);
  msg1_dump[ cs_pkt1_12_msg1_length] = '\0';   
  // for debugging
  log_event( "packet 12.1" );     
  //
  // Output the line to the display
  //
  if(display_mode == 0)
  {
    clear_pckt_message( 1 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg1_length, 4, message1_X , message1_Y, 4);  
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 3 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    // info displayed with list in msg2.  
  }
}
// *****************************************************************************
//
// Output message 2 from packet 12 to the display
//
//  Messages are list items of the active tab
//   contacts and zones, messages, 
//

void display_pkt1_12_msg2(void)
{
  uint8_t updn = 2;
  int nav_key;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_12_msg2, cs_pkt1_12_msg2_length);    
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_12_msg2, cs_pkt1_12_msg2_length);
  msg2_dump[ cs_pkt1_12_msg2_length] = '\0';  
  
  nav_key = get_nav_key();  

  // set updn by switches
  if( nav_key == UP) updn = 1;
  if( nav_key == DOWN) updn = 0;
  if( nav_key == SELECT) updn = 0;
  
  if(display_mode == 0)
  {
    list_xloc = 30; 
    list_ytop = 105;  
    
    if(cs_menu_page == 0) list_xloc = 30;  // contacts
    if(cs_menu_page == 2) list_xloc = screen_tab[2].x_loc;  // zones
    if(cs_menu_page == 3)
    {
      // for lists on messages tab
      list_xloc = 30;
      menu_current_level++;
    }
    draw_list( list_xloc, list_ytop, cs_pkt1_12_msg2_length, updn ); 
    //clear_pckt_message( 2 );   
    //ssd1963_cg_write_line(cs_pkt1_12_msg2_length, 4, message2_X , message2_Y, 4);
  }
  if(display_mode == 1)
  {
    clear_pckt_message( 4 );   
    ssd1963_cg_write_line(cs_pkt1_12_msg2_length, 3, message2_X , message2_Y, 4);
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 12 ", 10);       
    display_debug_p();    
  }
  if(display_mode == 2)
  {
     // clear popped up infobox
    if(info_box_active)
    {
      clear_rectangle( info_box_loc[cs_menu_page].x_loc,
                       info_box_loc[cs_menu_page].y_loc,
                       info_box_loc[cs_menu_page].width,
                       info_box_loc[cs_menu_page].height );
      info_box_active = 0;
    }
    list_xloc = 30; 
    list_ytop = 115;  
    
    if(cs_menu_page == 0) list_xloc = 30;  // contacts
    if(cs_menu_page == 2) list_xloc = screen_tab[2].x_loc;  // zones
    if(cs_menu_page == 3)
    {
      // for lists on messages tab
      list_xloc = 30;
      menu_current_level++;
    }
    // clear frame at start of list
    if(!list_active)
    {
      // clear list data
      clear_scroll_list();      
      clear_rectangle( 10, 100, 780, 140 );
      list_active = 1;
    }
    draw_list( list_xloc, list_ytop, cs_pkt1_12_msg2_length, updn ); 
    draw_navbuttons( 2 ); 
  }
  
}
// =============================================================================
// PACKET 13 ROUTINES
// =============================================================================
// *****************************************************************************
//
//
// Output ICON from packet 13 to the display
//
//   2 bits of byte 6
// 0 private
// 1 group
// 2 all
void display_pkt1_13_icon(void)
{
  draw_call_icon( cs_pkt1_13_msg1[6] & 0x03 );
  // debug
  // if((cs_pkt1_13_msg1[6] & 0x03) > 1) display_packet_dump();
}
//
// Output message 1 from packet 13 to the display
//
//  displays DMR messages
//
// array of recent contacts
uint8_t last_call_recs = 10;
uint8_t last_call_indx = 0;
Last_call_info Last_call_table[10];
uint8_t prev_pkt13_msg1[16];
uint8_t lc_new = 0;

void display_pkt1_13_msg1(void)
{
  uint32_t i;
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length);     
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length);
  msg1_dump[ cs_pkt1_13_msg1_length] = '\0';
  
  lc_new = 0;
  if( strncmp( (char const * ) Last_call_table[last_call_indx].text,
                (char const * ) cs_pkt1_13_msg1, cs_pkt1_13_msg1_length) != 0)
  {
    lc_new = 1;
  }
  // for debugging
  log_event( "packet 13.1" );
  
  if(lc_new && (cs_pkt1_13_msg1_length > 0))
  {
    cur_call_active = 1;
    last_call_indx++;
    if(last_call_indx > last_call_recs-1) last_call_indx = 0;                  
    
    Last_call_table[last_call_indx].channel = cs_channel_number;
      
    for (i = 0; i < cs_pkt1_13_msg1_length; i++)
    {
      Last_call_table[last_call_indx].text[i] = cs_pkt1_13_msg1[i];
    }
    // put a null terminator
    Last_call_table[last_call_indx].text[cs_pkt1_13_msg1_length] = '\0';
    Last_call_table[last_call_indx].has_data = 1;
    
    if(display_mode == 0)
    {
      // Output the line to the display, larger size for this contact info
      // also clear msg 1 because the larger size overlaps
      if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );
      // clear this message space
      clear_rectangle(  30 , 375, 280, 35 );
      ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 3, 30 , 375, 4);
      display_lastcall_icon();
    }
    if(display_mode == 1)
    {
      if(pckt_msg3_dirty == 1) clear_pckt_message( 3 );
      ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 3, message1_X , message1_Y, 4);
      pckt_msg3_dirty = 1;
    }
    if(display_mode == 2)
    {
      display_lastcall_icon();
      if(pckt_msg3_dirty == 1) clear_pckt_message( 1 );
      ssd1963_cg_write_line(cs_pkt1_13_msg1_length, 4, message1_X , message1_Y, 4);
      pckt_msg3_dirty = 1;
    }    
  }
}
// *****************************************************************************
//
// Output message 2 from packet 13 to the display
//
// packet 13 msg 1 is call sign or name
// packet 13 msg 2 is zone     

uint8_t prev_pkt13_msg2[16];

void display_pkt1_13_msg2(void)
{
  // Transfer data to line buffer
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length); 
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length);
  msg2_dump[ cs_pkt1_13_msg2_length] = '\0';  

  // for debugging
  log_event( "packet 13.2" );   
  // Output the line to the display
  if(cs_pkt1_13_msg2_length > 0)
  {
    if(display_mode == 0)
    {
      if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );  
      ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 4, message2_X , message2_Y, 4);
      pckt_msg2_dirty = 1;
    }
    if(display_mode == 1)
    {
      // check msg 2, and use last_call result from msg 1
      if(strncmp( (char const * ) prev_pkt13_msg2,
                  (char const * ) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length) ||
                  (lc_new == 1))
      { 
        clear_pckt_message( 4 ); 
        ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 3, message2_X , message2_Y, 4);    
        strncpy( (char*) cg_line_buffer, (char const*) "Packet 13 ", 10);
        display_debug_p();
      }
    }
    if(display_mode == 2)
    {
      // if it has changed
      if(strncmp( (char const * ) prev_pkt13_msg2,
                  (char const * ) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length) ||
                  (lc_new == 1))
      {
        clear_pckt_message( 2 ); 
        ssd1963_cg_write_line(cs_pkt1_13_msg2_length, 4, message2_X , message2_Y, 4);    
      }   
    }
    // save message for next compare
    strncpy( (char*) prev_pkt13_msg2, (char const*) cs_pkt1_13_msg2, cs_pkt1_13_msg2_length);    
  }  
  
}
// *****************************************************************************
//
// Output message 1 from packet 14 to the display
//
uint8_t prev_pkt14_msg1[16];

void display_pkt1_14_msg1(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);
  msg1_dump[ cs_pkt1_14_msg1_length] = '\0';
  msg1_buf[ cs_pkt1_14_msg1_length] = '\0';

   if(display_mode == 0)
  {
    // better to pop up a text box in this mode
    if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );      
    ssd1963_cg_write_line(cs_pkt1_14_msg1_length, 4, message1_X , message1_Y, 4);    
    pckt_msg1_dirty = 1;
  }   
  
  if(display_mode == 1)
  {
    // if not the same as the last
    if(strncmp((char const * ) prev_pkt14_msg1,
               (char const * ) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length))
    {
      clear_pckt_message( 3 ); 
    }
    ssd1963_cg_write_line(cs_pkt1_14_msg1_length, 3, message1_X , message1_Y, 4);
  }
  if(display_mode == 2)
  {
    // handled by text box in msg2
  }
  // save message for next compare
  strncpy( (char*) prev_pkt14_msg1, (char const*) cs_pkt1_14_msg1, cs_pkt1_14_msg1_length);   

}
// *****************************************************************************
//
// Output message 2 from packet 14 to the display
//
uint8_t prev_pkt14_msg2[16];

void display_pkt1_14_msg2(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);  
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);
  msg2_dump[ cs_pkt1_14_msg2_length] = '\0';
  msg2_buf[ cs_pkt1_14_msg2_length] = '\0';
  
  if(display_mode == 0)
  {
    // need to pop up a text box in this mode
    if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );      
    ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 4, message2_X , message2_Y, 4);    
    pckt_msg2_dirty = 1;
  }    

  if(display_mode == 1)
  {
    // if not the same as the last
    if(strncmp((char const * ) prev_pkt14_msg2,
               (char const * ) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length))
    {
      clear_pckt_message( 4 );
    }
    ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 3, message2_X , message2_Y, 4); 
    
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 14 ", 10);
    display_debug_p();
     
  }
  
  if(display_mode == 2)
  {
    // text box
    display_text_box( info_box_loc[cs_menu_page].x_loc,
                      info_box_loc[cs_menu_page].y_loc,
                      info_box_loc[cs_menu_page].width,
                      info_box_loc[cs_menu_page].height );
    info_box_active = 1;     
    //clear_pckt_message( 4 ); 
    // ssd1963_cg_write_line(cs_pkt1_14_msg2_length, 3, message2_X , message2_Y, 4);    
  }    
  // debug
  //display_packet_dump();
  // save message for next compare
  strncpy( (char*) prev_pkt14_msg2, (char const*) cs_pkt1_14_msg2, cs_pkt1_14_msg2_length);     
  
  
}
// *****************************************************************************
//
// Output message 1 from packet 15 to the display
//
//
uint8_t prev_pkt15_msg1[16];

void display_pkt1_15_msg1(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);
  strncpy( (char*) msg1_buf, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);  
  strncpy( (char*) msg1_dump, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);
  msg1_dump[ cs_pkt1_15_msg1_length] = '\0';
  msg1_buf[ cs_pkt1_15_msg1_length] = '\0';  

  if(display_mode == 0)
  {
    if(pckt_msg1_dirty == 1) clear_pckt_message( 1 );      
    ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 4, message1_X , message1_Y, 4);    
    pckt_msg1_dirty = 1;
  } 
  
  if(display_mode == 1)
  {
    clear_pckt_message( 3 ); 
    ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 3, message1_X , message1_Y, 4);    
  }  
  if(display_mode == 2)
  {
    // clear_pckt_message( 3 ); 
    // ssd1963_cg_write_line(cs_pkt1_15_msg1_length, 3, message1_X , message1_Y, 4);    
  }  
  // save message for next compare
  strncpy( (char*) prev_pkt15_msg1, (char const*) cs_pkt1_15_msg1, cs_pkt1_15_msg1_length);  
}
// *****************************************************************************
//
// Output message 2 from packet 15 to the display
//
uint8_t prev_pkt15_msg2[16];

void display_pkt1_15_msg2(void)
{
  strncpy( (char*) cg_line_buffer, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length); 
  strncpy( (char*) msg2_buf, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);
  strncpy( (char*) msg2_dump, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);
  msg2_dump[ cs_pkt1_15_msg2_length] = '\0';  
  msg2_buf[ cs_pkt1_15_msg2_length] = '\0';
  
   if(display_mode == 0)
  {
    if(pckt_msg2_dirty == 1) clear_pckt_message( 2 );      
    ssd1963_cg_write_line(cs_pkt1_15_msg2_length, 4, message2_X , message2_Y, 4);    
    pckt_msg2_dirty = 1;
  } 
  if(display_mode == 1)
  {  
    strncpy( (char*) cg_line_buffer, (char const*) "Packet 15 ", 10);
    display_debug_p();
  }
  if(display_mode == 2)
  {
    display_text_box( info_box_loc[cs_menu_page].x_loc,
                      info_box_loc[cs_menu_page].y_loc,
                      info_box_loc[cs_menu_page].width,
                      info_box_loc[cs_menu_page].height );
    info_box_active = 1;   
    
   // clear_pckt_message( 4 ); 
    //ssd1963_cg_write_line(cs_pkt1_15_msg2_length, 3, message2_X , message2_Y, 4);    
  }    
  // debug
  // display_packet_dump();
  // save message for next compare
  strncpy( (char*) prev_pkt15_msg2, (char const*) cs_pkt1_15_msg2, cs_pkt1_15_msg2_length);     
}
*/

/*
// =============================================================================
//
//  clear the message area
//   for message 1, and message 2 
//   location 1 and 2 for screen 1,
//   location 3 and 4 for screen 5 
//
uint8_t pckt_msg1_dirty = 0;
uint8_t pckt_msg2_dirty = 0;
uint8_t pckt_msg3_dirty = 0;
uint8_t pckt_msg4_dirty = 0;

void clear_pckt_message( uint8_t location )
{
  if(location == 1)
  {
    clear_rectangle(  message1_X , message1_Y, 250, 25 );
    pckt_msg1_dirty = 0;
  }
  if(location == 2)
  {
    clear_rectangle(  message2_X , message2_Y, 250, 25 );
    pckt_msg2_dirty = 0;
  }
  if(location == 3)
  {
    clear_rectangle(  message1_X , message1_Y, 740, 50 );
    pckt_msg3_dirty = 0;
  }  
  if(location == 4)
  {
    clear_rectangle(  message2_X , message2_Y, 740, 50);
    pckt_msg4_dirty = 0;
  }  
}
// =============================================================================
//
//   Packet data dump
//    for debugging puts packet data to screen
//    then waits for keypress on right knob
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History 07/26/2018
//
// Status
//  written, working
//
void display_packet_dump( void )
{
  int i;
  int n;
  
  int numbytes = 0;
  int cur_screen = active_screen_num;
  int xloc = 20;
  int yloc = 20;
  
  active_screen_num = 0; // to stay off while waiting
  
  // clear screen
  ssd1963_cg_clear_display();
  // dump packet in hex
  // get num of bytes
  numbytes = packet_process_buffer[2] + 4;
  
  for(i = 0; i < numbytes; i++)
  {
    n = sprintf((char*) cg_line_buffer, "%02X ", packet_process_buffer[i]);
    ssd1963_cg_write_line(n, 4, xloc , yloc, 3);
    xloc += 60;     // next byte
    if( xloc > 600 )  
    {
      xloc =  20;
      yloc += 24;   // next row
    }
  }
  // hang here and wait for a press of the right encoder switch
  while( read_switch_s2() );
  
  // back to last screen
  display_screen( cur_screen , 1);
}

//
// =============================================================================
//
//  Packet info display for debug on screen 5
//

void display_debug_p( void )
{
    // clear space
    clear_rectangle( 10, debug_p_y + debug_p_line * debug_p_step, 710, 25);
    
    ssd1963_cg_write_line( 10, 4, debug_p_x, debug_p_y + debug_p_line * debug_p_step, 2);
    
    strncpy( (char*) cg_line_buffer, (char const*) msg1_dump, 16);
    ssd1963_cg_write_line( 16, 4, 200 , debug_p_y + debug_p_line * debug_p_step, 2);
    
    strncpy( (char*) cg_line_buffer, (char const*) msg2_dump, 16);
    ssd1963_cg_write_line( 16, 4, 420 , debug_p_y + debug_p_line * debug_p_step, 2);    

    // clear marker
    clear_rectangle( 0, 240, 30, 240);

    if(uart2_good_data) ssd1963_cg_output_character(' ', 4, 160, debug_p_y + debug_p_line * debug_p_step, 0);
    else 
    {
      ssd1963_cg_output_character('x', 4, 160, debug_p_y + debug_p_line * debug_p_step, 0);
      //display_packet_dump();
    }
    
    // place new marker
    ssd1963_cg_output_character('>', 3, 10, debug_p_y + debug_p_line++ * debug_p_step - 7, 2);
   
    if(debug_p_line > 7) debug_p_line = 0;
}
*/
