#include "fcb_define.h"
#include "fcb.h"
#include <stdint.h>
#include "CSI_PROGRAMMING.h"
#include "CSI_SSD1963.h"
#include "CSI_eeprom.h"
#include "CSI_MC59.h"
// *****************************************************************************
// NAME OF ROUTINE           Function
//
// parameter_program         Mainline program for parameter programs
// ParameterStates           Controls the various parameter states
// param_state0              Initialize parameters
// param_state1              Set up the parameter programming screen
// param_state2              Wait for first character of parm number sequence
// param_state3              Setup the line for data entry
// param_state4              Enter parameter number
// param_state5              Gets parameter FCB and decides what to do next
// param_state10             Handles whats needed before data entry
// param_state11             Handles data entry for numeridc fields
// param_state12             Handles storing data
// param_state13             Handles post data entry special function
// param_state14             Determines how to exit from FCB processing
// param_state20             Handles whats needed before data entry
// param_state21             Handles data entry for numeridc fields
// param_state22             Handles storing data
// param_state23             Handles post data entry special function
// param_state24             Determines how to exit from FCB processing
// initialize_parameters     Factory initialization of all FCB and messages
// display_line_0            Displays "USER PROGRAMMING" on top line
// display_line_e            Displays "enter parameter number" on bottom line
// array_to_buffer          Moves message to display buffer
// clear_parm_buffer         Clear 32 byte paramater buffer with space character
// load_fcb_from_eeprom      Load FCB from EEPROM memory to FCB array
// set_cursor                Sets the cursor to the specified location
// remove_cursor             Clear the cursor from the current location
// move_cursor_right         Move cursor one position right
// move_cursor_left          Move the cursor one position left
// clear_parm_number_display Clear screen where parameter number is entered
// display_fatal_error       Display on display an irrecoverable error
// call_functions            Call function based on index number
//******************************************************************************
uint32_t ProgramFlag;
uint32_t factory_initialize_flag;

//Variables
uint32_t ParameterState;
uint32_t numeric_flag;
uint32_t pound_counter;
uint32_t ProgramFlag;

uint32_t dtmf_code_flag;
uint32_t dtmf_code_char;
uint32_t first_auto_advance_flag;
uint32_t special_characters;
uint32_t parm_buffer_pointer;
uint32_t parameter_number;
uint32_t parm_buffer[32];
uint16_t load_fcb_buffer[16];

uint16_t opcode;
uint16_t display_line_1;
uint16_t display_line_2;
uint16_t data_entry;
uint16_t display_old;
uint16_t function_begin;
uint16_t function_end;
uint16_t store_data;
uint16_t auto_advance;
uint16_t first_auto_advance;

void dummy_function(void);
void dummy_function(void)
{
}
static uint16_t fcb_array[][16] =
{
// FCB + 0	op code
// FCB + 1      length of field
// FCB + 2	eeprom address
// FCB + 3      horizontal cursor location to display old data
// FCB + 4      vertical cursor location to display old data
// FCB + 5      horizontal cursor location to display line 1
// FCB + 6      vertical cursor location to display line 1
// FCB + 7      display line 1 index into prompt table
// FCB + 8      horizontal cursor location to display line 2
// FCB + 9      vertical cursor location to display line 2
// FCB + 10     display line 2 index into prompt table
// FCB + 11     horizontal cursor location to enter new data
// FCB + 12     vertical cursor location to enter new data
// FCB + 13     minimum value
// FCB + 14     maximum value
// FCB + 15     Subroutine index number
//
// Parameter 1
//  
1,2,3,4,5,6,7,8,910,11,12,13,14,15,16,  
};

static unsigned char system_message[][32] = {
 
"USER PROGRAMMING                ",   // 0
"ENTER PARAMETER NUMBER          ",   // 1 
"Fatal Error  1                  ",   // 2
"Fatal Error  2                  ",   // 3
"Fatal Error  3                  ",   // 4
"Fatal Error  4                  ",   // 5
" PTC NUMBER                     ",   // 6
" VOICE MESSAGES                 ",   // 7
" VOICE PLAYBACK                 ",   // 8
};
//
// The system expects the programmer to set up a FCB (file control block) for
// each parameter that is used. The length of each parameter is 16 bits.
// The definition of the FCB is as shown below:
//
// FCB + 0	op code
// FCB + 1      length of field
// FCB + 2	eeprom address
// FCB + 3      horizontal cursor location to display old data
// FCB + 4      vertical cursor location to display old data
// FCB + 5      horizontal cursor location to display line 1
// FCB + 6      vertical cursor location to display line 1
// FCB + 7      display line 1 index into prompt table
// FCB + 8      horizontal cursor location to display line 2
// FCB + 9      vertical cursor location to display line 2
// FCB + 10     display line 2 index into prompt table
// FCB + 11     horizontal cursor location to enter new data
// FCB + 12     vertical cursor location to enter new data
// FCB + 13     minimum value
// FCB + 14     maximum value
// FCB + 15     Subroutine index number
//
// The following comments should be noted:
// 1. We currently have two op codes. The first opcode is used for entering a
//    field of up to 3 digits with a value between 0 and 999. The second opcode
//    is used for entering ASCII characters.
// 2. The lenght of the field can be between 1 and 3 digits for numeric fields
//    and between 1 and 32 characters for alphanumeric fields
// 3. The eeprom address is based on the 16 bit characters. Being that the
//    EEPROM is normally addressed in bytes, it is necessary to multiply the
//    EEPROM address by two to get an absolute address.
// 4. The old data is the data currently in the EEPROM and is displayed before
//    the user enters the new data. The horizontal starting position to display
//    the data is between 1 and 800 and the starting vertical position to 
//    display the data is between 1 and 480
// 5. The display line index for line 1 and line 2 is an index into a table
//    having all the ascii prompts for this program.
// 6. The minimum and maximum value is not used in the ASCII entry fields.
// 7. The subroutine index number is used by a program that calls various
//    subroutines based on the index number. The subroutines can only be used 
//    before the program starts or after the program ends. The LSB contains the
//    index. If the program uses both before and after the program options, then
//    bits 0 -7 is used for the subroutine before the program beginings and bits
//    8 - 15 is used for the subroutine after the program ends.
//    The OP Code is 16 bits long and here is the defintion of each bit:
//      bits 0 - 3 is the op code. 
//                 Value = 0: 3 digit numeric entry 
//                 Value = 1: Alphanumeric entry
//      bit  4     1 = display line 1
//      bit  5     1 = display line 2
//      bit  6     1 = enter data in field
//      bit  7     1 = display old data
//      bit  8     1 = call subroutine before program starts
//      bit  9     1 = call subrotuine after program ends
//      bit 10     1 = enter data into eeprom after data entered.
//                 0 = enter data in FCB_temp_storage after data is entered
//      bit 11     1 = automatically advance to next FCB on exit of current FCB
//      bit 12     1 = first field of automatic advances
//      bit 13 -15 for future expansion
//
// ****************************************************************************
// When people enter data, they use the following key sequence: *nnn#d....d#
//   * is used to start the sequence and also a reset if entered anyplace else
//
//   nnn is the parameter number
//
//   d....d is the data to be entered. Up to three digits for numeric data and
//   up to 32 characters for alphanumeric data
//
//   # is used as a delimiter between parameter number data and also an end to
//   the data entry.
//
//   If the user enters # without entering any data, then the eeprom is not
//   changed.
//
// The states to process this part of the field is as follows:
//
// state 0: Set watch dog timer
// state 1: House keeping state
// state 2: Get "*"
// state 3: House keeping for "*" character
//
// If its a numeric parameter, the entire sequence for this type of field is
// as follows:
//
// *nnn#ddd# 
//
// The states to process this part of the field is as follows:
//
// state 4: Get first number
// state 5: get second number
// state 6: get third number
// state 7: get "#"
// state 8: process results and transfer to appropriate next state
//


//
// The states to process three digit numeric fields is as follows:
// 
// state 10: Get first number
// state 11: Get send Number
// state 12: Get third number
// state 13: get "#"
// state 14: post process data
//
// ****************************************************************************
// If its an alphanumeric parameter, the entire sequence for this type of 
// field is as follows:
//
// *nnn#d....d#. The maximum lenght of the field is 32 characters.
//
// The states to process this part of the field is as follows:
// 
// State 20: Get character
// State 21: post process data
//
// ****************************************************************************

// ****************************************************************************
//
// The data from the entry is stored in ParmBuffer[] as follows:
// 
// ParmBuffer[0] -  ParmBuffer[2]  = data for numeric fields
// ParmBuffer[0] -  ParmBuffer[31] = data for alphanumeric fields
//
// While the parameter number is being entered, the following is the storage
// for those entries:
//
// ParmBuffer[0] = *
// ParmBuffer[1] = N
// ParmBuffer[2] = N
// ParmBuffer[3] = N
// ParmBuffer[4] = #
//
// This program initializes a few variables and runs the state programs
// until the program is terminated.
//


void parameter_program(void)
{
//
// When the program sets the ProgramFlag to 0, ParameterProgram terminates.
// The Parameter WatchdogTimer is used to make sure a character is entered
// every five minutes or sooner. If it does not the program terminates.
//
  WatchDogTimer = 100;
  ParameterState=0;
  numeric_flag = 0;
  pound_counter = 0;
//
// If ProgramFlag is true then the program executes with DTMF data comming
// from the radio. If the dtmf_code_flag is true, reset the watchdog because it
// means someone pressed the keypad.
//
  while (ProgramFlag)
  {
    if (dtmf_code_flag)
    {
      WatchDogTimer = 10000;
    }
    ParameterStates();
  }
  return;
}
void ParameterStates(void)
{
//
// If the watch dog timer has timed out clear the ProgramFlag to abort from the 
// programming mode. 
//
//  unsigned char i;
  if (!WatchDogTimer)
  {
    ProgramFlag = 0;
    return;
  }
  switch(ParameterState) {
//
// Type of field
//
    case 0:
	  param_state0();
	  break;
    case 1:
	  param_state1();
	  break;
    case 2:
	  param_state2();
	  break;
    case 3:
	  param_state3();
	  break;
    case 4:
	  param_state4();
	  break;
    case 5:
	  param_state5();
	  break;
//
// Numeric entry field
//
    case 10:
	  param_state10();
	  break;
    case 11:
	  param_state11();
	  break;
    case 12:
	  param_state12();
	  break;
    case 13:
	  param_state13();
	  break;
    case 14:
	  param_state14();
	  break;
//
// Alphanumeric field
//
    case 20:
	  param_state20();
	  break;
    case 21:
	  param_state21();
	  break;
    case 22:
	  param_state22();
	  break;
    case 23:
	  param_state23();
	  break;
    case 24:
	  param_state24();
	  break;
//
//
// If we got here there is an error
//
    default:
	  ParameterState = 0;
  }
}
//*****************************************************************************
//                                TYPE OF FIELD
//*****************************************************************************
//
// =============================================================================
//
// State 0: This is a spare state used to initialize parameters
//
// =============================================================================
//
void param_state0(void)
{
  parm_char_flag = 0;
  pound_counter = 0;
  first_auto_advance_flag = 0;
  ParameterState++;  
}
// =============================================================================
//
// State 1: Start the program by clearing the display and display a message on
// the top to indicating in programming mode. Also display a line on the bottom
// for the parameter number for the customer to enter. Set cursor to position 
// where the user is going to enter the parameter number. 
//
// =============================================================================
//
void param_state1(void)
{
  ssd1963_cg_clear_display();
  display_line_0(16);           // "USER PROGRAMMING                "
  display_line_e(1);            // "ENTER PARAMETER NUMBER          ",  
  special_characters = 0;
  parm_char_flag = 0;
  ParameterState++;
}
// =============================================================================
//
// State 2 If there is no characters waiting or if the character is other than 
// a DTMF_STAR, just exit from this routine, else advance to the next state.
//
// =============================================================================
//
void param_state2(void)
{
  unsigned char character;
//
// If there are no characters available, just return.
//
  if (!parm_char_flag)
  {
    return;
  }
  character = parm_char;
  parm_char_flag = 0;
//
//
// If the character is a DTMF_STAR, clear pound character and advance to the 
// next state
//
  if (character == '*')
  {
    pound_counter = 0;
    ParameterState++;
    return;
  }
//
// If there are three #'s in a row, clear the entire display and exit from 
// programming. 
//
  if (character == '#')
  {     
    pound_counter++;
    if (pound_counter >= 3)
     {
       ProgramFlag = 0;
       ssd1963_cg_clear_display();
     }
     return;
  }
//
// For any other character just clear the pound character
//
  pound_counter = 0;
}
// =============================================================================
//
// State 3: Clears line 2 and displays the * in the first position of line 2 
// before advancing to the next state. When we exit from this state the line
// should look like: "ENTER PARAMETER NUMBER *_   " Notice there is a star and
// the cursor is right after the star and there is three spaces including where
// the cursor is.
//
// =============================================================================
//
void param_state3(void)
{
  clear_parm_number_display();
  clear_parm_buffer();
  parm_buffer_pointer = 0;
  cg_next_address = ssd1963_cg_output_character('*',2,400, 300,0 );
  special_characters = 0;
  ParameterState++;
}
// =============================================================================
//
// State 4 is processes the data for all three characters
//
// =============================================================================
//
void param_state4(void)
{
  unsigned char character;
//
// If there are no characters available, just return.
//
  if (!parm_char_flag)
  {
    return;
  }
  character = parm_char;
  parm_char_flag = 0;
//
// If the character received is a star, clear data and initialze variables.
//
  if (character == '*')
  {
    clear_parm_number_display();
    clear_parm_buffer();
    parm_buffer_pointer = 0;
    return;
  }
//
// If the character received is a pound and we have already entered in the three
// numeric digits, then advance to the next state to setup the data entry. Else
// increment # counter. Three #'s in a row and you exit the programming.
//
  if (character == '#')
  {
    if (parm_buffer_pointer == 3)
    {
      ParameterState++;
      return;
    }
    else
    {
      pound_counter++;
      if (pound_counter >= 3)
      {
        ProgramFlag = 0;
        ssd1963_cg_clear_display();
      }
      return;
    }
  }
//
// If we got to this point, then there the character is other than a * or a #.
// Clear the pound counter.
//
  pound_counter = 0;
//
// If ">" character, then move the cursor to the right unless it is already at
// the far right indicated by parm_buffer_pointer having a value of 4
//
  if (character == '>')
  {
    if (parm_buffer_pointer != 3)
    {
      move_cursor_right();
      parm_buffer_pointer++;
    }
    return;
  }
//
// If the "<" character, then move te curosr to the left unless it is already
// at the far left, indicated by parm_buffer_pointer having a value of 0
//
  if (character == '<')
  {
    if (parm_buffer_pointer != 0)
    {
      move_cursor_left();
      parm_buffer_pointer--;
    }
    return;
  }         
//
// if parm_buffer_pointer has a value of 4, all three numeric characters have
// been entered. Ignore any characters until the # is entered.
//
  if (parm_buffer_pointer == 3)
  {
    return;
  }
//
// if valid numeric character, store in buffer 
//
  if ((character > 0x2f) && (character < 0x3A))
  {
    write_char_at_cursor(character);
    parm_buffer[parm_buffer_pointer] = character;
    move_cursor_right();     
    parm_buffer_pointer++;
    ParameterState++;
  }
//
// Any other characters just return
//
}
// =============================================================================
//
// State 5 gets the data from the eeprom and decides what to do next
//
// =============================================================================
//
void param_state5(void)
{
uint16_t i;
//
// Do a ASCII to Binary Conversion
//
  parameter_number = (100*(parm_buffer[0] & 0x0F))+ 
                      (10*(parm_buffer[1] & 0x0F))+ 
                       (1*(parm_buffer[2] & 0x0F));
//
// Get the 16 words of data from the EEPROM and put it in the load_fcb_buffer
//
  load_fcb_from_array(parameter_number);
//
// Break up the OPCODE for future use
//
    i = load_fcb_buffer[0];
    opcode = i & 0x000F;
    display_line_1 = (i>> 4) & 0x0001;
    display_line_2 = (i>> 5) & 0x0001;
    data_entry = (i>> 6) & 0x0001;
    display_old = (i>> 7) & 0x0001;
    function_begin = (i>> 8) & 0x0001;
    function_end = (i>> 9) & 0x0001;
    store_data = (i>> 10) & 0x0001;
    auto_advance = (i>> 11) & 0x0001;
    first_auto_advance = (i>> 12) & 0x0001;
//
// If the op code is for a numeric field set the next state to 10. If the op
// code is for an alphanumeric field set the next state to 20
//
  if (opcode == 0)
  {
    ParameterState = 10;
    return;
  }
  
  if (opcode == 1)
  {
    ParameterState = 20;
    return;
  }
//
// If we got to this point we have an illegal value. Indicate an irrecoverable
// error and stay in loop
//
  display_fatal_error(3);
  while (1){}
}
//
//*****************************************************************************
//                               Numeric Parameter
//
// These states process the data entry for numeric fields
// 
// state 10 Handles the preprocessing for the field
// state 11 enters the data for the field
// state 12 Stores the data
// state 13 Handles the postprocessing for the field
// state 14 Determines what to do on exit from field
//
//*****************************************************************************
//
// =============================================================================
//
// State 10 processes the following preliminary functions before it goes into
// data entry mode:
//     optionally calling special function
//     optionally displaying first line
//     optionally displaying second line
//     optionally skipping data entry
// =============================================================================
//
void param_state10(void)
{
//
// Optionally call a special function before any data entry
//
  if (function_begin)
  {
    call_functions(load_fcb_buffer[15] & 0x00FF);
  }
//
// Optionally display line1
//  
  if (display_line_1)
  {
  }
//
// Optionally display line2
//
  if (display_line_1)
  {
  }
//
// Optionally display the old data still in EEPROM
//  
  if (display_old)
  {
  }
//
// Skip data entry
//  
  if (!data_entry)
  {
    ParameterState = 13;
    return;
  }
//
// Set up cursor and parameters for data entry.
//
  ParameterState++;
}
       
//
// =============================================================================
//
// State 11 is used for processing the three digit numeric field
//
// ============================================================================
//
void param_state11(void)
{
  param_state4();
}
       
//
// =============================================================================
//
// State 12 is used for storing the three digit numeric field. Checks to make
// sure the value is within range.
//
// ============================================================================
//
void param_state12(void)
{
uint16_t value;
uint16_t address;

  value = (100*(parm_buffer[0] & 0x0F))+ 
          (10*(parm_buffer[1] & 0x0F))+ 
          (1*(parm_buffer[2] & 0x0F));
  address = load_fcb_buffer[2];
  if ((value >= load_fcb_buffer[2]) && (value <= load_fcb_buffer[2]))
  {
    eeprom_write(address,value);
    ParameterState++;
  }
  else
  {
    ParameterState = 10;
  }
}

//
// =============================================================================
//
// State 13 is used for optionally calling special function
//
// ============================================================================
// 
void param_state13(void)
{
  if (function_end)
  {
    if (function_begin)
    {
      call_functions((load_fcb_buffer[15]>>8) & 0x00FF);
    }
    else
    {
      call_functions(load_fcb_buffer[15] & 0x00FF);      
    }   
  }
  ParameterState++;
}
//
// =============================================================================
//
// State 14 is used to determine if we automatically advance to the next FCB or
// return to let the user determine what the next field should be.
//
// ============================================================================
// 
void param_state14(void) 
{
  if (!auto_advance)
  {
    ParameterState = 0;
    return;
  }
  if (first_auto_advance)
  {
    first_auto_advance_flag = 1;
  }
  if (!first_auto_advance_flag)
  {
    ParameterState = 0;
    return;
  }
  parameter_number++;
  ParameterState = 10;
}
//
//*****************************************************************************
//                            Alphanumeric Parameter
//
// These states process the data entry for alphanumeric fields
// 
// state 20 Handles the preprocessing for the field
// state 21 enters the data for the field
// state 22 Stores the data
// state 23 Handles the postprocessing for the field
// state 24 Determines what to do on exit from field
//
//*****************************************************************************
//
// =============================================================================
//
// State 20 processes the following preliminary functions before it goes into
// data entry mode:
//     optionally calling special function
//     optionally displaying first line
//     optionally displaying second line
//     optionally skipping data entry
// =============================================================================
void param_state20(void)
{
}
void param_state21(void)
{
}
void param_state22(void)
{
}
void param_state23(void)
{
}
void param_state24(void)
{
}

//
// This program initializes the eeprom for all the data
//
void initialize_parameters(void)
{
unsigned char i;
  for (i = 0; i < GLOBAL_INDEX; i++)
  {
    eeprom_write(GLOBAL_EEPROM + i,global_defaults[i]);    
  }
}
//
// This program displays "USER PROGRAMMING" on the top line of the display.
// This is a special routine for the programming function of the display
//
void display_line_0(uint8_t number_of_characters)
{
//
// Transfer the data from the message array to the fixed message buffer
//
  array_to_buffer(0);
//
// output the data
//
  ssd1963_cg_write_line(number_of_characters,0,50,20, 10); 
}
//
// This program displays "enter paraeterr number" on the bottom line of display.
// This is a special routine for the programming function of the display
//
void display_line_e(uint8_t number_of_characters)
{
//
// Transfer the data from the message array to the fixed message buffer
//
  array_to_buffer(1);
//
// output the data
//
  ssd1963_cg_write_line(number_of_characters,0,50,350, 0); 
}
void array_to_buffer(uint8_t message_number)
{
uint32_t i;
//
// Transfer the data from the message array to the fixed message buffer
//
  for (i = 0; i < 32; i++)
  {
    cg_line_buffer[i] = system_message[message_number][i];
  }
}
void clear_parm_buffer(void)
{
uint32_t i;
//
// clear buffer
//
  for (i = 0; i < 32; i++)
  {
    parm_buffer[i] = ' ';
  }
}
void load_fcb_from_array(uint32_t parameter_number)
{
uint32_t i;
//
// 
//
  parameter_number = parameter_number * 32;
//
// transfer 16 words of data from the FCB Array to load_fcb_buffer
//
  for (i = 0; i < 16; i++)
  {
    load_fcb_buffer[i] = fcb_array[parameter_number][i];
  }
}
//
// Set cursor puts the cursor at the first location of the entry field. It
// should put it at least one bit below the character. The key to the location
// where the line to be displayed is:
//   FCB + 11     horizontal cursor location to enter new data
//   FCB + 12     vertical cursor location to enter new data
// Thhe program that write the line is defined by:
//   ssd1963_cq_write_horizontal_line(cg_hori_location, cg_vert_location,
//                                    cg_hori_size)
void set_cursor(void)
{
  cg_hori_location = load_fcb_buffer[11] + parm_buffer_pointer * 20;
  cg_vert_location = load_fcb_buffer[12] + 15;
  ssd1963_cq_write_horizontal_line(cg_hori_location, cg_vert_location, 50);       
}
void remove_cursor(void)
{
  cg_hori_location = load_fcb_buffer[11] + parm_buffer_pointer * 20;
  cg_vert_location = load_fcb_buffer[12] + 15;
  ssd1963_cq_clear_horizontal_line(cg_hori_location, cg_vert_location, 50); 
}
void move_cursor_right(void)
{
  remove_cursor();
  parm_buffer_pointer++;
  set_cursor();
}
void move_cursor_left(void)
{
  remove_cursor();
  parm_buffer_pointer--;
  set_cursor();  
}
void write_char_at_cursor(uint8_t character)
{
  cg_hori_location = load_fcb_buffer[11] + parm_buffer_pointer * 20;
  cg_vert_location = load_fcb_buffer[12];
  
}
//
//******************************************************************************
//
// March 7, 2018		Design Documents       clear_parm_number_display
//
// This program clears the three digit parameter entry display area. The key
// to the location where the line to be displayed is:
//   FCB + 11     horizontal cursor location to enter new data
//   FCB + 12     vertical cursor location to enter new data
// Being that we know the character set, the height and length of the rectange
// is defined
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters provided by the program
//
// Exit Conditions
//  The section of the display holding the parameter entry is cleared.
// 
// External variables used
//  load_fcb_buffer[16] contains the FCB information needed
// 
// Functions used
//  ssd1963_cg_clear_line
//
// Constants
//
// History
//
// Status
//  unwritten
//
void clear_parm_number_display(void)
{
//
// Besides the horozontal and vertical locations defined in the function call
// below, we assume a height of 50 pixels and a length of 300 pixels.
//
  ssd1963_cg_clear_line(load_fcb_buffer[11],load_fcb_buffer[12],50, 300);
}
//
//******************************************************************************
//
// March 7, 2018		Design Documents             display_fatal_error
//
// This program display an error message on the screen in a specific location
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  message_number is the index to the message array
//
// Exit Conditions
//  The message is displayed on the display
// 
// External variables used
//  None
// 
// Functions used
//  array_to_buffer
//  ssd1963_cg_write_line
//
// Constants
//
// History
//
// Status
//  unwritten
//
void display_fatal_error(uint8_t message_number)
{
//
// transfer message array to fixed location
//  
  array_to_buffer(message_number);
//
// Output message to the display. 32 character, character set 3, horizontal 
// location is 20, vertical location is 400, and use fixed spacing.
//
  ssd1963_cg_write_line(32, 3, 20, 400, 0);
}
//
//******************************************************************************
//
// March 7, 2018		Design Documents                  call_functions
//
// This program calls different functions based on a numeric parameter
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  function_number is the index to call the specific function
//
// Exit Conditions
//  The specified function is executed
// 
// External variables used
//  None
// 
// Functions used
//  dummy_function        // This will evenually be replaced by a real function  
// Constants
//
// History
//
// Status
//  unwritten
//
void call_functions(uint16_t function_number)
{
  if (function_number == 0)
  {
    dummy_function();
    return;
  }
  if (function_number == 1)
  {
    dummy_function();
    return;    
  }
  if (function_number == 2)
  {
    dummy_function();
    return;    
  }
  if (function_number == 3)
  {
    dummy_function();
    return;    
  }
}

