//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// September 25, 2019         Design Documents                       CSI_radio.c
//
//	            routines for multiple radios
//                    Large panel display project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History:
//
// Status
//  written
//
// =============================================================================
#include <stdint.h>
#include "CSI_RADIO.h"

// =============================================================================
//                                                              get_radio_baud()
//   get baud rate register setting for a radio
//
// baud_div is the setting for the baud rate divider register
//  mantissa and fraction combined
//  Based on a 84 MHz clock
// calculation examples:
  // CS800D
  // baud rate 57.6K firmware version 20.42 and up 
  // 84 Mhz / (16 * 57,600 ) = 91.1458  91.125 is close
  // set baud rate register to 91.125
  // 0x5B for 91 mantissa 2/16 = 0.125 for fraction
  // 0x000005B2  
  
  // FT857D
  // baud rate 62.5K 
  // 84 Mhz/ (16 * 62500) = 84 = 0x54  exact
  // 0x54 for mantissa and 0 for fraction
  // 0x00000540
  
  // FT100D 
  // baud rate 96,000
  // 84 Mhz/ (16 * 96000) = 54.6875 = 0x36
  // 11/16 = 0.6875 exact
  // 0x36 for mantissa and 0x0B for fraction
  // 0x0000036B

  // ID4100
  // baud rate 38.4K 
  // 84 Mhz/ (16 * 38400) = 136.71875 = 0x88 
  // 12/16 = 0.75  + .03125    38391
  // 11/16 = 0.6875  -.03125   38408
  // 0x88 for mantissa and B for fraction
  // 0x0000088B

//  History: 09/25/2019 moved here for ID4100 
// 

uint32_t get_radio_baud( uint8_t radio_type )
{
  uint32_t baud_div = 0;
  
  switch( radio_type )
  {
  case CS800D: baud_div = 0x000005B2; break;
  case FT857D: baud_div = 0x00000540; break;
  case FT100:  baud_div = 0x0000036B; break;  
  case ID4100: baud_div = 0x0000088B; break;
  default: baud_div = 0x000005B2; break;
  }
  return baud_div;
}  