//
// Functional offsets for various peripherals
//
// 0x40023C00    Flash interface register
// 0x40007000    Power Controller
// 0x40023800    RCC Reset and Clock Control
// 0x40020000    GPIOA
// 0x40020400    GPIOB
// 0x40020800    GPIOC
// 0X40020C00    GPIOD
// 0x40021000    GPIOE
// 0x40021400    GPIOF
// 0x40021800    GPIOG
// 0x40013800    System Configuration Controller
// 0x40026000    DMA1
// 0x40026400    DMA2
// 0x40013C00    Extended Interrups and Events Controller
// 0xA0001000    Quad-SPI control register
// 0x40012000    ADC 1
// 0x40012100    ADC 2
// 0x40002200    ADC 3
// 0x40002300    ADC Common registers
// 0x40007400    DAC
// 0x40010000    TIMER 1
// 0x40010400    TIMER 8
// 0x40000000    TIMER 2
// 0x40000400    TIMER 3
// 0x40000800    TIMER 4
// 0x40000C00    TIMER 5

// 0x40014000    TIMER 9
// 0x40001800    TIMER 12

// 0x40014400    TIMER 10
// 0x40014800    TIMER 11
// 0x40001C00    TIMER 13
// 0x40002000    TIMER 14

// 0x40001000    TIMER 6
// 0x40001400    TIMER 7

// 0x40002400    Low Power Timer
// 0x40003000    Indendent Watchdog Timer
// 0x40002C00    System window watchdog
// 0x40002800    Real Time Clock


// 0x50000000    USB OTG FS
// 0x40040000    USB OTG HS
// 0x40013000    SPI1
// 0x40003800    SPI2
// 0x40005000    UART5
// 0x40007C00    UART8

// 0xE0000000    Debug Support

//
// Vector Table for Interrupts
//
#define IV_RESERVED_1             0x00000000   // Reserved
#define Reset                     0x00000004   // Reset
#define NMI                       0x00000008   // Non maskable interrupt
#define HardFault                 0x0000000C   // All class of fault
#define MemManage                 0x00000010   // Memory management
#define BusFault                  0x00000014   // Pre-fetch fault, memory access fault
#define UsageFault                0x00000018   // Undefined instruction or illegal state
#define IV_RESERVED_2             0x0000001C   // Reserved
#define IV_RESERVED_3             0x00000020   // Reserved
#define IV_RESERVED_4             0x00000024   // Reserved
#define IV_RESERVED_5             0x00000028   // Reserved
#define SVCall                    0x0000002C   // Reserved
#define DebugMonitor              0x00000030   // Debug Monitor
#define IV_RESERVED_6             0x00000034   // Reserved
#define PendSV                    0x00000038   // Pendable request for system service
#define SysTick                   0x0000003C   // System tick timer
#define WWDG                      0x00000040   // Window Watchdog interrupt
#define PVD                       0x00000044   // PVD through EXTI line detection interrupt
#define TAMP_STAMP                0x00000048   // Tamper and TimeStamp interrupts through the EXTI line
#define RTC_WKUP                  0x0000004C   // RTC Wakeup interrupt through the EXTI line
#define FLASH                     0x00000050   // Flash global interrupt
#define RCC                       0x00000054   // RCC global interrupt
#define EXTI0                     0x00000058   // EXTI Line0 interrupt
#define EXTI1                     0x0000005C   // EXTI Line1 interrupt
#define EXTI2                     0x00000060   // EXTI Line2 interrupt
#define EXTI3                     0x00000064   // EXTI Line3 interrupt
#define EXTI4                     0x00000068   // EXTI Line4 interrupt
#define DMA1_Stream0              0x0000006C   // DMA1 Stream0 global interrupt
#define DMA1_Stream1              0x00000070   // DMA1 Stream1 global interrupt
#define DMA1_Stream2              0x00000074   // DMA1 Stream2 global interrupt
#define DMA1_Stream3              0x00000078   // DMA1 Stream3 global interrupt
#define DMA1_Stream4              0x0000007C   // DMA1 Stream4 global interrupt
#define DMA1_Stream5              0x00000080   // DMA1 Stream5 global interrupt
#define DMA1_Stream6              0x00000084   // DMA1 Stream6 global interrupt
#define ADC                       0x00000088   // ADC1, ADC2 and ADC3 global interrupts
#define CAN1_TX                   0x0000008C   // CAN1 TX interrupts
#define CAN1_RX0                  0x00000090   // CAN1 RX0 interrupts
#define CAN1_RX1                  0x00000094   // CAN1 RX1 interrupt
#define CAN1_SCE                  0x00000098   // CAN1 SCE interrupt
#define EXTI9_5                   0x0000009C   // EXTI Line[9:5] interrupts
#define TIM1_BRK_TIM9             0x000000A0   // TIM1 Break interrupt and TIM9 global interrupt
#define TIM1_UP_TIM10             0x000000A4   // TIM1 Update interrupt and TIM10 global interrupt
#define TIM1_TRG_COM_TIM11        0x000000A8   // TIM1 Trigger and Commutation interrupts and TIM11 global interrupt
#define TIM1_CC                   0x000000AC   // TIM1 Capture Compare interrupt
#define TIM2                      0x000000B0   // TIM2 global interrupt
#define TIM3                      0x000000B4   // TIM3 global interrupt
#define TIM4                      0x000000B8   // TIM4 global interrupt
#define I2C1_EV                   0x000000BC   // I2C1 event interrupt
#define I2C1_ER                   0x000000C0   // I2C1 error interrupt
#define I2C2_EV                   0x000000C4   // I2C2 event interrupt
#define I2C2_ER                   0x000000C8   // I2C2 error interrupt
#define SPI1                      0x000000CC   // SPI1 global interrupt
#define SPI2                      0x000000D0   // SPI2 global interrupt
#define USART1                    0x000000D4   // USART1 global interrupt
#define USART2                    0x000000D8   // USART2 global interrupt
#define USART3                    0x000000DC   // USART3 global interrupt
#define EXTI15_10                 0x000000E0   // EXTI Line[15:10] interrupts
#define RTC_Alarm                 0x000000E4   // RTC Alarms (A and B) through EXTI line interrupt
#define OTG_FS_WKUP               0x000000E8   // USB On-The-Go FS Wakeup through EXTI line interrupt
#define TIM8_BRK_TIM12            0x000000EC   // TIM8 Break interrupt and TIM12 global interrupt
#define TIM8_UP_TIM13             0x000000F0   // TIM8 Update interrupt and TIM13 global interrupt
#define TIM8_TRG_COM_TIM14        0x000000F4   // TIM8 Trigger and Commutation interrupts and TIM14 global interrupt
#define TIM8_CC                   0x000000F8   // TIM8 Capture Compare interrupt
#define DMA1_Stream7              0x000000FC   // DMA1 Stream7 global interrupt
#define FSMC                      0x00000100   // FSMC global interrupt
#define SDIO                      0x00000104   // SDIO global interrupt
#define TIM5                      0x00000108   // TIM5 global interrupt
#define SPI3                      0x0000010C   // SPI3 global interrupt
#define UART4                     0x00000110   // UART4 global interrupt
#define UART5                     0x00000114   // UART5 global interrupt
#define TIM6_DAC                  0x00000118   // TIM6 global interrupt, DAC1 and DAC2 underrun error interrupt
#define TIM7                      0x0000011C   // TIM7 global interrupt
#define DMA2_Stream0              0x00000120   // DMA2 Stream0 global interrupt
#define DMA2_Stream1              0x00000124   // DMA2 Stream1 global interrupt
#define DMA2_Stream2              0x00000128   // DMA2 Stream2 global interrupt
#define DMA2_Stream3              0x0000012C   // DMA2 Stream3 global interrupt
#define DMA2_Stream4              0x00000130   // DMA2 Stream4 global interrupt
#define ETH                       0x00000134   // Ethernet global interrupt
#define ETH_WKUP                  0x00000138   // Ethernet Wakeup through EXTI line interrupt
#define CAN2_TX                   0x0000013C   // CAN2 TX interrupts
#define CAN2_RX0                  0x00000140   // CAN2 RX0 interrupts
#define CAN2_RX1                  0x00000144   // USART3 global interrupt
#define CAN2_SCE                  0x00000148   // USART3 global interrupt
#define OTG_FS                    0x0000014C   // USART3 global interrupt
#define DMA2_Stream5              0x00000150   // USART3 global interrupt
#define DMA2_Stream6              0x00000154   // USART3 global interrupt
#define DMA2_Stream7              0x00000158   // USART3 global interrupt
#define USART6                    0x0000015C   // USART3 global interrupt
#define I2C3_EV                   0x00000160   // USART3 global interrupt
#define I2C3_ER                   0x00000164   // USART3 global interrupt
#define OTG_HS_EP1_OUT            0x00000168   // USART3 global interrupt
#define OTG_HS_EP1_IN             0x0000016C   // USART3 global interrupt
#define OTG_HS_WKUP               0x00000170   // USART3 global interrupt
#define OTG_HS                    0x00000174   // USART3 global interrupt
#define DCMI                      0x00000178   // USART3 global interrupt
#define CRYP                      0x0000017C   // USART3 global interrupt
#define HASH_RNG                  0x00000180   // USART3 global interrupt
#define FPU                       0x00000184   // USART3 global interrupt
//
// Flash Interface Register
//
#define FLASH_ACR_ADDRESS         0x40023C00   // Flash access control register
#define FLASH_KEYR_ADDRESS        0x40023C04   // Flash key register
#define FLASH_OPTKEYR_ADDRESS     0x40023C08   // Flash option key register
#define FLASH_SR_ADDRESS          0x40023C0c   // Flash status register
#define FLASH_CR_ADDRESS          0x40023C10   // Flash control register
#define FLASH_OPTCR_ADDRESS       0x40023C14   // Flash option control register
#define FLASH_OPTCR1_ADDRESS      0x40023C18   // Flash option control register
//
// Power Controller Registers
//
#define PWR_CR_ADDRESS           0x40007000   // PWR Power control registers
#define PWR_CSR_ADDRESS          0x40007004   // PWR power control/status register
//#define PWR_CR2_ADDRESS           0x40007008   // PWR Power control registers
//#define PWR_CSR2_ADDRESS          0x4000700C   // PWR power control/status register
//
// RESET and Clock Control
//
#define RCC_CR_ADDRESS            0x40023800   // RCC Clock COntrol Register
#define RCC_PLLCFGR_ADDRESS       0X40023804   // RCC PLL Configuration register
#define RCC_CFGR_ADDRESS          0X40023808   // RCC clock configuration register
#define RCC_CIR_ADDRESS           0X4002380C   // RCC clock interrupt register
#define RCC_AHB1RSTR_ADDRESS      0X40023810   // RCC AHB1 peripheral reset register
#define RCC_AHB2RSTR_ADDRESS      0X40023814   // RCC AHB2 peripheral reset register
#define RCC_AHB3RSTR_ADDRESS      0X40023818   // RCC AHB3 peripheral reset register
#define RCC_APB1RSTR_ADDRESS      0X40023820   // RCC APB1 peripheral reset register
#define RCC_APB2RSTR_ADDRESS      0X40023824   // RCC APB2 peripheral reset register
#define RCC_AHB1ENR_ADDRESS       0X40023830   // RCC AHB1 peripheral clock enable register
#define RCC_AHB2ENR_ADDRESS       0X40023834   // RCC AHB2 peripheral clock enable register
#define RCC_AHB3ENR_ADDRESS       0X40023838   // RCC AHB3 peripheral clock enable register
#define RCC_APB1ENR_ADDRESS       0X40023840   // RCC APB1 peripheral clock enable register
#define RCC_APB2ENR_ADDRESS       0X40023844   // RCC APB2 peripheral clock enable register
#define RCC_AHB1LPENR_ADDRESS     0X40023850   // RCC AHB1 peripheral clock enable in low power mode register
#define RCC_AHB2LPENR_ADDRESS     0X40023854   // RCC AHB2 peripheral clock enable in low power mode register
#define RCC_AHB3LPENR_ADDRESS     0X40023858   // RCC AHB3 peripheral clock enable in low power mode register
#define RCC_APB1LPENR_ADDRESS     0X40023860   // RCC APB1 peripheral clock enable in low power mode register
#define RCC_APB2LPENR_ADDRESS     0X40023864   // RCC APB2 peripheral clock enable in low power mode register
#define RCC_BDCR_ADDRESS          0X40023870   // RCC Backup domain control register
#define RCC_CSR_ADDRESS           0X40023874   // RCC clock control and status register
#define RCC_SSCGR_ADDRESS         0X40023880   // RCC spread spectrum clock generation register
#define RCC_PLLI2SCFGR_ADDRESS    0X40023884   // RCC PLLI2S configuration register
#define RCC_PLLSAICFGR_ADDRESS    0X40023888   // RCC PLL configuration register
#define RCC_DCKCFGR1_ADDRESS      0X4002388C   // RCC dedicated clocks configuration register
#define RCC_DCKCFGR2_ADDRESS      0X40023890   // RCC dedicated clocks configuration register
//
// GPIOA Addresses
//
#define GPIOA_MODER_ADDRESS       0x40020000   // Port Mode Register
#define GPIOA_OTYPER_ADDRESS      0x40020004   // Output Type Register
#define GPIOA_OSPEEDR_ADDRESS     0x40020008   // Output Speed Register
#define GPIOA_PUPDR_ADDRESS       0x4002000C   // Push Up-Pull Down Register
#define GPIOA_IDR_ADDRESS         0x40020010   // Input Data Register
#define GPIOA_ODR_ADDRESS         0x40020014   // Output Data Register
#define GPIOA_BSRR_ADDRESS        0x40020018   // bit set/reset Register
#define GPIOA_LCKR_ADDRESS        0x4002001C   // Configuration Lock Register
#define GPIOA_AFRL_ADDRESS        0x40020020   // Alternate Function Low Register
#define GPIOA_AFRH_ADDRESS        0x40020024   // Alternate Function High Register
//
// GPIOB Addresses
//
#define GPIOB_MODER_ADDRESS       0x40020400   // Port Mode Register
#define GPIOB_OTYPER_ADDRESS      0x40020404   // Output Type Register
#define GPIOB_OSPEEDR_ADDRESS     0x40020408   // Output Speed Register
#define GPIOB_PUPDR_ADDRESS       0x4002040C   // Push Up-Pull Down Register
#define GPIOB_IDR_ADDRESS         0x40020410   // Input Data Register
#define GPIOB_ODR_ADDRESS         0x40020414   // Output Data Register
#define GPIOB_BSRR_ADDRESS        0x40020418   // bit set/reset Register
#define GPIOB_LCKR_ADDRESS        0x4002041C   // Configuration Lock Register
#define GPIOB_AFRL_ADDRESS        0x40020420   // Alternate Function Low Register
#define GPIOB_AFRH_ADDRESS        0x40020424   // Alternate Function High Register
//
// GPIOC Addresses
//
#define GPIOC_MODER_ADDRESS       0x40020800   // Port Mode Register
#define GPIOC_OTYPER_ADDRESS      0x40020804   // Output Type Register
#define GPIOC_OSPEEDR_ADDRESS     0x40020808   // Output Speed Register
#define GPIOC_PUPDR_ADDRESS       0x4002080C   // Push Up-Pull Down Register
#define GPIOC_IDR_ADDRESS         0x40020810   // Input Data Register
#define GPIOC_ODR_ADDRESS         0x40020814   // Output Data Register
#define GPIOC_BSRR_ADDRESS        0x40020818   // bit set/reset Register
#define GPIOC_LCKR_ADDRESS        0x4002081C   // Configuration Lock Register
#define GPIOC_AFRL_ADDRESS        0x40020820   // Alternate Function Low Register
#define GPIOC_AFRH_ADDRESS        0x40020824   // Alternate Function High Register
//
// GPIOD Addresses
//
#define GPIOD_MODER_ADDRESS       0x40020C00   // Port Mode Register
#define GPIOD_OTYPER_ADDRESS      0x40020C04   // Output Type Register
#define GPIOD_OSPEEDR_ADDRESS     0x40020C08   // Output Speed Register
#define GPIOD_PUPDR_ADDRESS       0x40020C0C   // Push Up-Pull Down Register
#define GPIOD_IDR_ADDRESS         0x40020C10   // Input Data Register
#define GPIOD_ODR_ADDRESS         0x40020C14   // Output Data Register
#define GPIOD_BSRR_ADDRESS        0x40020C18   // bit set/reset Register
#define GPIOD_LCKR_ADDRESS        0x40020C1C   // Configuration Lock Register
#define GPIOD_AFRL_ADDRESS        0x40020C20   // Alternate Function Low Register
#define GPIOD_AFRH_ADDRESS        0x40020C24   // Alternate Function High Register
//
// GPIOE Addresses
//
#define GPIOE_MODER_ADDRESS       0x40021000   // Port Mode Register
#define GPIOE_OTYPER_ADDRESS      0x40021004   // Output Type Register
#define GPIOE_OSPEEDR_ADDRESS     0x40021008   // Output Speed Register
#define GPIOE_PUPDR_ADDRESS       0x4002100C   // Push Up-Pull Down Register
#define GPIOE_IDR_ADDRESS         0x40021010   // Input Data Register
#define GPIOE_ODR_ADDRESS         0x40021014   // Output Data Register
#define GPIOE_BSRR_ADDRESS        0x40021018   // bit set/reset Register
#define GPIOE_LCKR_ADDRESS        0x4002101C   // Configuration Lock Register
#define GPIOE_AFRL_ADDRESS        0x40021020   // Alternate Function Low Register
#define GPIOE_AFRH_ADDRESS        0x40021024   // Alternate Function High Register
//
// GPIOF Addresses
//
#define GPIOF_MODER_ADDRESS       0x40021400   // Port Mode Register
#define GPIOF_OTYPER_ADDRESS      0x40021404   // Output Type Register
#define GPIOF_OSPEEDR_ADDRESS     0x40021408   // Output Speed Register
#define GPIOF_PUPDR_ADDRESS       0x4002140C   // Push Up-Pull Down Register
#define GPIOF_IDR_ADDRESS         0x40021410   // Input Data Register
#define GPIOF_ODR_ADDRESS         0x40021414   // Output Data Register
#define GPIOF_BSRR_ADDRESS        0x40021418   // bit set/reset Register
#define GPIOF_LCKR_ADDRESS        0x4002141C   // Configuration Lock Register
#define GPIOF_AFRL_ADDRESS        0x40021420   // Alternate Function Low Register
#define GPIOF_AFRH_ADDRESS        0x40021424   // Alternate Function High Register
//
// GPIOG addresses
//
#define GPIOG_MODER_ADDRESS       0x40021800   // Port Mode Register
#define GPIOG_OTYPER_ADDRESS      0x40021804   // Output Type Register
#define GPIOG_OSPEEDR_ADDRESS     0x40021808   // Output Speed Register
#define GPIOG_PUPDR_ADDRESS       0x4002180C   // Push Up-Pull Down Register
#define GPIOG_IDR_ADDRESS         0x40021810   // Input Data Register
#define GPIOG_ODR_ADDRESS         0x40021814   // Output Data Register
#define GPIOG_BSRR_ADDRESS        0x40021818   // bit set/reset Register
#define GPIOG_LCKR_ADDRESS        0x4002181C   // Configuration Lock Register
#define GPIOG_AFRL_ADDRESS        0x40021820   // Alternate Function Low Register
#define GPIOG_AFRH_ADDRESS        0x40021824   // Alternate Function High Register
//
//
// System Configuration Controller
//
#define SYSCFG_MEMRMP_ADDRESS     0x40013800   // SYSCFG memory remap register
#define SYSCFG_PMC_ADDRESS        0x40013804   // SYSCFG peripheral mode configuration register
#define SYSCFG_EXTICR1_ADDRESS    0x40013808   // SYSCFG external interript configuration register 1
#define SYSCFG_EXTICR2_ADDRESS    0x4001380C   // SYSCFG external interript configuration register 2
#define SYSCFG_EXTICR3_ADDRESS    0x40013810   // SYSCFG external interript configuration register 3
#define SYSCFG_EXTICR4_ADDRESS    0x40013814   // SYSCFG external interript configuration register 4
#define SYSCFG_CMPCR_ADDRESS      0x40013820   // SYSCFG compensation cell control register
//
// DMA 1 Registers
//
#define DMA1_LISR_ADDRESS         0x40026000   // DMA1 low interrupt status register
#define DMA1_HISR_ADDRESS         0x40026004   // DMA1 high interrupt status register
#define DMA1_LIFCR_ADDRESS        0x40026008   // DMA1 low interrupt flag clear register
#define DMA1_HIFCR_ADDRESS        0x4002600C   // DMA1 high interrupt flag clear register

#define DMA1_S0CR_ADDRESS         0x40026010   // DMA1 stream 0 configuration register
#define DMA1_S0NDTR_ADDRESS       0x40026014   // DMA1 stream 0 number of data register
#define DMA1_S0PAR_ADDRESS        0x40026018   // DMA1 stream 0 peripheral address register
#define DMA1_S0M0AR_ADDRESS       0x4002601C   // DMA1 stream 0 memory 0 address register
#define DMA1_S0M1AR_ADDRESS       0x40026020   // DMA1 stream 0 memory 1 address register
#define DMA1_S0FCR_ADDRESS        0x40026024   // DMA1 stream 0 FIFO control register

#define DMA1_S1CR_ADDRESS         0x40026028   // DMA1 stream 1 configuration register
#define DMA1_S1NDTR_ADDRESS       0x4002602C   // DMA1 stream 1 number of data register
#define DMA1_S1PAR_ADDRESS        0x40026030   // DMA1 stream 1 peripheral address register
#define DMA1_S1M0AR_ADDRESS       0x40026034   // DMA1 stream 1 memory 0 address register
#define DMA1_S1M1AR_ADDRESS       0x40026038   // DMA1 stream 1 memory 1 address register
#define DMA1_S1FCR_ADDRESS        0x4002603C   // DMA1 stream 1 FIFO control register

#define DMA1_S2CR_ADDRESS         0x40026040   // DMA1 stream 2 configuration register
#define DMA1_S2NDTR_ADDRESS       0x40026044   // DMA1 stream 2 number of data register
#define DMA1_S2PAR_ADDRESS        0x40026048   // DMA1 stream 2 peripheral address register
#define DMA1_S2M0AR_ADDRESS       0x4002604C   // DMA1 stream 2 memory 0 address register
#define DMA1_S2M1AR_ADDRESS       0x40026050   // DMA1 stream 2 memory 1 address register
#define DMA1_S2FCR_ADDRESS        0x40026054   // DMA1 stream 2 FIFO control register

#define DMA1_S3CR_ADDRESS         0x40026058   // DMA1 stream 3 configuration register
#define DMA1_S3NDTR_ADDRESS       0x4002605C   // DMA1 stream 3 number of data register
#define DMA1_S3PAR_ADDRESS        0x40026060   // DMA1 stream 3 peripheral address register
#define DMA1_S3M0A_ADDRESS        0x40026064   // DMA1 stream 3 memory 0 address register
#define DMA1_S3M1AR_ADDRESS       0x40026068   // DMA1 stream 3 memory 1 address register
#define DMA1_S3FCR_ADDRESS        0x4002606C   // DMA1 stream 3 FIFO control register

#define DMA1_S4CR_ADDRESS         0x40026070   // DMA1 stream 4 configuration register
#define DMA1_S4NDTR_ADDRESS       0x40026074   // DMA1 stream 4 number of data register
#define DMA1_S4PAR_ADDRESS        0x40026078   // DMA1 stream 4 peripheral address register
#define DMA1_S4M0AR_ADDRESS       0x4002607C   // DMA1 stream 4 memory 0 address register
#define DMA1_S4M1AR_ADDRESS       0x40026080   // DMA1 stream 4 memory 1 address register
#define DMA1_S4FCR_ADDRESS        0x40026084   // DMA1 stream 4 FIFO control register

#define DMA1_S5CR_ADDRESS         0x40026088   // DMA1 stream 5 configuration register
#define DMA1_S5NDTR_ADDRESS       0x4002608C   // DMA1 stream 5 number of data register
#define DMA1_S5PAR_ADDRESS        0x40026090   // DMA1 stream 5 peripheral address register
#define DMA1_S5M0AR_ADDRESS       0x40026094   // DMA1 stream 5 memory 0 address register
#define DMA1_S5M1AR_ADDRESS       0x40026098   // DMA1 stream 5 memory 1 address register
#define DMA1_S5FCR_ADDRESS        0x4002609C   // DMA1 stream 5 FIFO control register

#define DMA1_S6CR_ADDRESS         0x400260A0   // DMA1 stream 6 configuration register
#define DMA1_S6NDTR_ADDRESS       0x400260A4   // DMA1 stream 6 number of data register
#define DMA1_S6PAR_ADDRESS        0x400260A8   // DMA1 stream 6 peripheral address register
#define DMA1_S6M0AR_ADDRESS       0X400260AC   // DMA1 stream 6 memory 0 address register
#define DMA1_S6M1AR_ADDRESS       0x400260B0   // DMA1 stream 6 memory 1 address register
#define DMA1_S6FCR_ADDRESS        0x400260B4   // DMA1 stream 6 FIFO control register

#define DMA1_S7CR_ADDRESS         0x400260B8   // DMA1 stream 7 configuration register
#define DMA1_S7NDTR_ADDRESS       0x400260BC   // DMA1 stream 7 number of data register
#define DMA1_S7PAR_ADDRESS        0x400260C0   // DMA1 stream 7 peripheral address register
#define DMA1_S7M0AR_ADDRESS       0x400260C4   // DMA1 stream 7 memory 0 address register
#define DMA1_S7M1AR_ADDRESS       0x400260C8   // DMA1 stream 7 memory 1 address register
#define DMA1_S7FCR_ADDRESS        0x400260CC   // DMA1 stream 7 FIFO control register
//
// DMA 2 Registers
//
#define DMA2_LISR_ADDRESS         0x40026400   // DMA2 low interrupt status register
#define DMA2_HISR_ADDRESS         0x40026404   // DMA2 high interrupt status register
#define DMA2_LIFCR_ADDRESS        0x40026408   // DMA2 low interrupt flag clear register
#define DMA2_HIFCR_ADDRESS        0x4002640C   // DMA2 high interrupt flag clear register

#define DMA2_S0CR_ADDRESS         0x40026410   // DMA2 stream 0 configuration register
#define DMA2_S0NDTR_ADDRESS       0x40026414   // DMA2 stream 0 number of data register
#define DMA2_S0PAR_ADDRESS        0x40026418   // DMA2 stream 0 peripheral address register
#define DMA2_S0M0AR_ADDRESS       0x4002641C   // DMA2 stream 0 memory 0 address register
#define DMA2_S0M1AR_ADDRESS       0x40026420   // DMA2 stream 0 memory 1 address register
#define DMA2_S0FCR_ADDRESS        0x40026424   // DMA2 stream 0 FIFO control register

#define DMA2_S1CR_ADDRESS         0x40026428   // DMA2 stream 1 configuration register
#define DMA2_S1NDTR_ADDRESS       0x4002642C   // DMA2 stream 1 number of data register
#define DMA2_S1PAR_ADDRESS        0x40026430   // DMA2 stream 1 peripheral address register
#define DMA2_S1M0AR_ADDRESS       0x40026434   // DMA2 stream 1 memory 0 address register
#define DMA2_S1M1AR_ADDRESS       0x40026438   // DMA2 stream 1 memory 1 address register
#define DMA2_S1FCR_ADDRESS        0x4002643C   // DMA2 stream 1 FIFO control register

#define DMA2_S2CR_ADDRESS         0x40026440   // DMA2 stream 2 configuration register
#define DMA2_S2NDTR_ADDRESS       0x40026444   // DMA2 stream 2 number of data register
#define DMA2_S2PAR_ADDRESS        0x40026448   // DMA2 stream 2 peripheral address register
#define DMA2_S2M0AR_ADDRESS       0x4002644C   // DMA2 stream 2 memory 0 address register
#define DMA2_S2M1AR_ADDRESS       0x40026450   // DMA2 stream 2 memory 1 address register
#define DMA2_S2FCR_ADDRESS        0x40026454   // DMA2 stream 2 FIFO control register

#define DMA2_S3CR_ADDRESS         0x40026458   // DMA2 stream 3 configuration register
#define DMA2_S3NDTR_ADDRESS       0x4002645C   // DMA2 stream 3 number of data register
#define DMA2_S3PAR_ADDRESS        0x40026460   // DMA2 stream 3 peripheral address register
#define DMA2_S3M0A_ADDRESS        0x40026464   // DMA2 stream 3 memory 0 address register
#define DMA2_S3M1AR_ADDRESS       0x40026468   // DMA2 stream 3 memory 1 address register
#define DMA2_S3FCR_ADDRESS        0x4002646C   // DMA2 stream 3 FIFO control register

#define DMA2_S4CR_ADDRESS         0x40026470   // DMA2 stream 4 configuration register
#define DMA2_S4NDTR_ADDRESS       0x40026474   // DMA2 stream 4 number of data register
#define DMA2_S4PAR_ADDRESS        0x40026478   // DMA2 stream 4 peripheral address register
#define DMA2_S4M0AR_ADDRESS       0x4002647C   // DMA2 stream 4 memory 0 address register
#define DMA2_S4M1AR_ADDRESS       0x40026480   // DMA2 stream 4 memory 1 address register
#define DMA2_S4FCR_ADDRESS        0x40026484   // DMA2 stream 4 FIFO control register

#define DMA2_S5CR_ADDRESS         0x40026488   // DMA2 stream 5 configuration register
#define DMA2_S5NDTR_ADDRESS       0x4002648C   // DMA2 stream 5 number of data register
#define DMA2_S5PAR_ADDRESS        0x40026490   // DMA2 stream 5 peripheral address register
#define DMA2_S5M0AR_ADDRESS       0x40026494   // DMA2 stream 5 memory 0 address register
#define DMA2_S5M1AR_ADDRESS       0x40026498   // DMA2 stream 5 memory 1 address register
#define DMA2_S5FCR_ADDRESS        0x4002649C   // DMA2 stream 5 FIFO control register

#define DMA2_S6CR_ADDRESS         0x400264A0   // DMA2 stream 6 configuration register
#define DMA2_S6NDTR_ADDRESS       0x400264A4   // DMA2 stream 6 number of data register
#define DMA2_S6PAR_ADDRESS        0x400264A8   // DMA2 stream 6 peripheral address register
#define DMA2_S6M0AR_ADDRESS       0x400264AC   // DMA2 stream 6 memory 0 address register
#define DMA2_S6M1AR_ADDRESS       0x400264B0   // DMA2 stream 6 memory 1 address register
#define DMA2_S6FCR_ADDRESS        0x400264B4   // DMA2 stream 6 FIFO control register

#define DMA2_S7CR_ADDRESS         0x400264B8   // DMA2 stream 7 configuration register
#define DMA2_S7NDTR_ADDRESS       0x400264BC   // DMA2 stream 7 number of data register
#define DMA2_S7PAR_ADDRESS        0x400264C0   // DMA2 stream 7 peripheral address register
#define DMA2_S7M0AR_ADDRESS       0x400264C4   // DMA2 stream 7 memory 0 address register
#define DMA2_S7M1AR_ADDRESS       0x400264C8   // DMA2 stream 7 memory 1 address register
#define DMA2_S7FCR_ADDRESS        0x400264CC   // DMA2 stream 7 FIFO control register
//
// Extended interrupts and events controller
//
#define EXTI_IMR_ADDRESS          0x40013C00   // Interrupt mask register
#define EXTI_EMR_ADDRESS          0x40013C04   // Event mask register
#define EXTI_RTSR_ADDRESS         0x40013C08   // Rising trigger selection register
#define EXTI_FTSR_ADDRESS         0x40013C0C   // Falling trigger selection register
#define EXTI_SWIER_ADDRESS        0x40013C10   // Software interrupt event register
#define EXTI_PR_ADDRESS           0x40013C14   // Pending register
//
// Quad SPI Flash Memory Controller
//
#define QUADSPI_CR_ADDRESS        0xA0001000   // Control Register
#define QUADSPI_DCR_ADDRESS       0xA0001004   // Device Configuration Register
#define QUADSPI_SR_ADDRESS        0xA0001008   // Status Register
#define QUADSPI_FCR_ADDRESS       0xA000100C   // Flag Clear Register
#define QUADSPI_DLR_ADDRESS       0xA0001010   // Data Length Register
#define QUADSPI_CCR_ADDRESS       0xA0001014   // Communication Configuration Register
#define QUADSPI_AR_ADDRESS        0xA0001018   // Address Register
#define QUADSPI_ABR_ADDRESS       0xA000101C   // Alternate Byte Register
#define QUADSPI_DR_ADDRESS        0xA0001020   // Data Register
#define QUADSPI_PSMKR_ADDRESS     0xA0001024   // Polling Status Mask Register
#define QUADSPI_PSMAR_ADDRESS     0xA0001028   // Polling Status Match Register
#define QUADSPI_PIR_ADDRESS       0xA000102C   // Polling Interval Register
#define QUADSPI_LPTR_ADDRESS      0xA0001039   // Low Power Timeout Register
//
// A/D 1 Converters
//
#define ADC1_SR_ADDRESS           0x40012000   // ADC1 status register
#define ADC1_CR1_ADDRESS          0x40012004   // ADC1 control register 1
#define ADC1_CR2_ADDRESS          0x40012008   // ADC1 control register 2
#define ADC1_SMPR1_ADDRESS        0x4001200C   // ADC1 sample time register 1
#define ADC1_SMPR2_ADDRESS        0x40012010   // ADC1 sample time register 2
#define ADC1_JOFR1_ADDRESS        0x40012014   // ADC1 injected channel data offset register 1
#define ADC1_JOFR2_ADDRESS        0x40012018   // ADC1 injected channel data offset register 2
#define ADC1_JOFR3_ADDRESS        0x4001201C   // ADC1 injected channel data offset register 3
#define ADC1_JOFR4_ADDRESS        0x40012020   // ADC1 injected channel data offset register 4
#define ADC1_HTR_ADDRESS          0x40012024   // ADC1 watchdog higher threshold register
#define ADC1_LTR_ADDRESS          0x40012028   // ADC1 watchdog lower threshold register
#define ADC1_SQR1_ADDRESS         0x4001202C   // ADC1 regular sequence register 1
#define ADC1_SQR2_ADDRESS         0x40012030   // ADC1 regular sequence register 2
#define ADC1_SQR3_ADDRESS         0x40012034   // ADC1 regular sequence register 3
#define ADC1_JSQR_ADDRESS         0x40012038   // ADC1 injected sequence register
#define ADC1_JDR1_ADDRESS         0x4001203C   // ADC1 injected data register 1
#define ADC1_JDR2_ADDRESS         0x40012040   // ADC1 injected data register 2
#define ADC1_JDR3_ADDRESS         0x40012044   // ADC1 injected data register 3
#define ADC1_JDR4_ADDRESS         0x40012048   // ADC1 injected data register 4
#define ADC1_DR_ADDRESS           0x4001204C   // ADC1 regular data register
//
// A/D 2 Converters
//
#define ADC2_SR_ADDRESS           0x40012100   // ADC2 status register
#define ADC2_CR1_ADDRESS          0x40012104   // ADC2 control register 1
#define ADC2_CR2_ADDRESS          0x40012108   // ADC2 control register 2
#define ADC2_SMPR1_ADDRESS        0x4001210C   // ADC2 sample time register 1
#define ADC2_SMPR2_ADDRESS        0x40012110   // ADC2 sample time register 2
#define ADC2_JOFR1_ADDRESS        0x40012114   // ADC2 injected channel data offset register 1
#define ADC2_JOFR2_ADDRESS        0x40012118   // ADC2 injected channel data offset register 2
#define ADC2_JOFR3_ADDRESS        0x4001211C   // ADC2 injected channel data offset register 3
#define ADC2_JOFR4_ADDRESS        0x40012120   // ADC2 injected channel data offset register 4
#define ADC2_HTR_ADDRESS          0x40012124   // ADC2 watchdog higher threshold register
#define ADC2_LTR_ADDRESS          0x40012128   // ADC2 watchdog lower threshold register
#define ADC2_SQR1_ADDRESS          0x4001212C   // ADC2 regular sequence register 1
#define ADC2_SQR2_ADDRESS         0x40012130   // ADC2 regular sequence register 2
#define ADC2_SQR3_ADDRESS         0x40012134   // ADC2 regular sequence register 3
#define ADC2_JSQR_ADDRESS         0x40012138   // ADC2 injected sequence register
#define ADC2_JDR1_ADDRESS         0x4001213C   // ADC2 injected data register 1
#define ADC2_JDR2_ADDRESS         0x40012140   // ADC2 injected data register 2
#define ADC2_JDR3_ADDRESS         0x40012144   // ADC2 injected data register 3
#define ADC2_JDR4_ADDRESS         0x40012148   // ADC2 injected data register 4
#define ADC2_DR_ADDRESS           0x4001214C   // ADC2 regular data register
//
// A/D 3 Converters
//
#define ADC3_SR_ADDRESS           0x40012200   // ADC3 status register
#define ADC3_CR1_ADDRESS          0x40012204   // ADC3 control register 1
#define ADC3_CR2_ADDRESS          0x40012208   // ADC3 control register 2
#define ADC3_SMPR1_ADDRESS        0x4001220C   // ADC3 sample time register 1
#define ADC3_SMPR2_ADDRESS        0x40012210   // ADC3 sample time register 2
#define ADC3_JOFR1_ADDRESS        0x40012214   // ADC3 injected channel data offset register 1
#define ADC3_JOFR2_ADDRESS        0x40012218   // ADC3 injected channel data offset register 2
#define ADC3_JOFR3_ADDRESS        0x4001221C   // ADC3 injected channel data offset register 3
#define ADC3_JOFR4_ADDRESS        0x40012220   // ADC3 injected channel data offset register 4
#define ADC3_HTR_ADDRESS          0x40012224   // ADC3 watchdog higher threshold register
#define ADC3_LTR_ADDRESS          0x40012228   // ADC3 watchdog lower threshold register
#define ADC3_SQR1_ADDRESS         0x4001222C   // ADC3 regular sequence register 1
#define ADC3_SQR2_ADDRESS         0x40012230   // ADC3 regular sequence register 2
#define ADC3_SQR3_ADDRESS         0x40012234   // ADC3 regular sequence register 3
#define ADC3_JSQR_ADDRESS         0x40012238   // ADC3 injected sequence register
#define ADC3_JDR1_ADDRESS         0x4001223C   // ADC3 injected data register 1
#define ADC3_JDR2_ADDRESS         0x40012240   // ADC3 injected data register 2
#define ADC3_JDR3_ADDRESS         0x40012244   // ADC3 injected data register 3
#define ADC3_JDR4_ADDRESS         0x40012248   // ADC3 injected data register 4
#define ADC3_DR_ADDRESS           0x4001224C   // ADC3 regular data register
//
// A/D Common Registers
//
#define ADC_CSR_ADDRESS           0x40012300   // ADC Common status register
#define ADC_CCR_ADDRESS           0x40012304   // ADC common control register
#define ADC_CDR_ADDRESS           0x40012308   // ADC common regular data register for dual and triple modes
//
// D/A Converter
//
#define DAC_CR_ADDRESS            0x40007400   // DAC Control Register
#define DAC_SWTRIGR_ADDRESS       0x40007404   // DAC Software Trigger Register
#define DAC_DHR12R1_ADDRESS       0x40007408   // DAC channel1 12-bit right-aligned data holding register
#define DAC_DHR12L1_ADDRESS       0x4000740C   // DAC channel1 12-bit left aligned data holding register
#define DAC_DHR8R1_ADDRESS        0x40007410   // DAC channel1 8-bit right aligned data holding register
#define DAC_DHR12R2_ADDRESS       0x40007414   // DAC channel2 12-bit right aligned data holding register
#define DAC_DHR12L2_ADDRESS       0x40007418   // DAC channel2 12-bit left aligned data holding register
#define DAC_DHR8R2_ADDRESS        0x4000741C   // DAC channel2 8-bit right-aligned data holding register
#define DAC_DHR12RD_ADDRESS       0x40007420   // DAC Dual DAC 12-bit right-aligned data holding register
#define DAC_DHR12LD_ADDRESS       0x40007424   // DAC DUAL DAC 12-bit left aligned data holding register
#define DAC_DHR8RD_ADDRESS        0x40007428   // DAC DUAL DAC 8-bit right aligned data holding register
#define DAC_DOR1_ADDRESS          0x4000742C   // DAC channel1 data output register (DAC_DOR1)
#define DAC_DOR2_ADDRESS          0x40007430   // DAC channel2 data output register (DAC_DOR2)
#define DAC_SR_ADDRESS            0x40007434   // DAC status register
//
// Timer 1
//
#define TIM1_CR1_ADDRESS          0x40010000   // TIM1 control register 1
#define TIM1_CR2_ADDRESS          0x40010004   // TIM1 control register 2
#define TIM1_SMCR_ADDRESS         0x40010008   // TIM1 slave mode control register
#define TIM1_DIER_ADDRESS         0x4001000C   // TIM1 DMA/interrupt enable register
#define TIM1_SR_ADDRESS           0x40010010   // TIM1 status register
#define TIM1_EGR_ADDRESS          0x40010014   // TIM1 event generation register
#define TIM1_CCMR1_ADDRESS        0x40010018   // TIM1 capture/compare mode register 1
#define TIM1_CCMR2_ADDRESS        0x4001001C   // TIM1 capture/compare mode register 2
#define TIM1_CCER_ADDRESS         0x40010020   // TIM1 capture/compare enable register
#define TIM1_CNT_ADDRESS          0x40010024   // TIM1 counter
#define TIM1_PSC_ADDRESS          0x40010028   // TIM1 prescaler
#define TIM1_ARR_ADDRESS          0x4001002C   // TIM1 auto-reload register
#define TIM1_RCR_ADDRESS          0x40010030   // TIM1 repetition counter register
#define TIM1_CCR1_ADDRESS         0x40010034   // TIM1 capture/compare register 1
#define TIM1_CCR2_ADDRESS         0x40010038   // TIM1 capture/compare register 2
#define TIM1_CCR3_ADDRESS         0x4001003C   // TIM1 capture/compare register 3
#define TIM1_CCR4_ADDRESS         0x40010040   // TIM1 capture/compare register 4
#define TIM1_BDTR_ADDRESS         0x40010044   // TIM1 break and dead-time register
#define TIM1_DCR_ADDRESS          0x40010048   // TIM1 DMA control register
#define TIM1_DMAR_ADDRESS         0x4001004C   // TIM1 DMA address for full transfer
#define TIM1_CCMR3_ADDRESS        0x40010054   // TIM1 capture/compare mode register 3
#define TIM1_CCR5_ADDRESS         0X40010058   // TIM1 capture/compare register 5
#define TIM1_CCR6_ADDRESS         0X4001005C   // TIM1 capture/compare register 6
//
// Timer 8
//
#define TIM8_CR1_ADDRESS          0x40010400   // TIM8 control register 1
#define TIM8_CR2_ADDRESS          0x40010404   // TIM8 control register 2
#define TIM8_SMCR_ADDRESS         0x40010408   // TIM8 slave mode control register
#define TIM8_DIER_ADDRESS         0x4001040C   // TIM8 DMA/interrupt enable register
#define TIM8_SR_ADDRESS           0x40010410   // TIM8 status register
#define TIM8_EGR_ADDRESS          0x40010414   // TIM8 event generation register
#define TIM8_CCMR1_ADDRESS        0x40010418   // TIM8 capture/compare mode register 1
#define TIM8_CCMR2_ADDRESS        0x4001041C   // TIM8 capture/compare mode register 2
#define TIM8_CCER_ADDRESS         0x40010420   // TIM8 capture/compare enable register
#define TIM8_CNT_ADDRESS          0x40010424   // TIM8 counter
#define TIM8_PSC_ADDRESS          0x40010428   // TIM8 prescaler
#define TIM8_ARR_ADDRESS          0x4001042C   // TIM8 auto-reload register
#define TIM8_RCR_ADDRESS          0x40010430   // TIM8 repetition counter register
#define TIM8_CCR1_ADDRESS         0x40010434   // TIM8 capture/compare register 1
#define TIM8_CCR2_ADDRESS         0x40010438   // TIM8 capture/compare register 2
#define TIM8_CCR3_ADDRESS         0x4001043C   // TIM8 capture/compare register 3
#define TIM8_CCR4_ADDRESS         0x40010440   // TIM8 capture/compare register 4
#define TIM8_BDTR_ADDRESS         0x40010444   // TIM8 break and dead-time register
#define TIM8_DCR_ADDRESS          0x40010448   // TIM8 DMA control register
#define TIM8_DMAR_ADDRESS         0x4001044C   // TIM8 DMA address for full transfer
#define TIM1_CCMR3_ADDRESS        0x40010054   // TIM1 capture/compare mode register 3
#define TIM1_CCR5_ADDRESS         0X40010058   // TIM1 capture/compare register 5
#define TIM1_CCR6_ADDRESS         0X4001005C   // TIM1 capture/compare register 6
//
// Timer 2
//
#define TIM2_CR1_ADDRESS          0x40000000   // TIM2 control register 1
#define TIM2_CR2_ADDRESS          0x40000004   // TIM2 control register 2
#define TIM2_SMCR_ADDRESS         0x40000008   // TIM2 slave mode control register
#define TIM2_DIER_ADDRESS         0x4000000C   // TIM2 DMA/interrupt enable register
#define TIM2_SR_ADDRESS           0x40000010   // TIM2 status register
#define TIM2_EGR_ADDRESS          0x40000014   // TIM2 event generation register
#define TIM2_CCMR1_ADDRESS        0x40000018   // TIM2 capture/compare mode register 1
#define TIM2_CCMR2_ADDRESS        0x4000001C   // TIM2 capture/compare mode register 2
#define TIM2_CCER_ADDRESS         0x40000020   // TIM2 capture/compare enable register
#define TIM2_CNT_ADDRESS          0x40000024   // TIM2 counter
#define TIM2_PSC_ADDRESS          0x40000028   // TIM2 prescaler
#define TIM2_ARR_ADDRESS          0x4000002C   // TIM2 auto-reload register
#define TIM2_RCR_ADDRESS          0x40000030   // TIM2 repetition counter register
#define TIM2_CCR1_ADDRESS         0x40000034   // TIM2 capture/compare register 1
#define TIM2_CCR2_ADDRESS         0x40000038   // TIM2 capture/compare register 2
#define TIM2_CCR3_ADDRESS         0x4000003C   // TIM2 capture/compare register 3
#define TIM2_CCR4_ADDRESS         0x40000040   // TIM2 capture/compare register 4
#define TIM2_DCR_ADDRESS          0x40000048   // TIM2 DMA control register
#define TIM2_DMAR_ADDRESS         0x4000004C   // TIM2 DMA address for full transfer
#define TIM2_OR_ADDRESS           0x40000050   // TIM2 option register
//
// Timer 3
//
#define TIM3_CR1_ADDRESS          0x40000400   // TIM3 control register 1
#define TIM3_CR2_ADDRESS          0x40000404   // TIM3 control register 2
#define TIM3_SMCR_ADDRESS         0x40000408   // TIM3 slave mode control register
#define TIM3_DIER_ADDRESS         0x4000040C   // TIM3 DMA/interrupt enable register
#define TIM3_SR_ADDRESS           0x40000410   // TIM3 status register
#define TIM3_EGR_ADDRESS          0x40000414   // TIM3 event generation register
#define TIM3_CCMR1_ADDRESS        0x40000418   // TIM3 capture/compare mode register 1
#define TIM3_CCMR2_ADDRESS        0x4000041C   // TIM3 capture/compare mode register 2
#define TIM3_CCER_ADDRESS         0x40000420   // TIM3 capture/compare enable register
#define TIM3_CNT_ADDRESS          0x40000424   // TIM3 counter
#define TIM3_PSC_ADDRESS          0x40000428   // TIM3 prescaler
#define TIM3_ARR_ADDRESS          0x4000042C   // TIM3 auto-reload register
#define TIM3_RCR_ADDRESS          0x40000430   // TIM3 repetition counter register
#define TIM3_CCR1_ADDRESS         0x40000434   // TIM3 capture/compare register 1
#define TIM3_CCR2_ADDRESS         0x40000438   // TIM3 capture/compare register 2
#define TIM3_CCR3_ADDRESS         0x4000043C   // TIM3 capture/compare register 3
#define TIM3_CCR4_ADDRESS         0x40000440   // TIM3 capture/compare register 4
#define TIM3_DCR_ADDRESS          0x40000448   // TIM3 DMA control register
#define TIM3_DMAR_ADDRESS         0x4000044C   // TIM3 DMA address for full transfer
//
// Timer 4
//
#define TIM4_CR1_ADDRESS          0x40000800   // TIM4 control register 1
#define TIM4_CR2_ADDRESS          0x40000804   // TIM4 control register 2
#define TIM4_SMCR_ADDRESS         0x40000808   // TIM4 slave mode control register
#define TIM4_DIER_ADDRESS         0x4000080C   // TIM4 DMA/interrupt enable register
#define TIM4_SR_ADDRESS           0x40000810   // TIM4 status register
#define TIM4_EGR_ADDRESS          0x40000814   // TIM4 event generation register
#define TIM4_CCMR1_ADDRESS        0x40000818   // TIM4 capture/compare mode register 1
#define TIM4_CCMR2_ADDRESS        0x4000081C   // TIM4 capture/compare mode register 2
#define TIM4_CCER_ADDRESS         0x40000820   // TIM4 capture/compare enable register
#define TIM4_CNT_ADDRESS          0x40000824   // TIM4 counter
#define TIM4_PSC_ADDRESS          0x40000828   // TIM4 prescaler
#define TIM4_ARR_ADDRESS          0x4000082C   // TIM4 auto-reload register
#define TIM4_RCR_ADDRESS          0x40000830   // TIM4 repetition counter register
#define TIM4_CCR1_ADDRESS         0x40000834   // TIM4 capture/compare register 1
#define TIM4_CCR2_ADDRESS         0x40000838   // TIM4 capture/compare register 2
#define TIM4_CCR3_ADDRESS         0x4000083C   // TIM4 capture/compare register 3
#define TIM4_CCR4_ADDRESS         0x40000840   // TIM4 capture/compare register 4
#define TIM4_DCR_ADDRESS          0x40000848   // TIM4 DMA control register
#define TIM4_DMAR_ADDRESS         0x4000084C   // TIM4 DMA address for full transfer
//
// Timer 5
//
#define TIM5_CR1_ADDRESS          0x40010C00   // TIM5 control register 1
#define TIM5_CR2_ADDRESS          0x40010C04   // TIM5 control register 2
#define TIM5_SMCR_ADDRESS         0x40010C08   // TIM5 slave mode control register
#define TIM5_DIER_ADDRESS         0x40010C0C   // TIM5 DMA/interrupt enable register
#define TIM5_SR_ADDRESS           0x40010C10   // TIM5 status register
#define TIM5_EGR_ADDRESS          0x40010C14   // TIM5 event generation register
#define TIM5_CCMR1_ADDRESS        0x40010C18   // TIM5 capture/compare mode register 1
#define TIM5_CCMR2_ADDRESS        0x40010C1C   // TIM5 capture/compare mode register 2
#define TIM5_CCER_ADDRESS         0x40010C20   // TIM5 capture/compare enable register
#define TIM5_CNT_ADDRESS          0x40010C24   // TIM5 counter
#define TIM5_PSC_ADDRESS          0x40010C28   // TIM5 prescaler
#define TIM5_ARR_ADDRESS          0x40010C2C   // TIM5 auto-reload register
#define TIM5_RCR_ADDRESS          0x40010C30   // TIM5 repetition counter register
#define TIM5_CCR1_ADDRESS         0x40010C34   // TIM5 capture/compare register 1
#define TIM5_CCR2_ADDRESS         0x40010C38   // TIM5 capture/compare register 2
#define TIM5_CCR3_ADDRESS         0x40010C3C   // TIM5 capture/compare register 3
#define TIM5_CCR4_ADDRESS         0x40010C40   // TIM5 capture/compare register 4
#define TIM5_DCR_ADDRESS          0x40010C48   // TIM5 DMA control register
#define TIM5_DMAR_ADDRESS         0x40010C4C   // TIM5 DMA address for full transfer
#define TIM5_OR_ADDRESS           0x40010C50   // TIM5 option register
//
// Timer 9
//
#define TIM9_CR1                  0x40014000   //TIM9 Control Reister 1
#define TIM9_SMCR                 0x40014008   //TIM9 Slave Mode Control Register
#define TIM9_DIER                 0x4001400C   //TIM9 Interrupt Enable Register
#define TIM9_SR                   0x40014010   //TIM9 Status Register
#define TIM9_EGR                  0x40014014   //TIM9 Event Generation Register
#define TIM9_CCMR1                0x40014018   //TIM9 Capture/Compare Mode Register 1
#define TIM9_CCER                 0x40014020   //TIM9 Capture/Compare Enable Register
#define TIM9_CNT                  0x40014024   //TIM9 Counter
#define TIM9_PSC                  0x40014028   //TIM9 Prescaler
#define TIM9_ARR                  0x4001402C   //TIM9 Auto-Reload Register
#define TIM9_CCR1                 0x40014034   //TIM9 Capture/Compare Register 1
#define TIM9_CCR2                 0x40014038   //TIM9 Capture/Compare Register 2
//
// Timer 12
//
#define TIM12_CR1                 0x40001800   //TIM12 Control Reister 1
#define TIM12_SMCR                0x40001808   //TIM12 Slave Mode Control Register
#define TIM12_DIER                0x4000180C   //TIM12 Interrupt Enable Register
#define TIM12_SR                  0x40001810   //TIM12 Status Register
#define TIM12_EGR                 0x40001814   //TIM12 Event Generation Register
#define TIM12_CCMR1               0x40001818   //TIM12 Capture/Compare Mode Register 1
#define TIM12_CCER                0x40001820   //TIM12 Capture/Compare Enable Register
#define TIM12_CNT                 0x40001824   //TIM12 Counter
#define TIM12_PSC                 0x40001828   //TIM12 Prescaler
#define TIM12_ARR                 0x4000182C   //TIM12 Auto-Reload Register
#define TIM12_CCR1                0x40001834   //TIM12 Capture/Compare Register 1
#define TIM12_CCR2                0x40001838   //TIM12 Capture/Compare Register 2
//
// Timer 10
//
#define TIM10_CR1                 0x40014400   //TIM10 Control Reister 1
#define TIM10_SMCR                0x40014408   //TIM10 Slave Mode Control Register
#define TIM10_DIER                0x4001440C   //TIM10 Interrupt Enable Register
#define TIM10_SR                  0x40014410   //TIM10 Status Register
#define TIM10_EGR                 0x40014414   //TIM10 Event Generation Register
#define TIM10_CCMR1               0x40014418   //TIM10 Capture/Compare Mode Register 1
#define TIM10_CCER                0x40014420   //TIM10 Capture/Compare Enable Register
#define TIM10_CNT                 0x40014424   //TIM10 Counter
#define TIM10_PSC                 0x40014428   //TIM10 Prescaler
#define TIM10_ARR                 0x4001442C   //TIM10 Auto-Reload Register
#define TIM10_CCR1                0x40014434   //TIM10 Capture/Compare Register 1
//
// Timer 11
//
#define TIM11_CR1                 0x40014800   //TIM11 Control Reister 1
#define TIM11_SMCR                0x40014808   //TIM11 Slave Mode Control Register
#define TIM11_DIER                0x4001480C   //TIM11 Interrupt Enable Register
#define TIM11_SR                  0x40014810   //TIM11 Status Register
#define TIM11_EGR                 0x40014814   //TIM11 Event Generation Register
#define TIM11_CCMR1               0x40014818   //TIM11 Capture/Compare Mode Register 1
#define TIM11_CCER                0x40014820   //TIM11 Capture/Compare Enable Register
#define TIM11_CNT                 0x40014824   //TIM11 Counter
#define TIM11_PSC                 0x40014828   //TIM11 Prescaler
#define TIM11_ARR                 0x4001482C   //TIM11 Auto-Reload Register
#define TIM11_CCR1                0x40014834   //TIM11 Capture/Compare Register 1
#define TIM11_OR                  0x40014850   //TIM11 Option Register 1
//
// Timer 13
//
#define TIM13_CR1                 0x40001C00   //TIM13 Control Reister 1
#define TIM13_SMCR                0x40001C08   //TIM13 Slave Mode Control Register
#define TIM13_DIER                0x40001C0C   //TIM13 Interrupt Enable Register
#define TIM13_SR                  0x40001C10   //TIM13 Status Register
#define TIM13_EGR                 0x40001C14   //TIM13 Event Generation Register
#define TIM13_CCMR1               0x40001C18   //TIM13 Capture/Compare Mode Register 1
#define TIM13_CCER                0x40001C20   //TIM13 Capture/Compare Enable Register
#define TIM13_CNT                 0x40001C24   //TIM13 Counter
#define TIM13_PSC                 0x40001C28   //TIM13 Prescaler
#define TIM13_ARR                 0x40001C2C   //TIM13 Auto-Reload Register
#define TIM13_CCR1                0x40001C34   //TIM13 Capture/Compare Register 1
//
// Timer 14
//
#define TIM14_CR1                 0x40002000   //TIM14 Control Reister 1
#define TIM14_SMCR                0x40002008   //TIM14 Slave Mode Control Register
#define TIM14_DIER                0x4000200C   //TIM14 Interrupt Enable Register
#define TIM14_SR                  0x40002010   //TIM14 Status Register
#define TIM14_EGR                 0x40002014   //TIM14 Event Generation Register
#define TIM14_CCMR1               0x40002018   //TIM14 Capture/Compare Mode Register 1
#define TIM14_CCER                0x40002020   //TIM14 Capture/Compare Enable Register
#define TIM14_CNT                 0x40002024   //TIM14 Counter
#define TIM14_PSC                 0x40002028   //TIM14 Prescaler
#define TIM14_ARR                 0x4000202C   //TIM14 Auto-Reload Register
#define TIM14_CCR1                0x40002034   //TIM14 Capture/Compare Register 1
//
// Timer 6
//
#define TIM6_CR1                  0x40001000   //TIM6 Control Reister 1
#define TIM6_CR2                  0x40001004   //TIM6 Control Reister 2
#define TIM6_DIER                 0x4000100C   //TIM6 Interrupt Enable Register
#define TIM6_SR                   0x40001010   //TIM6 Status Register
#define TIM6_EGR                  0x40001014   //TIM6 Event Generation Register
#define TIM6_CNT                  0x40001024   //TIM6 Counter
#define TIM6_PSC                  0x40001028   //TIM6 Prescaler
#define TIM6_ARR                  0x4000102C   //TIM6 Auto-Reload Register
//
// Timer 7
//
#define TIM7_CR1                  0x40001400   //TIM7 Control Reister 1
#define TIM7_CR2                  0x40001404   //TIM7 Control Reister 2
#define TIM7_DIER                 0x4000140C   //TIM7 Interrupt Enable Register
#define TIM7_SR                   0x40001410   //TIM7 Status Register
#define TIM7_EGR                  0x40001414   //TIM7 Event Generation Register
#define TIM7_CNT                  0x40001424   //TIM7 Counter Register
#define TIM7_PSC                  0x40001428   //TIM7 Prescaler Register
#define TIM7_ARR                  0x4000142C   //TIM7 Auto-Reload Register
//
// Low Power Timer 1
//
#define LPTIM1_ISR                0x40002400   //LPTIM1 Interrupt and Status Register
#define LPTIM1_ICR                0x40002404   //LPTIM1 Interrupt Clear Register
#define LPTIM1_IER                0x40002408   //LPTIM1 Interrupt Enable Register
#define LPTIM1_CFGR               0x4000240C   //LPTIM1 Configuration Register
#define LPTIM1_CR                 0x40002410   //LPTIM1 Control Register
#define LPTIM1_CMP                0x40002414   //LPTIM1 Compare Register
#define LPTIM1_ARR                0x40002418   //LPTIM1 Auto-Reload Register
#define LPTIM1_CNT                0x4000241C   //LPTIM1 Counter Register
#define LPTIM1_OR                 0x40002420   //LPTIM1 Option Register
//
// Independent Watch Dog Timer
//
#define IWDG_KR_ADDRESS           0x40003000   // IWDG Key Register
#define IWDG_PR_ADDRESS           0x40003004   // IWDG Prescale Register
#define IWDG_RLR_ADDRESS          0x40003008   // IWDG Reload Register
#define IWDG_SR_ADDRESS           0x4000300C   // IWDG Status Register
#define IWDG_WINR_ADDRESS         0x40003010   // IWDG Window Register
//
// Window Watch Dog Timer
//
#define WWDG_CR_ADDRESS           0x40002C00   // WWDG Control Register
#define WWDG_CFR_ADDRESS          0x40002C04   // WWDG Configuration Register
#define WWDG_SR_ADDRESS           0x40002C08   // WWDG Status Register
//
// Real Time Clock
//
#define RTC_TR_ADDRESS            0x40002800   //RTC Time Register
#define RTC_DR_ADDRESS            0x40002804   //RTC Date Register
#define RTC_CR_ADDRESS            0x40002808   //RTC Control Register
#define RTC_ISR_ADDRESS           0x4000280C   //RTC Initialization and Status Register
#define RTC_PRER_ADDRESS          0x40002810   //RTC Prescale Register
#define RTC_WUTR_ADDRESS          0x40002814   //RTC Wakeup Timer Register
#define RTC_ALRMAR_ADDRESS        0x4000281C   //RTC Alarm A Register
#define RTC_ALRMBR_ADDRESS        0x40002820   //RTC Alarm B Register
#define RTC_WPR_ADDRESS           0x40002824   //RTC Write Protection Register
#define RTC_SSR_ADDRESS           0x40002828   //RTC Sub Second Register
#define RTC_SHIFTR_ADDRESS        0x4000282C   //RTC Shift Control Register
#define RTC_TSTR_ADDRESS          0x40002830   //RTC Timestamp Time Register
#define RTC_TSDR_ADDRESS          0x40002834   //RTC Timestamp Date Register
#define RTC_TSSSR_ADDRESS         0x40002838   //RTC Timestamp Sub Second Register
#define RTC_CALR_ADDRESS          0x4000283C   //RTC Calibration Register
#define RTC_TAMPCR_ADDRESS        0x40002840   //RTC Tamper Configuration Register
#define RTC_ALRMASSR_ADDRESS      0x40002844   //RTC Alarm A Sub Second Register
#define RTC_ALRMBSSR_ADDRESS      0x40002848   //RTC Alarm B Sub Second Register 
#define RTC_OR_ADDRESS            0x4000284C   //RTC Option Register
#define RTC_BKP0R_ADDRESS         0x40002850   //RTC Backup Register 0
#define RTC_BKP1R_ADDRESS         0x40002854   //RTC Backup Register 1
#define RTC_BKP2R_ADDRESS         0x40002858   //RTC Backup Register 2
#define RTC_BKP3R_ADDRESS         0x4000285C   //RTC Backup Register 3
#define RTC_BKP4R_ADDRESS         0x40002860   //RTC Backup Register 4
#define RTC_BKP5R_ADDRESS         0x40002864   //RTC Backup Register 5
#define RTC_BKP6R_ADDRESS         0x40002868   //RTC Backup Register 6
#define RTC_BKP7R_ADDRESS         0x4000286C   //RTC Backup Register 7
#define RTC_BKP8R_ADDRESS         0x40002870   //RTC Backup Register 8
#define RTC_BKP9R_ADDRESS         0x40002874   //RTC Backup Register 9
#define RTC_BKP10R_ADDRESS        0x40002878   //RTC Backup Register 10
#define RTC_BKP11R_ADDRESS        0x4000287C   //RTC Backup Register 11
#define RTC_BKP12R_ADDRESS        0x40002880   //RTC Backup Register 12
#define RTC_BKP13R_ADDRESS        0x40002884   //RTC Backup Register 13
#define RTC_BKP14R_ADDRESS        0x40002888   //RTC Backup Register 14
#define RTC_BKP15R_ADDRESS        0x4000288C   //RTC Backup Register 15
#define RTC_BKP16R_ADDRESS        0x40002890   //RTC Backup Register 16
#define RTC_BKP17R_ADDRESS        0x40002894   //RTC Backup Register 17
#define RTC_BKP18R_ADDRESS        0x40002898   //RTC Backup Register 18
#define RTC_BKP19R_ADDRESS        0x4000289C   //RTC Backup Register 19
#define RTC_BKP20R_ADDRESS        0x400028A0   //RTC Backup Register 20
#define RTC_BKP21R_ADDRESS        0x400028A4   //RTC Backup Register 21
#define RTC_BKP22R_ADDRESS        0x400028A8   //RTC Backup Register 22
#define RTC_BKP23R_ADDRESS        0x400028AC   //RTC Backup Register 23
#define RTC_BKP24R_ADDRESS        0x400028B0   //RTC Backup Register 24
#define RTC_BKP25R_ADDRESS        0x400028B4   //RTC Backup Register 25
#define RTC_BKP26R_ADDRESS        0x400028B8   //RTC Backup Register 26
#define RTC_BKP27R_ADDRESS        0x400028BC   //RTC Backup Register 27
#define RTC_BKP28R_ADDRESS        0x400028C0   //RTC Backup Register 28
#define RTC_BKP29R_ADDRESS        0x400028C4   //RTC Backup Register 29
#define RTC_BKP30R_ADDRESS        0x400028C8   //RTC Backup Register 30
#define RTC_BKP31R_ADDRESS        0x400028CC   //RTC Backup Register 31
//
// SPI Port 1
//
#define SPI1_CR1_ADDRESS          0x40013000   //SPI1 Control Register 1
#define SPI1_CR2_ADDRESS          0x40013004   //SPI1 Control Register 2
#define SPI1_SR_ADDRESS           0x40013008   //SPI1 Status Register
#define SPI1_DR_ADDRESS           0x4001300C   //SPI1 Data Register
#define SPI1_CRCPR_ADDRESS        0x40013010   //SPI1 CRC Polynomial Register
#define SPI1_RXCRCR_ADDRESS       0x40013014   //SPI1 Rx CRC Register
#define SPI1_TXCRCR_ADDRESS       0x40013018   //SPI1 Tx CRC Register
#define SPI1_I2SCFGR_ADDRESS      0x4001301C   //SPI1 I2S Configuration Register
#define SPI1_I2SPR_ADDRESS        0x40013020   //SPI1 I2S Prescale Register
//
// SPI Port 2
//
#define SPI2_CR1_ADDRESS          0x40013800   //SPI2 Control Register 1
#define SPI2_CR2_ADDRESS          0x40013804   //SPI2 Control Register 2
#define SPI2_SR_ADDRESS           0x40013808   //SPI2 Status Register
#define SPI2_DR_ADDRESS           0x4001380C   //SPI2 Data Register
#define SPI2_CRCPR_ADDRESS        0x40013810   //SPI2 CRC Polynomial Register
#define SPI2_RXCRCR_ADDRESS       0x40013814   //SPI2 Rx CRC Register
#define SPI2_TXCRCR_ADDRESS       0x40013818   //SPI2 Tx CRC Register
#define SPI2_I2SCFGR_ADDRESS      0x4001381C   //SPI2 I2S Configuration Register
#define SPI2_I2SPR_ADDRESS        0x40013820   //SPI2 I2S Prescale Register
//
// UART 1 base 40011000
//
#define USART1_SR_ADDRESS         0x40011000   //UART1 Status Register
#define USART1_DR_ADDRESS         0x40011004   //UART1 Data Register
#define USART1_BRR_ADDRESS        0x40011008   //UART1 Baud Rate Register
#define USART1_CR1_ADDRESS        0x4001100C   //UART1 Control Register 1
#define USART1_CR2_ADDRESS        0x40011010   //UART1 Control Register 2 
#define USART1_CR3_ADDRESS        0x40011014   //UART1 Control Register 3 
#define USART1_GTPR_ADDRESS       0x40011018   //UART1 Guard Time and Prescale Register
//
// USART 2
//
#define USART2_SR_ADDRESS        0x40004400   //status register
#define USART2_DR_ADDRESS        0x40004404   //data register
#define USART2_BRR_ADDRESS       0x40004408   //baud rate register
#define USART2_CR1_ADDRESS       0x4000440C   //control register 1
#define USART2_CR2_ADDRESS       0x40004410   //control register 2
#define USART2_CR3_ADDRESS       0x40004414   //control register 3
#define USART2_GTPR_ADDRESS      0x40004418   //guard time and prescaler
//
// USART 3
//
#define USART3_SR_ADDRESS        0x40004800   //status register
#define USART3_DR_ADDRESS        0x40004804   //data register
#define USART3_BRR_ADDRESS       0x40004808   //baud rate register
#define USART3_CR1_ADDRESS       0x4000480C   //control register 1
#define USART3_CR2_ADDRESS       0x40004810   //control register 2
#define USART3_CR3_ADDRESS       0x40004814   //control register 3
#define USART3_GTPR_ADDRESS      0x40004818   //guard time and prescaler
//
// Debug Support
//
#define DBGMCU_IDCODE            0xE0042000   //DBGMCU Device ID Code
#define DBGMCU_CR                0xE0042004   //DBGMCU Configuration Register
#define DBGMCU_APB1_FZ           0xE0042008   //DBGMCU APB1 Freeze Register
#define DBGMCU_APB2_FZ           0xE004200C   //DBGMCU APB2 Freeze Register
//
// Define bit pattern used for BSRR register used to turn on port bit
//
#define BIT00_ON            0x00000001
#define BIT01_ON            0x00000002
#define BIT02_ON            0x00000004
#define BIT03_ON            0x00000008
#define BIT04_ON            0x00000010
#define BIT05_ON            0x00000020
#define BIT06_ON            0x00000040
#define BIT07_ON            0x00000080
#define BIT08_ON            0x00000100
#define BIT09_ON            0x00000200
#define BIT10_ON            0x00000400
#define BIT11_ON            0x00000800
#define BIT12_ON            0x00001000
#define BIT13_ON            0x00002000
#define BIT14_ON            0x00004000
#define BIT15_ON            0x00008000
//
// Define bit pattern used for BSRR register used to turn off port bit
//
#define BIT00_OFF           0x00010000
#define BIT01_OFF           0x00020000
#define BIT02_OFF           0x00040000
#define BIT03_OFF           0x00080000
#define BIT04_OFF           0x00100000
#define BIT05_OFF           0x00200000
#define BIT06_OFF           0x00400000
#define BIT07_OFF           0x00800000
#define BIT08_OFF           0x01000000
#define BIT09_OFF           0x02000000
#define BIT10_OFF           0x04000000
#define BIT11_OFF           0x08000000
#define BIT12_OFF           0x10000000
#define BIT13_OFF           0x20000000
#define BIT14_OFF           0x40000000
#define BIT15_OFF           0x80000000
//
// Define bit mask
//
#define BIT00_MASK          0x00000001
#define BIT01_MASK          0x00000002
#define BIT02_MASK          0x00000004
#define BIT03_MASK          0x00000008
#define BIT04_MASK          0x00000010
#define BIT05_MASK          0x00000020
#define BIT06_MASK          0x00000040
#define BIT07_MASK          0x00000080
#define BIT08_MASK          0x00000100
#define BIT09_MASK          0x00000200
#define BIT10_MASK          0x00000400
#define BIT11_MASK          0x00000800
#define BIT12_MASK          0x00001000
#define BIT13_MASK          0x00002000
#define BIT14_MASK          0x00004000
#define BIT15_MASK          0x00008000




