//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// May 3, 2018  		Design Documents                   CSI_screens.h 
// 
//       Header for Graphics routines for LCD display
//           for large display project
//
// Programmer
//  Dean S. Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  05/03/2018  started
//
// Status
//  writing
//
//==============================================================================
#ifndef CSI_SCREENS_H_
#define CSI_SCREENS_H_

#include <stdint.h>
#include "CSI_COLOR.h"

// variables
extern int wait_for_radio_onscreen;
extern int radio_connected_onscreen;

extern int active_screen_num;
extern int display_mode;
extern uint8_t screen_refresh;
extern uint8_t rssi_icon_isactive;
extern uint8_t rssi_graph_on;

extern uint8_t lines_per_page;
extern uint8_t menu_line_num;

extern uint8_t lc_poped_up;
extern uint8_t list_hasdata;
extern uint8_t cur_call_active;
extern uint8_t chan_nums_active;
extern uint8_t active_button_mode;
extern uint8_t prev_powlvl;

extern uint8_t tx_on;

extern int message1_X;
extern int message1_Y;
extern int message2_X;
extern int message2_Y;


extern RGB_color adj_color;

extern uint16_t color_adj_x;
extern uint16_t color_adj_y;

typedef struct { 
  uint8_t has_data;
  char text[16];
} List_text;


typedef struct { 
  uint8_t has_data;
  uint32_t channel;
  char text[16];
} Last_call_info;

extern List_text Scroll_list[9];
extern List_text pop_up_list[20];
extern List_text Menu_list[25];
extern int list_top_item;

typedef struct 
  {
  char passed;
  char *text; 
  }test_info;

// x, y upper left corner
typedef struct 
  {
  uint16_t x_loc;
  uint16_t y_loc;
  uint8_t *icon; 
  }tab_info;

// x, y upper left corner
typedef struct 
  {
  uint16_t x_loc;
  uint16_t y_loc;
  char *text; 
  }menu_info;

extern tab_info screen_tab[7];


// touch screen locations
extern uint16_t current_radio_volume;
extern uint16_t volume_up_touch_minx;
extern uint16_t volume_up_touch_maxx; 
extern uint16_t volume_up_touch_miny; 
extern uint16_t volume_up_touch_maxy;

extern uint16_t volume_dn_touch_minx;
extern uint16_t volume_dn_touch_maxx;
extern uint16_t volume_dn_touch_miny;
extern uint16_t volume_dn_touch_maxy;

extern uint16_t channel_up_touch_miny;
extern uint16_t channel_up_touch_maxy; 
extern uint16_t channel_up_touch_minx; 
extern uint16_t channel_up_touch_maxx;

extern uint16_t channel_dn_touch_miny;
extern uint16_t channel_dn_touch_maxy;
extern uint16_t channel_dn_touch_minx;
extern uint16_t channel_dn_touch_maxx;

extern uint16_t lastcall_touch_miny;
extern uint16_t lastcall_touch_maxy;
extern uint16_t lastcall_touch_minx;
extern uint16_t lastcall_touch_maxx;

// icon states for refresh
extern uint8_t prev_talka;
extern uint8_t prev_monitor;
extern uint8_t prev_scan;
extern uint8_t prev_call_type;
extern uint8_t prev_env;
extern uint8_t prev_tdis;

// screen 1
void display_screen( uint8_t screen_num, uint8_t clear );
// screen functions
void display_volume_bar(uint8_t volume_value);
void draw_rssi_icon( uint8_t rssi_value, uint16_t pos_x, uint16_t pos_y );
void display_channel_name( uint8_t numchars );
void draw_scan_icon( uint8_t scan, uint16_t pos_x, uint16_t pos_y );
void display_tx_power( uint8_t powlvl, uint8_t txonoff,
                      uint16_t pos_x, uint16_t pos_y, uint8_t font );
void draw_lock_icon( uint8_t lock, uint16_t pos_x, uint16_t pos_y );
void draw_envelope_icon( uint8_t env, uint16_t pos_x, uint16_t pos_y );
void draw_inboxfull_icon( uint8_t inbox );
void draw_tone_disable_icon( uint8_t tdis );
void draw_talk_around_icon( uint8_t talka );
// void update_talk_around_icon( uint8_t talka );
void draw_battery_icon( uint8_t batt );
void draw_monitor_icon( uint8_t monitor, uint16_t pos_x, uint16_t pos_y );
void draw_button( uint8_t B_num, uint8_t bordr_color, uint8_t face_color );
void draw_gps(void);
void draw_menutab( uint8_t tabnum, uint8_t active);
void draw_navbuttons( uint8_t mode );
void draw_cancel_button( void );
void clear_scroll_list( void );
void draw_list( uint16_t x_loc, uint16_t y_loc, uint32_t txt_len, uint8_t updown );
void clear_menupage( uint8_t menupage );
void draw_TXonoff( uint8_t onoff );
void display_POM_message( uint8_t msg_num, uint8_t msg_length );
void draw_call_icon( uint8_t call_type );
void popup_lastcall_list( void );
void display_lastcall_icon( void );
void popup_list( uint8_t poped_up, uint8_t numrecs, uint8_t list_indx,
       uint16_t pop_x, uint16_t pop_y, uint16_t pop_width, uint16_t pop_height);
void wide_list( uint8_t numrecs, uint8_t list_indx,
       uint16_t loc_x, uint16_t loc_y, uint16_t width, uint16_t height);
void display_chan_selector( uint8_t current_chan, uint16_t pos_x, uint16_t pos_y );

void display_color(  uint16_t startX, uint16_t startY, RGB_color c_color );
void display_color_adj_block( void );
void display_packet_dump( uint8_t * pk_buffer );

void blink_red ( unsigned int blinks );
void blink_green ( unsigned int blinks );
void blink_alternate( void );

void display_dmr_color( int color_code,
                       uint16_t pos_x, uint16_t pos_y, uint8_t font);
void display_dmr_slot( int slot_num,
                      uint16_t pos_x, uint16_t pos_y, uint8_t font);

void display_radio_wait( uint8_t font, uint16_t pos_x,
                         uint16_t pos_y, uint8_t spacing);

void display_radio_connected( uint8_t font, uint16_t pos_x,
                             uint16_t pos_y, uint8_t spacing);

void display_heartbeat( uint16_t pos_x, uint16_t pos_y, uint8_t count );
void display_message( char* msg, uint16_t pos_x, uint16_t pos_y, uint8_t font );
void display_RSSI_graph( void );
void draw_RXonoff( uint8_t onoff );



#endif /* CSI_SCREENS_H_ */
