//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 3, 2019             Design Documents                       CSI_SSD1963.h
//
//	       header file for SSD1963 LCD screen controller
//                    Large panel display project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//
// Status
//  written
//
//******************************************************************************
#ifndef CSI_SSD1963_H_
#define CSI_SSD1963_H_

//
// SSD1963 Utilities Functions
//
void ssd1963_write_command(uint8_t);
void ssd1963_write_data(uint8_t);
uint8_t ssd1963_read_data(void);
void ssd1963_set_port_direction(uint8_t);
void ssd1963_nop(void);
void ssd1963_soft_reset(void);
uint8_t ssd1963_get_power_mode(void);
uint8_t ssd1963_get_address_mode(void);
uint8_t ssd1963_get_display_mode(void);
uint8_t ssd1963_get_tear_effect_status(void);
void ssd1963_enter_sleep_mode(void);
void ssd1963_exit_sleep_mode(void);
void ssd1963_enter_partial_mode(void);
void ssd1963_enter_normal_mode(void);
void ssd1963_exit_invert_mode(void);
void ssd1963_enter_invert_mode(void);
void ssd1963_set_gamma_curve(void);
void ssd1963_set_display_off(void);
void ssd1963_set_display_on(void);
void ssd1963_set_column_address(uint16_t, uint16_t);
void ssd1963_set_row_address(uint16_t,uint16_t);
void ssd1963_write_memory_start(void);
void ssd1963_read_memory_start(void);
void ssd1963_set_partial_area(uint16_t, uint16_t);
void ssd1963_set_tear_off(void);
void ssd1963_set_address_mode(uint8_t);
void ssd1963_exit_idle_mode(void);
void ssd1963_enter_idle_mode(void);
void ssd1963_write_memory_continue(void);
void ssd1963_read_memory_continue(void);
void ssd1963_set_tear_scanline(uint16_t);
uint16_t ssd1963_get_scanline(uint16_t);
uint16_t ssd1963_read_device_desciptor(void);
void ssd1963_set_lcd_mode(uint8_t, uint8_t, uint16_t, uint16_t, uint8_t);
void ssd1963_set_hori_period(uint16_t,uint16_t, uint8_t, uint16_t, uint8_t);
void ssd1963_set_vert_period(uint16_t,uint16_t, uint8_t, uint16_t);
void ssd1963_set_post_proc(uint8_t, uint8_t, uint8_t, uint8_t);
uint32_t ssd1963_get_post_proc(void);
void ssd1963_set_brightness( uint8_t bright_level );

void ssd1963_set_pwm_conf(uint8_t, uint8_t, uint8_t,uint8_t, uint8_t, uint8_t);                         
void ssd1963_set_dbc_conf(uint8_t);
void ssd1963_set_dbc_th(uint32_t, uint32_t, uint32_t);
void ssd1963_set_pll(uint8_t parm1);
void ssd1963_set_pll_mn(uint8_t, uint8_t, uint8_t);
uint8_t ssd1963_get_pll_status(void);
void ssd1963_set_deep_sleep(void);
void ssd1963_set_lshift_freq(uint32_t);
void ssd1963_set_pixel_data_interface(uint8_t);

void ssd1963_set_port_direction( uint8_t );
//
// CSI_SSD1963 CHARACTER_GENERATION Funtions
//
void ssd1963_cg_initialize(void);
uint8_t ssd1963_cg_read_array(uint16_t, uint16_t);
void ssd1963_cg_read_descriptor(uint16_t, uint8_t);
void ssd1963_cg_read_char_size(uint16_t);
void ssd1963_cg_write_byte(uint8_t,uint8_t);
void ssd1963_cg_blank(uint32_t);
uint16_t ssd1963_cg_line(uint16_t, uint16_t, uint16_t, uint16_t);
void ssd1963_cg_write_progressive(uint16_t, uint16_t, uint16_t,uint16_t,
                                         uint16_t, uint16_t, uint16_t);   
void ssd1963_cg_write_centered(uint16_t, uint16_t, uint16_t, uint16_t,
                                      uint16_t, uint16_t, uint16_t, uint16_t);
void ssd1963_cg_location(uint16_t, uint16_t, uint16_t, uint16_t,uint16_t,
                                uint16_t);
extern uint16_t ssd1963_cg_next_location(uint16_t, uint16_t, uint16_t,uint16_t);  
extern uint16_t ssd1963_cg_output_character(uint8_t ,uint8_t, uint16_t,uint16_t,
                                            uint16_t);
void ssd1963_cg_write_line(uint8_t,uint8_t,uint16_t, uint16_t, uint16_t);
void ssd1963_cg_clear_line(uint16_t, uint16_t,  uint16_t, uint16_t);
void ssd1963_cg_clear_display(void);
void ssd1963_cg_clear_display_fast(void);
void ssd1963_cq_write_horizontal_line(uint16_t, uint16_t, uint16_t);
void ssd1963_cq_clear_horizontal_line(uint16_t, uint16_t, uint16_t);
extern uint16_t ssd1963_cq_calculate_center(uint16_t,uint16_t);
void ssd1963_cq_display_center(uint16_t, uint16_t, uint16_t); 
                                   
//
// SSD1963 variables
//
extern uint16_t cg_space_above; 
extern uint16_t cg_space_below; 
extern uint16_t cg_space_before;
extern uint16_t cg_space_after; 
extern uint16_t cg_max_hori_size;
extern uint16_t cg_max_vert_size;
extern uint16_t cg_hori_size;
extern uint16_t cg_next_address;

extern uint16_t cg_hori_location;
extern uint16_t cg_vert_location;

extern uint8_t  cg_line_buffer[32];

#endif /* CSI_SSD1963_H_ */

