//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// July 9, 2018 		Design Documents      	          CSI_switches.h
//
//                 Header for switch routines
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  07/09/2018 moved header info from CSI_RAM and CSI_FUNCTIONS
//
// Status
//  writing
//
//******************************************************************************
#ifndef CSI_SWITCHES_H_
#define CSI_SWITCHES_H_

#include <stdint.h>
#include "CSI_PROCESS_CS800D.h"

// each button has properties 
// the whole list of buttons in a table
// a separate table for each radio

// each button
typedef struct 
{
  uint8_t send_to_radio;   // to indicate button press will be sent
  uint8_t key_number_1;
  uint8_t key_number_2;
  uint8_t key_type;        // 0-1 for a press release type. 2-3 for knob
  uint8_t press_flag;
  uint32_t release_flag;  // 32 bit so address can be used in short timer
}panel_button;

extern panel_button CS800_buttons[14];

// button controls for touch screen
extern uint8_t channel_up_button;
extern uint8_t channel_up_longp;
extern uint8_t channel_dn_button;
extern uint8_t channel_dn_longp;
extern uint8_t volume_up_button;
extern uint8_t volume_up_longp;
extern uint8_t volume_dn_button;
extern uint8_t volume_dn_longp;


//
// SWITCH Functions
//
void process_switch_s1(void);
void process_switch_s2(void);
void process_switch_s3(void);
void process_switch_s4(void);
void process_switch_s5(void);
void process_switch_s6(void);
void process_encoder_1(void);
void FT857D_encoder_right(void);
void process_encoder_2(void);
void FT857D_encoder_left(void);
void process_ptt(void);
void process_switches(void);
void setup_keypad_switches(void);

void volume_updown( int updown );
void channel_updown( int updown );
void knob_nav_control( uint8_t updown );

void short_press_timer( void );

extern uint32_t last_key_press;
extern int short_press_timeout;
extern uint32_t *short_release_flag;
extern int short_press_holdoff;

void deactivate_keypress( uint8_t keynum );
void deactivate_all_keypress( void );
void activate_keypress( uint8_t keynum );
void activate_all_keypress( void );

void FB_button_write( void );
void FB_button_read( void );
int FB_button_compare( FB_KeysProgFunctionDef_t* FB_a,
                       FB_KeysProgFunctionDef_t* FB_b );
void get_button_assign( void );
uint8_t radio_to_panel_button( uint8_t btn );


//
// SWITCH Variables
//
extern uint32_t FT857D_key_released;
extern uint32_t FT857D_left_encoder_switch;
extern uint32_t FT857D_right_encoder_switch;
extern uint32_t FT857D_p1_switch;
extern uint32_t FT857D_p2_switch;
extern uint32_t FT857D_p3_switch;
extern uint32_t FT857D_p4_switch;

extern uint32_t FT857D_encoder_rt_cw;
extern uint32_t FT857D_encoder_rt_ccw;
extern uint32_t FT857D_encoder_lft_cw;
extern uint32_t FT857D_encoder_lft_ccw;

extern uint32_t ptt_state;
extern uint32_t ptt_debounce_timer;
extern uint32_t ptt_timer;

extern uint32_t key_number_1;
extern uint32_t key_number_2;
extern uint32_t key_type;
extern uint32_t key_active_flag;
extern uint32_t key_release;

extern uint8_t r_knob_for_nav;

#endif /* CSI_SWITCHES_H_ */