//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 7, 2018             Design Documents                    CSI_TouchPanel.h
//
//			header file for Touch Panel
//                     Large panel display project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  06/7/2018 
//
// Status
//  writing
//
//******************************************************************************

#ifndef CSI_TOUCHPANEL_H_
#define CSI_EEPROM_H_

#include <stdint.h>

extern uint8_t TS_touch_count;
extern uint8_t volatile TS_status;
extern uint8_t volatile TS_touch_ready;
extern int TS_waitcount;

typedef struct
{
  uint8_t   id;  // touch id
  uint16_t  x_coord; 
  uint16_t  y_coord;
  uint16_t  size;
} Touch_info;

typedef struct
{
  uint8_t   is_active; 
  uint8_t   touch;
  uint8_t   tp_state;
  uint16_t  xmin; 
  uint16_t  xmax;
  uint16_t  ymin;
  uint16_t  ymax;
} Touch_zone;

extern Touch_info Panel_Touch[5];

extern Touch_zone CS800D_touch_point[66];
// extern const uint16_t FT857D_touch_point[18];

void CS800D_process_touches( void );
void FT857D_process_touches( void );
void FT857D_mechanical_switches(void);
void  FT857D_touch_state_program(uint32_t cur_y, uint32_t cur_x, uint32_t i);

void all_touches_off( void );

void TouchPanel_int( uint8_t hilo );
void TouchPanel_reset( uint8_t hilo );
void TouchPanel_init( void );
void TouchPanel_disable( void );
void TouchPanel_interrupt( uint8_t onoff );
uint8_t TouchPanel_read_register( uint16_t regAddr );
void TouchPanel_read_multi( uint16_t regAddr, uint8_t numbytes );
void TouchPanel_product_ID( void );
int TouchPanel_waitfor_I2C_RX( void );
int TouchPanel_waitfor_I2C_TX( void );
void TouchPanel_read_touch1( void );
void TouchPanel_command ( uint8_t command );
void TouchPanel_write ( uint16_t regnum, uint8_t data );
void TouchPanel_get_touch_data( uint8_t );
uint8_t TouchPanel_read_touch_buffer( void );

void touch_state( void );

void TouchPanel_process( void );

// int TouchPanel_config_check( void );

// register per GT911 programming guide, firmware V1040 or later
// 2 byte register data is stored lowbyte in lower number register then high

// These are the only registers that are used now
#define   GT911_Command            0x8040
#define   GT911_Config_Version     0x8047
#define   GT911_Product_ID         0x8140   // 4 bytes 
#define   GT911_Status             0x814E 

// The whole list for reference
/*
// Command registers, write only
#define   GT911_Command            0x8040
#define   GT911_ESD_Check          0x8041
#define   GT911_Command_check      0x8046

// config registers
#define   GT911_Config_Version     0x8047
#define   GT911_X_out_max          0x8048    // 2 bytes
#define   GT911_Y_out_max          0x804A    // 2 bytes
#define   GT911_Touch_Number       0x804C  
#define   GT911_Module_SW1         0x804D
#define   GT911_Module_SW2         0x804E
#define   GT911_Shake_count        0x804F
#define   GT911_Filter             0x8050
#define   GT911_Large_touch        0x8051
#define   GT911_Noise_reduc        0x8052
#define   GT911_Scr_touch_lvl      0x8053
#define   GT911_Scr_leave_lvl      0x8054
#define   GT911_Low_pwr_ctrl       0x8055
#define   GT911_Refresh_rate       0x8056
#define   GT911_x_thresh           0x8057
#define   GT911_y_thresh           0x8058
#define   GT911_X_speed_limit      0x8059  // reserved
#define   GT911_Y_speed_limit      0x805A  // reserved
#define   GT911_Space_top_bot      0x805B
#define   GT911_Space_L_R          0x805C
#define   GT911_Mini_filter        0x805D
#define   GT911_Stretch_R0         0x805E
#define   GT911_Stretch_R1         0x805F
#define   GT911_Stretch_R2         0x8060
#define   GT911_Stretch_RM         0x8061
#define   GT911_Drive_groupA       0x8062
#define   GT911_Drive_groupB       0x8063
#define   GT911_Sensor_num         0x8064
#define   GT911_FreqA_factor       0x8065
#define   GT911_FreqB_factor       0x8066
#define   GT911_Panel_BitFreq      0x8067   // 2 bytes
#define   GT911_Panel_Sens_timeL   0x8069   // reserved 
#define   GT911_Panel_Sens_timeH   0x806A   // reserved 
#define   GT911_Panel_TX_gain      0x806B
#define   GT911_Panel_RX_gain      0x806C
#define   GT911_Panel_dump_shift   0x806D 
#define   GT911_Drv_frame_ctrl     0x806E
#define   GT911_Drv_Charg_lvl_Up   0x806F
#define   GT911_Module_Switch3     0x8070
#define   GT911_Gesture_dist       0x8071
#define   GT911_Gest_lng_press_t   0x8072
#define   GT911_XY_slope_adj       0x8073
#define   GT911_Gesture_ctrl       0x8074
#define   GT911_Gesture_Switch1    0x8075
#define   GT911_Gesture_Switch2    0x8076
#define   GT911_Gest_refresh       0x8077
#define   GT911_Gest_touch_lvl     0x8078
#define   GT911_New_grn_wake_levl  0x8079
#define   GT911_Freq_hop_start     0x807A
#define   GT911_Freq_hop_end       0x807B
#define   GT911_Noise_det_time     0x807C
#define   GT911_Hop_flag           0x807D
#define   GT911_Hop_thresh         0x807E
// 0x807F (reserved)
#define   GT911_Noise_min_thresh   0x8080
// 0x8081 (reserved)
#define   GT911_Hop_sensor_group   0x8082
#define   GT911_Hop_seg1_normalize 0x8083
#define   GT911_Hop_seg1_Factor    0x8084
#define   GT911_Main_clock_adj     0x8085
#define   GT911_Hop_seg2_normalize 0x8086
#define   GT911_Hop_seg2_Factor    0x8087
// 0x8088 (reserved)
#define   GT911_Hop_seg3_normalize 0x8089
#define   GT911_Hop_seg3_Factor    0x808A
// 0x808B (reserved)
#define   GT911_Hop_seg4_normalize 0x808C
#define   GT911_Hop_seg4_Factor    0x808D
// 0x808E (reserved)
#define   GT911_Hop_seg5_normalize 0x808F  
#define   GT911_Hop_seg5_Factor    0x8090
// 0x8091 (reserved)
#define   GT911_Hop_seg6_normalize 0x8092  
#define   GT911_Key1               0x8093
#define   GT911_Key2               0x8094
#define   GT911_Key3               0x8095
#define   GT911_Key4               0x8096
#define   GT911_Key_area           0x8097
#define   GT911_Key_Touch_lvl      0x8098
#define   GT911_Key_Leave_lvl      0x8099
#define   GT911_Key_sens_12        0x809A
#define   GT911_Key_sens_34        0x809B
#define   GT911_Key_restrain       0x809C
#define   GT911_Key_restr_time     0x809D
#define   GT911_Gesture_lg_touch   0x809E
// 0x809F (reserved)
// 0x80A0 (reserved)
#define   GT911_Hotknot_nois_map   0x80A1
#define   GT911_link_threshold     0x80A2
#define   GT911_Pxy_threshold      0x80A3
#define   GT911_GHot_dump_shift    0x80A4
#define   GT911_GHot_Rx_gain       0x80A5
#define   GT911_freq_gain0         0x80A6
#define   GT911_freq_gain1         0x80A7
#define   GT911_freq_gain2         0x80A8
#define   GT911_freq_gain3         0x80A9
// 0x80AA-0x80B2 (reserved)
#define   GT911_Combine_dist       0x80B3
#define   GT911_Split_set          0x80B4
// 0x80B5 (reserved)
// 0x80B6 (reserved)
// ITO sensor chip channel number, 14 bytes 0x80B7 to 0x80C4
#define   GT911_Sensor             0x80B7
// 0x80C5-0x80D4 (reserved)
// ITO driver chip channel number, 26 bytes 0x80D5 to 0x80EE
#define   GT911_Driver             0x80D5
// 0x80EF-0x80FE (reserved)
#define   GT911_Config_chksum      0x80FF
#define   GT911_Config_fresh       0x8100

// coordinate information registers

#define   GT911_Product_ID         0x8140   // 4 bytes 
#define   GT911_Firmware_ver       0x8144   // 2 bytes
#define   GT911_X_coord_res        0x8146   // 2 bytes
#define   GT911_Y_coord_res        0x8148   // 2 bytes
#define   GT911_Vendor_ID          0x814A
// 0x814B-0x814D (reserved)
#define   GT911_Status             0x814E                

#define   GT911_track_0id          0x814F   // hot knot detect
#define   GT911_PxyOK1             0x8150
#define   GT911_PxyOK2             0x8151
// 0x8152-0x8156 (reserved)

#define   GT911_track_1id          0x8157 
#define   GT911_point_1x           0x8158   // 2 bytes
#define   GT911_point_1y           0x815A   // 2 bytes
#define   GT911_point_1size        0x815C   // 2 bytes
// 0x815E (reserved)
#define   GT911_track_2id          0x815F 
#define   GT911_point_2x           0x8160   // 2 bytes
#define   GT911_point_2y           0x8162   // 2 bytes
#define   GT911_point_2size        0x8164   // 2 bytes
// 0x8166 (reserved)
#define   GT911_track_3id          0x8167 
#define   GT911_point_3x           0x8168   // 2 bytes
#define   GT911_point_3y           0x816A   // 2 bytes
#define   GT911_point_3size        0x816C   // 2 bytes
// 0x816E (reserved)
#define   GT911_track_4id          0x816F 
#define   GT911_point_4x           0x8170   // 2 bytes
#define   GT911_point_4y           0x8172   // 2 bytes
#define   GT911_point_4size        0x8174   // 2 bytes
// 0x8176 (reserved)
#define   GT911_track_5id          0x8177 
#define   GT911_point_5x           0x8178   // 2 bytes
#define   GT911_point_5y           0x817A   // 2 bytes
#define   GT911_point_5size        0x817C   // 2 bytes
// 0x817E (reserved)
#define   GT911_KeyValue           0x819F
*/

#endif /* CSI_TOUCHPANEL_H_ */