//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 27, 2018             Design Documents                              uart.h
//
//			header file for UART
//                        for STM32F405
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  03/27/2018 copied from CS700 for STM32F7
//  01/15/2019 merged with UART2.h and uart.h
//  06/25/2019 removed extern variables that are not used externally
//  10/08/2019 revised uart initialize to work with multiple radios
//
// Status
//  writing revision
//
// *****************************************************************************
#ifndef UART_H_
#define UART_H_
// *****************************************************************************
// functions for USART 1 
// *****************************************************************************
void uart1_initialize( uint8_t radio_type );

void uart_FT857D_send_0xA5(void);
void USART1_Handler(void);
void USART1_CS800D_Handler(void);
void USART1_FT857D_Handler(void);

void uart1_CS800D_receive_interrupt(void);
void uart1_CS800D_receive_interruptB(void);

void uart1_CS800D_transmit_interrupt(void);
void uart1_FT857D_receive_interrupt(void);
void uart1_FT857D_receive_sniffing(void);
void uart1_FT857D_transmit_interrupt(void);
void uart1_FT857D_send_0xA5(void);
void uart_FT857D_send_0xA5(void);

uint8_t FT857D_uart1_isready( void );

void uart1_tx_coordinate(void);
void set_UART1TXRX_to_GPIO (void);
void set_UART1TXRX_to_UART (void);
extern uint8_t get_UART1RX(void);
// *****************************************************************************
// Functions for UART2
// *****************************************************************************
void USART2_Handler(void);
void uart2_FT857D_receive_interrupt(void);
void uart2_FT857D_transmit_interrupt(void);
void uart2_FT857D_send_0xA5(void);
void uart2_FT857D_initialize(void);
// *****************************************************************************
// global pointers for USART1, RCC, and GPIO
// *****************************************************************************
extern uint32_t volatile *USART1_SR;
extern uint32_t volatile *USART1_DR;
extern uint32_t volatile *USART1_CR1;  
extern uint32_t volatile *USART1_CR2;
extern uint32_t volatile *USART1_CR3;
extern uint32_t volatile *USART1_BRR;

extern uint32_t volatile *USART2_SR;
extern uint32_t volatile *USART2_DR;
extern uint32_t volatile *USART2_CR1;
extern uint32_t volatile *USART2_CR2;
extern uint32_t volatile *USART2_CR3;
extern uint32_t volatile *USART2_BRR;
  
extern uint32_t volatile *RCC_APB2ENR;
extern uint8_t  uart1_good_data;

extern uint8_t uart2_rx_checksum;
extern uint8_t uart2_tx_checksum;
// *****************************************************************************
// Defines for UART register bit setting
// *****************************************************************************

#define TXE                 ((uint32_t)0x1 <<7)
#define TC                  ((uint32_t)0x1 <<6)
#define RXNE                ((uint32_t)0x1 <<5)
#define IDLE                ((uint32_t)0x1 <<4)
#define ORE                 ((uint32_t)0x1 <<3)

#define RE                  ((uint32_t)0x1 <<2)
#define TE                  ((uint32_t)0x1 <<3)
#define IDLEIE              ((uint32_t)0x1 <<4)
#define RXNEIE              ((uint32_t)0x1 <<5)
#define TCIE                ((uint32_t)0x1 <<6)
#define TXEIE               ((uint32_t)0x1 <<7)
#define UE                  ((uint32_t)0x1 <<13)
// uart1, bit 4 of APB2ENR uart2, bit 17 of APB1ENR
#define RCC_APB2ENR_UART1EN ((uint32_t)0x00000010) // 0x1 <<4
#define RCC_APB1ENR_UART2EN ((uint32_t)0x00020000) // 0x1 <<17
// *****************************************************************************
// Common buffers and pointers between UART1 and UART2
// *****************************************************************************
extern uint8_t  FT857D_tx_buffer_index;
extern uint8_t  FT857D_tx_buffer[];
extern uint8_t  FT857D_rx_buffer_index;
extern uint8_t  FT857D_rx_buffer[];
extern uint32_t FT857D_rx_fifo_input_pointer;
extern uint32_t FT857D_rx_fifo_output_pointer;
extern uint8_t  FT857D_rx_fifo[];
// *****************************************************************************
// UART1 FT857D Variables
// *****************************************************************************
extern uint32_t TC_flag;
extern uint32_t FT857D_auto_init_trigger;
extern uint32_t FT857D_uart1_tx_flag;
// *****************************************************************************
// UART1 CS800D Variables
// *****************************************************************************
#define UART_DATA_BUF_SIZE 256  
extern uint8_t uart1_rx_buffer[UART_DATA_BUF_SIZE];
extern uint8_t uart1_tx_buffer[UART_DATA_BUF_SIZE];
//
// UART1 Variables
//
extern uint32_t uart1_rx_buffer_index;
extern uint32_t uart1_rx_watch_dog_timer;
extern uint32_t uart1_rx_flag;
extern uint8_t  key_buffer[32];
extern uint32_t key_buffer_flag; 
extern uint8_t  ack_buffer[32]; 
extern uint32_t ack_buffer_flag;
extern uint32_t ack_buffer_delay_flag;
// 
// UART2 Variables
//

#endif /* UART_H_ */
