//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
///
// Mar 30, 2018			Design Documents               CSI_diagnostics.c 
// 
//           Diagnostic tests and board checkout routines
//                 for large display project
//
//  These were used during development, but many are not used now
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  03/30/2018  started
//
// Status:  written, tested
//
//==============================================================================
// NAME OF ROUTINE                      Function
//
//  test_pin                        toggle test pin GPIO A-9
//  keypad_backlight                tests mic keypad backlight
//  screen_backlight                tests LCD screen backlight
//  cpu_clocks                      sends clock to clock ouutput pin
//  systick_check                   sends system tick to test pin 
//  keypad_check                    checks all buttons on the mic keypad
//  Rotary_enc_test                 tests rotary encoder 1 or 2
//  LCD_pins_test                   sends data to LCD 
//  LCD_controller_check            reads device data block of SSD1963
//  UART1_transmit_test             sends test data to UART1 
//  uart2_diagnostic                sends test data to UART2 for FT857
//  clock_nop_test                  test to measure clock
//  LCD_color_test                  displays color blocks to test LCD
//  LCD_font_test                   displays 95 characters of a font
//  LCD_rssiIcon_test               tests rssi icon bars
//  LCD_batteryIcon_test            tests battery icon bars
//  diagnostic                      runs a chosen test routine
//  set_clock_out                   sets up clock out assignement to oupput pin
//  I2C_EEPROM_test                 tests writes to EEPROM without ISR
//  I2C_EEPROM_test2                tests writes to EEPROM using ISR
//  SPI_FLASH_test                  reads mfr id of flash chip
//  process_board_tests             Board test routine, runs panel diagnostics
//  Board_I2C_EEPROM                board test, multiple tests of EEPROM
//  Board_Flash_mfrid               board test, Flash chip mfr id
//  Board_Flash_uniqueid            board test, Flash chip unique id
//  Board_Flash_RW                  board test, Flash chip read, write
//  clear_message_area              board test, clears screen message area
//  Board_switches                  board test, panel buttons
//  Board_Lknob_switch              board test, left knob button
//  Board_Rknob_switch              board test, right knob button
//  Board_Lknob_rotate              board test, left knob rotation
//  Board_Rknob_rotate              board test, right knob rotation
//  Board_PTT                       board test, mic PTT
//  Board_allMICkeys                board test, all mic keypad keys
//  Board_MICkeypad                 board test, mic kepad test 1 & 9
//  Board_Touchscreen               board test, a single touch anywhere
//  Board_Volume                    board test, checks radio volume adjustment
//  Board_Channel                   board test, checks radio channel change
//  Board_UART1                     board test, UART1 with loopback connector
//  Board_UART2                     board test, Uart2 with loopback connector
//  Board_USB1                      board test, USB1 with loopback connector
//  Board_USB2                      board test, USB2 with loopback connector

// test to write: 
// touch panel calibration to verify x,y coordinates

#include <string.h>
#include <stdio.h>
#include "CSI_diagnostics.h"
#include "CSI_REGISTERS.h"
#include "CSI_SSD1963.h"
#include "CSI_MC59.h"
#include "CSI_GPIO.h"
#include "CSI_UART.h"
#include "CSI_i2c.h"
#include "CSI_eeprom.h"
#include "CSI_spi.h"
#include "CSI_ExtFlash.h"
#include "CSI_COLOR.h"
#include "CSI_GRAPHICS.h"
#include "CSI_SCREENS.h"
#include "CSI_SWITCHES.h"
#include "CSI_TouchPanel.h"
#include "CSI_PROCESS_CS800D.h"
#include "CSI_TIMERS.h"
#include "CSI_ERROR_LOG.h"

extern unsigned long seconds;

extern void set_clock_out( int );
extern void test_pin( void );

//========================================================
//
//  toggle test pin GPIO A-9 pin 101
//     changes output pin each call
//

void test_pin( void )
{
int static toggle;
  
  toggle ^= 1;
  if(toggle) *GPIO_A_BSRR = 0x00000200; // BIT09_ON
  else *GPIO_A_BSRR = 0x02000000;   // BIT09_OFF
}

//========================================================
//
// keypad backlight test
//
// diagnostic 3

 void keypad_backlight ( unsigned int msec )
{
  key_backlight_on();
  delay( msec * 1000);
  key_backlight_off();
} 
//========================================================
//
// lcd backlight test
//
// diagnostic 4

 void screen_backlight ( unsigned int timesec )
{
  while (timesec--)
  {
    lcd_backlight_on();
    delay( 1000000 ); 
    lcd_backlight_off();
    delay( 1000000 );
  }
}
//========================================================
//
// send CPU clocks to test pin, verify frequencies
//
//  used during debugging
// diagnostic 5

void cpu_clocks( void )
{
  //set_clock_out( 0 );  // sysclk
  //msec_delay( 5000 ); 
  //set_clock_out( 1 );  // PLLI2S 
  //msec_delay( 5000 ); 
  //set_clock_out( 2 );  // HSE
  //msec_delay( 5000 ); 
  set_clock_out( 3 );  // PLL
  //msec_delay( 5000 ); 
  // restore
  //uint32_t volatile *GPIOC_MODER = (uint32_t *) GPIOC_MODER_ADDRESS;  
  //uint32_t volatile *GPIOC_OSPEEDR = (uint32_t *) GPIOC_OSPEEDR_ADDRESS;  
  
  //*GPIOC_MODER = 0x56944000;
  //*GPIOC_OSPEEDR = 0x02C02000;  
}
//========================================================
//
// send systick timer interrupt outputs to test pin
//  test pin is GPIO A-8  pin 100 - unconnected
//
// can put test_pin(); into timer based interrupt handlers
//  to check different rates
  
// diagnostic 6
// didn't run the test this way. Put test_pin() into systick ISR 
void systick_check( int msec )
{
  while(1)
  {
    if(millisec % msec) test_pin();
  }
}
//========================================================
//
// keypad test, confirm all buttons
//
// 
// diagnostic 7

// extern volatile uint32_t mc59_key;
//
// Status written, untested

uint8_t keylist[17] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void keypad_check( void )
{
  int i;
  int sum = 0;
  // after all 17 have been pressed the sum will be 17
  while( sum < 17 )
  {
     // mc59_read_keypad() should be running in ISR
     keylist[mc59_key] = 1;
     sum = 0;
     for(i=0;i<17;i++) sum += keylist[i]; 
  }
}
//========================================================
//
// rotary encoder test
//
//  s7_flag indicates encoder 1 turning clockwise
//  s8_flag indicates encoder 1 turning counter clockwise
//  s9_flag indicates encoder 2 turning clockwise
//  s10_flag indicaes encoder 2 turning counter clockwise
//
// diagnostic 8 (1)
// diagnostic 9 (2)

extern uint32_t s7_flag;  // in CSI_RAM.h 
extern uint32_t s8_flag;  // in CSI_RAM.h
uint32_t s9_flag;  // not in CSI_RAM.h !!!
uint32_t s10_flag; // not in CSI_RAM.h !!!

uint8_t Rotary_enc_test( uint8_t rencoder )
{
    uint8_t direction = 0;
    
    if(rencoder == 1)
    {
      if(CS800_buttons[7].press_flag == 1) direction = 11;
      if(CS800_buttons[8].press_flag == 1) direction = 12;
    }
    if(rencoder == 2)
    {
      if(CS800_buttons[9].press_flag == 1)  direction = 21;
      if(CS800_buttons[10].press_flag == 1) direction = 22;
    }
    // if no flags direction will be 0
    
    return direction;  
}


// display tests
//========================================================
//
//  Test LCD pins, parallel data and serial port
//
//  use scope to look at pins
//
void LCD_pins_test( unsigned int testime ) 
{
   uint8_t counter = 0;
   uint8_t toggle = 0;
   
   while(testime)
   {
     toggle ^= 1;

     if(toggle)
     {
        // reset
       lcd_reset_high();
       // serial port
       lcd_cs_high();
       lcd_d_c_high();
       lcd_rd_high();
       lcd_wr_high();
       // parallel port
       write_lcd_port( 0xFF );
       //write_lcd_port( counter );
     }
     else
     {
       lcd_reset_low();
       lcd_cs_low();
       lcd_d_c_low();
       lcd_rd_low();
       lcd_wr_low();
       write_lcd_port( 0x00 );       
     }
     if( counter++ == 0 ) testime--;  // 256 msec * 1000 ~ 4.2 min
     delay( 1000 );   // 1 K hz
   };
}

//========================================================
//
// LCD controller test
//    reads device data block of the ssd1963 LCD controller
//    tests Uart communication in both directions, send command
//    and receive response.
// 
extern uint8_t ssd1963_read_ddb( void );


uint8_t LCD_controller_check( void )
{
   return ssd1963_read_device_desciptor();
}
//==============================================================================
//
//   UART transmit test 
//     UART 1 must be initialized by initialize() 
//     change commented out section to modify output
//     Needs ISR written to use uart_tx_buffer[]
//
//
//  Status
//    Written, tested, working - only used for test
//     Broken, UART1_send_tx_data() was a function to start transmitting.
//       the number was the number of bytes to transmit.    
//       Jerry renamed it to a macro that just writes one byte to the
//       uart data reg
/* 
void UART1_transmit_test( void )
{
  // fill transmit buffer with 256 bytes
  int i;
  for(i=0; i< 256; i++) uart1_tx_buffer[i] = (uint8_t) i; 
  //for(i=0; i< 256; i++) uart_tx_buffer[i] = 0xFF;
 
  i  = 200;
  while(i--)
  {
    // send first byte and number of bytes
    UART1_send_tx_data( 256 );
    // ISR should do the rest...
    delay( 1000000 );  
  }
}
*/
// =============================================================================
//
//  Uart 2 diagnostic test for FT857 connection
//
// place a call in main for debugging

// 05/07/2019  Jerry Wanger
//  
void uart2_diagnostic( void )
{
  uart2_FT857D_send_0xA5();
  
  while(1)
  {
    if(ms_1_timer == 0 )
    {
      ms_1_timer = 100;
      *USART2_DR = 0x55;
    }
  }
}
//==============================================================================
//
//   Clock test loop with NOP timing for clock rate checking 
//     measure on test pin A9
//
void clock_nop_test( void )
{
  int i = 10000;
  while (i--)
  {
    test_pin();
    delay( 500 );  // delay in micro seconds, clock should be 1 Khz
  }
  
}
//==============================================================================
//
//  LCD Color test
//
//
void LCD_color_test( void )
{
  int i;
  uint32_t hpoz = 0;
  uint32_t vpoz = 0;
  uint8_t colornum = 0;
  
  for( colornum = 0 ; colornum < 16; colornum++)
  {
    ssd1963_cg_location( 0, hpoz, vpoz, 100, 100, 100);
   
    for( i = 0; i < 10000; i++)
    {
      // write block of color
      ssd1963_write_data( standard_palette[colornum].red );
      ssd1963_write_data( standard_palette[colornum].green );    
      ssd1963_write_data( standard_palette[colornum].blue );
    }     
    hpoz += 100;
    if(hpoz > 700 )
    {
      hpoz = 0;
      vpoz += 100;
    }
  }
  // next pallette
  for( colornum = 0 ; colornum < 16; colornum++)
  {
    ssd1963_cg_location( 0, hpoz, vpoz, 100, 100, 100);
   
    for( i = 0; i < 10000; i++)
    {
      // write block of color
      ssd1963_write_data( rainbow_palette[colornum].red );
      ssd1963_write_data( rainbow_palette[colornum].green );    
      ssd1963_write_data( rainbow_palette[colornum].blue );
    }     
    hpoz += 100;
    if(hpoz > 700 )
    {
      hpoz = 0;
      vpoz += 100;
    }
  }

}
//   fonts
//   verify character sets
//     must run cg_initialize first
//
//  01/28/2019 modified to use font number, and max sizes for spacing
//
void LCD_font_test( uint8_t font_num )
{
  cg_font.red = 255;
  cg_font.green = 255;
  cg_font.blue = 255;
  
  cg_background.red = 20;
  cg_background.green = 20;
  cg_background.blue = 100;
  
  ssd1963_cg_clear_display();
  
  // set font background (not the same as background to show spacing gaps)
  cg_background.red = 0;
  cg_background.green = 0;
  cg_background.blue = 0;
  
  uint8_t testchar = ' ';
  uint32_t horizpos = 5;
  uint32_t vertpos =  5;
  uint32_t horizstep;
  
  ssd1963_cg_read_char_size( font_num );
  
  horizstep = cg_max_hori_size;

  // display font 
  int i;
  for( i =0;i < 95; i++)
  {
    ssd1963_cg_output_character( testchar + i, font_num,  horizpos, vertpos, 0); 
    horizpos += horizstep;
    if(horizpos > 740)
    {
      horizpos = 5;
      vertpos += cg_max_vert_size;
    }
  }

}
// =============================================================================
//
//   check rssi icon bar display

void LCD_rssiIcon_test( void )
{
  draw_rssi_icon( 0, 690, 15 );
  msec_delay( 1000 );
  draw_rssi_icon( 1, 690, 15 );
  msec_delay( 1000 );
  draw_rssi_icon( 2, 690, 15 );
  msec_delay( 1000 );
  draw_rssi_icon( 3, 690, 15 );
  msec_delay( 1000 );
  draw_rssi_icon( 4, 690, 15 );
}

void LCD_batteryIcon_test( void )
{
  draw_battery_icon( 0 );
  msec_delay( 1000 );  
  draw_battery_icon( 1 );
  msec_delay( 1000 );  
  draw_battery_icon( 2 );
  msec_delay( 1000 );  
  draw_battery_icon( 3 );
  msec_delay( 1000 );  
  draw_battery_icon( 4 );
  msec_delay( 1000 );  
  draw_battery_icon( 5 );
}





//==============================================================================
//
//   run a selected diagnostic 
//    

void diagnostic( int dnumber )
{
  switch(dnumber)
  {
  case 1: blink_red( 10 ); break;
  case 2: blink_green( 10 ); break;
  case 3: keypad_backlight( 500 ); break;  
  case 4: screen_backlight( 10 ); break;    
  case 5: cpu_clocks(); break;
  case 6: systick_check( 500 );break;
  case 7: keypad_check();break;
  case 8: Rotary_enc_test( 1 );break;
  case 9: Rotary_enc_test( 2 );break;
  case 10: LCD_pins_test( 1000 );break;
  case 11: LCD_controller_check();break;
  case 12: clock_nop_test();break;
  case 13: UART1_transmit_test();break;
  case 14: LCD_color_test();break;
  case 15: LCD_font_test( 4 );break;
  case 16: LCD_rssiIcon_test(); break;
  case 17: LCD_batteryIcon_test(); break;
  
  default: break;  
  }
}
//******************************************************************************
//
//  !! PROTOTYPE TESTING ONLY !!
//  Direct clock output to pin for testing
//
//  Ref. Manual says to only do this before turning on HSE and PLL
//
//  MCO1 can output HSI, LSE, HSE, PLL
//  MCO2 can output HSE, PLL, SYSCLK, PLLI2S     
//    will use MCO2. It's available on GPIO C 9
//
void set_clock_out( int clock )
{
  uint32_t volatile *RCC_CFGR = (uint32_t *) RCC_CFGR_ADDRESS;
  uint32_t volatile *RCC_CR   = (uint32_t *) RCC_CR_ADDRESS;
  // set bits in RCC clock configuration register
  //  bits 30,31 00 for system clock
  //             01 PLLI2S 
  //             10 HSE
  //             11 PLL
  // Input variable is used to select which clock 0-3
  *RCC_CFGR &= 0x3FFFFFFF;  // clear 30,31 it's set to sysclock
  if(clock == 1) *RCC_CR |= 0x04000000;  // bit 26 PLLI2S on
  else *RCC_CR &= ~0x04000000;
  if(clock > 0)*RCC_CFGR |= clock << 30; // set to other clock value
  
  // output has a prescaler which can divide output. bit 27 - 29
  // maximum clock output is 100Mhz must divide to use sysclck at 168
  // set to 000, divide by 1  0x00
  // set to 100, divide by 2  0x20
  // set to 101, divide by 3  0x28
  // set to 110, divide by 4  0x30
  // set to 111, divide by 5  0x38
  //*RCC_CFGR &= ~0x38000000; // clear bits if not setting all of them
  *RCC_CFGR |= 0x38000000;  // 168/5 = 33.6 Mhz 
  // *RCC_CFGR |= 0x20000000;  // 168/2 = 84 Mhz
  // Leave at 0 for no division
  // set gpio port C 9 for MCO2 is Alt function 0
  //  GPIO mode was  0x00010000 set to 0x00090000
  //  GPIO speed was 0x00000000 leave 
  uint32_t volatile *GPIOC_MODER = (uint32_t *) GPIOC_MODER_ADDRESS;  
  //uint32_t volatile *GPIOC_OSPEEDR = (uint32_t *) GPIOC_OSPEEDR_ADDRESS;  
  
  *GPIOC_MODER = 0x00090000;
  //*GPIOC_OSPEEDR = 0x000C0000;
  
  // reset cpu to restore normal operation 
 
}
// =============================================================================
//
// read and write to eprom
//
// written for single byte random 
// uses I2C module without interrupts

// Status
//     written, tested, working. Don't use

void I2C_EEPROM_test ( void )
{
  uint8_t block = 0;
  uint8_t add_H_byte = 1;
  uint8_t add_L_byte = 21;
  
  // test sending one byte
  EEPROM_MemWrite( block, add_H_byte, add_L_byte++, 0xAA);  
  EEPROM_MemWrite( block, add_H_byte, add_L_byte++, 0xFE); 
  EEPROM_MemWrite( block, add_H_byte, add_L_byte, 0x30); 
  
  // get data from I2C
  EEPROM_MemRead( block, 1, 21, 3 );  
}
// =============================================================================
//
//  Test I2C and eeprom chip using ISR
//
void I2C_EEPROM_test2( void )
{
    //  address
    uint8_t block = 0;
  
    // test multiple
    // put address in output buffer
    I2C_put_dataout( 0x00 );  
    I2C_put_dataout( 0x26 );
    // put some data in the output buffer
    I2C_put_dataout( 0x03 );
    I2C_put_dataout( '.' );
    I2C_put_dataout( 0x01 );
    I2C_put_dataout( 0x04 );
    I2C_put_dataout( 0x01 );
    I2C_put_dataout( 0x06 );

    // count needs to include control byte
    EEPROM_MultipleWrite( block, 9);
    // wait for I2C to finish
    //while(!I2C_2_txrx_complete);
    EEPROM_waitfor_I2C();
    // wait for EEPROM write to finish    
    I2C_2_EEPROM_busypoll( I2C_eprom );

    // read multiple
    EEPROM_MemRead( block, 0x00, 0x26,  6);
    // wait for I2C to finish
    while(!I2C_2_txrx_complete);
}
// =============================================================================
//
//  Test SPI and FLASH chip
//
uint32_t SPI_FLASH_test( void )
{
  uint32_t mfrid;
  //uint32_t jdecid;
  //uint64_t uniqueid;
  // with interrupts
  mfrid = flash_mfrid();
  //jdecid = flash_jdecid();
  //uniqueid = flash_uniqueid();

  return mfrid;  
}
// =============================================================================
// =============================================================================
//
//   Tests for board checkout
//
//
// 
// =============================================================================
//
//  Process all board tests
//
//
// Status
//     writing, working.
int all_passed = 0;

int waittime = 50000000;

extern test_info test_list[];
extern uint32_t HSEStatus;

void process_board_tests( void )
{
  int i;
//  int n;
  int t_num = 0;
  int retry_count = 0;
  int max_retry = 1;
  
  while( all_passed == 0)
  {
    deactivate_all_keypress();  
    
    retry_count++;
    all_passed = 1;
    waittime = 50000000;
    // test 0 clock
    t_num = 0;
    // read HSE status from variable set at startup
    if(HSEStatus == 1) test_list[t_num].passed = 1;
    else all_passed = 0;    
    // test 1  LCD - must be working 
    t_num = 1;
    // test 2  LCD controller - must be working, could test and blink lights     
    t_num = 2;
    // test 3  External Flash, mfr id, tests SPI communication also 
    t_num = 3;

    if( Board_Flash_mfrid() == 1) test_list[t_num].passed = 1;
    else all_passed = 0;
    // test 4  External Flash unique id
    t_num = 4;    
    if( Board_Flash_uniqueid() == 1) test_list[t_num].passed = 1;
    else all_passed = 0;
    display_screen( 2, 0 );
    // test 5 External Flash read write, erase
    t_num = 5;
    {
      if( Board_Flash_RW() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
    }
    // test 6
    t_num = 6;
    // TouchPanel_disable();

    if( test_list[t_num].passed == 0 )
    {
      if( Board_I2C_EEPROM() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );
    }
    
    // test 7  Switch 1
    t_num = 7;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_switches( 1 ) == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );
    }
    // test 8  Switch 2
    t_num = 8;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_switches( 2 ) == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );      
    }
    // test 9  Switch 3
    t_num = 9;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_switches( 3 ) == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );      
    }
    // test 10  Switch 4
    t_num = 10;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_switches( 4 ) == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );
    }
    // test 11  Left knob button
    t_num = 11;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_Lknob_switch() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );      
    }
    // test 12  Left knob rotate
    t_num = 12;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_Lknob_rotate() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );      
    }
    // test 13  Right knob button
    t_num = 13;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_Rknob_switch() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }
    // test 14  Right knob rotate
    t_num = 14;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_Rknob_rotate() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );
    }
    // test 15  PTT
    t_num = 15;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_PTT() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }
    // test 16  Mic kepad
    t_num = 16;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_MICkeypad() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    } 
    // test 17  Touch screen
    i2c_2_initialize( I2C_touch );
    TouchPanel_init( );

    t_num = 17;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_Touchscreen() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }
    // send buttons to radio 
    activate_all_keypress();
    
    // check to see if radio is attached
    int cs800D_attached = 0;
    int packet_check = 0;

    if( test_list[18].passed == 0 )
    {    
      // display checking for radio
      strncpy( (char *) cg_line_buffer, "Checking for radio", 19);
      clear_rectangle( 300, 160, 490, 73 );
      ssd1963_cg_write_line( 25, 3, 300 , 180, 4);     
      while ( packet_check++ < 200 )
      {
        process_all_packet();
        msec_delay(5);
        if( packet_counter > 0 )
        {
          cs800D_attached = 1;
          clear_rectangle( 300, 160, 490, 73 );        
          break;       
        }
      }      
    }
    // test 18  Radio volume
    t_num = 18;
    if( test_list[t_num].passed == 0 )
    {
      // if there is an 800D radio attached
      if( cs800D_attached )
      {
        if( Board_Volume() == 1) test_list[t_num].passed = 1;
        else all_passed = 0;
        display_screen( 2, 1 );  
      }
      else // no radio, test uart lines as gpio
      {
        // test 
        if( Board_UART1() == 1) test_list[t_num].passed = 1;
        else all_passed = 0;
        display_screen( 2, 1 );        
      }
    }  
    // test 19  Radio channel
    t_num = 19;
    if( cs800D_attached )
    {
      if( test_list[t_num].passed == 0 )
      {
        if( Board_Channel() == 1) test_list[t_num].passed = 1;
        else all_passed = 0;
      }  
      display_screen( 2, 1 ); 
    }
    else // no radio, uart1 was tested use the same result as volume
    {
      test_list[t_num].passed = test_list[t_num-1].passed;
    }
    // test 20  UART1
    t_num = 20;
    
    // UART1 will have been tested by Volume and Channel    
    if( (test_list[17].passed == 1) || (test_list[18].passed == 1) )
    {
      test_list[t_num].passed = 1;
    }
    display_screen( 2, 0 );
    waittime = 50000000;
   
    // test 21  UART2
    t_num = 21;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_UART2() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }   
    // test 22  USB1
    t_num = 22;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_USB1() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }       
    // test 23  USB2
    t_num = 23;
    if( test_list[t_num].passed == 0 )
    {
      if( Board_USB2() == 1) test_list[t_num].passed = 1;
      else all_passed = 0;
      display_screen( 2, 0 );  
    }          
    
    display_screen( 2, 0 );
    if(retry_count > max_retry) break;
  }
  if( all_passed )
  {
    log_event( "diagnostics pass" );       
    // display test finished screen
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 )  );
    ssd1963_cg_clear_display();
    // border shaded frame
    RGB_color f_colr;
    f_colr.red = 15;
    f_colr.green = 70;
    f_colr.blue = 120;
    RGB_color s_colr;
    s_colr.red = 30;
    s_colr.green = 130;
    s_colr.blue = 200;
    draw_shaded_frame( 0, 0, 800, 480, 5, s_colr, f_colr);
    char *message = "ALL TESTS PASSED";
    for ( i = 0 ; i < 16; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 16, 2, 140 , 220, 6);
  }
  else
  {
    log_event( "diagnostics fail" );  
    // don't clear screen so tests that failed are seen
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 )  );    
    char *message = "TESTS FAILED";
    for ( i = 0 ; i < 12; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 12, 2, 200 , 220, 6);
    
  }
  all_passed = 0; // for a restart 
}

// =============================================================================
//
//  Test I2C and eeprom chip read and write 
//   Test for board checkout
//
// Status
//     written, tested, working.
//
//char *Company = "Test123 Test123\n";
char *Company = "Connect Systems\n";

int Board_I2C_EEPROM( void )
{
    int i;
    int success = 1;
 
    // TouchPanel_disable(); // be sure it's off and not sending touch data
    //  address
    uint8_t block = 0;
    // read first, don't write if it's already been written 
    // read multiple
    EEPROM_MemRead( block, 0x00, 0x01,  16);
    // wait for I2C to finish
    while(!I2C_2_txrx_complete);
    // check data read
    for( i = 0; i < 16; i++) 
    {
      if( Company[i] != I2C_2_data_in[I2C_dbufin_tail++]) success = 0;
    }    
    
    if(success == 0)
    {
      // put address in output buffer
      I2C_put_dataout( 0x00 );  
      I2C_put_dataout( 0x01 );
      // put data in the output buffer    
      for( i =0; i< 16;i++) I2C_put_dataout( Company[i] );
      
      // count, control byte, + 2 address, + 16 data
      EEPROM_MultipleWrite( block, 19);
      // wait for I2C to finish
      //while(!I2C_2_txrx_complete);
      EEPROM_waitfor_I2C();
      msec_delay(10);
      // read multiple
      EEPROM_MemRead( block, 0x00, 0x01,  16);
      // wait for I2C to finish
      while(!I2C_2_txrx_complete);
      // check data read
      success = 1;
      for( i = 0; i < 16; i++) 
      {
        if( Company[i] != I2C_2_data_in[I2C_dbufin_tail++]) success = 0;
      }
    }
    return success;
}

// =============================================================================
//
//  Test SPI and flash chip read mfr id
//   Test for board checkout
//
//
// Status
//     written, tested, working.

int Board_Flash_mfrid( void )
{
    int success = 0;  
    int mfrid;
    
    mfrid  = flash_mfrid();
    
    if( mfrid == FLASH_MFR_ID) success = 1;

    return success;
}
// =============================================================================
//
//  Test SPI and flash chip read unique id
//   Test for board checkout
//
// unique ID is 32 bit. It will be different for each chip
// need at least 3 of 4 bytes to be non zero.
// check each byte
//
// History
//      10/09/2018 revised, different test, and display UID to screen 
// 
// Status
//     written, tested, working.
int Board_Flash_uniqueid( void )
{
    int success = 0;
//    int n;
    //uint8_t b_check = 0;
    uint64_t uniqueid;
    uint32_t uid_hi = 0;
    uint32_t uid_lo = 0;
    
    
    uniqueid  = flash_uniqueid();

    uid_hi = uniqueid >> 32;
    uid_lo = uniqueid & 0xFFFFFFFF;
    
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    // show unique ID, test 3
//    n = sprintf((char*) cg_line_buffer, "%08X ", uid_hi);
    sprintf((char*) cg_line_buffer, "%08X ", uid_hi);
//    n = sprintf((char*) cg_line_buffer + 8, "%08X ", uid_lo);
    sprintf((char*) cg_line_buffer + 8, "%08X ", uid_lo);
    
    ssd1963_cg_write_line( 16, 4, 180, 7 + 4 * 24, 2); 
    
    if( uniqueid > 0) success = 1;
       
    return success;
}
// =============================================================================
//
//  Test SPI and Flash chip read, write and erase
//   Test for board checkout
//
// Status
//     written, tested, working, can try to reduce msec waits 

int Board_Flash_RW( void )
{
  uint8_t i;
  int success = 1;
  uint8_t testdata[50];
  
  // erase in case the chip has garbage
  flash_erase( 0x00001000, 0 );
  msec_delay(100);
  
  // make test data
  for( i = 0; i < 50; i++) testdata[i] = i;

  // test flash write
  flash_write( testdata, 0x00001000, 50 );
  msec_delay(100);  
  
  // read to test flash written
  flash_read( 0x00001000, 50 );  
  msec_delay(100); 
  // check the write
  for( i = 0; i < 50; i++)
  {
    if(spi_receive_buffer[rx_bufr_tail++] != i) success = 0;
  }
 
  // erase again 
  flash_erase( 0x00001000, 0 );
  msec_delay(100);  
  
  // read to test flash erased
  flash_read( 0x00001000, 50 );
  msec_delay(100);  
  
  // check the erase
  for( i = 0; i < 50; i++)
  {
    if( spi_receive_buffer[rx_bufr_tail++] != 0xFF) success = 0;
  }

  return success;
}
//
// =============================================================================
//   clear message area
//
//
int mes_x_pos = 350;
int mes_y_pos = 400;
uint8_t backg_color = 13;

void clear_message_area( void )
{
  clear_rectangle( mes_x_pos, mes_y_pos, 800 - mes_x_pos - 5, 40);
}

// =============================================================================
//
//  Test board switches
//   Test for board checkout
//
// Status
//     written, tested, working.

int Board_switches( uint8_t switch_num )
{
    int i;
    int success = 0;
    int volatile timeout;
    char *message = "                ";

    switch( switch_num)
    {
    case 1:  // left panel button P1  read_switch_s6 PE8  Left soft key 
      {
        // display "press P1"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Press P1";
        for ( i = 0 ; i < 8; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 8, 3, mes_x_pos , mes_y_pos, 4);
        timeout = waittime;        
        // wait for push (w timeout)
        while( !read_switch_s3() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;  
        // display "release P1"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Release P1";
        for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 10, 3, mes_x_pos , mes_y_pos, 4);
        
        timeout = waittime;
        // wait for release (w timeout)
        while( read_switch_s3() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;
        // good press and release
        success = 1;
        break;
      }
    case 2:  // left middle panel button P2
      {
        // display "press P2"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Press P2";
        for ( i = 0 ; i < 8; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 8, 3, mes_x_pos , mes_y_pos, 4);
        timeout = waittime;        
        // wait for push (w timeout)
        while( !read_switch_s4() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;  
        // display "release P2"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Release P2";
        for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 10, 3, mes_x_pos , mes_y_pos, 4);
        
        timeout = waittime;
        // wait for release (w timeout)
        while( read_switch_s4() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;
        // good press and release
        success = 1;        
        break;
      }
    case 3:  // right middle panel button P3
      {
        // display "press P3"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Press P3";
        for ( i = 0 ; i < 8; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 8, 3, mes_x_pos , mes_y_pos, 4);
        timeout = waittime;        
        // wait for push (w timeout)
        while( !read_switch_s5() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;  
        // display "release P3"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Release P3";
        for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 10, 3, mes_x_pos , mes_y_pos, 4);
        
        timeout = waittime;
        // wait for release (w timeout)
        while( read_switch_s5() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;
        // good press and release
        success = 1;           
        break;
      }
    case 4:  // right panel button P4
      {
        // display "press P4"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Press P4";
        for ( i = 0 ; i < 8; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 8, 3, mes_x_pos , mes_y_pos, 4);
        timeout = waittime;        
        // wait for push (w timeout)
        while( !read_switch_s6() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;  
        // display "release P4"
        set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
        message = "Release P4";
        for ( i = 0 ; i < 10; i++) cg_line_buffer[i] = message[i];
        ssd1963_cg_write_line( 10, 3, mes_x_pos , mes_y_pos, 4);
        
        timeout = waittime;
        // wait for release (w timeout)
        while( read_switch_s6() )
        {
          if( --timeout == 0) break;  
        }
        // if timedout leave the switch
        if( timeout == 0) break;
        // good press and release
        success = 1;           
        break;
      }
    default: break;      
    }
    clear_message_area();
   
    return success;
}
// =============================================================================
//
//  Test board left knob switch
//   Test for board checkout
//
// Status
//     written, tested, working.

int Board_Lknob_switch( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message = "                                     ";

    // display "press left knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Press left knob";
    for ( i = 0 ; i < 15; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 15, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for push (w timeout)
    while( !read_switch_s1() )
    {
      if( --timeout == 0) break;  
    }
    // if timedout fail
    if( timeout == 0) success = 0;
    clear_message_area(); 
    // display "release left knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Release left knob";
    for ( i = 0 ; i < 17; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 17, 3, mes_x_pos , mes_y_pos, 4);
    
    timeout = waittime;
    // wait for release (w timeout)
    while( read_switch_s1() )
    {
      if( --timeout == 0) break;  
    }
    // if timedout leave the switch
    if( timeout == 0) success = 0;
    // good press and release
    clear_message_area();  
   
    return success;
}
// =============================================================================
//
//  Test board right knob switch
//   Test for board checkout
//
// Status
//     written, tested, working.

int Board_Rknob_switch( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message = "                                     ";

    // display "press right knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Press right knob";
    for ( i = 0 ; i < 16; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 16, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for push (w timeout)
    while( !read_switch_s2() )
    {
      if( --timeout == 0) break;  
    }
    // if timedout fail
    if( timeout == 0) success = 0;
    clear_message_area(); 
    // display "release right knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Release right knob";
    for ( i = 0 ; i < 18; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 18, 3, mes_x_pos , mes_y_pos, 4);
    
    timeout = waittime;
    // wait for release (w timeout)
    while( read_switch_s2() )
    {
      if( --timeout == 0) break;  
    }
    // if timedout 
    if( timeout == 0) success = 0;
    // good press and release
    clear_message_area();  
   
    return success;
}
// =============================================================================
//
//  Test board left knob rotate
//   Test for board checkout
//
//  using a 4 bit indicator for the two switches. One bit for each
//  position. The indicator will = 15 if each switch can be 0 or 1 
//
// Status
//     written, tested, working.

int Board_Lknob_rotate( void )
{
    int i;
    int success = 0;
    int rotate = 0;  // 4 bit indicator
    int volatile timeout;
    char *message = "                ";

    // display "turn left knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Turn left knob";
    for ( i = 0 ; i < 14; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 14, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for rotated (w timeout)
    while( rotate < 15 )
    {
      if ( read_rotary_encoder_2_0() ) rotate |= 2;
      else rotate |= 1;
      if ( read_rotary_encoder_2_1() ) rotate |= 8;
      else rotate |= 4;      

      if( --timeout == 0) break;  
    }
    if(rotate == 15) success = 1;
    clear_message_area();  
   
    return success;
}
// =============================================================================
//
//  Test board right knob rotate
//   Test for board checkout
//
//  using a 4 bit indicator for the two switches. One bit for each
//  position. The indicator will = 15 if each switch can be 0 or 1 
//
// Status
//     written, tested, working.

int Board_Rknob_rotate( void )
{
    int i;
    int success = 0;
    int rotate = 0;  // 4 bit indicator
    int volatile timeout;
    char *message = "                ";

    // display "turn right knob"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Turn right knob";
    for ( i = 0 ; i < 15; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 15, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for rotated (w timeout)
    while( rotate < 15 )
    {
      if ( read_rotary_encoder_1_0() ) rotate |= 2;
      else rotate |= 1;
      if ( read_rotary_encoder_1_1() ) rotate |= 8;
      else rotate |= 4;      

      if( --timeout == 0) break;  
    }
    if(rotate == 15) success = 1;
    clear_message_area();  
   
    return success;
}
// =============================================================================
//
//  Test board external mic PTT
//   Test for board checkout
//
// Status
//     written, tested, working.

int Board_PTT( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message = "          ";

    // display "Press PTT"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Press PTT";
    for ( i = 0 ; i < 9; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 9, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for ptt (w timeout)
    while( read_mike_ptt() )
    {
      if( --timeout == 0) 
      {
        success = 0; 
        break;
      }
    }
    clear_message_area(); 
   
    return success;
}
// =============================================================================
//
//  Test all mic keypad keys
//    special test to check out mic's
//    displays key pressed until the cancel key is pressed 

void Board_allMICkeys( void )
{
  char *message = "          ";  
  uint8_t prev_key;  
  
  clear_message_area();         
  message = " ";
  cg_line_buffer[0] = message[0];
  ssd1963_cg_write_line( 1, 3, mes_x_pos , mes_y_pos, 4);
  cg_hori_location = mes_x_pos;
  cg_vert_location = mes_y_pos;
  // test all keys until a cancel
  while( mc59_key != 0x0B)
  {
    if( mc59_key != prev_key )
    {
      cg_hori_location = ssd1963_cg_output_character(ascii_convert_string[mc59_key],
                       3, cg_hori_location, cg_vert_location, 2 );
      prev_key = mc59_key;
    }
  }
  clear_message_area();      
}

// =============================================================================
//
//  Test board external mic keypad
//   Test for board checkout
//
// Status
//     written, tested, working.

int Board_MICkeypad( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message = "               ";

    // display "Press micpad 1"
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "Press micpad 1";
    for ( i = 0 ; i < 14; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 14, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;        
    // wait for keypad press (w timeout)
     while( (mc59_key != 2) && (mc59_key != 0x0E))
     {
       if( --timeout == 0)
       {
        success = 0; 
        break;  
       }
     }
    if( mc59_key == 0x0E )
    {
      Board_allMICkeys();      
    }     
    
    message = "Press micpad 9";
    for ( i = 0 ; i < 14; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 14, 3, mes_x_pos , mes_y_pos, 4);
    timeout = waittime;    
    // wait for keypad press (w timeout)
    while( mc59_key != 0x0A )
    {
      if( --timeout == 0)
      {
        success = 0;
        break;
      }
    }    

    clear_message_area(); 
   
    return success;
}
// =============================================================================
//
//  Test board touch screen
//   Test for board checkout
//
// Status
//     written, testing, working
//     06/26/2018 added reset line shutdown at end, seems to have fixed
//                failures after restarting test

typedef struct
{
  uint16_t  x; 
  uint16_t  y;
  uint16_t  width;
  uint16_t  height;
} xy_box;

// touch point rectangles
xy_box board_tp[4] = {
  {  0,   0,  80,  50 },
  {720,   0,  80,  50 },
  {  0, 430,  80,  50 },
  {720, 430,  80,  50 }
};
extern int b_test_tp;


int Board_Touchscreen( void )
{
    int i;
    int success = 0;
    int volatile timeout;
    char *message = "                 ";
    uint8_t box_touched[4];
    
    for( i = 0; i< 4; i++) box_touched[i] = 0;
    // touch in four corners
    
    // clear screen
    ssd1963_cg_clear_display();    
    // draw 4 boxes with numbers
    for ( i = 0; i < 4; i++)
    {
      // set bkgd and font color
      set_font_colors( get_pal_color( 0 ), get_pal_color( 6+i ) );      
      // draw box
      draw_rectangle( board_tp[i].x, board_tp[i].y,
                     board_tp[i].width, board_tp[i].height, cg_background );
      // draw digit
      sprintf((char*) cg_line_buffer, "%i", i+1);
      ssd1963_cg_write_line(1, 3, board_tp[i].x + 25, board_tp[i].y + 5, 0); 
    }
    // display message "touch 1,2,3,4
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "touch 1, 2, 3, 4";
    for ( i = 0 ; i < 16; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 16, 3, 300 , 225, 4);    
    
    // activate touch points
    CS800D_touch_point[59].is_active = 1;  // top left
    CS800D_touch_point[60].is_active = 1;  // top right
    CS800D_touch_point[61].is_active = 1;  // bottom left
    CS800D_touch_point[62].is_active = 1;  // bottom right    
    
    // while not all touched and timeout not expired 
    timeout = waittime;
    while(timeout--)
    {
      // CS800D_process_touches();  // running in the timer
      // which box touched? b_test_tp is 0 for no touch, 1-4 per touch
      if( b_test_tp )
      {
        if( box_touched[ b_test_tp-1 ] == 0 )
        {
          // clear box
          clear_rectangle( board_tp[b_test_tp-1].x, board_tp[b_test_tp-1].y,
                     board_tp[b_test_tp-1].width, board_tp[b_test_tp-1].height );           
          box_touched[ b_test_tp-1 ] = 1;          
        }
      }
      // if all touched 
      if((box_touched[ 0 ] + box_touched[ 1 ] +
          box_touched[ 2 ] + box_touched[ 3 ] ) == 4)
      {
        success = 1;
        break;
      }
    }
    // repaint screen
    display_screen( 2, 0 );
    all_touches_off();    
        
      // get touches
      // if touch is in a box clear the box
    
    // when all touched, set passed flag, restore board test screen

/*    
    // touch anywhere
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "touch anywhere";
    for ( i = 0 ; i < 14; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 14, 3, 300 , 225, 4);
    
    timeout = waittime;
     
    while(timeout--)
    {
      CS800D_process_touches();
      if(TS_touch_count > 0) 
      {
        success = 1;
        break;
      }
    }

    // clear message
    clear_rectangle(  300 , 225, 495, 40 );
*/
  

    return success;
}
// =============================================================================
//
//  Test board radio control of Volume
//   Test for board checkout
//
// History
//  11/28/2018 revised to use cs_channel_volume instead of current_radio_volume
//
// Status
//     written, tested, working
//
uint16_t volume;

int Board_Volume( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message;
    
    process_all_packet();
    // volume = current_radio_volume;
    volume = cs_channel_volume;
    // display volume bar
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    display_volume_bar( volume );    
    
    // adjust volume
    message = "adjust volume";
    for ( i = 0 ; i < 13; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 13, 3, 300 , 225, 4);
    timeout = waittime;
    msec_delay( 10 );
    // wait for volume change to get a good volume
    //while ( volume == current_radio_volume )
    while ( volume == cs_channel_volume )  
    {
      process_all_packet();
      if( --timeout <= 0)
      {
        success = 0;
        break;
      }
      msec_delay( 10 );
    }
    display_volume_bar( volume );      
    // volume = current_radio_volume;
    volume = cs_channel_volume;
    // wait for volume change again
    if(success)
    {
      //while ( volume == current_radio_volume )
      while ( volume == cs_channel_volume )    
      {
        process_all_packet();
        if( --timeout <= 0)
        {
          success = 0;
          break;
        }
        display_volume_bar( volume );          
        msec_delay( 10 );  
      }
    }
  
    clear_message_area(); 
    // repaint screen
    display_screen( 2, 1 );
    
    return success;
}
// =============================================================================
//
//  Test board radio control of Channel
//   Test for board checkout
//
// channel name stored by process packet buffer 5
// Status
//     written, tested, sort of working, flakey response to knob

//extern char current_radio_channel[20];

int Board_Channel( void )
{
    int i;
    int success = 1;
    int volatile timeout;
    char *message = "               ";
    char prev_channel[16];
    int ch_change = 0;    

    process_all_packet();
    // copy current channel to previous for compare
    for(i = 0; i < 16; i++) prev_channel[i] = cs_channel_alias[i];
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );    
    // display current channel name
    for ( i = 0 ; i < 16; i++) 
    {
      if( cs_channel_alias[i] == 0) cg_line_buffer[i] = ' '; 
      else  cg_line_buffer[i] = cs_channel_alias[i];    
    }
    clear_rectangle( 300, 160, 490, 73 );
    ssd1963_cg_write_line( 16, 3, 300 , 180, 4);      
    
    // change channel
    set_font_colors( get_pal_color( 0 ), get_pal_color( 1 ) );
    message = "change channel";
    for ( i = 0 ; i < 14; i++) cg_line_buffer[i] = message[i];
    ssd1963_cg_write_line( 14, 3, 300 , 250, 4);
    timeout = waittime;
    msec_delay( 10 );
    // wait for channel change
    while ( !ch_change )
    {
      process_all_packet();
      if(strncmp((char const * ) prev_channel,
                 (char const * ) cs_channel_alias, 10) != 0)
      {
          ch_change = 1;
      }
      
      if( --timeout <= 0)
      {
        success = 0;
        break;
      }
      msec_delay( 10 );      
    }
    clear_rectangle( 300, 160, 490, 73 );
    // display current channel name
    for ( i = 0 ; i < 16; i++) 
    {
      if( cs_channel_alias[i] == 0) cg_line_buffer[i] = ' '; 
      else  cg_line_buffer[i] = cs_channel_alias[i];    
    }   
    ssd1963_cg_write_line( 16, 3, 300 , 180, 4);   
 
    msec_delay( 2000 ); 
    
    // repaint screen
    display_screen( 2, 1 );
    
    return success;
}
// =============================================================================
//
//  Test board connector UART1
//   Test for board checkout
//  GPIO

//  USART1_RXD   A 10
//  USART1_TXD   B 6
//
// Status
//     writing

int Board_UART1( void )
{
  int success = 0;
  int volatile timeout;
  int pin = 0;
    
  // display insert loopback message
  strncpy( (char *) cg_line_buffer, "Insert loopback in UART1", 25);
  clear_rectangle( 300, 160, 490, 73 );
  ssd1963_cg_write_line( 25, 3, 300 , 180, 4);       

  // clear both from alt mode, makes them inputs
  *GPIO_A_MODER &= ~0x00300000;  // A 10
  *GPIO_B_MODER &= ~0x00003000;  // B 6
  // set B6 to output
  *GPIO_B_MODER |= 0x00001000;   // B 6
  
  timeout = waittime;
  while ( success == 0 )
  {
    // read input
    pin = *GPIO_A_IDR & BIT10_MASK;
    // toggle output
    if (pin) *GPIO_B_BSRR = BIT06_OFF;
    else     *GPIO_B_BSRR = BIT06_ON; 
    // re read input
    if ( (*GPIO_A_IDR & BIT10_MASK) != pin) success = 1;
    if( --timeout == 0) break;
  }

  return success;
}
// =============================================================================
//
//  Test board connector UART2
//   Test for board checkout
//  GPIO

//  USART2_RXD   A 3
//  USART2_TXD   D 5
//
// Status
//     written, tested, working.

int Board_UART2( void )
{
  int success = 0;
  int volatile timeout;
  int pin = 0;
    
  // display insert loopback message
  strncpy( (char *) cg_line_buffer, "Insert loopback in UART2", 25);
  clear_rectangle( 300, 160, 490, 73 );
  ssd1963_cg_write_line( 25, 3, 300 , 180, 4);       
  
  /*  
  // change GPIO from ALT for uart to input D+ and output D-
  uint32_t volatile *GPIOA_MODER = (uint32_t *) GPIOA_MODER_ADDRESS;
  uint32_t volatile *GPIOD_MODER = (uint32_t *) GPIOD_MODER_ADDRESS;
  uint32_t volatile *GPIOD_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;    
  uint32_t volatile *GPIOA_IDRR = (uint32_t *) GPIOA_IDR_ADDRESS;
  */
  
  // clear both from alt mode, makes them inputs
  *GPIO_A_MODER &= ~0x000000C0;  // A 3
  *GPIO_D_MODER &= ~0x00000C00;  // D 5
  // set D5 to output
  *GPIO_D_MODER |= 0x00000400;   // D 5
  
  timeout = waittime;
  while ( success == 0 )
  {
    // read input
    pin = *GPIO_A_IDR & BIT03_MASK;
    // toggle output
    if (pin) *GPIO_D_BSRR = BIT05_OFF;
    else     *GPIO_D_BSRR = BIT05_ON; 
    // re read input
    if ( (*GPIO_A_IDR & BIT03_MASK) != pin) success = 1;
    if( --timeout == 0) break;
  }

  // TODO: later
  // exchange lines input D- output D+
  // test again
  // restore GPIO to correct settings
    
  return success;
}
// =============================================================================
//
//  Test board connector USB1
//   Test for board checkout
//  GPIO

//  USB1_DP     B 15
//  USB1 DM     B 14
//
// Status


int Board_USB1( void )
{
  int success = 0;
  int volatile timeout;
  int pin = 0;

      
  // display insert loopback message
  strncpy( (char *) cg_line_buffer, "Insert loopback in USB1", 25);
  clear_rectangle( 300, 160, 490, 73 );
  ssd1963_cg_write_line( 25, 3, 300 , 180, 4);     
    
  // change GPIO from ALT for uart to input DP and output DM
  uint32_t volatile *GPIOB_MODER = (uint32_t *) GPIOB_MODER_ADDRESS;
  uint32_t volatile *GPIOB_BSRR = (uint32_t *) GPIOB_BSRR_ADDRESS;    
  uint32_t volatile *GPIOB_IDRR = (uint32_t *) GPIOB_IDR_ADDRESS;
  
  // clear both from alt mode, makes them inputs
  *GPIOB_MODER &= ~0xF0000000;  // B 14, 15
  // set B14 to output
  *GPIOB_MODER |= 0x10000000;   // B 14
  
  timeout = waittime;
  while ( success == 0 )
  {
    // read input
    pin = *GPIOB_IDRR & BIT15_MASK;
    // toggle output
    if (pin) *GPIOB_BSRR = BIT14_OFF;
    else     *GPIOB_BSRR = BIT14_ON; 
    // re read input
    if ( (*GPIOB_IDRR & BIT15_MASK) != pin) success = 1;
    if( --timeout == 0) break;
  }

  // TODO: later
  // exchange lines input DM output DP
  // test again
  // restore GPIO to correct settings
    
  return success;
}
// =============================================================================
//
//  Test board connector USB2
//   Test for board checkout
//  GPIO

//  USB2_DP     A 12
//  USB2 DM     A 11
//
// Status


int Board_USB2( void )
{
  int success = 0;
  int volatile timeout;
  int pin = 0;

  // display insert loopback message
  strncpy( (char *) cg_line_buffer, "Insert loopback in USB2", 25);
  clear_rectangle( 300, 160, 490, 73 );
  ssd1963_cg_write_line( 25, 3, 300 , 180, 4);        
    
  // change GPIO from ALT for uart to input DP and output DM
  uint32_t volatile *GPIOA_MODER = (uint32_t *) GPIOA_MODER_ADDRESS;
  uint32_t volatile *GPIOA_BSRR = (uint32_t *) GPIOA_BSRR_ADDRESS;    
  uint32_t volatile *GPIOA_IDRR = (uint32_t *) GPIOA_IDR_ADDRESS;
  
  // clear both from alt mode, makes them inputs
  *GPIOA_MODER &= ~0x03C00000;  // A 11, 12
  // set A11 to output
  *GPIOA_MODER |= 0x00400000;   // A 11
  
  timeout = waittime;
  while ( success == 0 )
  {
    // read input
    pin = *GPIOA_IDRR & BIT12_MASK;
    // toggle output
    if (pin) *GPIOA_BSRR = BIT11_OFF;
    else     *GPIOA_BSRR = BIT11_ON; 
    // re read input
    if ( (*GPIOA_IDRR & BIT12_MASK) != pin) success = 1;
    if( --timeout == 0) break;
  }

  // TODO: later
  // exchange lines input DM output DP
  // test again
  // restore GPIO to correct settings
    
  return success;
}