//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 7, 2018             Design Documents                        CSI_eeprom.h
//
//			header file for EEPROM
//                    Large panel display project
//
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  06/7/2018 moved from CSI_i2c.c
//
// Status
//  written
//
//******************************************************************************

#ifndef CSI_EEPROM_H_
#define CSI_EEPROM_H_

#include <stdint.h>

// debugging
void EEPROM_TestWrite( uint8_t block, uint8_t AddrHigh,
                      uint8_t AddrLow,uint8_t MemData);
void EEPROM_TestRead( uint8_t block, uint8_t AddrHigh, uint8_t AddrLow );

// for eprom
void EEPROM_MemRead( uint8_t block, uint8_t AddrHigh,
                       uint8_t AddrLow, uint16_t count_to_read);
void EEPROM_MemWrite( uint8_t block,uint8_t AddrHigh,
                     uint8_t AddrLow,uint8_t MemData);
void EEPROM_MultipleWrite( uint8_t block, uint16_t num_bytes);
void EEPROM_Mem_fill( uint8_t block, uint8_t AddrHigh,
                      uint8_t AddrLow, uint8_t num_bytes, uint8_t fillbyte );
void EEPROM_busypoll( void );

void EEPROM_waitfor_I2C( void );

// old routines (not working, left to prevent compile errors)
// wherever these were used needs to be revised
void eeprom_write(uint32_t address, uint8_t data);
void eeprom_write16(uint32_t address, uint16_t data);
uint8_t eeprom_read(uint32_t address);
uint16_t eeprom_read16(uint32_t address);

void save_radio_type( uint8_t r_type );
uint8_t load_radio_type( void );

extern uint8_t panel_config_flag;  

// 
// EEPROM Memory map
// block 0 0x0000 to 0xFFFF
// block 1 0x0000 to 0xFFFF

// 128 bytes per page
//  page 0  0x0000 to 0x007F
//  page 1  0x0080 to 0x00FF
//  page 2  0x0100 to 0x017F
//  page 3  0x0180 to 0x01FF
// ... 
//  page 511 0xFF80  to FFFF 

// color settings             0x2000 - 0x203B
// Function button settings   0x2100 - 0x212D
// zone favorites             0x2200 - 0x22F8
//                            0x2300 - 0x2378
// channel favorites          0x2400 - 0x24F8
//                            0x2500 - 0x2578
// radio type                 0x2600
// FT857 info volume and sql  0x2700




#endif /* CSI_EEPROM_H_ */
