//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// June 29, 2018 		Design Documents      	            CSI_secure.c
//
//                 Security and code protection routines
//
//      !!!======= CONFIDENTIAL, NOT FOR PUBLIC RELEASE ======= !!!!
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// History
//  06/29/2018 started
//
// Status
//  writing
//
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working
//
//  read_cpuID                      reads CPU unique id
//  read_cpu_waferXY                reads CPU wafer XY coordinates
//  read_cpu_Lot                    reads CPU lot number
//  store_securepage2               writes secure info to EEPROM scattered
//  check_securepage2               compares stored info with current CPU info
//
// Status: written, tested, not used
//
//  store_securepage                writes secure info to EEPROM single location
//  check_securepage                compares stored info with current CPU info
//==============================================================================
#include "CSI_secure.h"
#include <stdint.h>
#include <stdlib.h>
#include "CSI_ExtFlash.h"
#include "CSI_i2c.h"
#include "CSI_eeprom.h"

// store in top of EEPROM bank 0
// bank 0 page 511 0xFF80  to FFFF
// 15 bytes company name  0xFF80 0xFF8E
// 4 bytes CPUID          0xFF8F 0xFF92
// 4 bytes CPU wafer      0xFF93 0xFF96
// 4 bytes CPU lot        0xFF97 0xFF9A
// 8 bytes FLASH id       0xFF9B 0xFFA2
// 1 byte  secure flag    0xFFA3
// ... may add more


// 8 bytes FLASH unique id 
// =============================================================================
//
// read cpu ID 
uint32_t read_cpuID( void )
{
  uint32_t *CPU_UNIQUE_ID = (uint32_t *) CPU_UNIQUE_ID_ADDRESS;

  // read
  return *CPU_UNIQUE_ID;
}
// =============================================================================
//
// read cpu wafer xy
uint32_t read_cpu_waferXY( void )
{
  uint32_t *CPU_WAFER = (uint32_t *) CPU_WAFER_ADDRESS;

  // read
  return *CPU_WAFER;
}
// =============================================================================
//
// read cpu Lot number
//
uint32_t read_cpu_Lot( void )
{
  uint32_t *CPU_LOT_NUMBER = (uint32_t *) CPU_LOT_NUMBER_ADDRESS;

  // read
  return *CPU_LOT_NUMBER;
}
// =============================================================================
//
// electronic signature
// store all information in the page in one write
// placed in the last page 0xFF80 - 0xFFFF 
//
// using 38 bytes, room to add
// todo: add
// hardware version number
// software version number

char *company = "(C) Connect Systems";

void store_securepage( void )
{
  int i;
  uint32_t CPU_ID;
  uint32_t CPU_wafer;
  uint32_t CPU_lot;
  uint64_t Flash_ID;
  
  // read cpu and chip info
  CPU_ID = read_cpuID();
  CPU_wafer = read_cpu_waferXY();
  CPU_lot = read_cpu_Lot();
  Flash_ID = flash_uniqueid();
  
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout(  SECURE_EEPROM_ADD_HI );  
  I2C_put_dataout(  SECURE_EEPROM_ADD_LO );
  // company name
  for( i = 0; i < 19; i++)
  {
    I2C_put_dataout( (uint8_t) company[i] );   
  }
  // put cpu id 32 bit
  for( i = 24; i >= 0; i -= 8)
  {
    I2C_put_dataout( (CPU_ID & 0xFF << i) >> i );   
  }
  // put cpu wafer xy  32 bit
    for( i = 24; i >= 0; i -= 8)
  {
    I2C_put_dataout( (CPU_wafer & 0xFF << i) >> i );   
  }
  // put cpu lotnumber  32 bit
  for( i = 24; i >= 0; i -= 8)
  {
    I2C_put_dataout( (CPU_lot & 0xFF << i) >> i );   
  }
  // put flash id 64 bit
  for( i = 56; i >= 0; i -= 8)
  {
    I2C_put_dataout( ((uint64_t) Flash_ID & ((uint64_t) 0xFF << i)) >> i );   
  }
   
  // set flag in EEPROM, secure has been run
  I2C_put_dataout( 0xC5 );   // C5 is for Connect Systems
   
  // write to EEPROM  3+19+4+4+4+8+1  43 bytes
  EEPROM_MultipleWrite( 0, 43);
  
  //while(!I2C_2_txrx_complete);
  EEPROM_waitfor_I2C();
  EEPROM_busypoll(); 
   
}
// =============================================================================
//
//  revison for first release, 
//

// locations of secure id storage
// starting data has 27 bytes, one byte at the end for the stored flag
// staying in one page 128 bytes, storage locations must be between 27 and 126
uint8_t sec_loc[12][2] =
{
{ 62,  0 }, // cpuid
{ 74,  0 },
{ 56,  0 },
{ 120, 0 },  
{ 63,  0 }, // wafer xy
{ 39,  0 },
{ 97,  0 },
{ 82,  0 },
{ 124, 0 }, // wafer num and lot num
{ 41,  0 },
{ 48,  0 },
{ 30,  0 },
};

void store_securepage2( void )
{
  int i;
  int j;
  uint32_t CPU_ID;
  uint32_t CPU_wafer;
  uint32_t CPU_lot;  
  uint64_t Flash_ID;
  
  // read cpu and chip info
  CPU_ID = read_cpuID();
  CPU_wafer = read_cpu_waferXY();
  CPU_lot = read_cpu_Lot();  
  Flash_ID = flash_uniqueid();
  
  // break cpuid into bytes, xor
  for(i = 0; i < 4; i++)
  {
    sec_loc[i][1] = (CPU_ID & 0xFF) ^ 0x55;
    CPU_ID >>= 8;
  }
  // break wafer into bytes, xor
  for(i = 4; i < 8; i++)
  {
    sec_loc[i][1] = (CPU_wafer & 0xFF) ^ 0x55;
    CPU_wafer >>= 8;
  }
  // break wafer into bytes, xor
  for(i = 8; i < 12; i++)
  {
    sec_loc[i][1] = (CPU_lot & 0xFF) ^ 0x55;
    CPU_lot >>= 8;
  }  
  
  // load transmit buffer  
  // put address in output buffer
  I2C_put_dataout(  SECURE_EEPROM_ADD_HI );  
  I2C_put_dataout(  0  );  // using full page
  // company name (19)
  for( i = 0; i < 19; i++)
  {
    I2C_put_dataout( (uint8_t) company[i] );
  }
  // put flash id 64 bit (8)
  for( i = 56; i >= 0; i -= 8)
  {
    I2C_put_dataout( ((uint64_t) Flash_ID & ((uint64_t) 0xFF << i)) >> i );   
  }
  // 0-18 19-26 start at 27
  // ============

  int skip;
  
  for(i = 27; i < 126; i++)
  {
    skip = 0;
    // if it's a sec_loc byte  
    for(j = 0; j < 12; j++)
    {
      if(i == sec_loc[j][0])
      {
        // store id byte
        I2C_put_dataout( (uint8_t) sec_loc[j][1] );
        skip = 1;
      }
    }
    if( skip == 0)
    {
      I2C_put_dataout( (uint8_t) rand() & 0xFF );
    }
  }
   
  // set flag in EEPROM, secure has been run
  I2C_put_dataout( 0xC5 );   // C5 is for Connect Systems
   
  // write to EEPROM  3+15+8+100+1 = 127
  EEPROM_MultipleWrite( 0, 128);
  
  //while(!I2C_2_txrx_complete);
  EEPROM_waitfor_I2C();
  EEPROM_busypoll(); 
   
}

// =============================================================================
//
//  check securepage
//
//  read securepage data
//  read chip data
//  if securepage has not been written (all 0xFF)
//     write to secure page 
//  compare chip data and secure page data
//  if sourcepage does not match - report an error
// 

  uint32_t CPU_ID;
  uint32_t CPU_wafer;
  uint32_t CPU_lot;
  uint64_t Flash_ID;
  uint8_t comparedata[43]; 
  uint32_t test;
  
int check_securepage( void )
{
  int i;
  int n = 0;
  
  // read cpu and chip info
  CPU_ID = read_cpuID();
  CPU_wafer = read_cpu_waferXY();
  CPU_lot = read_cpu_Lot();
  Flash_ID = flash_uniqueid();
  
  // put info into comparison array
  for( i = 0 ; i < 19 ; i++)
  {
    comparedata[n++] = (uint8_t) company[i];
  }
  for(i = 24 ; i >= 0 ; i -= 8)
  {
    comparedata[n++] = (CPU_ID & (0xFF << i)) >> i;
  }  
  for(i = 24 ; i >= 0 ; i -= 8)
  {
    comparedata[n++] = (CPU_wafer & (0xFF << i)) >> i;
  }    
  for(i = 24 ; i >= 0 ; i -= 8)
  {
    comparedata[n++] = (CPU_lot & (0xFF << i)) >> i;
  }
  for(i = 56 ; i >= 0 ; i -= 8)
  {
    comparedata[n++] = ((uint64_t) Flash_ID & ((uint64_t) 0xFF << i)) >> i;
  }
  comparedata[n] = 0xC5;
  
  // read securepage
  EEPROM_MemRead( 0, SECURE_EEPROM_ADD_HI, SECURE_EEPROM_ADD_LO, 43);
  // wait for I2C to finish
  while(!I2C_2_txrx_complete);  

  // data is in rx buffer
  // test for never written, not incrementing buffer pointer here
  n = 0;
  for( i = 0; i < 6; i++)
  {
    if(I2C_2_data_in[I2C_dbufin_tail + i] == 0xFF) n++; 
  }
  if(n > 3)
  {
    // set buffer pointer to all read
    I2C_dbufin_tail = I2C_dbufin_head;
    // write the info
    store_securepage();
  
    return 1;    
  }
  // something has been written
  // check all
  for( i = 0; i < 43; i++)
  {
    if(comparedata[i] != I2C_get_datain() )
    {
      // set buffer pointer to all read
      I2C_dbufin_tail = I2C_dbufin_head;
      return  0;      
    }
    
  }
  // all checks passed
  return 1;
}
// =============================================================================
//
//  check securepage
//
// revision for first release

// fixed array so locations are independent of buffer
uint8_t Eprom_data[I2C_DATA_BUF_SIZE];

int check_securepage2( void )
{
  int i;
  int n = 0;
  
  // read cpu and chip info
  CPU_ID = read_cpuID();
  CPU_wafer = read_cpu_waferXY();
  CPU_lot = read_cpu_Lot();  
  Flash_ID = flash_uniqueid();
  
  // put data into sec_loc array
  // break cpuid into bytes, xor
  for(i = 0; i < 4; i++)
  {
    sec_loc[i][1] = (CPU_ID & 0xFF) ^ 0x55;
    CPU_ID >>= 8;
  }
  // break wafer into bytes, xor
  for(i = 4; i < 8; i++)
  {
    sec_loc[i][1] = (CPU_wafer & 0xFF) ^ 0x55;
    CPU_wafer >>= 8;
  }
  // break wafer into bytes, xor
  for(i = 8; i < 12; i++)
  {
    sec_loc[i][1] = (CPU_lot & 0xFF) ^ 0x55;
    CPU_lot >>= 8;
  }  
  
  // put other info into comparison array
  for( i = 0 ; i < 19 ; i++)
  {
    comparedata[n++] = (uint8_t) company[i];
  }
  for(i = 56 ; i >= 0 ; i -= 8)
  {
    comparedata[n++] = ((uint64_t) Flash_ID & ((uint64_t) 0xFF << i)) >> i;
  }
  
  // read securepage
  EEPROM_MemRead( 0, SECURE_EEPROM_ADD_HI, 0, 128);
  // wait for I2C to finish
  while(!I2C_2_txrx_complete);  

  // data is in rx buffer
  // copy to array
  for(i = 0; i < 128; i++)
  {
    Eprom_data[i] = I2C_get_datain();
  }
  // test for never written, FF's
  n = 0;
  for( i = 0; i < 6; i++ )
  {
    if(Eprom_data[i] == 0xFF) n++; 
  }
  if(n > 3)
  {
    // set buffer pointer to all read
    I2C_dbufin_tail = I2C_dbufin_head;
    // write the info
    store_securepage2();
    return 1;    
  }
  // something has been written
  // check Co and Flash ID
  for( i = 0; i < 27; i++)
  {
    if(comparedata[i] != Eprom_data[i] )  return  0;      
  }
  // check sec_loc data
  for(i = 0; i < 12; i++)
  {
    if( sec_loc[i][1] != Eprom_data[sec_loc[i][0]] ) return 0;
  }
  // check final byte
  //   Eprom_data[ n?? ] == 0xC5;
  
  // set buffer pointer to all read, just in case it's not
  I2C_dbufin_tail = I2C_dbufin_head;    
  // all checks passed
  return 1;
}




