///       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//==============================================================================
//
// Mar 29, 2018             Design Documents                           CSI_spi.h
//
//			header file for SPI
// Programmer
//  Dean S. Graham
//
// Copyright
//  Connect Systems Inc.
//
// History
//  03/29/2018 originaly written for prototype STM32F7
//  05/23/2018 added header guard 
//
// Status
//  writing revising for use with STM32F405
//
//******************************************************************************
#ifndef CSI_SPI_H_
#define CSI_SPI_H_

#define RCC_APB2ENR_SPI1EN    0x00001000
#define RCC_APB1ENR_SPI2EN    0x00004000

// functions that can be used externally
int spi1_init( void );
int spi1_write_bytes( void );
int spi1_sleep( void );
void SPI1_tx_buffer_add( uint8_t data );
void SPI1_Handler( void );

// functions used internally
int spi1_check_error( void );
void spi_rxtx( int );
void spi1_csn( int hilo );
void spi1_interrupts( int onoff );
void spi1_enable( int onoff );

// buffers
extern uint8_t spi_receive_buffer[256];
extern uint8_t rx_bufr_head;
extern uint8_t rx_bufr_tail;

extern uint8_t spi_transmit_buffer[256];
extern uint8_t tx_bufr_head;
extern uint8_t tx_bufr_tail;

extern volatile int spi1_num_bytes_to_rx;
extern volatile int spi1_num_bytes_to_tx;

// SPI control register CR1 structure 16 bit
typedef struct
{
  unsigned int cpha     : 1;  // clock phase (bit 0)
  unsigned int cpol     : 1;  // clock polarity
  unsigned int mstr     : 1;  // master selection
  unsigned int rate     : 3;  // baud rate control
  unsigned int spe      : 1;  // SPI enable
  unsigned int lsbfirst : 1;  // msb or lsb transmit first
  unsigned int ssi      : 1;  // internal slave select
  unsigned int ssm      : 1;  // software slave management
  unsigned int rxonly   : 1;  // receive only enabled
  unsigned int dff      : 1;  // data frame format
  unsigned int crcnext  : 1;  // transmit CRC next
  unsigned int crcen    : 1;  // CRC enable
  unsigned int bidioe   : 1;  // output enable in bidirectional mode
  unsigned int bidimode : 1;  // enable bidirectional mode (bit 15)
  
} SPI_CR1_reg;

// SPI control register CR2 structure 16 bit
typedef struct
{
  unsigned int rxdmaen  : 1;  // rx DMA buffer enable
  unsigned int txdmaen  : 1;  // tx DMA buffer enable
  unsigned int ssoe     : 1;  // SS output enable
  unsigned int res1     : 1;  // reserved
  unsigned int frf      : 1;  // frame format
  unsigned int errie    : 1;  // error interrupt enable
  unsigned int rxneie   : 1;  // rx buffer not empty interrupt enable
  unsigned int txeie    : 1;  // tx buffer empty interrupt enable
  unsigned int res2     : 8;  // reserved
  
} SPI_CR2_reg;

typedef struct
{
  unsigned int rxne      : 1;  // Receive buffer not empty
  unsigned int txe       : 1;  // Transmit buffer empty
  unsigned int chside    : 1;  // Channel side
  unsigned int udr       : 1;  // Underrun flag
  unsigned int crcerr    : 1;  // CRC error
  unsigned int modf      : 1;  // Mode fault
  unsigned int ovr       : 1;  // Overrun flag
  unsigned int bsy       : 1;  // Busy flag
  unsigned int fre       : 1;  // Frame format error
  unsigned int res1      : 7;  // reserved
  
} SPI_SR_reg;

// pointers to the registers
// SPI 1
extern SPI_CR1_reg *SPI1_CR1;
extern SPI_CR2_reg *SPI1_CR2;
extern uint16_t *SPI1_CRCPR;
// different sized pointers to the same data register
extern uint8_t  volatile *SPI1_DR8;
extern uint16_t volatile *SPI1_DR16;
extern SPI_SR_reg volatile *SPI1_SR;

// make global variables with the structures
extern SPI_CR1_reg  SPI1_CR1_temp;
extern SPI_CR2_reg  SPI1_CR2_temp;
extern SPI_SR_reg   SPI1_SR_temp;

// flag for ISR to send dummy tx for continuous reads
extern uint8_t spi_repeat_tx;

#define SPI_ERR_MASK   0x0178  // udr, CRC, Mode, odr, fre 

#endif /* CSI_SPI_H_ */
