// LCD DATA: PG0 - PG7

// read_switch_s1                  PB1
// read_switch_s2                  PB2
// read_switch_s3                  PE11
// read_switch_s4                  PE10
// read_switch_s5                  PE9
// read_switch_s6                  PE8           
// read_rotary_encoder_1_0         PE14
// read_rotary_encoder_1_1         PE15
// read_rotary_encoder_2_0         PE12
// read_rotary_encoder_2_1         PE13
//
// rx_led_on                       PD14
// rx_led_off                      PD14
// tx_led_on                       PD15
// tx_led_off                      PD15
// key_backlight_on                PC8
// key_backlight_off               PC8
// lcd_backlight_on                PF7
// lcd_backlight_off               PF7
//
// lcd_cs_high                     PB2
// lcd_cs_low                      PB2
// lcd_d_c_high                    PF11
// lcd_d_c_low                     PF11
// lcd_rd_high                     PF12
// lcd_rd_low                      PF12
// lcd_wr_high                     PF13
// lcd_wr_low                      PF13
// lcd_reset_high                  PF14
// lcd_reset_low                   PF14
// read_lcd_tear                   PF15
// read_lcd_port                   PG
// write_lcd_port                  PG

// read_mike_hook                  PD10
// read_mike_ptt                   PD9
// read_mike_dm                    PD12
// read_mike_mbl                   PD11

// mike_dm_high                    PD12
// mike_dm_low                     PD12
// mike_mbl_high                   PD11
// mike_mbl_low                    PD11

//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s1
//
// Read switch on rotary encoder 1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s1 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_B_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOB_IDR_ADDRESS
//    BIT01_MASK 
//
// History 
//
// Status
//  written
//
uint32_t read_switch_s1(void)
{
  uint32_t volatile *GPIO_B_IDR = (uint32_t *) GPIOB_IDR_ADDRESS;
  return (*GPIO_B_IDR) & BIT01_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s2
//
// Read switch on rotary encoder 2.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s2 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_B_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOB_IDR_ADDRESS
//    BIT00_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_switch_s2(void)
{
  uint32_t volatile *GPIO_B_IDR = (uint32_t *) GPIOB_IDR_ADDRESS;
  return (*GPIO_B_IDR) & BIT00_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s3
//
// Read switch s3.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s3 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT11_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_switch_s3(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT11_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s4
//
// Read switch s4.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s4 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT10_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_switch_s4(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT10_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s5
//
// Read switch s5.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s5 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT09_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_switch_s5(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT09_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_switch_s6
//
// Read switch s6.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns switch s6 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT08_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_switch_s6(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT08_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_1_0
//
// Read rotary encoder signal 1_0.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns rotary_encoder_1_0 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT14_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_rotary_encoder_1_0(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT14_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_1_1
//
// Read rotary encoder signal 1_1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns rotary_encoder_1_1 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT15_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_rotary_encoder_1_1(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT15_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_2_0
//
// Read rotary encoder signal 2_0.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns rotary_encoder_2_0 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT12_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_rotary_encoder_2_0(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT12_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents         read_rotary_encoder_2_1
//
// Read rotary encoder signal 2_1.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns rotary_encoder_2_1 bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_E_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOE_IDR_ADDRESS
//    BIT13_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_rotary_encoder_2_1(void)
{
  uint32_t volatile *GPIO_E_IDR = (uint32_t *) GPIOE_IDR_ADDRESS;
  return (*GPIO_E_IDR) & BIT13_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		       rx_led_on
//
// Turns on RX LED.  RX LED is connected to Port D bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The RX LED is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT14_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void rx_led_on(void)
{
//
// Turn on RX LED by setting bit 14 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT14_ON;
}
//
//******************************************************************************
//
// Feb 19, 2016			Design Documents      		      rx_led_off
//
// Turns off RX LED.  RX LED is connected to Port D bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The RX LED is turned off
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT14_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void rx_led_off(void)
{
//
// Turn off RX LED by clearing bit 14 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT14_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		       tx_led_on
//
// Turns on TX LED.  TX LED is connected to Port D bit 15 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The TX LED is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT15_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void tx_led_on(void)
{
//
// Turn on TX LED by setting bit 14 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT15_ON;
}
//
//******************************************************************************
//
// Feb 19, 2016			Design Documents      		      tx_led_off
//
// Turns off TX LED.  TX LED is connected to Port D bit 15 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The TX LED is turned off
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT15_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void tx_led_off(void)
{
//
// Turn off TX LED by clearing bit 15 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT15_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		key_backlight_on
//
// Turns on the key backlight.  KLED is connected to Port C bit 8 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The Key backlight is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_C_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOC_BSRR_ADDRESS 
//    BIT08_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void key_backlight_on(void)
{
//
// Turn on key backlight by setting bit 8 on port C
//
  uint32_t volatile *GPIO_C_BSRR = (uint32_t *) GPIOC_BSRR_ADDRESS;
  *GPIO_C_BSRR = BIT08_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	       key_backlight_off
//
// Turns off the key backlight.  KLED is connected to Port C bit 8 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The Key backlight is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_C_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOC_BSRR_ADDRESS 
//    BIT08_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void key_backlight_off(void)
{
//
// Turn off key backlight by clearing bit 8 on port C
//
  uint32_t volatile *GPIO_C_BSRR = (uint32_t *) GPIOC_BSRR_ADDRESS;
  *GPIO_C_BSRR = BIT08_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		lcd_backlight_on
//
// Turns on the lcd backlight.  LCD_BACKLIGHT is connected to Port F bit 7 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD backlight is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT07_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_backlight_on(void)
{
//
// Turn on lcd backlight by setting bit 7 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT07_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	       lcd_backlight_off
//
// Turns off the lcd backlight.  LCD_BACKLIGHT is connected to Port F bit 7 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The lcd backlight is turned on
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT07_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_backlight_off(void)
{
//
// Turn off lcd backlight by clearing bit 7 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOC_BSRR_ADDRESS;
  *GPIO_C_BSRR = BIT07_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		     lcd_cs_high
//
// Set the LCD CS line high.  LCD_CS/ is connected to Port B bit 2 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD CS line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_B_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOB_BSRR_ADDRESS 
//    BIT02_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_cs_high(void)
{
//
// Set the CS line high by setting bit 2 on port B
//
  uint32_t volatile *GPIO_B_BSRR = (uint32_t *) GPIOB_BSRR_ADDRESS;
  *GPIO_B_BSRR = BIT02_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_cs_low
//
// Set the LCD CS line low.  LCD_CS/ is connected to Port B bit 2 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD CS line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_B_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOB_BSRR_ADDRESS 
//    BIT07_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_cs_low(void)
{
//
// Set the LCD CS line low by clearing bit 2 on port B
//
  uint32_t volatile *GPIO_B_BSRR = (uint32_t *) GPIOB_BSRR_ADDRESS;
  *GPIO_B_BSRR = BIT07_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_d_c_high
//
// Set the LCD D/C line high.  LCD_D/C is connected to Port F bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD D/C line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT11_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_d_c_high(void)
{
//
// Set the LCD D_C line high by setting bit 11 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT11_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	             lcd_d_c_low
//
// Set the LCD D/C line low.  LCD_D/C is connected to Port F bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD D/C line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT11_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_d_c_low(void)
{
//
// Set the LCD D/C line low by clearing bit 11 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT11_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_rd_high
//
// Set the LCD RD/ line high.  LCD_RD/ is connected to Port F bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD RD/ line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT12_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_rd_high(void)
{
//
// Set the LCD RD/ line high by setting bit 12 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT12_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_rd_low
//
// Set the LCD RD/ line low.  LCD_RD is connected to Port F bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD RD/ line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT12_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_rd_low(void)
{
//
// Set the LCD RD/ line low by clearing bit 12 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT12_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    lcd_wr_high
//
// Set the LCD WR/ line high.  LCD_WR/ is connected to Port F bit 13 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD WR/ line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT13_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_wr_high(void)
{
//
// Set the LCD WR/ line high by setting bit 13 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT13_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	              lcd_wr_low
//
// Set the LCD WR/ line low.  LCD_WR/ is connected to Port F bit 13 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD WR/ line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT13_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_wr_low(void)
{
//
// Set the LCD WR/ line low by clearing bit 13 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT13_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		  lcd_reset_high
//
// Set the LCD RESET line high.  LCD_RESET is connected to Port F bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD RESET line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT14_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_reset_high(void)
{
//
// Set the LCD RESET line high by setting bit 14 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT14_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	           lcd_reset_low
//
// Set the LCD RESET line low.  LCD_RESET is connected to Port F bit 14 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The LCD RESET line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_F_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT14_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void lcd_reset_low(void)
{
//
// Set the LCD RESET line low by clearing bit 14 on port F
//
  uint32_t volatile *GPIO_F_BSRR = (uint32_t *) GPIOF_BSRR_ADDRESS;
  *GPIO_F_BSRR = BIT14_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_lcd_tear 
//
// Read LCD TEAR.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns lcd_tear bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_F_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_IDR_ADDRESS
//    BIT15_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_lcd_tear(void)
{
  uint32_t volatile *GPIO_F_IDR = (uint32_t *) GPIOF_IDR_ADDRESS;
  return (*GPIO_F_IDR) & BIT15_MASK;   
}
//
//******************************************************************************
//
// Feb 20, 2018			Design Documents                   read_lcd_port 
//
// This data reads the microprocessor port attached to the LCD 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns microprocessor port attached to LCD display with unused bits set to
//  zero. The LCD is on the LSB of port G.
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_G_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_IDR_ADDRESS 
//
// History
//
// Status
//  written
//
uint32_t read_lcd_port(void)
{
  uint32_t volatile *GPIO_G_IDR = (uint32_t *) GPIOG_IDR_ADDRESS;
  return (*GPIO_G_IDR) & 0x000000FF;   
}

//
//******************************************************************************
//
// Feb 20, 2018			Design Documents                  write_lcd_port 
//
// This data writes to the microprocessor port attached to the LCD 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  Data supplied by calling program
//
// Exit Conditions
//  Output the data passed to this program to Port G. The MSB are set to zero at
//  this time so the other pins on this port cannot currently be used.
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_G_ODR
//
// Constants
//  defined in register_definitions.h
//    GPIOG_ODR_ADDRESS 
//
// History
//
// Status
//  written
//
void write_lcd_port(uint32_t lcd_data)
{
  uint32_t volatile *GPIO_G_ODR = (uint32_t *) GPIOG_ODR_ADDRESS;
  *GPIO_G_ODR = lcd_data & 0x000000FF;   
}

//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                  read_mike_hook
//
// Read MIKE HOOK.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns mike hook bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_D_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_IDR_ADDRESS
//    BIT10_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_mike_hook(void)
{
  uint32_t volatile *GPIO_D_IDR = (uint32_t *) GPIOD_IDR_ADDRESS;
  return (*GPIO_D_IDR) & BIT10_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_mike_ptt
//
// Read MIKE PTT.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns mike ptt bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_D_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_IDR_ADDRESS
//    BIT09_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_mike_ptt(void)
{
  uint32_t volatile *GPIO_D_IDR = (uint32_t *) GPIOD_IDR_ADDRESS;
  return (*GPIO_D_IDR) & BIT09_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                    read_mike_dm
//
// Read MIKE DM.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns mike dm bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_D_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_IDR_ADDRESS
//    BIT12_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_mike_dm(void)
{
  uint32_t volatile *GPIO_D_IDR = (uint32_t *) GPIOD_IDR_ADDRESS;
  return (*GPIO_D_IDR) & BIT12_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents                   read_mike_mbl
//
// Read MIKE MBL.
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by program
//
// Exit Conditions
//  Returns mike mbl bit with other bits in the register masked out 
//
// External variables used
//  None
//
// Internal variables used
//    uint32_t volatile *GPIO_D_IDR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_IDR_ADDRESS
//    BIT11_MASK 
//
// History
//
// Status
//  written
//
uint32_t read_mike_mbl(void)
{
  uint32_t volatile *GPIO_D_IDR = (uint32_t *) GPIOD_IDR_ADDRESS;
  return (*GPIO_D_IDR) & BIT11_MASK;   
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		    mike_dm_high
//
// Set the MIKE DM line high.  MIKE_DM is connected to Port D bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The MIKE_DM line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOF_BSRR_ADDRESS 
//    BIT12_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void mike_dm_high(void)
{
//
// Set the MIKE DM line high by setting bit 12 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT12_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	             mike_dm_low
//
// Set the MIKE DM line low.  MIKE_DM is connected to Port D bit 12 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The MIKE_DM line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT12_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void mike_dm_low(void)
{
//
// Set the MIKE DM line low by clearing bit 12 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT12_OFF;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      		   mike_mbl_high
//
// Set the MIKE MBL line high.  MIKE_MBL is connected to Port D bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The MIKE MBL line is set high
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT11_ON
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void mike_mbl_high(void)
{
//
// Set the MIKE MBL line high by setting bit 11 on port D
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT11_ON;
}
//
//******************************************************************************
//
// Feb 19, 2018			Design Documents      	            mike_mbl_low
//
// Set the MIKE MBL line low.  MIKE_MBL is connected to Port D bit 11 
//
// Programmer
//  Jerry Wanger
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions
//  All parameters supplied by the program
//
// Exit Conditions
//  The MIKE MBL line is set low
// 
// External variables used
//  None
// 
// Internal variables used
//  uint32_t volatile *GPIO_D_BSRR
//
// Constants
//  defined in register_definitions.h
//    GPIOD_BSRR_ADDRESS 
//    BIT11_OFF
//
// History
//
// Status
//  written
//
//******************************************************************************
//
void mike_mbl_low(void)
{
//
// Set the LCD WR/ line low by clearing bit 13 on port F
//
  uint32_t volatile *GPIO_D_BSRR = (uint32_t *) GPIOD_BSRR_ADDRESS;
  *GPIO_D_BSRR = BIT11_OFF;
}
