//       1          2        3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//
//
// Mar 19,2018			Design Documents                   cstartupCSI.c
//
// Initialize interrupt vectors and run IAR startup 
//
// For Large Display project  CPU STM32F405
//
// Programmer
//  Dean Graham
// 
// Copyright 
//  Connect Systems Inc.
//
// Entry conditions - none
//
// Exit Conditions
//   calls main
// 
// Functions used - iar_init_vfp
//                  iar_init_core
//
// History
//  03/19/2018 Copied from CS7000 project for STM32F7
//  
//==============================================================================
// NAME OF ROUTINE                      Function
// Status: written, tested, working for STM32F405
//
//   __iar_program_start              Program start function
//  interrupt_error                   interrupt without handler halt function
//
//==============================================================================
#include "CSI_REGISTERS.h"
#include "CSI_CORE_REGISTERS.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "CSI_TIMERS.h"
#include "CSI_Bootload_Jump.h"

extern uint32_t dtmf_codes_timer;
extern uint32_t WatchDogTimer;

// 
void interrupt_error( int );
//
// These routines below are interrupts that are never used. In the future if
// they are used, take them out of here and put them in CSI_FUNCTIONS.h
// !!! NO !!!
// These are the declarations of the interrupt routines that are all used
// in the interrupt vector table setup below.
// The definitions of most below are one line that calls the interrupt error
// function (at the bottom of this file). Interrupts that are used have 
// interrupt handling routines located in their respective modules. These
// interrupt routines are defined here as __weak so that the correct routine
// in their module overrides the weak definition.
// These all need to be here as place holders for the interrupt vector table.
// CSTACK must be used for the first IAR_program_start routine. The name is
// fixed and required by the IAR debugger
#pragma language=extended
#pragma segment="CSTACK"
extern void __iar_program_start(void);

extern void NMI_Handler(void);
extern void HardFault_Handler(void);
extern void MemManage_Handler(void);
extern void BusFault_Handler(void);
extern void UsageFault_Handler(void);
extern void SVC_Handler(void);
extern void DebugMon_Handler(void);
extern void PendSV_Handler(void);
extern void SysTick_Handler(void);
extern void WWDG_Handler(void);
extern void PVD_Handler(void);
extern void TAMP_STAMP_Handler(void);
extern void RTC_WKUP_Handler(void);
extern void FLASH_Handler(void);
extern void RCC_Handler(void);
extern void EXTI0_Handler(void);
extern void EXTI1_Handler(void);
extern void EXTI2_Handler(void);
extern void EXTI3_Handler(void);
extern void EXTI4_Handler(void);
extern void DMA1_STREAM0_Handler(void);
extern void DMA1_STREAM1_Handler(void);
extern void DMA1_STREAM2_Handler(void);
extern void DMA1_STREAM3_Handler(void);
extern void DMA1_STREAM4_Handler(void);
extern void DMA1_STREAM5_Handler(void);
extern void DMA1_STREAM6_Handler(void);
extern void ADC_Handler(void);
extern void CAN1_TX_Handler(void);
extern void CAN1_RX0_Handler(void);
extern void CAN1_RX1_Handler(void);
extern void CAN1_SCE_Handler(void);
extern void EXTI9_5_Handler(void);
extern void TIM1_BRK_TIM9_Handler(void);
extern void TIM1_UP_TIM10_Handler(void);
extern void TIM1_TRG_COM_TIM11_Handler(void);
extern void TIM1_CC_Handler(void);
extern void TIM2_Handler(void);
extern void TIM3_Handler(void);
extern void TIM4_Handler(void);
extern void I2C1_EV_Handler(void);
extern void I2C1_ER_Handler(void);
extern void I2C2_EV_Handler(void);
extern void I2C2_ER_Handler(void);
extern void SPI1_Handler(void);
extern void SPI2_Handler(void);
extern void USART1_Handler(void);
extern void USART2_Handler(void);
extern void USART3_Handler(void);
extern void EXTI15_10_Handler(void);
extern void RTC_Alarm_Handler(void);
extern void OTG_FS_WKUP_Handler(void);
extern void TIM8_BRK_TIM12_Handler(void);
extern void TIM8_UP_TIM13_Handler(void);
extern void TIM8_TRG_COM_TIM14_Handler(void);
extern void TIM8_CC_Handler(void);
extern void DMA1_Stream7_Handler(void);
extern void FSMC_Handler(void);
extern void SDIO_Handler(void);
extern void TIM5_Handler(void);
extern void SPI3_Handler(void);
extern void UART4_Handler(void);
extern void UART5_Handler(void);
extern void TIM6_DAC_Handler(void);
extern void TIM7_Handler(void);
extern void DMA2_Stream0_Handler(void);
extern void DMA2_Stream1_Handler(void);
extern void DMA2_Stream2_Handler(void);
extern void DMA2_Stream3_Handler(void);
extern void DMA2_Stream4_Handler(void);
extern void ETH_Handler(void);
extern void ETH_WKUP_Handler(void);
extern void CAN2_TX_Handler(void);
extern void CAN2_RX0_Handler(void);
extern void CAN2_RX1_Handler(void);
extern void CAN2_SCE_Handler(void);
extern void OTG_FS_Handler(void);
extern void DMA2_Stream5_Handler(void);
extern void DMA2_Stream6_Handler(void);
extern void DMA2_Stream7_Handler(void);
extern void USART6_Handler(void);
extern void I2C3_EV_Handler(void);
extern void I2C3_ER_Handler(void);
extern void OTG_HS_EP1_OUT_Handler(void);
extern void OTG_HS_EP1_IN_Handler(void);
extern void OTG_HS_WKUP_Handler(void);
extern void OTG_HS_Handler(void);
extern void DCMI_Handler(void);
extern void CRYP_Handler(void);
extern void HASH_RNG_Handler(void);
extern void FPU_Handler(void);

typedef void( *intfunc )(void);
typedef union { intfunc __fun; void * __ptr; } intvec_elem;

// The vector table is normally located at address 0.
// When debugging in RAM, it can be located in RAM, aligned to at least 2^6.
// If you need to define interrupt service routines,
// make a copy of this file and include it in your project.
// The name "__vector_table" has special meaning for C-SPY, which
// is where to find the SP start value.
// If vector table is not located at address 0, the user has to initialize
// the  NVIC vector table register (VTOR) before using interrupts.

#pragma location = ".intvec"
const intvec_elem __vector_table[] =
{                             //Acronym            Address   Priority  Type
  { .__ptr = __sfe( "CSTACK" )},
  __iar_program_start,        //Reset              0004      -3        Fixed             
  NMI_Handler,                //NMI                0008      -2        Fixed
  HardFault_Handler,          //HardFault          000C      -1        Fixed
  MemManage_Handler,          //MemManage          0010       0        Settable
  BusFault_Handler,           //BusFault           0014       1        Settable
  UsageFault_Handler,         //UsageFault         0018       2        Settable
  0,                          //Reserved           001C       -        -
  0,                          //Reserved           0020       -        -
  0,                          //Reserved           0024       -        -
  0,                          //Reserved           0028       -        -
  SVC_Handler,                //SVCall             002C       3        Settable
  DebugMon_Handler,           //DebugMonitor       0030       4        Settable
  0,                          //Reserved           0034       -        -
  PendSV_Handler,             //PendSV             0038       5        Settable
  SysTick_Handler,            //SysTick            003C       6        Settable
  WWDG_Handler,               //WWDG               0040       7        Settable
  PVD_Handler,                //PVD                0044       8        Settable
  TAMP_STAMP_Handler,         //TampStamp          0048       9        Settable
  RTC_WKUP_Handler,           //RTC_WKUP           004C       10       Settable
  FLASH_Handler,              //FLASH              0050       11       Settable
  RCC_Handler,                //RCC                0054       12       Settable
  EXTI0_Handler,              //EXTI0              0058       13       Settable
  EXTI1_Handler,              //EXTI1              005C       14       Settable
  EXTI2_Handler,              //EXTI2              0060       15       Settable
  EXTI3_Handler,              //EXTI3              0064       16       Settable
  EXTI4_Handler,              //EXTI4              0068       17       Settable
  DMA1_STREAM0_Handler,       //DMA1_Stream0       006C       18       Settable
  DMA1_STREAM1_Handler,       //DMA1_Stream1       0070       19       Settable
  DMA1_STREAM2_Handler,       //DMA1_Stream2       0074       20       Settable
  DMA1_STREAM3_Handler,       //DMA1_Stream3       0078       21       Settable
  DMA1_STREAM4_Handler,       //DMA1_Stream4       007C       22       Settable
  DMA1_STREAM5_Handler,       //DMA1_Stream5       0080       23       Settable
  DMA1_STREAM6_Handler,       //DMA1_Stream6       0084       24       Settable
  ADC_Handler,                //ADC                0088       25       Settable
  CAN1_TX_Handler,            //CAN1_TX            008C       26       Settable
  CAN1_RX0_Handler,           //CAN1_RX0           0090       27       Settable
  CAN1_RX1_Handler,           //CAN1_RX1           0094       28       Settable
  CAN1_SCE_Handler,           //CAN1_SCE           0098       29       Settable
  EXTI9_5_Handler,            //EXTI9_5            009C       30       Settable
  TIM1_BRK_TIM9_Handler,      //TIM1_BRK_TIM9      00A0       31       Settable
  TIM1_UP_TIM10_Handler,      //TIM1_up_TIM10      00A4       32       Settable
  TIM1_TRG_COM_TIM11_Handler, //TIM1_TRG_COM_TIM11 00A8       33       Settable
  TIM1_CC_Handler,            //TIM1_CC            00AC       34       Settable
  TIM2_Handler,               //TIM2               00B0       35       Settable 
  TIM3_Handler,               //TIM3               00B4       36       Settable
  TIM4_Handler,               //TIM4               00B8       37       Settable
  I2C1_EV_Handler,            //I2C1_EV            00BC       38       Settable
  I2C1_ER_Handler,            //I2C1_ER            00C0       39       Settable
  I2C2_EV_Handler,            //I2C1_EV            00C4       40       Settable
  I2C2_ER_Handler,            //I2C1_EV            00C8       41       Settable
  SPI1_Handler,               //SPI1               00CC       42       Settable
  SPI2_Handler,               //SPI2               00D0       43       Settable
  USART1_Handler,             //USART1             00D4       44       Settable
  USART2_Handler,             //USART2             00D8       45       Settable
  USART3_Handler,             //USART3             00DC       46       Settable
  EXTI15_10_Handler,          //EXTI15_10          00E0       47       Settable
  RTC_Alarm_Handler,          //RTC_Alarm          00E4       48       Settable
  OTG_FS_WKUP_Handler,        //OTG_FS_WKUP        00E8       49       Settable
  TIM8_BRK_TIM12_Handler,     //TIM8_BRK_TIM12     00EC       50       Settable
  TIM8_UP_TIM13_Handler,      //TIM8_UP_TIM13      00F0       51       Settable
  TIM8_TRG_COM_TIM14_Handler, //TIM8_TRG_COM_TIM14 00F4       52       Settable
  TIM8_CC_Handler,            //TIM8_CC            00F8       53       Settable
  DMA1_Stream7_Handler,       //DMA1_Stream7       00FC       54       Settable
  FSMC_Handler,               //FSMC               0100       55       Settable
  SDIO_Handler,               //SDIO               0104       56       Settable
  TIM5_Handler,               //TIM5               0108       57       Settable
  SPI3_Handler,               //SPI3               010C       58       Settable
  UART4_Handler,              //UART4              0110       59       Settable
  UART5_Handler,              //UART5              0114       60       Settable
  TIM6_DAC_Handler,           //TIM6_DAC           0118       61       Settable
  TIM7_Handler,               //TIM7               011C       62       Settable
  DMA2_Stream0_Handler,       //DMA2_Stream0       0120       63       Settable
  DMA2_Stream1_Handler,       //DMA2_Stream1       0124       64       Settable
  DMA2_Stream2_Handler,       //DMA2_Stream2       0128       65       Settable
  DMA2_Stream3_Handler,       //DMA2_Stream3       012C       66       Settable
  DMA2_Stream4_Handler,       //DMA2_Stream4       0130       67       Settable
  ETH_Handler,                //ETH                0134       68       Settable
  ETH_WKUP_Handler,           //ETH_WKUP           0138       69       Settable
  CAN2_TX_Handler,            //CAN2_TX            013C       70       Settable
  CAN2_RX0_Handler,           //CAN2_RX0           0140       71       Settable
  CAN2_RX1_Handler,           //CAN2_RX1           0144       72       Settable
  CAN2_SCE_Handler,           //CAN2_SCE           0148       73       Settable
  OTG_FS_Handler,             //OTG_FS             014C       74       Settable
  DMA2_Stream5_Handler,       //DMA2_Stream5       0150       75       Settable
  DMA2_Stream6_Handler,       //DMA2_Stream6       0154       76       Settable
  DMA2_Stream7_Handler,       //DMA2_Stream7       0158       77       Settable
  USART6_Handler,             //USART6             015C       78       Settable
  I2C3_EV_Handler,            //I2C3_EV            0160       79       Settable
  I2C3_ER_Handler,            //I2C3_ER            0164       80       Settable
  OTG_HS_EP1_OUT_Handler,     //OTG_HS_EP1_OUT     0168       81       Settable
  OTG_HS_EP1_IN_Handler,      //OTG_HS_EP1_IN      016C       82       Settable
  OTG_HS_WKUP_Handler,        //OTG_HS_WKUP        0170       83       Settable
  OTG_HS_Handler,             //OTG_HS             0174       84       Settable
  DCMI_Handler,               //DCMI               0178       85       Settable
  CRYP_Handler,               //CRYP               017C       86       Settable
  HASH_RNG_Handler,           //HASH_RNG           0180       87       Settable
  FPU_Handler                 //FPU                0184       88       Settable
};
//
//interrupts are sent to interrupt_error unless interrupt handlers are written
// 
//#pragma call_graph_root = "interrupt"
//void __iar_program_start(void) {interrupt_error(-3);}
#pragma call_graph_root = "interrupt"
void NMI_Handler(void){ interrupt_error(-2);}
#pragma call_graph_root = "interrupt"
void HardFault_Handler(void)  {interrupt_error(-1);}
#pragma call_graph_root = "interrupt"
void MemManage_Handler(void)  {interrupt_error(0);}
#pragma call_graph_root = "interrupt"
void BusFault_Handler(void)   {interrupt_error(1);}
#pragma call_graph_root = "interrupt"
void UsageFault_Handler(void) {interrupt_error(2);}
#pragma call_graph_root = "interrupt"
void SVC_Handler(void)        {interrupt_error(3);}
#pragma call_graph_root = "interrupt"
void DebugMon_Handler(void)   {interrupt_error(4);}
#pragma call_graph_root = "interrupt"
void PendSV_Handler(void)     {interrupt_error(5);}
#pragma call_graph_root = "interrupt"
__weak void SysTick_Handler(void)    {interrupt_error(6);}
//
#pragma call_graph_root = "interrupt"
void WWDG_Handler(void)       {interrupt_error(7);}
#pragma call_graph_root = "interrupt"
void PVD_Handler(void)        {interrupt_error(8);}
#pragma call_graph_root = "interrupt"
void TAMP_STAMP_Handler(void) {interrupt_error(9);}
#pragma call_graph_root = "interrupt"
void RTC_WKUP_Handler(void)   {interrupt_error(10);}
#pragma call_graph_root = "interrupt"
void FLASH_Handler(void)      {interrupt_error(11);}
#pragma call_graph_root = "interrupt"
void RCC_Handler(void)        {interrupt_error(12);}
#pragma call_graph_root = "interrupt"
void EXTI0_Handler(void)      {interrupt_error(13);}
#pragma call_graph_root = "interrupt"
void EXTI1_Handler(void)      {interrupt_error(14);}
#pragma call_graph_root = "interrupt"
__weak void EXTI2_Handler(void)      {interrupt_error(15);}
#pragma call_graph_root = "interrupt"
void EXTI3_Handler(void)      {interrupt_error(16);}
#pragma call_graph_root = "interrupt"
void EXTI4_Handler(void)      {interrupt_error(17);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM0_Handler(void) {interrupt_error(18);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM1_Handler(void) {interrupt_error(19);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM2_Handler(void) {interrupt_error(20);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM3_Handler(void) {interrupt_error(21);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM4_Handler(void) {interrupt_error(22);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM5_Handler(void) {interrupt_error(23);}
#pragma call_graph_root = "interrupt"
void DMA1_STREAM6_Handler(void) {interrupt_error(24);}
#pragma call_graph_root = "interrupt"
void ADC_Handler(void)          {interrupt_error(25);}
#pragma call_graph_root = "interrupt"
void CAN1_TX_Handler(void)      {interrupt_error(26);} 
#pragma call_graph_root = "interrupt"
void CAN1_RX0_Handler(void)     {interrupt_error(27);}
#pragma call_graph_root = "interrupt"
void CAN1_RX1_Handler(void)     {interrupt_error(28);}
#pragma call_graph_root = "interrupt"
 void CAN1_SCE_Handler(void)    {interrupt_error(29);}
#pragma call_graph_root = "interrupt"
__weak void EXTI9_5_Handler(void)     {interrupt_error(30);}
#pragma call_graph_root = "interrupt"
 // Interrupt service routine in CSI_interrupt.c
 // EXTI6 for PIN F[6] Touch Panel 
 // Interrupt #23
 // see pg.291 of Reference Manual
 // priority of this is changed in the initialize to 50

// GPIO interrupts  
 void TIM1_BRK_TIM9_Handler(void)  {interrupt_error(31);}
#pragma call_graph_root = "interrupt"
 void TIM1_UP_TIM10_Handler(void)  {interrupt_error(32);}
#pragma call_graph_root = "interrupt"
 void TIM1_TRG_COM_TIM11_Handler(void) {interrupt_error(33);}
#pragma call_graph_root = "interrupt"
 void TIM1_CC_Handler(void)     {interrupt_error(34);}
#pragma call_graph_root = "interrupt"
__weak  void TIM2_Handler(void) {interrupt_error(35);}
// ISR in timer_two.c
// interruput #28
#pragma call_graph_root = "interrupt"
__weak  void TIM3_Handler(void)        {interrupt_error(36);}
// ISR in timer_two.c
// interruput #29
#pragma call_graph_root = "interrupt"
 void TIM4_Handler(void)        {interrupt_error(37);}
#pragma call_graph_root = "interrupt"
 void I2C1_EV_Handler(void)     {interrupt_error(38);}
#pragma call_graph_root = "interrupt"
 void I2C1_ER_Handler(void)     {interrupt_error(39);}
#pragma call_graph_root = "interrupt"
 __weak void I2C2_EV_Handler(void) {interrupt_error(40);}
 // Event handler routine is in CSI_i2c.c
 // I2C2 Interrupt Event handler
 // Interrupt #33
 // see pg.860 of Refernce Manual
#pragma call_graph_root = "interrupt"
 __weak  void I2C2_ER_Handler(void) {interrupt_error(41);}
 // Interrupt service routine is in CSI_i2c.c
 // I2C2 Interrupt Error handler
 // Interrupt #34
 // see pg.860 of Refernce Manual
#pragma call_graph_root = "interrupt"
__weak void SPI1_Handler(void)   {interrupt_error(42);}
// Interrupt service routine is in spi.c
// SPI 1 for FLASH memory
// Interrupt #35
// see pg.901 of Refernce Manual
#pragma call_graph_root = "interrupt"
 void SPI2_Handler(void)        {interrupt_error(43);}
#pragma call_graph_root = "interrupt"
__weak void USART1_Handler(void) {interrupt_error(44);}
// Interrupt service routine is in CSI_UART1.C
// USART 1 for main Radio connection
// Interrupt #37
// see pg.1009 of Refernce Manual
#pragma call_graph_root = "interrupt"
__weak void USART2_Handler(void) {interrupt_error(45);}
// USART2 future 
#pragma call_graph_root = "interrupt"
 void USART3_Handler(void)      {interrupt_error(46);}
#pragma call_graph_root = "interrupt"
 __weak  void EXTI15_10_Handler(void)   {interrupt_error(47);}
#pragma call_graph_root = "interrupt"
 void RTC_Alarm_Handler(void)   {interrupt_error(48);}
#pragma call_graph_root = "interrupt"
 void OTG_FS_WKUP_Handler(void) {interrupt_error(49);}
#pragma call_graph_root = "interrupt"
 void TIM8_BRK_TIM12_Handler(void){interrupt_error(50);}
#pragma call_graph_root = "interrupt"
 void TIM8_UP_TIM13_Handler(void) {interrupt_error(51);}
#pragma call_graph_root = "interrupt"
 void TIM8_TRG_COM_TIM14_Handler(void){interrupt_error(52);}
#pragma call_graph_root = "interrupt"
 void TIM8_CC_Handler(void)      {interrupt_error(53);}
#pragma call_graph_root = "interrupt"
 void DMA1_Stream7_Handler(void) {interrupt_error(54);}
#pragma call_graph_root = "interrupt"
 void FSMC_Handler(void)         {interrupt_error(55);}
#pragma call_graph_root = "interrupt"
 void SDIO_Handler(void)         {interrupt_error(56);}
#pragma call_graph_root = "interrupt"
 void TIM5_Handler(void)         {interrupt_error(57);}
#pragma call_graph_root = "interrupt"
 void SPI3_Handler(void)         {interrupt_error(58);}
#pragma call_graph_root = "interrupt"
 void UART4_Handler(void)        {interrupt_error(59);}
#pragma call_graph_root = "interrupt"
 void UART5_Handler(void)        {interrupt_error(60);}
#pragma call_graph_root = "interrupt"
 void TIM6_DAC_Handler(void)     {interrupt_error(61);}
#pragma call_graph_root = "interrupt"
 void TIM7_Handler(void)         {interrupt_error(62);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream0_Handler(void) {interrupt_error(63);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream1_Handler(void) {interrupt_error(64);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream2_Handler(void) {interrupt_error(65);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream3_Handler(void) {interrupt_error(66);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream4_Handler(void) {interrupt_error(67);}
#pragma call_graph_root = "interrupt"
 void ETH_Handler(void)          {interrupt_error(68);}
#pragma call_graph_root = "interrupt"
 void ETH_WKUP_Handler(void)     {interrupt_error(69);}
#pragma call_graph_root = "interrupt"
 void CAN2_TX_Handler(void)      {interrupt_error(70);}
#pragma call_graph_root = "interrupt"
 void CAN2_RX0_Handler(void)     {interrupt_error(71);}
#pragma call_graph_root = "interrupt"
 void CAN2_RX1_Handler(void)     {interrupt_error(72);}
#pragma call_graph_root = "interrupt"
 void CAN2_SCE_Handler(void)     {interrupt_error(73);}
#pragma call_graph_root = "interrupt"
 void OTG_FS_Handler(void)       {interrupt_error(74);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream5_Handler(void) {interrupt_error(75);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream6_Handler(void) {interrupt_error(76);}
#pragma call_graph_root = "interrupt"
 void DMA2_Stream7_Handler(void) {interrupt_error(77);}
#pragma call_graph_root = "interrupt"
 void USART6_Handler(void)       {interrupt_error(78);}
#pragma call_graph_root = "interrupt"
 void I2C3_EV_Handler(void)      {interrupt_error(79);}
#pragma call_graph_root = "interrupt"
 void I2C3_ER_Handler(void)      {interrupt_error(80);}
#pragma call_graph_root = "interrupt"
 void OTG_HS_EP1_OUT_Handler(void) {interrupt_error(81);}
// USB interrupt will be here
#pragma call_graph_root = "interrupt"
 void OTG_HS_EP1_IN_Handler(void)  {interrupt_error(82);}
// USB interrupt will be here
#pragma call_graph_root = "interrupt"
 void OTG_HS_WKUP_Handler(void)    {interrupt_error(83);}
// USB interrupt will be here
#pragma call_graph_root = "interrupt"
 void OTG_HS_Handler(void)      {interrupt_error(84);}
// USB interrupt will be here
#pragma call_graph_root = "interrupt"
 void DCMI_Handler(void)        {interrupt_error(85);}
#pragma call_graph_root = "interrupt"
 void CRYP_Handler(void)        {interrupt_error(86);}
// 80
#pragma call_graph_root = "interrupt"
 void HASH_RNG_Handler(void)    {interrupt_error(87);}
#pragma call_graph_root = "interrupt"
 void FPU_Handler(void)         {interrupt_error(88);}
// =======================================================
//
// call iar init routines
//
//   if buttons P1 an P4 are held at startup it runs the bootloader,
//   otherwise it runs the c main prgram.

void __cmain(void);
__weak void __iar_init_core(void);
__weak void __iar_init_vfp(void);
extern void SystemInit(void);

#pragma required = __vector_table

void __iar_program_start(void)
{
  uint32_t volatile *PWR_CR         = (uint32_t *) PWR_CR_ADDRESS;
  uint32_t volatile *RCC_AHB1ENR    = (uint32_t *) RCC_AHB1ENR_ADDRESS;
  uint32_t volatile *RCC_APB1ENR    = (uint32_t *) RCC_APB1ENR_ADDRESS;
  uint32_t volatile *GPIOE_PUPDR    = (uint32_t *) GPIOE_PUPDR_ADDRESS;
  uint32_t volatile *GPIOE_IDR      = (uint32_t *) GPIOE_IDR_ADDRESS;
  
  // turn on some clocks to be able to check GPIO pins
  *PWR_CR      = 0x00004000;  // PWR Power control register
  *RCC_AHB1ENR = 0x0010007F;  // RCC AHB1 peripheral clock enable register
                              // CCMDATA RAM, GPIO A-G on
  *RCC_APB1ENR = 0x10000000;  // RCC APB1 peripheral clock enable register
                              // PWR bit
 
  // set pull ups on gpio for buttons
  *GPIOE_PUPDR = 0x55550000; // Pull Up-Pull Down Register
  
  // check if buttons are pressed for jump to bootloader
  int buttons_pressed = 0;
  int i;
  // fast debounce, check each switch 10 times.
  // if they are both pressed buttons_pressed will be 10.
  // The high speed cpu clock has not been set.
  // The starting clock is 16 Mhz 62.5 nsec per cycle.
  // delay() has 14 NOP. 875 nsec per delay 87.5 usec x 10  .875 msec 

  // Can not use the GPIO read switch routines as the variables needed have
  // not been initialized yet.
  // Pressing P1 and P4 at the same time will cause the jump to the bootloader.
  // P1 and P4 are on bit 8 and bit 11 of port E so, 0x00000900 is used to test
  // both buttons. When a button is pressed the bit changes to a zero.
  //  
  for( i = 0; i < 10; i++)
  {
    // check if both buttons are 0, pressed 
    if (!(*GPIOE_IDR & 0x00000900)) buttons_pressed++;
    delay(100);
  }
  if( buttons_pressed == 10) Bootloader_Jump();
  
  // Standard program start 
  SystemInit();

   __iar_init_core();
   __iar_init_vfp();
  // relocate vector table not being done now
  //uint32_t volatile *VTOR = (uint32_t *) VTOR_ADDRESS;   
  //*VTOR = 0x20000000;
  __cmain();
}
// ===================================================
//
// Interrupt error reporting and debug routine
// all interrupts without valid interrupt handlers are sent here
//
uint32_t ENum;
void interrupt_error( int ErrorNum )
{ 
  // if error needs to go to debugging screen then use printf
  // printf("error on interrupt %d \n", ErrorNum);

  // show error on lcd
  //sprintf(display_line1, "Error: %4d\n", ErrorNum);
  //strcpy(display_line2, "Program Halted\n");
  //clear_display();
  //display_string(0, 0, display_line2);  
  //display_string(0, 2, display_line1);
  
  ENum = ErrorNum;
  // do nothing Halt
  while (1) {};
}

